@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2015, 2020, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -24,16 +24,15 @@
  */
 
 #include "precompiled.hpp"
+#include "pauth_aarch64.hpp"
 #include "runtime/arguments.hpp"
 #include "runtime/globals_extension.hpp"
 #include "runtime/java.hpp"
-#include "runtime/os.hpp"
+#include "runtime/os.inline.hpp"
 #include "runtime/vm_version.hpp"
 #include "utilities/formatBuffer.hpp"
 #include "utilities/macros.hpp"
 
-#include OS_HEADER_INLINE(os)
-
 int VM_Version::_cpu;
 int VM_Version::_model;
 int VM_Version::_model2;
@@ -45,6 +44,8 @@ int VM_Version::_zva_length;
 int VM_Version::_dcache_line_size;
 int VM_Version::_icache_line_size;
 int VM_Version::_initial_sve_vector_length;
+bool VM_Version::_rop_protection;
+uintptr_t VM_Version::_pac_mask;
 
 SpinWait VM_Version::_spin_wait;
 
@@ -123,7 +124,7 @@ void VM_Version::initialize() {
     // if dcpop is available publish data cache line flush size via
     // generic field, otherwise let if default to zero thereby
     // disabling writeback
-    if (_features & CPU_DCPOP) {
+    if (VM_Version::supports_dcpop()) {
       _data_cache_line_flush_size = dcache_line;
     }
   }
@@ -197,8 +198,10 @@ void VM_Version::initialize() {
     }
   }
 
-  // Neoverse N1
-  if (_cpu == CPU_ARM && (_model == 0xd0c || _model2 == 0xd0c)) {
+  // Neoverse N1, N2 and V1
+  if (_cpu == CPU_ARM && ((_model == 0xd0c || _model2 == 0xd0c)
+                          || (_model == 0xd49 || _model2 == 0xd49)
+                          || (_model == 0xd40 || _model2 == 0xd40))) {
     if (FLAG_IS_DEFAULT(UseSIMDForMemoryOps)) {
       FLAG_SET_DEFAULT(UseSIMDForMemoryOps, true);
     }
@@ -218,22 +221,20 @@ void VM_Version::initialize() {
     }
   }
 
-  if (_cpu == CPU_ARM && (_model == 0xd07 || _model2 == 0xd07)) _features |= CPU_STXR_PREFETCH;
-
   char buf[512];
   sprintf(buf, "0x%02x:0x%x:0x%03x:%d", _cpu, _variant, _model, _revision);
   if (_model2) sprintf(buf+strlen(buf), "(0x%03x)", _model2);
-#define ADD_FEATURE_IF_SUPPORTED(id, name, bit) if (_features & CPU_##id) strcat(buf, ", " name);
+#define ADD_FEATURE_IF_SUPPORTED(id, name, bit) if (VM_Version::supports_##name()) strcat(buf, ", " #name);
   CPU_FEATURE_FLAGS(ADD_FEATURE_IF_SUPPORTED)
 #undef ADD_FEATURE_IF_SUPPORTED
 
   _features_string = os::strdup(buf);
 
   if (FLAG_IS_DEFAULT(UseCRC32)) {
-    UseCRC32 = (_features & CPU_CRC32) != 0;
+    UseCRC32 = VM_Version::supports_crc32();
   }
 
-  if (UseCRC32 && (_features & CPU_CRC32) == 0) {
+  if (UseCRC32 && !VM_Version::supports_crc32()) {
     warning("UseCRC32 specified, but not supported on this CPU");
     FLAG_SET_DEFAULT(UseCRC32, false);
   }
@@ -247,7 +248,7 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseVectorizedMismatchIntrinsic, false);
   }
 
-  if (_features & CPU_LSE) {
+  if (VM_Version::supports_lse()) {
     if (FLAG_IS_DEFAULT(UseLSE))
       FLAG_SET_DEFAULT(UseLSE, true);
   } else {
@@ -257,7 +258,7 @@ void VM_Version::initialize() {
     }
   }
 
-  if (_features & CPU_AES) {
+  if (VM_Version::supports_aes()) {
     UseAES = UseAES || FLAG_IS_DEFAULT(UseAES);
     UseAESIntrinsics =
         UseAESIntrinsics || (UseAES && FLAG_IS_DEFAULT(UseAESIntrinsics));
@@ -288,7 +289,7 @@ void VM_Version::initialize() {
     UseCRC32Intrinsics = true;
   }
 
-  if (_features & CPU_CRC32) {
+  if (VM_Version::supports_crc32()) {
     if (FLAG_IS_DEFAULT(UseCRC32CIntrinsics)) {
       FLAG_SET_DEFAULT(UseCRC32CIntrinsics, true);
     }
@@ -305,7 +306,8 @@ void VM_Version::initialize() {
     UseMD5Intrinsics = true;
   }
 
-  if (_features & (CPU_SHA1 | CPU_SHA2 | CPU_SHA3 | CPU_SHA512)) {
+  if (VM_Version::supports_sha1() || VM_Version::supports_sha256() ||
+      VM_Version::supports_sha3() || VM_Version::supports_sha512()) {
     if (FLAG_IS_DEFAULT(UseSHA)) {
       FLAG_SET_DEFAULT(UseSHA, true);
     }
@@ -314,7 +316,7 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseSHA, false);
   }
 
-  if (UseSHA && (_features & CPU_SHA1)) {
+  if (UseSHA && VM_Version::supports_sha1()) {
     if (FLAG_IS_DEFAULT(UseSHA1Intrinsics)) {
       FLAG_SET_DEFAULT(UseSHA1Intrinsics, true);
     }
@@ -323,7 +325,7 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseSHA1Intrinsics, false);
   }
 
-  if (UseSHA && (_features & CPU_SHA2)) {
+  if (UseSHA && VM_Version::supports_sha256()) {
     if (FLAG_IS_DEFAULT(UseSHA256Intrinsics)) {
       FLAG_SET_DEFAULT(UseSHA256Intrinsics, true);
     }
@@ -332,7 +334,7 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseSHA256Intrinsics, false);
   }
 
-  if (UseSHA && (_features & CPU_SHA3)) {
+  if (UseSHA && VM_Version::supports_sha3()) {
     // Do not auto-enable UseSHA3Intrinsics until it has been fully tested on hardware
     // if (FLAG_IS_DEFAULT(UseSHA3Intrinsics)) {
       // FLAG_SET_DEFAULT(UseSHA3Intrinsics, true);
@@ -342,7 +344,7 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseSHA3Intrinsics, false);
   }
 
-  if (UseSHA && (_features & CPU_SHA512)) {
+  if (UseSHA && VM_Version::supports_sha512()) {
     // Do not auto-enable UseSHA512Intrinsics until it has been fully tested on hardware
     // if (FLAG_IS_DEFAULT(UseSHA512Intrinsics)) {
       // FLAG_SET_DEFAULT(UseSHA512Intrinsics, true);
@@ -356,7 +358,7 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseSHA, false);
   }
 
-  if (_features & CPU_PMULL) {
+  if (VM_Version::supports_pmull()) {
     if (FLAG_IS_DEFAULT(UseGHASHIntrinsics)) {
       FLAG_SET_DEFAULT(UseGHASHIntrinsics, true);
     }
@@ -381,18 +383,26 @@ void VM_Version::initialize() {
     FLAG_SET_DEFAULT(UseBlockZeroing, false);
   }
 
-  if (_features & CPU_SVE) {
+  if (VM_Version::supports_sve2()) {
     if (FLAG_IS_DEFAULT(UseSVE)) {
-      FLAG_SET_DEFAULT(UseSVE, (_features & CPU_SVE2) ? 2 : 1);
+      FLAG_SET_DEFAULT(UseSVE, 2);
     }
-    if (UseSVE > 0) {
-      _initial_sve_vector_length = get_current_sve_vector_length();
+  } else if (VM_Version::supports_sve()) {
+    if (FLAG_IS_DEFAULT(UseSVE)) {
+      FLAG_SET_DEFAULT(UseSVE, 1);
+    } else if (UseSVE > 1) {
+      warning("SVE2 specified, but not supported on current CPU. Using SVE.");
+      FLAG_SET_DEFAULT(UseSVE, 1);
     }
   } else if (UseSVE > 0) {
     warning("UseSVE specified, but not supported on current CPU. Disabling SVE.");
     FLAG_SET_DEFAULT(UseSVE, 0);
   }
 
+  if (UseSVE > 0) {
+    _initial_sve_vector_length = get_current_sve_vector_length();
+  }
+
   // This machine allows unaligned memory accesses
   if (FLAG_IS_DEFAULT(UseUnalignedAccesses)) {
     FLAG_SET_DEFAULT(UseUnalignedAccesses, true);
@@ -407,6 +417,48 @@ void VM_Version::initialize() {
     UsePopCountInstruction = true;
   }
 
+  if (UseBranchProtection == nullptr || strcmp(UseBranchProtection, "none") == 0) {
+    _rop_protection = false;
+  } else if (strcmp(UseBranchProtection, "standard") == 0) {
+    _rop_protection = false;
+    // Enable PAC if this code has been built with branch-protection, the CPU/OS
+    // supports it, and incompatible preview features aren't enabled.
+#ifdef __ARM_FEATURE_PAC_DEFAULT
+    if (VM_Version::supports_paca() && !Arguments::enable_preview()) {
+      _rop_protection = true;
+    }
+#endif
+  } else if (strcmp(UseBranchProtection, "pac-ret") == 0) {
+    _rop_protection = true;
+#ifdef __ARM_FEATURE_PAC_DEFAULT
+    if (!VM_Version::supports_paca()) {
+      warning("ROP-protection specified, but not supported on this CPU.");
+      // Disable PAC to prevent illegal instruction crashes.
+      _rop_protection = false;
+    } else if (Arguments::enable_preview()) {
+      // Not currently compatible with continuation freeze/thaw.
+      warning("PAC-RET is incompatible with virtual threads preview feature.");
+      _rop_protection = false;
+    }
+#else
+    warning("ROP-protection specified, but this VM was built without ROP-protection support.");
+#endif
+  } else {
+    vm_exit_during_initialization(err_msg("Unsupported UseBranchProtection: %s", UseBranchProtection));
+  }
+
+  if (_rop_protection == true) {
+    // Determine the mask of address bits used for PAC. Clear bit 55 of
+    // the input to make it look like a user address.
+    _pac_mask = (uintptr_t)pauth_strip_pointer((address)~(UINT64_C(1) << 55));
+
+    // The frame pointer must be preserved for ROP protection.
+    if (FLAG_IS_DEFAULT(PreserveFramePointer) == false && PreserveFramePointer == false ) {
+      vm_exit_during_initialization(err_msg("PreserveFramePointer cannot be disabled for ROP-protection"));
+    }
+    PreserveFramePointer = true;
+  }
+
 #ifdef COMPILER2
   if (FLAG_IS_DEFAULT(UseMultiplyToLenIntrinsic)) {
     UseMultiplyToLenIntrinsic = true;
@@ -488,3 +540,22 @@ void VM_Version::initialize() {
 
   _spin_wait = get_spin_wait_desc();
 }
+
+void VM_Version::initialize_cpu_information(void) {
+  // do nothing if cpu info has been initialized
+  if (_initialized) {
+    return;
+  }
+
+  _no_of_cores  = os::processor_count();
+  _no_of_threads = _no_of_cores;
+  _no_of_sockets = _no_of_cores;
+  snprintf(_cpu_name, CPU_TYPE_DESC_BUF_SIZE - 1, "AArch64");
+
+  int desc_len = snprintf(_cpu_desc, CPU_DETAILED_DESC_BUF_SIZE, "AArch64 ");
+  get_compatible_board(_cpu_desc + desc_len, CPU_DETAILED_DESC_BUF_SIZE - desc_len);
+  desc_len = (int)strlen(_cpu_desc);
+  snprintf(_cpu_desc + desc_len, CPU_DETAILED_DESC_BUF_SIZE - desc_len, " %s", _features_string);
+
+  _initialized = true;
+}
