@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,7 +31,8 @@
 #include "interpreter/interpreter.hpp"
 #include "memory/allocation.inline.hpp"
 #include "nativeInst_arm.hpp"
-#include "os_share_linux.hpp"
+#include "os_linux.hpp"
+#include "os_posix.hpp"
 #include "prims/jniFastGetField.hpp"
 #include "prims/jvm_misc.hpp"
 #include "runtime/arguments.hpp"
@@ -44,6 +45,7 @@
 #include "runtime/safepointMechanism.hpp"
 #include "runtime/sharedRuntime.hpp"
 #include "runtime/stubRoutines.hpp"
+#include "runtime/threads.hpp"
 #include "runtime/timer.hpp"
 #include "signals_posix.hpp"
 #include "utilities/debug.hpp"
@@ -75,6 +77,13 @@
 
 #define SPELL_REG_SP  "sp"
 
+#ifndef __thumb__
+enum {
+  // Offset to add to frame::_fp when dealing with non-thumb C frames
+  C_frame_offset =  -1,
+};
+#endif
+
 // Don't #define SPELL_REG_FP for thumb because it is not safe to use, so this makes sure we never fetch it.
 #ifndef __thumb__
 #define SPELL_REG_FP "fp"
@@ -154,7 +163,7 @@ address os::fetch_frame_from_context(const void* ucVoid,
 #ifndef __thumb__
       if (CodeCache::find_blob(epc) == NULL) {
         // It's a C frame. We need to adjust the fp.
-        fp += os::C_frame_offset;
+        fp += C_frame_offset;
       }
 #endif
       // Clear FP when stack walking is dangerous so that
@@ -192,7 +201,7 @@ frame os::get_sender_for_C_frame(frame* fr) {
   if (! is_safe_for_fp(pc)) {
     return frame(fr->sender_sp(), (intptr_t *)NULL, pc);
   } else {
-    return frame(fr->sender_sp(), fr->link() + os::C_frame_offset, pc);
+    return frame(fr->sender_sp(), fr->link() + C_frame_offset, pc);
   }
 #endif
 }
@@ -211,7 +220,7 @@ frame os::current_frame() {
 #else
   register intptr_t* fp __asm__ (SPELL_REG_FP);
   // fp is for os::current_frame. We want the fp for our caller.
-  frame myframe((intptr_t*)os::current_stack_pointer(), fp + os::C_frame_offset,
+  frame myframe((intptr_t*)os::current_stack_pointer(), fp + C_frame_offset,
                  CAST_FROM_FN_PTR(address, os::current_frame));
   frame caller_frame = os::get_sender_for_C_frame(&myframe);
 
@@ -273,7 +282,7 @@ bool PosixSignals::pd_hotspot_signal_handler(int sig, siginfo_t* info,
         if (overflow_state->in_stack_yellow_reserved_zone(addr)) {
           overflow_state->disable_stack_yellow_reserved_zone();
           if (thread->thread_state() == _thread_in_Java) {
-            // Throw a stack overflow exception.  Guard pages will be reenabled
+            // Throw a stack overflow exception.  Guard pages will be re-enabled
             // while unwinding the stack.
             stub = SharedRuntime::continuation_for_implicit_exception(thread, pc, SharedRuntime::STACK_OVERFLOW);
           } else {
@@ -282,7 +291,7 @@ bool PosixSignals::pd_hotspot_signal_handler(int sig, siginfo_t* info,
           }
         } else if (overflow_state->in_stack_red_zone(addr)) {
           // Fatal red zone violation.  Disable the guard pages and fall through
-          // to handle_unexpected_exception way down below.
+          // to the exception handling code below.
           overflow_state->disable_stack_red_zone();
           tty->print_raw_cr("An irrecoverable stack overflow has occurred.");
         } else {
@@ -314,7 +323,7 @@ bool PosixSignals::pd_hotspot_signal_handler(int sig, siginfo_t* info,
         // BugId 4454115: A read from a MappedByteBuffer can fault
         // here if the underlying file has been truncated.
         // Do not crash the VM in such a case.
-        CodeBlob* cb = CodeCache::find_blob_unsafe(pc);
+        CodeBlob* cb = CodeCache::find_blob(pc);
         CompiledMethod* nm = (cb != NULL) ? cb->as_compiled_method_or_null() : NULL;
         if ((nm != NULL && nm->has_unsafe_access()) || (thread->doing_unsafe_access() && UnsafeCopyMemory::contains_pc(pc))) {
           unsafe_access = true;
@@ -322,12 +331,12 @@ bool PosixSignals::pd_hotspot_signal_handler(int sig, siginfo_t* info,
       } else if (sig == SIGSEGV &&
                  MacroAssembler::uses_implicit_null_check(info->si_addr)) {
           // Determination of interpreter/vtable stub/compiled code null exception
-          CodeBlob* cb = CodeCache::find_blob_unsafe(pc);
+          CodeBlob* cb = CodeCache::find_blob(pc);
           if (cb != NULL) {
             stub = SharedRuntime::continuation_for_implicit_exception(thread, pc, SharedRuntime::IMPLICIT_NULL);
           }
-      } else if (sig == SIGILL && *(int *)pc == NativeInstruction::zombie_illegal_instruction) {
-        // Zombie
+      } else if (sig == SIGILL && *(int *)pc == NativeInstruction::not_entrant_illegal_instruction) {
+        // Not entrant
         stub = SharedRuntime::get_handle_wrong_method_stub();
       }
     } else if ((thread->thread_state() == _thread_in_vm ||
@@ -418,9 +427,9 @@ void os::setup_fpu() {
 
 // Minimum usable stack sizes required to get to user code. Space for
 // HotSpot guard pages is added later.
-size_t os::Posix::_compiler_thread_min_stack_allowed = (32 DEBUG_ONLY(+ 4)) * K;
-size_t os::Posix::_java_thread_min_stack_allowed = (32 DEBUG_ONLY(+ 4)) * K;
-size_t os::Posix::_vm_internal_thread_min_stack_allowed = (48 DEBUG_ONLY(+ 4)) * K;
+size_t os::_compiler_thread_min_stack_allowed = (32 DEBUG_ONLY(+ 4)) * K;
+size_t os::_java_thread_min_stack_allowed = (32 DEBUG_ONLY(+ 4)) * K;
+size_t os::_vm_internal_thread_min_stack_allowed = (48 DEBUG_ONLY(+ 4)) * K;
 
 // return default stack size for thr_type
 size_t os::Posix::default_stack_size(os::ThreadType thr_type) {
@@ -444,7 +453,20 @@ void os::print_context(outputStream *st, const void *context) {
   }
 #define U64_FORMAT "0x%016llx"
   // now print flag register
-  st->print_cr("  %-4s = 0x%08lx", "cpsr",uc->uc_mcontext.arm_cpsr);
+  uint32_t cpsr = uc->uc_mcontext.arm_cpsr;
+  st->print_cr("  %-4s = 0x%08x", "cpsr", cpsr);
+  // print out instruction set state
+  st->print("isetstate: ");
+  const int isetstate =
+      ((cpsr & (1 << 5))  ? 1 : 0) | // T
+      ((cpsr & (1 << 24)) ? 2 : 0); // J
+  switch (isetstate) {
+  case 0: st->print_cr("ARM"); break;
+  case 1: st->print_cr("Thumb"); break;
+  case 2: st->print_cr("Jazelle"); break;
+  case 3: st->print_cr("ThumbEE"); break;
+  default: ShouldNotReachHere();
+  };
   st->cr();
 }
 
@@ -453,9 +475,8 @@ void os::print_tos_pc(outputStream *st, const void *context) {
 
   const ucontext_t* uc = (const ucontext_t*)context;
 
-  intptr_t *sp = (intptr_t *)os::Linux::ucontext_get_sp(uc);
-  st->print_cr("Top of Stack: (sp=" INTPTR_FORMAT ")", p2i(sp));
-  print_hex_dump(st, (address)sp, (address)(sp + 8*sizeof(intptr_t)), sizeof(intptr_t));
+  address sp = (address)os::Linux::ucontext_get_sp(uc);
+  print_tos(st, sp);
   st->cr();
 
   // Note: it may be unsafe to inspect memory near pc. For example, pc may
@@ -481,16 +502,20 @@ void os::print_register_info(outputStream *st, const void *context) {
   st->cr();
 }
 
-typedef int64_t cmpxchg_long_func_t(int64_t, int64_t, volatile int64_t*);
 
-cmpxchg_long_func_t* os::atomic_cmpxchg_long_func = os::atomic_cmpxchg_long_bootstrap;
+ARMAtomicFuncs::cmpxchg_long_func_t ARMAtomicFuncs::_cmpxchg_long_func = ARMAtomicFuncs::cmpxchg_long_bootstrap;
+ARMAtomicFuncs::load_long_func_t    ARMAtomicFuncs::_load_long_func    = ARMAtomicFuncs::load_long_bootstrap;
+ARMAtomicFuncs::store_long_func_t   ARMAtomicFuncs::_store_long_func   = ARMAtomicFuncs::store_long_bootstrap;
+ARMAtomicFuncs::atomic_add_func_t   ARMAtomicFuncs::_add_func          = ARMAtomicFuncs::add_bootstrap;
+ARMAtomicFuncs::atomic_xchg_func_t  ARMAtomicFuncs::_xchg_func         = ARMAtomicFuncs::xchg_bootstrap;
+ARMAtomicFuncs::cmpxchg_func_t      ARMAtomicFuncs::_cmpxchg_func      = ARMAtomicFuncs::cmpxchg_bootstrap;
 
-int64_t os::atomic_cmpxchg_long_bootstrap(int64_t compare_value, int64_t exchange_value, volatile int64_t* dest) {
+int64_t ARMAtomicFuncs::cmpxchg_long_bootstrap(int64_t compare_value, int64_t exchange_value, volatile int64_t* dest) {
   // try to use the stub:
-  cmpxchg_long_func_t* func = CAST_TO_FN_PTR(cmpxchg_long_func_t*, StubRoutines::atomic_cmpxchg_long_entry());
+  cmpxchg_long_func_t func = CAST_TO_FN_PTR(cmpxchg_long_func_t, StubRoutines::atomic_cmpxchg_long_entry());
 
   if (func != NULL) {
-    os::atomic_cmpxchg_long_func = func;
+    _cmpxchg_long_func = func;
     return (*func)(compare_value, exchange_value, dest);
   }
   assert(Threads::number_of_threads() == 0, "for bootstrap only");
@@ -500,16 +525,13 @@ int64_t os::atomic_cmpxchg_long_bootstrap(int64_t compare_value, int64_t exchang
     *dest = exchange_value;
   return old_value;
 }
-typedef int64_t load_long_func_t(const volatile int64_t*);
 
-load_long_func_t* os::atomic_load_long_func = os::atomic_load_long_bootstrap;
-
-int64_t os::atomic_load_long_bootstrap(const volatile int64_t* src) {
+int64_t ARMAtomicFuncs::load_long_bootstrap(const volatile int64_t* src) {
   // try to use the stub:
-  load_long_func_t* func = CAST_TO_FN_PTR(load_long_func_t*, StubRoutines::atomic_load_long_entry());
+  load_long_func_t func = CAST_TO_FN_PTR(load_long_func_t, StubRoutines::atomic_load_long_entry());
 
   if (func != NULL) {
-    os::atomic_load_long_func = func;
+    _load_long_func = func;
     return (*func)(src);
   }
   assert(Threads::number_of_threads() == 0, "for bootstrap only");
@@ -518,16 +540,12 @@ int64_t os::atomic_load_long_bootstrap(const volatile int64_t* src) {
   return old_value;
 }
 
-typedef void store_long_func_t(int64_t, volatile int64_t*);
-
-store_long_func_t* os::atomic_store_long_func = os::atomic_store_long_bootstrap;
-
-void os::atomic_store_long_bootstrap(int64_t val, volatile int64_t* dest) {
+void ARMAtomicFuncs::store_long_bootstrap(int64_t val, volatile int64_t* dest) {
   // try to use the stub:
-  store_long_func_t* func = CAST_TO_FN_PTR(store_long_func_t*, StubRoutines::atomic_store_long_entry());
+  store_long_func_t func = CAST_TO_FN_PTR(store_long_func_t, StubRoutines::atomic_store_long_entry());
 
   if (func != NULL) {
-    os::atomic_store_long_func = func;
+    _store_long_func = func;
     return (*func)(val, dest);
   }
   assert(Threads::number_of_threads() == 0, "for bootstrap only");
@@ -535,15 +553,11 @@ void os::atomic_store_long_bootstrap(int64_t val, volatile int64_t* dest) {
   *dest = val;
 }
 
-typedef int32_t  atomic_add_func_t(int32_t add_value, volatile int32_t *dest);
-
-atomic_add_func_t * os::atomic_add_func = os::atomic_add_bootstrap;
-
-int32_t  os::atomic_add_bootstrap(int32_t add_value, volatile int32_t *dest) {
-  atomic_add_func_t * func = CAST_TO_FN_PTR(atomic_add_func_t*,
-                                            StubRoutines::atomic_add_entry());
+int32_t ARMAtomicFuncs::add_bootstrap(int32_t add_value, volatile int32_t *dest) {
+  atomic_add_func_t func = CAST_TO_FN_PTR(atomic_add_func_t,
+                                          StubRoutines::atomic_add_entry());
   if (func != NULL) {
-    os::atomic_add_func = func;
+    _add_func = func;
     return (*func)(add_value, dest);
   }
 
@@ -552,15 +566,11 @@ int32_t  os::atomic_add_bootstrap(int32_t add_value, volatile int32_t *dest) {
   return (old_value + add_value);
 }
 
-typedef int32_t  atomic_xchg_func_t(int32_t exchange_value, volatile int32_t *dest);
-
-atomic_xchg_func_t * os::atomic_xchg_func = os::atomic_xchg_bootstrap;
-
-int32_t  os::atomic_xchg_bootstrap(int32_t exchange_value, volatile int32_t *dest) {
-  atomic_xchg_func_t * func = CAST_TO_FN_PTR(atomic_xchg_func_t*,
-                                            StubRoutines::atomic_xchg_entry());
+int32_t ARMAtomicFuncs::xchg_bootstrap(int32_t exchange_value, volatile int32_t *dest) {
+  atomic_xchg_func_t func = CAST_TO_FN_PTR(atomic_xchg_func_t,
+                                           StubRoutines::atomic_xchg_entry());
   if (func != NULL) {
-    os::atomic_xchg_func = func;
+    _xchg_func = func;
     return (*func)(exchange_value, dest);
   }
 
@@ -569,16 +579,12 @@ int32_t  os::atomic_xchg_bootstrap(int32_t exchange_value, volatile int32_t *des
   return (old_value);
 }
 
-typedef int32_t cmpxchg_func_t(int32_t, int32_t, volatile int32_t*);
-
-cmpxchg_func_t* os::atomic_cmpxchg_func = os::atomic_cmpxchg_bootstrap;
-
-int32_t os::atomic_cmpxchg_bootstrap(int32_t compare_value, int32_t exchange_value, volatile int32_t* dest) {
+int32_t ARMAtomicFuncs::cmpxchg_bootstrap(int32_t compare_value, int32_t exchange_value, volatile int32_t* dest) {
   // try to use the stub:
-  cmpxchg_func_t* func = CAST_TO_FN_PTR(cmpxchg_func_t*, StubRoutines::atomic_cmpxchg_entry());
+  cmpxchg_func_t func = CAST_TO_FN_PTR(cmpxchg_func_t, StubRoutines::atomic_cmpxchg_entry());
 
   if (func != NULL) {
-    os::atomic_cmpxchg_func = func;
+    _cmpxchg_func = func;
     return (*func)(compare_value, exchange_value, dest);
   }
   assert(Threads::number_of_threads() == 0, "for bootstrap only");
