@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -31,9 +31,10 @@
 #include "code/debugInfoRec.hpp"
 #include "code/dependencies.hpp"
 #include "code/exceptionHandlerTable.hpp"
+#include "compiler/compiler_globals.hpp"
 #include "compiler/compilerThread.hpp"
 #include "oops/methodData.hpp"
-#include "runtime/thread.hpp"
+#include "runtime/javaThread.hpp"
 
 class CompileTask;
 class OopMapSet;
@@ -164,6 +165,9 @@ private:
                            Bytecodes::Code  bc,
                            constantTag      tag);
 
+  ciConstant unbox_primitive_value(ciObject* cibox, BasicType expected_bt = T_ILLEGAL);
+  ciConstant get_resolved_constant(const constantPoolHandle& cpool, int obj_index);
+
   // Get a ciObject from the object factory.  Ensures uniqueness
   // of ciObjects.
   ciObject* get_object(oop o) {
@@ -321,7 +325,7 @@ public:
   // Reason this compilation is failing, such as "too many basic blocks".
   const char* failure_reason() { return _failure_reason; }
 
-  // Return state of appropriate compilability
+  // Return state of appropriate compatibility
   int compilable() { return _compilable; }
 
   const char* retry_message() const {
@@ -379,9 +383,9 @@ public:
                        AbstractCompiler*         compiler,
                        bool                      has_unsafe_access,
                        bool                      has_wide_vectors,
-                       RTMState                  rtm_state = NoRTM,
-                       const GrowableArrayView<RuntimeStub*>& native_invokers = GrowableArrayView<RuntimeStub*>::EMPTY);
-
+                       bool                      has_monitors,
+                       int                       immediate_oops_patched,
+                       RTMState                  rtm_state = NoRTM);
 
   // Access to certain well known ciObjects.
 #define VM_CLASS_FUNC(name, ignore_s) \
@@ -419,6 +423,10 @@ public:
   }
   ciInstance* unloaded_ciinstance();
 
+  ciInstanceKlass* get_box_klass_for_primitive_type(BasicType type);
+
+  ciKlass*  find_system_klass(ciSymbol* klass_name);
+
   // Note:  To find a class from its name string, use ciSymbol::make,
   // but consider adding to vmSymbols.hpp instead.
 
@@ -498,8 +506,8 @@ public:
   void record_lambdaform(Thread* thread, oop obj);
   void record_member(Thread* thread, oop obj);
   void record_mh(Thread* thread, oop obj);
-  void record_call_site_obj(Thread* thread, const constantPoolHandle& pool, const Handle appendix);
-  void record_call_site_method(Thread* thread, const constantPoolHandle& pool, Method* adapter);
+  void record_call_site_obj(Thread* thread, oop obj);
+  void record_call_site_method(Thread* thread, Method* adapter);
   void process_invokedynamic(const constantPoolHandle &cp, int index, JavaThread* thread);
   void process_invokehandle(const constantPoolHandle &cp, int index, JavaThread* thread);
   void find_dynamic_call_sites();
