@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -34,6 +34,7 @@
 #include "classfile/systemDictionary.hpp"
 #include "compiler/compilationPolicy.hpp"
 #include "compiler/compileBroker.hpp"
+#include "compiler/compilerDefinitions.inline.hpp"
 #include "interpreter/linkResolver.hpp"
 #include "memory/allocation.inline.hpp"
 #include "memory/oopFactory.hpp"
@@ -50,12 +51,12 @@
 #include "runtime/globals_extension.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/java.hpp"
+#include "runtime/jniHandles.inline.hpp"
+#include "runtime/threads.hpp"
 #include "utilities/copy.hpp"
 #include "utilities/macros.hpp"
 #include "utilities/utf8.hpp"
 
-#ifndef PRODUCT
-
 // ciReplay
 
 typedef struct _ciMethodDataRecord {
@@ -64,7 +65,7 @@ typedef struct _ciMethodDataRecord {
   const char* _signature;
 
   int _state;
-  int _current_mileage;
+  int _invocation_counter;
 
   intptr_t* _data;
   char*     _orig_data;
@@ -90,6 +91,11 @@ typedef struct _ciMethodRecord {
   int _backedge_counter;
 } ciMethodRecord;
 
+typedef struct _ciInstanceKlassRecord {
+  const InstanceKlass* _klass;
+  jobject _java_mirror; // Global handle to java mirror to prevent unloading
+} ciInstanceKlassRecord;
+
 typedef struct _ciInlineRecord {
   const char* _klass_name;
   const char* _method_name;
@@ -97,6 +103,7 @@ typedef struct _ciInlineRecord {
 
   int _inline_depth;
   int _inline_bci;
+  bool _inline_late;
 } ciInlineRecord;
 
 class  CompileReplay;
@@ -107,10 +114,13 @@ class CompileReplay : public StackObj {
   FILE*   _stream;
   Thread* _thread;
   Handle  _protection_domain;
+  bool    _protection_domain_initialized;
   Handle  _loader;
+  int     _version;
 
   GrowableArray<ciMethodRecord*>     _ci_method_records;
   GrowableArray<ciMethodDataRecord*> _ci_method_data_records;
+  GrowableArray<ciInstanceKlassRecord*> _ci_instance_klass_records;
 
   // Use pointer because we may need to return inline records
   // without destroying them.
@@ -121,7 +131,6 @@ class CompileReplay : public StackObj {
   char* _bufptr;
   char* _buffer;
   int   _buffer_length;
-  int   _buffer_pos;
 
   // "compile" data
   ciKlass* _iklass;
@@ -134,8 +143,9 @@ class CompileReplay : public StackObj {
     _thread = THREAD;
     _loader = Handle(_thread, SystemDictionary::java_system_loader());
     _protection_domain = Handle();
+    _protection_domain_initialized = false;
 
-    _stream = fopen(filename, "rt");
+    _stream = os::fopen(filename, "rt");
     if (_stream == NULL) {
       fprintf(stderr, "ERROR: Can't open replay file %s\n", filename);
     }
@@ -146,12 +156,12 @@ class CompileReplay : public StackObj {
     _buffer_length = 32;
     _buffer = NEW_RESOURCE_ARRAY(char, _buffer_length);
     _bufptr = _buffer;
-    _buffer_pos = 0;
 
     _imethod = NULL;
     _iklass  = NULL;
     _entry_bci  = 0;
     _comp_level = 0;
+    _version = 0;
 
     test();
   }
@@ -182,10 +192,6 @@ class CompileReplay : public StackObj {
 
   void report_error(const char* msg) {
     _error_message = msg;
-    // Restore the _buffer contents for error reporting
-    for (int i = 0; i < _buffer_pos; i++) {
-      if (_buffer[i] == '\0') _buffer[i] = ' ';
-    }
   }
 
   int parse_int(const char* label) {
@@ -225,6 +231,10 @@ class CompileReplay : public StackObj {
     }
   }
 
+  // Ignore the rest of the line
+  void skip_remaining() {
+    _bufptr = &_bufptr[strlen(_bufptr)]; // skip ahead to terminator
+  }
 
   char* scan_and_terminate(char delim) {
     char* str = _bufptr;
@@ -370,6 +380,9 @@ class CompileReplay : public StackObj {
     char* ref = parse_string();
     if (strcmp(ref, "bci") == 0) {
       Method* m = parse_method(CHECK_NULL);
+      if (m == NULL) {
+        return NULL;
+      }
 
       InstanceKlass* ik = m->method_holder();
       const constantPoolHandle cp(Thread::current(), ik->constants());
@@ -385,7 +398,12 @@ class CompileReplay : public StackObj {
 
       ik->link_class(CHECK_NULL);
 
-      Bytecode_invoke bytecode(caller, bci);
+      Bytecode_invoke bytecode = Bytecode_invoke_check(caller, bci);
+      if (!Bytecodes::is_defined(bytecode.code()) || !bytecode.is_valid()) {
+        report_error("no invoke found at bci");
+        return NULL;
+      }
+      bytecode.verify();
       int index = bytecode.index();
 
       ConstantPoolCacheEntry* cp_cache_entry = NULL;
@@ -424,7 +442,7 @@ class CompileReplay : public StackObj {
       } else if (strcmp(dyno_ref, "<bsm>") == 0) {
         int pool_index = cp_cache_entry->constant_pool_index();
         BootstrapInfo bootstrap_specifier(cp, pool_index, index);
-        obj = cp->resolve_possibly_cached_constant_at(bootstrap_specifier.bsm_index(), thread);
+        obj = cp->resolve_possibly_cached_constant_at(bootstrap_specifier.bsm_index(), CHECK_NULL);
       } else {
         report_error("unrecognized token");
         return NULL;
@@ -437,6 +455,9 @@ class CompileReplay : public StackObj {
       }
 
       Klass* k = parse_klass(CHECK_NULL);
+      if (k == NULL) {
+        return NULL;
+      }
       InstanceKlass* ik = InstanceKlass::cast(k);
       const constantPoolHandle cp(Thread::current(), ik->constants());
 
@@ -450,57 +471,57 @@ class CompileReplay : public StackObj {
         report_error("no method handle found at cpi");
         return NULL;
       }
-      {
-        bool found_it;
-        obj = cp->find_cached_constant_at(cpi, found_it, thread);
-      }
+      ik->link_class(CHECK_NULL);
+      obj = cp->resolve_possibly_cached_constant_at(cpi, CHECK_NULL);
+    }
+    if (obj == NULL) {
+      report_error("null cp object found");
+      return NULL;
     }
     Klass* k = NULL;
-    if (obj != NULL) {
-      skip_ws();
-      // loop: read fields
-      char* field = NULL;
-      do {
-        field = parse_string();
-        if (field == NULL) {
-          report_error("no field found");
-          return NULL;
-        }
-        if (strcmp(field, ";") == 0) {
-          break;
-        }
-        // raw Method*
-        if (strcmp(field, "<vmtarget>") == 0) {
-          Method* vmtarget = java_lang_invoke_MemberName::vmtarget(obj);
-          k = (vmtarget == NULL) ? NULL : vmtarget->method_holder();
-          if (k == NULL) {
-            report_error("null vmtarget found");
-            return NULL;
-          }
-          if (!parse_terminator()) {
-            report_error("missing terminator");
-            return NULL;
-          }
-          return k;
-        }
-        obj = ciReplay::obj_field(obj, field);
-        // array
-        if (obj != NULL && obj->is_objArray()) {
-          objArrayOop arr = (objArrayOop)obj;
-          int index = parse_int("index");
-          if (index >= arr->length()) {
-            report_error("bad array index");
-            return NULL;
-          }
-          obj = arr->obj_at(index);
-        }
-      } while (obj != NULL);
-      if (obj == NULL) {
-        report_error("null field found");
+    skip_ws();
+    // loop: read fields
+    char* field = NULL;
+    do {
+      field = parse_string();
+      if (field == NULL) {
+        report_error("no field found");
         return NULL;
       }
-      k = obj->klass();
+      if (strcmp(field, ";") == 0) {
+        break;
+      }
+      // raw Method*
+      if (strcmp(field, "<vmtarget>") == 0) {
+        Method* vmtarget = java_lang_invoke_MemberName::vmtarget(obj);
+        k = (vmtarget == NULL) ? NULL : vmtarget->method_holder();
+        if (k == NULL) {
+          report_error("null vmtarget found");
+          return NULL;
+        }
+        if (!parse_terminator()) {
+          report_error("missing terminator");
+          return NULL;
+        }
+        return k;
+      }
+      obj = ciReplay::obj_field(obj, field);
+      // array
+      if (obj != NULL && obj->is_objArray()) {
+        objArrayOop arr = (objArrayOop)obj;
+        int index = parse_int("index");
+        if (index >= arr->length()) {
+          report_error("bad array index");
+          return NULL;
+        }
+        obj = arr->obj_at(index);
+      }
+    } while (obj != NULL);
+    if (obj == NULL) {
+      report_error("null field found");
+      return NULL;
     }
+    k = obj->klass();
     return k;
   }
 
@@ -568,8 +589,9 @@ class CompileReplay : public StackObj {
   }
 
   int get_line(int c) {
+    int buffer_pos = 0;
     while(c != EOF) {
-      if (_buffer_pos + 1 >= _buffer_length) {
+      if (buffer_pos + 1 >= _buffer_length) {
         int new_length = _buffer_length * 2;
         // Next call will throw error in case of OOM.
         _buffer = REALLOC_RESOURCE_ARRAY(char, _buffer, _buffer_length, new_length);
@@ -581,13 +603,12 @@ class CompileReplay : public StackObj {
       } else if (c == '\r') {
         // skip LF
       } else {
-        _buffer[_buffer_pos++] = c;
+        _buffer[buffer_pos++] = c;
       }
       c = getc(_stream);
     }
     // null terminate it, reset the pointer
-    _buffer[_buffer_pos] = '\0'; // NL or EOF
-    _buffer_pos = 0;
+    _buffer[buffer_pos] = '\0'; // NL or EOF
     _bufptr = _buffer;
     return c;
   }
@@ -601,7 +622,8 @@ class CompileReplay : public StackObj {
       c = get_line(c);
       process_command(THREAD);
       if (had_error()) {
-        tty->print_cr("Error while parsing line %d: %s\n", line_no, _error_message);
+        int pos = _bufptr - _buffer + 1;
+        tty->print_cr("Error while parsing line %d at position %d: %s\n", line_no, pos, _error_message);
         if (ReplayIgnoreInitErrors) {
           CLEAR_PENDING_EXCEPTION;
           _error_message = NULL;
@@ -611,6 +633,7 @@ class CompileReplay : public StackObj {
       }
       line_no++;
     }
+    reset();
   }
 
   void process_command(TRAPS) {
@@ -619,7 +642,16 @@ class CompileReplay : public StackObj {
       return;
     }
     if (strcmp("#", cmd) == 0) {
-      // ignore
+      // comment line, print or ignore
+      if (Verbose) {
+        tty->print_cr("# %s", _bufptr);
+      }
+      skip_remaining();
+    } else if (strcmp("version", cmd) == 0) {
+      _version = parse_int("version");
+      if (_version < 0 || _version > REPLAY_VERSION) {
+        tty->print_cr("# unrecognized version %d, expected 0 <= version <= %d", _version, REPLAY_VERSION);
+      }
     } else if (strcmp("compile", cmd) == 0) {
       process_compile(CHECK);
     } else if (strcmp("ciMethod", cmd) == 0) {
@@ -639,6 +671,9 @@ class CompileReplay : public StackObj {
     } else {
       report_error("unknown command");
     }
+    if (!had_error() && *_bufptr != '\0') {
+      report_error("line not properly terminated");
+    }
   }
 
   // validation of comp_level
@@ -692,18 +727,12 @@ class CompileReplay : public StackObj {
     return NULL;
   }
 
-  // compile <klass> <name> <signature> <entry_bci> <comp_level> inline <count> (<depth> <bci> <klass> <name> <signature>)*
+  // compile <klass> <name> <signature> <entry_bci> <comp_level> inline <count> (<depth> <bci> <inline_late> <klass> <name> <signature>)*
   void process_compile(TRAPS) {
     Method* method = parse_method(CHECK);
     if (had_error()) return;
     int entry_bci = parse_int("entry_bci");
-    const char* comp_level_label = "comp_level";
-    int comp_level = parse_int(comp_level_label);
-    // old version w/o comp_level
-    if (had_error() && (error_message() == comp_level_label)) {
-      // use highest available tier
-      comp_level = CompilationPolicy::highest_compile_level();
-    }
+    int comp_level = parse_int("comp_level");
     if (!is_valid_comp_level(comp_level)) {
       return;
     }
@@ -734,11 +763,19 @@ class CompileReplay : public StackObj {
         if (had_error()) {
           break;
         }
+        int inline_late = 0;
+        if (_version >= 2) {
+          inline_late = parse_int("inline_late");
+          if (had_error()) {
+              break;
+          }
+        }
+
         Method* inl_method = parse_method(CHECK);
         if (had_error()) {
           break;
         }
-        new_ciInlineRecord(inl_method, bci, depth);
+        new_ciInlineRecord(inl_method, bci, depth, inline_late);
       }
     }
     if (_imethod != NULL) {
@@ -766,7 +803,6 @@ class CompileReplay : public StackObj {
     CompileBroker::compile_method(methodHandle(THREAD, method), entry_bci, comp_level,
                                   methodHandle(), 0, CompileTask::Reason_Replay, THREAD);
     replay_state = NULL;
-    reset();
   }
 
   // ciMethod <klass> <name> <signature> <invocation_counter> <backedge_counter> <interpreter_invocation_count> <interpreter_throwout_count> <instructions_size>
@@ -781,7 +817,7 @@ class CompileReplay : public StackObj {
     rec->_instructions_size = parse_int("instructions_size");
   }
 
-  // ciMethodData <klass> <name> <signature> <state> <current_mileage> orig <length> <byte>* data <length> <ptr>* oops <length> (<offset> <klass>)* methods <length> (<offset> <klass> <name> <signature>)*
+  // ciMethodData <klass> <name> <signature> <state> <invocation_counter> orig <length> <byte>* data <length> <ptr>* oops <length> (<offset> <klass>)* methods <length> (<offset> <klass> <name> <signature>)*
   void process_ciMethodData(TRAPS) {
     Method* method = parse_method(CHECK);
     if (had_error()) return;
@@ -791,7 +827,7 @@ class CompileReplay : public StackObj {
     // method to be rewritten (number of arguments at a call for
     // instance)
     method->method_holder()->link_class(CHECK);
-    // Method::build_interpreter_method_data(method, CHECK);
+    // Method::build_profiling_method_data(method, CHECK);
     {
       // Grab a lock here to prevent multiple
       // MethodData*s from being created.
@@ -806,7 +842,11 @@ class CompileReplay : public StackObj {
     // collect and record all the needed information for later
     ciMethodDataRecord* rec = new_ciMethodData(method);
     rec->_state = parse_int("state");
-    rec->_current_mileage = parse_int("current_mileage");
+    if (_version < 1) {
+      parse_int("current_mileage");
+    } else {
+      rec->_invocation_counter = parse_int("invocation_counter");
+    }
 
     rec->_orig_data = parse_data("orig", rec->_orig_data_length);
     if (rec->_orig_data == NULL) {
@@ -855,6 +895,19 @@ class CompileReplay : public StackObj {
   void process_instanceKlass(TRAPS) {
     // just load the referenced class
     Klass* k = parse_klass(CHECK);
+
+    if (_version >= 1) {
+      if (!_protection_domain_initialized && k != NULL) {
+        assert(_protection_domain() == NULL, "must be uninitialized");
+        // The first entry is the holder class of the method for which a replay compilation is requested.
+        // Use the same protection domain to load all subsequent classes in order to resolve all classes
+        // in signatures of inlinees. This ensures that inlining can be done as stated in the replay file.
+        _protection_domain = Handle(_thread, k->protection_domain());
+      }
+
+      _protection_domain_initialized = true;
+    }
+
     if (k == NULL) {
       return;
     }
@@ -864,9 +917,13 @@ class CompileReplay : public StackObj {
       report_error("hidden class with comment expected");
       return;
     }
-    if (is_comment && Verbose) {
-      const char* hidden = parse_string();
-      tty->print_cr("Found %s for %s", k->name()->as_quoted_ascii(), hidden);
+    // comment, print or ignore
+    if (is_comment) {
+      if (Verbose) {
+        const char* hidden = parse_string();
+        tty->print_cr("Found %s for %s", k->name()->as_quoted_ascii(), hidden);
+      }
+      skip_remaining();
     }
   }
 
@@ -876,8 +933,9 @@ class CompileReplay : public StackObj {
   // constant pool is the same length as 'length' and make sure the
   // constant pool tags are in the same state.
   void process_ciInstanceKlass(TRAPS) {
-    InstanceKlass* k = (InstanceKlass *)parse_klass(CHECK);
+    InstanceKlass* k = (InstanceKlass*)parse_klass(CHECK);
     if (k == NULL) {
+      skip_remaining();
       return;
     }
     int is_linked = parse_int("is_linked");
@@ -899,6 +957,7 @@ class CompileReplay : public StackObj {
     } else if (is_linked) {
       k->link_class(CHECK);
     }
+    new_ciInstanceKlass(k);
     ConstantPool* cp = k->constants();
     if (length != cp->length()) {
       report_error("constant pool length mismatch: wrong class files?");
@@ -945,10 +1004,10 @@ class CompileReplay : public StackObj {
           break;
 
         case JVM_CONSTANT_Class:
-          if (tag == JVM_CONSTANT_Class) {
-          } else if (tag == JVM_CONSTANT_UnresolvedClass) {
-            tty->print_cr("Warning: entry was unresolved in the replay data");
-          } else {
+          if (tag == JVM_CONSTANT_UnresolvedClass) {
+            Klass* k = cp->klass_at(i, CHECK);
+            tty->print_cr("Warning: entry was unresolved in the replay data: %s", k->name()->as_utf8());
+          } else if (tag != JVM_CONSTANT_Class) {
             report_error("Unexpected tag");
             return;
           }
@@ -976,6 +1035,7 @@ class CompileReplay : public StackObj {
 
     if (k == NULL || ReplaySuppressInitializers == 0 ||
         (ReplaySuppressInitializers == 2 && k->class_loader() == NULL)) {
+      skip_remaining();
       return;
     }
 
@@ -1126,6 +1186,28 @@ class CompileReplay : public StackObj {
     return NULL;
   }
 
+  // Create and initialize a record for a ciInstanceKlass which was present at replay dump time.
+  void new_ciInstanceKlass(const InstanceKlass* klass) {
+    ciInstanceKlassRecord* rec = NEW_RESOURCE_OBJ(ciInstanceKlassRecord);
+    rec->_klass = klass;
+    oop java_mirror = klass->java_mirror();
+    Handle h_java_mirror(_thread, java_mirror);
+    rec->_java_mirror = JNIHandles::make_global(h_java_mirror);
+    _ci_instance_klass_records.append(rec);
+  }
+
+  // Check if a ciInstanceKlass was present at replay dump time for a klass.
+  ciInstanceKlassRecord* find_ciInstanceKlass(const InstanceKlass* klass) {
+    for (int i = 0; i < _ci_instance_klass_records.length(); i++) {
+      ciInstanceKlassRecord* rec = _ci_instance_klass_records.at(i);
+      if (klass == rec->_klass) {
+        // ciInstanceKlass for this klass was resolved.
+        return rec;
+      }
+    }
+    return NULL;
+  }
+
   // Create and initialize a record for a ciMethodData
   ciMethodDataRecord* new_ciMethodData(Method* method) {
     ciMethodDataRecord* rec = NEW_RESOURCE_OBJ(ciMethodDataRecord);
@@ -1153,13 +1235,14 @@ class CompileReplay : public StackObj {
   }
 
   // Create and initialize a record for a ciInlineRecord
-  ciInlineRecord* new_ciInlineRecord(Method* method, int bci, int depth) {
+  ciInlineRecord* new_ciInlineRecord(Method* method, int bci, int depth, int inline_late) {
     ciInlineRecord* rec = NEW_RESOURCE_OBJ(ciInlineRecord);
     rec->_klass_name =  method->method_holder()->name()->as_utf8();
     rec->_method_name = method->name()->as_utf8();
     rec->_signature = method->signature()->as_utf8();
     rec->_inline_bci = bci;
     rec->_inline_depth = depth;
+    rec->_inline_late = inline_late;
     _ci_inline_records->append(rec);
     return rec;
   }
@@ -1259,6 +1342,10 @@ void ciReplay::replay(TRAPS) {
   vm_exit(exit_code);
 }
 
+bool ciReplay::no_replay_state() {
+  return replay_state == NULL;
+}
+
 void* ciReplay::load_inline_data(ciMethod* method, int entry_bci, int comp_level) {
   if (FLAG_IS_DEFAULT(InlineDataFile)) {
     tty->print_cr("ERROR: no inline replay data file specified (use -XX:InlineDataFile=inline_pid12345.txt).");
@@ -1295,7 +1382,7 @@ int ciReplay::replay_impl(TRAPS) {
   if (ReplaySuppressInitializers > 2) {
     // ReplaySuppressInitializers > 2 means that we want to allow
     // normal VM bootstrap but once we get into the replay itself
-    // don't allow any intializers to be run.
+    // don't allow any initializers to be run.
     ReplaySuppressInitializers = 1;
   }
 
@@ -1330,7 +1417,7 @@ int ciReplay::replay_impl(TRAPS) {
 }
 
 void ciReplay::initialize(ciMethodData* m) {
-  if (replay_state == NULL) {
+  if (no_replay_state()) {
     return;
   }
 
@@ -1348,7 +1435,7 @@ void ciReplay::initialize(ciMethodData* m) {
     tty->cr();
   } else {
     m->_state = rec->_state;
-    m->_current_mileage = rec->_current_mileage;
+    m->_invocation_counter = rec->_invocation_counter;
     if (rec->_data_length != 0) {
       assert(m->_data_size + m->_extra_data_size == rec->_data_length * (int)sizeof(rec->_data[0]) ||
              m->_data_size == rec->_data_length * (int)sizeof(rec->_data[0]), "must agree");
@@ -1384,7 +1471,7 @@ void ciReplay::initialize(ciMethodData* m) {
 
 
 bool ciReplay::should_not_inline(ciMethod* method) {
-  if (replay_state == NULL) {
+  if (no_replay_state()) {
     return false;
   }
   VM_ENTRY_MARK;
@@ -1392,23 +1479,33 @@ bool ciReplay::should_not_inline(ciMethod* method) {
   return replay_state->find_ciMethodRecord(method->get_Method()) == NULL;
 }
 
-bool ciReplay::should_inline(void* data, ciMethod* method, int bci, int inline_depth) {
+bool ciReplay::should_inline(void* data, ciMethod* method, int bci, int inline_depth, bool& should_delay) {
   if (data != NULL) {
-    GrowableArray<ciInlineRecord*>*  records = (GrowableArray<ciInlineRecord*>*)data;
+    GrowableArray<ciInlineRecord*>* records = (GrowableArray<ciInlineRecord*>*)data;
     VM_ENTRY_MARK;
     // Inline record are ordered by bci and depth.
-    return CompileReplay::find_ciInlineRecord(records, method->get_Method(), bci, inline_depth) != NULL;
+    ciInlineRecord* record = CompileReplay::find_ciInlineRecord(records, method->get_Method(), bci, inline_depth);
+    if (record == NULL) {
+      return false;
+    }
+    should_delay = record->_inline_late;
+    return true;
   } else if (replay_state != NULL) {
     VM_ENTRY_MARK;
     // Inline record are ordered by bci and depth.
-    return replay_state->find_ciInlineRecord(method->get_Method(), bci, inline_depth) != NULL;
+    ciInlineRecord* record = replay_state->find_ciInlineRecord(method->get_Method(), bci, inline_depth);
+    if (record == NULL) {
+      return false;
+    }
+    should_delay = record->_inline_late;
+    return true;
   }
   return false;
 }
 
 bool ciReplay::should_not_inline(void* data, ciMethod* method, int bci, int inline_depth) {
   if (data != NULL) {
-    GrowableArray<ciInlineRecord*>*  records = (GrowableArray<ciInlineRecord*>*)data;
+    GrowableArray<ciInlineRecord*>* records = (GrowableArray<ciInlineRecord*>*)data;
     VM_ENTRY_MARK;
     // Inline record are ordered by bci and depth.
     return CompileReplay::find_ciInlineRecord(records, method->get_Method(), bci, inline_depth) == NULL;
@@ -1421,7 +1518,7 @@ bool ciReplay::should_not_inline(void* data, ciMethod* method, int bci, int inli
 }
 
 void ciReplay::initialize(ciMethod* m) {
-  if (replay_state == NULL) {
+  if (no_replay_state()) {
     return;
   }
 
@@ -1450,8 +1547,17 @@ void ciReplay::initialize(ciMethod* m) {
   }
 }
 
+void ciReplay::initialize(ciInstanceKlass* ci_ik, InstanceKlass* ik) {
+  assert(!no_replay_state(), "must have replay state");
+
+  ASSERT_IN_VM;
+  ciInstanceKlassRecord* rec = replay_state->find_ciInstanceKlass(ik);
+  assert(rec != NULL, "ciInstanceKlass must be whitelisted");
+  ci_ik->_java_mirror = CURRENT_ENV->get_instance(JNIHandles::resolve(rec->_java_mirror));
+}
+
 bool ciReplay::is_loaded(Method* method) {
-  if (replay_state == NULL) {
+  if (no_replay_state()) {
     return true;
   }
 
@@ -1461,7 +1567,16 @@ bool ciReplay::is_loaded(Method* method) {
   ciMethodRecord* rec = replay_state->find_ciMethodRecord(method);
   return rec != NULL;
 }
-#endif // PRODUCT
+
+bool ciReplay::is_klass_unresolved(const InstanceKlass* klass) {
+  if (no_replay_state()) {
+    return false;
+  }
+
+  // Check if klass is found on whitelist.
+  ciInstanceKlassRecord* rec = replay_state->find_ciInstanceKlass(klass);
+  return rec == NULL;
+}
 
 oop ciReplay::obj_field(oop obj, Symbol* name) {
   InstanceKlass* ik = InstanceKlass::cast(obj->klass());
