@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,64 +26,15 @@
 #define SHARE_CLASSFILE_JAVACLASSES_HPP
 
 #include "classfile/vmClasses.hpp"
-#include "oops/oop.hpp"
 #include "oops/instanceKlass.hpp"
-#include "oops/symbol.hpp"
-#include "runtime/os.hpp"
+#include "oops/oopsHierarchy.hpp"
+#include "runtime/handles.hpp"
+#include "utilities/macros.hpp"
 #include "utilities/vmEnums.hpp"
 
+class JvmtiThreadState;
 class RecordComponent;
 
-// Interface for manipulating the basic Java classes.
-
-#define BASIC_JAVA_CLASSES_DO_PART1(f) \
-  f(java_lang_Class) \
-  f(java_lang_String) \
-  f(java_lang_ref_Reference) \
-  //end
-
-#define BASIC_JAVA_CLASSES_DO_PART2(f) \
-  f(java_lang_System) \
-  f(java_lang_ClassLoader) \
-  f(java_lang_Throwable) \
-  f(java_lang_Thread) \
-  f(java_lang_ThreadGroup) \
-  f(java_lang_InternalError) \
-  f(java_lang_AssertionStatusDirectives) \
-  f(java_lang_ref_SoftReference) \
-  f(java_lang_invoke_MethodHandle) \
-  f(java_lang_invoke_DirectMethodHandle) \
-  f(java_lang_invoke_MemberName) \
-  f(java_lang_invoke_ResolvedMethodName) \
-  f(java_lang_invoke_LambdaForm) \
-  f(java_lang_invoke_MethodType) \
-  f(java_lang_invoke_CallSite) \
-  f(java_lang_invoke_ConstantCallSite) \
-  f(java_lang_invoke_MethodHandleNatives_CallSiteContext) \
-  f(java_security_AccessControlContext) \
-  f(java_lang_reflect_AccessibleObject) \
-  f(java_lang_reflect_Method) \
-  f(java_lang_reflect_Constructor) \
-  f(java_lang_reflect_Field) \
-  f(java_lang_reflect_RecordComponent) \
-  f(reflect_ConstantPool) \
-  f(reflect_UnsafeStaticFieldAccessorImpl) \
-  f(java_lang_reflect_Parameter) \
-  f(java_lang_Module) \
-  f(java_lang_StackTraceElement) \
-  f(java_lang_StackFrameInfo) \
-  f(java_lang_LiveStackFrameInfo) \
-  f(java_util_concurrent_locks_AbstractOwnableSynchronizer) \
-  f(jdk_internal_invoke_NativeEntryPoint) \
-  f(jdk_internal_misc_UnsafeConstants) \
-  f(java_lang_boxing_object) \
-  f(vector_VectorPayload) \
-  //end
-
-#define BASIC_JAVA_CLASSES_DO(f) \
-        BASIC_JAVA_CLASSES_DO_PART1(f) \
-        BASIC_JAVA_CLASSES_DO_PART2(f)
-
 #define CHECK_INIT(offset)  assert(offset != 0, "should be initialized"); return offset;
 
 // Interface to java.lang.Object objects
@@ -273,7 +224,6 @@ class java_lang_Class : AllStatic {
   static int _static_oop_field_count_offset;
 
   static int _protection_domain_offset;
-  static int _init_lock_offset;
   static int _signers_offset;
   static int _class_loader_offset;
   static int _module_offset;
@@ -288,7 +238,6 @@ class java_lang_Class : AllStatic {
   static GrowableArray<Klass*>* _fixup_mirror_list;
   static GrowableArray<Klass*>* _fixup_module_field_list;
 
-  static void set_init_lock(oop java_class, oop init_lock);
   static void set_protection_domain(oop java_class, oop protection_domain);
   static void set_class_loader(oop java_class, oop class_loader);
   static void set_component_mirror(oop java_class, oop comp_mirror);
@@ -344,10 +293,6 @@ class java_lang_Class : AllStatic {
 
   // Support for embedded per-class oops
   static oop  protection_domain(oop java_class);
-  static oop  init_lock(oop java_class);
-  static void clear_init_lock(oop java_class) {
-    set_init_lock(java_class, NULL);
-  }
   static oop  component_mirror(oop java_class);
   static objArrayOop  signers(oop java_class);
   static void set_signers(oop java_class, objArrayOop signers);
@@ -365,8 +310,8 @@ class java_lang_Class : AllStatic {
   static oop source_file(oop java_class);
   static void set_source_file(oop java_class, oop source_file);
 
-  static int oop_size(oop java_class);
-  static void set_oop_size(HeapWord* java_class, int size);
+  static size_t oop_size(oop java_class);
+  static void set_oop_size(HeapWord* java_class, size_t size);
   static int static_oop_field_count(oop java_class);
   static void set_static_oop_field_count(oop java_class, int size);
 
@@ -390,23 +335,27 @@ class java_lang_Class : AllStatic {
 
 // Interface to java.lang.Thread objects
 
+#define THREAD_INJECTED_FIELDS(macro)                                  \
+  macro(java_lang_Thread, jvmti_thread_state, intptr_signature, false) \
+  JFR_ONLY(macro(java_lang_Thread, jfr_epoch, short_signature, false))
+
 class java_lang_Thread : AllStatic {
+  friend class java_lang_VirtualThread;
  private:
   // Note that for this class the layout changed between JDK1.2 and JDK1.3,
   // so we compute the offsets at startup rather than hard-wiring them.
+  static int _holder_offset;
   static int _name_offset;
-  static int _group_offset;
   static int _contextClassLoader_offset;
   static int _inheritedAccessControlContext_offset;
-  static int _priority_offset;
   static int _eetop_offset;
+  static int _jvmti_thread_state_offset;
   static int _interrupted_offset;
-  static int _daemon_offset;
-  static int _stillborn_offset;
-  static int _stackSize_offset;
   static int _tid_offset;
-  static int _thread_status_offset;
+  static int _continuation_offset;
   static int _park_blocker_offset;
+  static int _extentLocalBindings_offset;
+  JFR_ONLY(static int _jfr_epoch_offset;)
 
   static void compute_offsets();
 
@@ -417,6 +366,8 @@ class java_lang_Thread : AllStatic {
   static JavaThread* thread(oop java_thread);
   // Set JavaThread for instance
   static void set_thread(oop java_thread, JavaThread* thread);
+  // FieldHolder
+  static oop holder(oop java_thread);
   // Interrupted status
   static bool interrupted(oop java_thread);
   static void set_interrupted(oop java_thread, bool val);
@@ -428,9 +379,6 @@ class java_lang_Thread : AllStatic {
   static void set_priority(oop java_thread, ThreadPriority priority);
   // Thread group
   static oop  threadGroup(oop java_thread);
-  // Stillborn
-  static bool is_stillborn(oop java_thread);
-  static void set_stillborn(oop java_thread);
   // Alive (NOTE: this is not really a field, but provides the correct
   // definition without doing a Java call)
   static bool is_alive(oop java_thread);
@@ -444,7 +392,16 @@ class java_lang_Thread : AllStatic {
   // Stack size hint
   static jlong stackSize(oop java_thread);
   // Thread ID
-  static jlong thread_id(oop java_thread);
+  static int64_t thread_id(oop java_thread);
+  static ByteSize thread_id_offset();
+  // Continuation
+  static inline oop continuation(oop java_thread);
+
+  static JvmtiThreadState* jvmti_thread_state(oop java_thread);
+  static void set_jvmti_thread_state(oop java_thread, JvmtiThreadState* state);
+
+  // Clear all extent local bindings on error
+  static void clear_extentLocalBindings(oop java_thread);
 
   // Blocker object responsible for thread parking
   static oop park_blocker(oop java_thread);
@@ -456,23 +413,78 @@ class java_lang_Thread : AllStatic {
 
   static const char*  thread_status_name(oop java_thread_oop);
 
+  // Fill in current stack trace, can cause GC
+  static oop async_get_stack_trace(oop java_thread, TRAPS);
+
+  JFR_ONLY(static u2 jfr_epoch(oop java_thread);)
+  JFR_ONLY(static void set_jfr_epoch(oop java_thread, u2 epoch);)
+  JFR_ONLY(static int jfr_epoch_offset() { CHECK_INIT(_jfr_epoch_offset); })
+
   // Debugging
   friend class JavaClasses;
 };
 
+// Interface to java.lang.Thread$FieldHolder objects
+
+class java_lang_Thread_FieldHolder : AllStatic {
+ private:
+  static int _group_offset;
+  static int _priority_offset;
+  static int _stackSize_offset;
+  static int _daemon_offset;
+  static int _thread_status_offset;
+
+  static void compute_offsets();
+
+ public:
+  static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
+
+  static oop threadGroup(oop holder);
+
+  static ThreadPriority priority(oop holder);
+  static void set_priority(oop holder, ThreadPriority priority);
+
+  static jlong stackSize(oop holder);
+
+  static bool is_daemon(oop holder);
+  static void set_daemon(oop holder);
+
+  static void set_thread_status(oop holder, JavaThreadStatus);
+  static JavaThreadStatus get_thread_status(oop holder);
+
+  friend class JavaClasses;
+};
+
+// Interface to java.lang.Thread$Constants objects
+
+class java_lang_Thread_Constants : AllStatic {
+ private:
+  static int _static_VTHREAD_GROUP_offset;
+  static int _static_NOT_SUPPORTED_CLASSLOADER_offset;
+
+  static void compute_offsets();
+  static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
+
+ public:
+  static oop get_VTHREAD_GROUP();
+  static oop get_NOT_SUPPORTED_CLASSLOADER();
+
+  friend class JavaClasses;
+};
+
 // Interface to java.lang.ThreadGroup objects
 
 class java_lang_ThreadGroup : AllStatic {
  private:
   static int _parent_offset;
   static int _name_offset;
-  static int _threads_offset;
-  static int _groups_offset;
   static int _maxPriority_offset;
-  static int _destroyed_offset;
   static int _daemon_offset;
-  static int _nthreads_offset;
+
   static int _ngroups_offset;
+  static int _groups_offset;
+  static int _nweaks_offset;
+  static int _weaks_offset;
 
   static void compute_offsets();
 
@@ -480,29 +492,75 @@ class java_lang_ThreadGroup : AllStatic {
   static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
 
   // parent ThreadGroup
-  static oop  parent(oop java_thread_group);
+  static oop parent(oop java_thread_group);
   // name
   static const char* name(oop java_thread_group);
-  // ("name as oop" accessor is not necessary)
-  // Number of threads in group
-  static int nthreads(oop java_thread_group);
-  // threads
-  static objArrayOop threads(oop java_thread_group);
-  // Number of threads in group
-  static int ngroups(oop java_thread_group);
-  // groups
-  static objArrayOop groups(oop java_thread_group);
   // maxPriority in group
   static ThreadPriority maxPriority(oop java_thread_group);
-  // Destroyed
-  static bool is_destroyed(oop java_thread_group);
   // Daemon
   static bool is_daemon(oop java_thread_group);
+
+  // Number of strongly reachable thread groups
+  static int ngroups(oop java_thread_group);
+  // Strongly reachable thread groups
+  static objArrayOop groups(oop java_thread_group);
+  // Number of weakly reachable thread groups
+  static int nweaks(oop java_thread_group);
+  // Weakly reachable thread groups
+  static objArrayOop weaks(oop java_thread_group);
+
   // Debugging
   friend class JavaClasses;
 };
 
 
+// Interface to java.lang.VirtualThread objects
+
+class java_lang_VirtualThread : AllStatic {
+ private:
+  static int static_notify_jvmti_events_offset;
+  static int static_vthread_scope_offset;
+  static int _carrierThread_offset;
+  static int _continuation_offset;
+  static int _state_offset;
+  JFR_ONLY(static int _jfr_epoch_offset;)
+ public:
+  enum {
+    NEW          = 0,
+    STARTED      = 1,
+    RUNNABLE     = 2,
+    RUNNING      = 3,
+    PARKING      = 4,
+    PARKED       = 5,
+    PINNED       = 6,
+    YIELDING     = 7,
+    TERMINATED   = 99,
+
+    // can be suspended from scheduling when unmounted
+    SUSPENDED    = 1 << 8,
+    RUNNABLE_SUSPENDED = (RUNNABLE | SUSPENDED),
+    PARKED_SUSPENDED   = (PARKED | SUSPENDED)
+  };
+
+  static void compute_offsets();
+  static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
+
+  // Testers
+  static bool is_subclass(Klass* klass) {
+    return klass->is_subclass_of(vmClasses::VirtualThread_klass());
+  }
+  static bool is_instance(oop obj);
+
+  static oop vthread_scope();
+  static oop carrier_thread(oop vthread);
+  static oop continuation(oop vthread);
+  static int state(oop vthread);
+  static JavaThreadStatus map_state_to_thread_status(int state);
+  static bool notify_jvmti_events();
+  static void set_notify_jvmti_events(bool enable);
+  static void init_static_notify_jvmti_events();
+};
+
 
 // Interface to java.lang.Throwable objects
 
@@ -517,9 +575,10 @@ class java_lang_Throwable: AllStatic {
     trace_bcis_offset    = 1,
     trace_mirrors_offset = 2,
     trace_names_offset   = 3,
-    trace_next_offset    = 4,
-    trace_hidden_offset  = 5,
-    trace_size           = 6,
+    trace_conts_offset   = 4,
+    trace_next_offset    = 5,
+    trace_hidden_offset  = 6,
+    trace_size           = 7,
     trace_chunk_size     = 32
   };
 
@@ -560,8 +619,9 @@ class java_lang_Throwable: AllStatic {
   // Fill in current stack trace, can cause GC
   static void fill_in_stack_trace(Handle throwable, const methodHandle& method, TRAPS);
   static void fill_in_stack_trace(Handle throwable, const methodHandle& method = methodHandle());
+
   // Programmatic access to stack trace
-  static void get_stack_trace_elements(Handle throwable, objArrayHandle stack_trace, TRAPS);
+  static void get_stack_trace_elements(int depth, Handle backtrace, objArrayHandle stack_trace, TRAPS);
 
   // For recreating class initialization error exceptions.
   static Handle get_cause_with_stack_trace(Handle throwable, TRAPS);
@@ -905,6 +965,7 @@ class java_lang_ref_Reference: AllStatic {
  public:
   // Accessors
   static inline oop weak_referent_no_keepalive(oop ref);
+  static inline oop weak_referent(oop ref);
   static inline oop phantom_referent_no_keepalive(oop ref);
   static inline oop unknown_referent_no_keepalive(oop ref);
   static inline void clear_referent(oop ref);
@@ -920,6 +981,8 @@ class java_lang_ref_Reference: AllStatic {
   static bool is_referent_field(oop obj, ptrdiff_t offset);
   static inline bool is_final(oop ref);
   static inline bool is_phantom(oop ref);
+  static inline bool is_weak(oop ref);
+  static inline bool is_soft(oop ref);
 
   static int referent_offset()    { CHECK_INIT(_referent_offset); }
   static int queue_offset()       { CHECK_INIT(_queue_offset); }
@@ -1038,16 +1101,12 @@ class java_lang_invoke_LambdaForm: AllStatic {
 // Interface to java.lang.invoke.NativeEntryPoint objects
 // (These are a private interface for managing adapter code generation.)
 
-class jdk_internal_invoke_NativeEntryPoint: AllStatic {
+class jdk_internal_foreign_abi_NativeEntryPoint: AllStatic {
   friend class JavaClasses;
 
  private:
-  static int _shadow_space_offset;
-  static int _argMoves_offset;
-  static int _returnMoves_offset;
-  static int _need_transition_offset;
   static int _method_type_offset;
-  static int _name_offset;
+  static int _downcall_stub_address_offset;
 
   static void compute_offsets();
 
@@ -1055,12 +1114,8 @@ class jdk_internal_invoke_NativeEntryPoint: AllStatic {
   static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
 
   // Accessors
-  static jint       shadow_space(oop entry);
-  static oop        argMoves(oop entry);
-  static oop        returnMoves(oop entry);
-  static jboolean   need_transition(oop entry);
   static oop        method_type(oop entry);
-  static oop        name(oop entry);
+  static jlong      downcall_stub_address(oop entry);
 
   // Testers
   static bool is_subclass(Klass* klass) {
@@ -1070,12 +1125,92 @@ class jdk_internal_invoke_NativeEntryPoint: AllStatic {
   static bool is_instance(oop obj);
 
   // Accessors for code generation:
-  static int shadow_space_offset_in_bytes()    { return _shadow_space_offset;    }
-  static int argMoves_offset_in_bytes()        { return _argMoves_offset;        }
-  static int returnMoves_offset_in_bytes()     { return _returnMoves_offset;     }
-  static int need_transition_offset_in_bytes() { return _need_transition_offset; }
-  static int method_type_offset_in_bytes()     { return _method_type_offset;     }
-  static int name_offset_in_bytes()            { return _name_offset;            }
+  static int method_type_offset_in_bytes()           { return _method_type_offset; }
+  static int downcall_stub_address_offset_in_bytes() { return _downcall_stub_address_offset; }
+};
+
+class jdk_internal_foreign_abi_ABIDescriptor: AllStatic {
+  friend class JavaClasses;
+
+ private:
+  static int _inputStorage_offset;
+  static int _outputStorage_offset;
+  static int _volatileStorage_offset;
+  static int _stackAlignment_offset;
+  static int _shadowSpace_offset;
+  static int _targetAddrStorage_offset;
+  static int _retBufAddrStorage_offset;
+
+  static void compute_offsets();
+
+ public:
+  static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
+
+  // Accessors
+  static objArrayOop inputStorage(oop entry);
+  static objArrayOop outputStorage(oop entry);
+  static objArrayOop volatileStorage(oop entry);
+  static jint        stackAlignment(oop entry);
+  static jint        shadowSpace(oop entry);
+  static oop         targetAddrStorage(oop entry);
+  static oop         retBufAddrStorage(oop entry);
+
+  // Testers
+  static bool is_subclass(Klass* klass) {
+    return vmClasses::ABIDescriptor_klass() != NULL &&
+      klass->is_subclass_of(vmClasses::ABIDescriptor_klass());
+  }
+  static bool is_instance(oop obj);
+};
+
+class jdk_internal_foreign_abi_VMStorage: AllStatic {
+  friend class JavaClasses;
+
+ private:
+  static int _type_offset;
+  static int _index_offset;
+  static int _debugName_offset;
+
+  static void compute_offsets();
+
+ public:
+  static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
+
+  // Accessors
+  static jint        type(oop entry);
+  static jint        index(oop entry);
+  static oop         debugName(oop entry);
+
+  // Testers
+  static bool is_subclass(Klass* klass) {
+    return vmClasses::VMStorage_klass() != NULL &&
+      klass->is_subclass_of(vmClasses::VMStorage_klass());
+  }
+  static bool is_instance(oop obj);
+};
+
+class jdk_internal_foreign_abi_CallConv: AllStatic {
+  friend class JavaClasses;
+
+ private:
+  static int _argRegs_offset;
+  static int _retRegs_offset;
+
+  static void compute_offsets();
+
+ public:
+  static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
+
+  // Accessors
+  static objArrayOop argRegs(oop entry);
+  static objArrayOop retRegs(oop entry);
+
+  // Testers
+  static bool is_subclass(Klass* klass) {
+    return vmClasses::CallConv_klass() != NULL &&
+      klass->is_subclass_of(vmClasses::CallConv_klass());
+  }
+  static bool is_instance(oop obj);
 };
 
 // Interface to java.lang.invoke.MemberName objects
@@ -1485,15 +1620,17 @@ private:
   static int _memberName_offset;
   static int _bci_offset;
   static int _version_offset;
+  static int _contScope_offset;
 
   static Method* get_method(Handle stackFrame, InstanceKlass* holder, TRAPS);
 
 public:
   // Setters
-  static void set_method_and_bci(Handle stackFrame, const methodHandle& method, int bci, TRAPS);
+  static void set_method_and_bci(Handle stackFrame, const methodHandle& method, int bci, oop cont, TRAPS);
   static void set_bci(oop info, int value);
 
   static void set_version(oop info, short value);
+  static void set_contScope(oop info, oop value);
 
   static void compute_offsets();
   static void serialize_offsets(SerializeClosure* f) NOT_CDS_RETURN;
@@ -1752,23 +1889,11 @@ class InjectedField {
   }
 };
 
-#define DECLARE_INJECTED_FIELD_ENUM(klass, name, signature, may_be_java) \
-  klass##_##name##_enum,
-
-#define ALL_INJECTED_FIELDS(macro)          \
-  STRING_INJECTED_FIELDS(macro)             \
-  CLASS_INJECTED_FIELDS(macro)              \
-  CLASSLOADER_INJECTED_FIELDS(macro)        \
-  RESOLVEDMETHOD_INJECTED_FIELDS(macro)     \
-  MEMBERNAME_INJECTED_FIELDS(macro)         \
-  CALLSITECONTEXT_INJECTED_FIELDS(macro)    \
-  STACKFRAMEINFO_INJECTED_FIELDS(macro)     \
-  MODULE_INJECTED_FIELDS(macro)             \
-  INTERNALERROR_INJECTED_FIELDS(macro)
-
 
 // Interface to hard-coded offset checking
 
+enum class InjectedFieldID : int;
+
 class JavaClasses : AllStatic {
  private:
 
@@ -1776,10 +1901,6 @@ class JavaClasses : AllStatic {
 
   static bool check_offset(const char *klass_name, int offset, const char *field_name, const char* field_sig) PRODUCT_RETURN0;
  public:
-  enum InjectedFieldID {
-    ALL_INJECTED_FIELDS(DECLARE_INJECTED_FIELD_ENUM)
-    MAX_enum
-  };
 
   static int compute_injected_offset(InjectedFieldID id);
 
@@ -1788,9 +1909,15 @@ class JavaClasses : AllStatic {
   static void serialize_offsets(SerializeClosure* soc) NOT_CDS_RETURN;
   static InjectedField* get_injected(Symbol* class_name, int* field_count);
   static bool is_supported_for_archiving(oop obj) NOT_CDS_JAVA_HEAP_RETURN_(false);
+
+  static void compute_offset(int &dest_offset,
+                             InstanceKlass* ik, Symbol* name_symbol, Symbol* signature_symbol,
+                             bool is_static = false);
+  static void compute_offset(int& dest_offset, InstanceKlass* ik,
+                             const char* name_string, Symbol* signature_symbol,
+                             bool is_static = false);
 };
 
-#undef DECLARE_INJECTED_FIELD_ENUM
-
 #undef CHECK_INIT
+
 #endif // SHARE_CLASSFILE_JAVACLASSES_HPP
