@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -103,7 +103,6 @@ static THREAD_LOCAL bool _lookup_shared_first = false;
 // Static arena for symbols that are not deallocated
 Arena* SymbolTable::_arena = NULL;
 
-static bool _rehashed = false;
 static uint64_t _alt_hash_seed = 0;
 
 static inline void log_trace_symboltable_helper(Symbol* sym, const char* msg) {
@@ -162,18 +161,12 @@ public:
   }
 };
 
-static size_t ceil_log2(size_t value) {
-  size_t ret;
-  for (ret = 1; ((size_t)1 << ret) < value; ++ret);
-  return ret;
-}
-
 void SymbolTable::create_table ()  {
   size_t start_size_log_2 = ceil_log2(SymbolTableSize);
   _current_size = ((size_t)1) << start_size_log_2;
   log_trace(symboltable)("Start size: " SIZE_FORMAT " (" SIZE_FORMAT ")",
                          _current_size, start_size_log_2);
-  _local_table = new SymbolTableHash(start_size_log_2, END_SIZE, REHASH_LEN);
+  _local_table = new SymbolTableHash(start_size_log_2, END_SIZE, REHASH_LEN, true);
 
   // Initialize the arena for global symbols, size passed in depends on CDS.
   if (symbol_alloc_arena_size == 0) {
@@ -536,10 +529,17 @@ TableStatistics SymbolTable::get_table_statistics() {
   return ts;
 }
 
-void SymbolTable::print_table_statistics(outputStream* st,
-                                         const char* table_name) {
+void SymbolTable::print_table_statistics(outputStream* st) {
   SizeFunc sz;
-  _local_table->statistics_to(Thread::current(), sz, st, table_name);
+  _local_table->statistics_to(Thread::current(), sz, st, "SymbolTable");
+
+  if (!_shared_table.empty()) {
+    _shared_table.print_table_statistics(st, "Shared Symbol Table");
+  }
+
+  if (!_dynamic_shared_table.empty()) {
+    _dynamic_shared_table.print_table_statistics(st, "Dynamic Shared Symbol Table");
+  }
 }
 
 // Verification
@@ -563,6 +563,14 @@ void SymbolTable::verify() {
   }
 }
 
+static void print_symbol(outputStream* st, Symbol* sym) {
+  const char* utf8_string = (const char*)sym->bytes();
+  int utf8_length = sym->utf8_length();
+  st->print("%d %d: ", utf8_length, sym->refcount());
+  HashtableTextDump::put_utf8(st, utf8_string, utf8_length);
+  st->cr();
+}
+
 // Dumping
 class DumpSymbol : StackObj {
   Thread* _thr;
@@ -572,19 +580,24 @@ public:
   bool operator()(Symbol** value) {
     assert(value != NULL, "expected valid value");
     assert(*value != NULL, "value should point to a symbol");
-    Symbol* sym = *value;
-    const char* utf8_string = (const char*)sym->bytes();
-    int utf8_length = sym->utf8_length();
-    _st->print("%d %d: ", utf8_length, sym->refcount());
-    HashtableTextDump::put_utf8(_st, utf8_string, utf8_length);
-    _st->cr();
+    print_symbol(_st, *value);
     return true;
   };
 };
 
+class DumpSharedSymbol : StackObj {
+  outputStream* _st;
+public:
+  DumpSharedSymbol(outputStream* st) : _st(st) {}
+  void do_value(Symbol* value) {
+    assert(value != NULL, "value should point to a symbol");
+    print_symbol(_st, value);
+  };
+};
+
 void SymbolTable::dump(outputStream* st, bool verbose) {
   if (!verbose) {
-    print_table_statistics(st, "SymbolTable");
+    print_table_statistics(st);
   } else {
     Thread* thr = Thread::current();
     ResourceMark rm(thr);
@@ -593,6 +606,20 @@ void SymbolTable::dump(outputStream* st, bool verbose) {
     if (!_local_table->try_scan(thr, ds)) {
       log_info(symboltable)("dump unavailable at this moment");
     }
+    if (!_shared_table.empty()) {
+      st->print_cr("#----------------");
+      st->print_cr("# Shared symbols:");
+      st->print_cr("#----------------");
+      DumpSharedSymbol dss(st);
+      _shared_table.iterate(&dss);
+    }
+    if (!_dynamic_shared_table.empty()) {
+      st->print_cr("#------------------------");
+      st->print_cr("# Dynamic shared symbols:");
+      st->print_cr("#------------------------");
+      DumpSharedSymbol dss(st);
+      _dynamic_shared_table.iterate(&dss);
+    }
   }
 }
 
@@ -602,7 +629,7 @@ void SymbolTable::copy_shared_symbol_table(GrowableArray<Symbol*>* symbols,
   ArchiveBuilder* builder = ArchiveBuilder::current();
   int len = symbols->length();
   for (int i = 0; i < len; i++) {
-    Symbol* sym = ArchiveBuilder::get_relocated_symbol(symbols->at(i));
+    Symbol* sym = ArchiveBuilder::get_buffered_symbol(symbols->at(i));
     unsigned int fixed_hash = hash_shared_symbol((const char*)sym->bytes(), sym->utf8_length());
     assert(fixed_hash == hash_symbol((const char*)sym->bytes(), sym->utf8_length(), false),
            "must not rehash during dumping");
@@ -750,7 +777,7 @@ bool SymbolTable::do_rehash() {
 
   // We use current size
   size_t new_size = _local_table->get_size_log2(Thread::current());
-  SymbolTableHash* new_table = new SymbolTableHash(new_size, END_SIZE, REHASH_LEN);
+  SymbolTableHash* new_table = new SymbolTableHash(new_size, END_SIZE, REHASH_LEN, true);
   // Use alt hash from now on
   _alt_hash = true;
   if (!_local_table->try_move_nodes_to(Thread::current(), new_table)) {
@@ -766,39 +793,13 @@ bool SymbolTable::do_rehash() {
   return true;
 }
 
-bool SymbolTable::should_grow() {
-  return get_load_factor() > PREF_AVG_LIST_LEN && !_local_table->is_max_size_reached();
-}
-
-bool SymbolTable::rehash_table_expects_safepoint_rehashing() {
-  // No rehashing required
-  if (!needs_rehashing()) {
-    return false;
-  }
-
-  // Grow instead of rehash
-  if (should_grow()) {
-    return false;
-  }
-
-  // Already rehashed
-  if (_rehashed) {
-    return false;
-  }
-
-  // Resizing in progress
-  if (!_local_table->is_safepoint_safe()) {
-    return false;
-  }
-
-  return true;
-}
-
 void SymbolTable::rehash_table() {
+  static bool rehashed = false;
   log_debug(symboltable)("Table imbalanced, rehashing called.");
 
   // Grow instead of rehash.
-  if (should_grow()) {
+  if (get_load_factor() > PREF_AVG_LIST_LEN &&
+      !_local_table->is_max_size_reached()) {
     log_debug(symboltable)("Choosing growing over rehashing.");
     trigger_cleanup();
     _needs_rehashing = false;
@@ -806,7 +807,7 @@ void SymbolTable::rehash_table() {
   }
 
   // Already rehashed.
-  if (_rehashed) {
+  if (rehashed) {
     log_warning(symboltable)("Rehashing already done, still long lists.");
     trigger_cleanup();
     _needs_rehashing = false;
@@ -816,7 +817,7 @@ void SymbolTable::rehash_table() {
   _alt_hash_seed = AltHashing::compute_seed();
 
   if (do_rehash()) {
-    _rehashed = true;
+    rehashed = true;
   } else {
     log_info(symboltable)("Resizes in progress rehashing skipped.");
   }
