@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,10 +27,12 @@
 
 #include "code/compiledMethod.hpp"
 
+class CompileTask;
 class DepChange;
 class DirectiveSet;
 class DebugInformationRecorder;
 class JvmtiThreadState;
+class OopIterateClosure;
 
 // nmethods (native methods) are the compiled code versions of Java methods.
 //
@@ -65,13 +67,12 @@ class JVMCINMethodData;
 class nmethod : public CompiledMethod {
   friend class VMStructs;
   friend class JVMCIVMStructs;
-  friend class NMethodSweeper;
   friend class CodeCache;  // scavengable oops
   friend class JVMCINMethodData;
 
  private:
-  // Shared fields for all nmethod's
-  int       _entry_bci;        // != InvocationEntryBci if this nmethod is an on-stack replacement method
+
+  uint64_t  _gc_epoch;
 
   // To support simple linked-list chaining of nmethods:
   nmethod*  _osr_link;         // from InstanceKlass::osr_nmethods_head
@@ -129,7 +130,7 @@ class nmethod : public CompiledMethod {
   // Unclaimed (C)-> N|SD (C)-> X|SD: the nmethod has been processed strongly from
   //   the beginning by a single thread.
   //
-  // "|" describes the concatentation of bits in _oops_do_mark_link.
+  // "|" describes the concatenation of bits in _oops_do_mark_link.
   //
   // The diagram also describes the threads responsible for changing the nmethod to
   // the next state by marking the _transition_ with (C) and (O), which mean "current"
@@ -195,6 +196,11 @@ class nmethod : public CompiledMethod {
   address _verified_entry_point;             // entry point without class check
   address _osr_entry_point;                  // entry point for on stack replacement
 
+  nmethod* _unlinked_next;
+
+  // Shared fields for all nmethod's
+  int _entry_bci;      // != InvocationEntryBci if this nmethod is an on-stack replacement method
+
   // Offsets for different nmethod parts
   int  _exception_offset;
   // Offset of the unwind handler if it exists
@@ -207,7 +213,6 @@ class nmethod : public CompiledMethod {
   int _scopes_data_offset;
   int _scopes_pcs_offset;
   int _dependencies_offset;
-  int _native_invokers_offset;
   int _handler_table_offset;
   int _nul_chk_table_offset;
 #if INCLUDE_JVMCI
@@ -223,21 +228,6 @@ class nmethod : public CompiledMethod {
   int _orig_pc_offset;
 
   int _compile_id;                           // which compilation made this nmethod
-  int _comp_level;                           // compilation level
-
-  // protected by CodeCache_lock
-  bool _has_flushed_dependencies;            // Used for maintenance of dependencies (CodeCache_lock)
-
-  // used by jvmti to track if an event has been posted for this nmethod.
-  bool _unload_reported;
-  bool _load_reported;
-
-  // Protected by CompiledMethod_lock
-  volatile signed char _state;               // {not_installed, in_use, not_entrant, zombie, unloaded}
-
-#ifdef ASSERT
-  bool _oops_are_stale;  // indicates that it's no longer safe to access oops section
-#endif
 
 #if INCLUDE_RTM_OPT
   // RTM state at compile time. Used during deoptimization to decide
@@ -245,29 +235,6 @@ class nmethod : public CompiledMethod {
   RTMState _rtm_state;
 #endif
 
-  // Nmethod Flushing lock. If non-zero, then the nmethod is not removed
-  // and is not made into a zombie. However, once the nmethod is made into
-  // a zombie, it will be locked one final time if CompiledMethodUnload
-  // event processing needs to be done.
-  volatile jint _lock_count;
-
-  // not_entrant method removal. Each mark_sweep pass will update
-  // this mark to current sweep invocation count if it is seen on the
-  // stack.  An not_entrant method can be removed when there are no
-  // more activations, i.e., when the _stack_traversal_mark is less than
-  // current sweep traversal index.
-  volatile long _stack_traversal_mark;
-
-  // The _hotness_counter indicates the hotness of a method. The higher
-  // the value the hotter the method. The hotness counter of a nmethod is
-  // set to [(ReservedCodeCacheSize / (1024 * 1024)) * 2] each time the method
-  // is active while stack scanning (do_stack_scanning()). The hotness
-  // counter is decreased (by 1) while sweeping.
-  int _hotness_counter;
-
-  // Local state used to keep track of whether unloading is happening or not
-  volatile uint8_t _is_unloading_state;
-
   // These are used for compiled synchronized native methods to
   // locate the owner and stack slot for the BasicLock. They are
   // needed because there is no debug information for compiled native
@@ -279,7 +246,19 @@ class nmethod : public CompiledMethod {
   ByteSize _native_receiver_sp_offset;
   ByteSize _native_basic_lock_sp_offset;
 
-  friend class nmethodLocker;
+  CompLevel _comp_level;               // compilation level
+
+  // Local state used to keep track of whether unloading is happening or not
+  volatile uint8_t _is_unloading_state;
+
+  // protected by CodeCache_lock
+  bool _has_flushed_dependencies;      // Used for maintenance of dependencies (CodeCache_lock)
+
+  // used by jvmti to track if an event has been posted for this nmethod.
+  bool _load_reported;
+
+  // Protected by CompiledMethod_lock
+  volatile signed char _state;         // {not_installed, in_use, not_used, not_entrant}
 
   // For native wrappers
   nmethod(Method* method,
@@ -309,8 +288,7 @@ class nmethod : public CompiledMethod {
           ExceptionHandlerTable* handler_table,
           ImplicitExceptionTable* nul_chk_table,
           AbstractCompiler* compiler,
-          int comp_level,
-          const GrowableArrayView<RuntimeStub*>& native_invokers
+          CompLevel comp_level
 #if INCLUDE_JVMCI
           , char* speculations,
           int speculations_len,
@@ -327,14 +305,13 @@ class nmethod : public CompiledMethod {
 
   // Returns true if this thread changed the state of the nmethod or
   // false if another thread performed the transition.
-  bool make_not_entrant_or_zombie(int state);
   bool make_entrant() { Unimplemented(); return false; }
   void inc_decompile_count();
 
   // Inform external interfaces that a compiled method has been unloaded
   void post_compiled_method_unload();
 
-  // Initailize fields to their default values
+  // Initialize fields to their default values
   void init_defaults();
 
   // Offsets
@@ -358,8 +335,7 @@ class nmethod : public CompiledMethod {
                               ExceptionHandlerTable* handler_table,
                               ImplicitExceptionTable* nul_chk_table,
                               AbstractCompiler* compiler,
-                              int comp_level,
-                              const GrowableArrayView<RuntimeStub*>& native_invokers = GrowableArrayView<RuntimeStub*>::EMPTY
+                              CompLevel comp_level
 #if INCLUDE_JVMCI
                               , char* speculations = NULL,
                               int speculations_len = 0,
@@ -372,9 +348,9 @@ class nmethod : public CompiledMethod {
   // Only used for unit tests.
   nmethod()
     : CompiledMethod(),
-      _is_unloading_state(0),
       _native_receiver_sp_offset(in_ByteSize(-1)),
-      _native_basic_lock_sp_offset(in_ByteSize(-1)) {}
+      _native_basic_lock_sp_offset(in_ByteSize(-1)),
+      _is_unloading_state(0) {}
 
 
   static nmethod* new_native_nmethod(const methodHandle& method,
@@ -385,7 +361,8 @@ class nmethod : public CompiledMethod {
                                      int frame_size,
                                      ByteSize receiver_sp_offset,
                                      ByteSize basic_lock_sp_offset,
-                                     OopMapSet* oop_maps);
+                                     OopMapSet* oop_maps,
+                                     int exception_handler = -1);
 
   // type info
   bool is_nmethod() const                         { return true; }
@@ -408,9 +385,7 @@ class nmethod : public CompiledMethod {
   PcDesc* scopes_pcs_begin      () const          { return (PcDesc*)(header_begin() + _scopes_pcs_offset   ); }
   PcDesc* scopes_pcs_end        () const          { return (PcDesc*)(header_begin() + _dependencies_offset) ; }
   address dependencies_begin    () const          { return           header_begin() + _dependencies_offset  ; }
-  address dependencies_end      () const          { return           header_begin() + _native_invokers_offset ; }
-  RuntimeStub** native_invokers_begin() const     { return (RuntimeStub**)(header_begin() + _native_invokers_offset) ; }
-  RuntimeStub** native_invokers_end  () const     { return (RuntimeStub**)(header_begin() + _handler_table_offset); }
+  address dependencies_end      () const          { return           header_begin() + _handler_table_offset ; }
   address handler_table_begin   () const          { return           header_begin() + _handler_table_offset ; }
   address handler_table_end     () const          { return           header_begin() + _nul_chk_table_offset ; }
   address nul_chk_table_begin   () const          { return           header_begin() + _nul_chk_table_offset ; }
@@ -438,10 +413,6 @@ class nmethod : public CompiledMethod {
 
   int total_size        () const;
 
-  void dec_hotness_counter()        { _hotness_counter--; }
-  void set_hotness_counter(int val) { _hotness_counter = val; }
-  int  hotness_counter() const      { return _hotness_counter; }
-
   // Containment
   bool oops_contains         (oop*    addr) const { return oops_begin         () <= addr && addr < oops_end         (); }
   bool metadata_contains     (Metadata** addr) const   { return metadata_begin     () <= addr && addr < metadata_end     (); }
@@ -455,15 +426,17 @@ class nmethod : public CompiledMethod {
   // flag accessing and manipulation
   bool  is_not_installed() const                  { return _state == not_installed; }
   bool  is_in_use() const                         { return _state <= in_use; }
-  bool  is_alive() const                          { return _state < unloaded; }
   bool  is_not_entrant() const                    { return _state == not_entrant; }
-  bool  is_zombie() const                         { return _state == zombie; }
-  bool  is_unloaded() const                       { return _state == unloaded; }
 
   void clear_unloading_state();
+  // Heuristically deduce an nmethod isn't worth keeping around
+  bool is_cold();
   virtual bool is_unloading();
   virtual void do_unloading(bool unloading_occurred);
 
+  nmethod* unlinked_next() const                  { return _unlinked_next; }
+  void set_unlinked_next(nmethod* next)           { _unlinked_next = next; }
+
 #if INCLUDE_RTM_OPT
   // rtm state accessing and manipulating
   RTMState  rtm_state() const                     { return _rtm_state; }
@@ -477,22 +450,16 @@ class nmethod : public CompiledMethod {
   // alive.  It is used when an uncommon trap happens.  Returns true
   // if this thread changed the state of the nmethod or false if
   // another thread performed the transition.
-  bool  make_not_entrant() {
-    assert(!method()->is_method_handle_intrinsic(), "Cannot make MH intrinsic not entrant");
-    return make_not_entrant_or_zombie(not_entrant);
-  }
+  bool  make_not_entrant();
   bool  make_not_used()    { return make_not_entrant(); }
-  bool  make_zombie()      { return make_not_entrant_or_zombie(zombie); }
 
   int get_state() const {
     return _state;
   }
 
-  void  make_unloaded();
-
   bool has_dependencies()                         { return dependencies_size() != 0; }
   void print_dependencies()                       PRODUCT_RETURN;
-  void flush_dependencies(bool delete_immediately);
+  void flush_dependencies();
   bool has_flushed_dependencies()                 { return _has_flushed_dependencies; }
   void set_has_flushed_dependencies()             {
     assert(!has_flushed_dependencies(), "should only happen once");
@@ -510,7 +477,6 @@ class nmethod : public CompiledMethod {
   oop*  oop_addr_at(int index) const {  // for GC
     // relocation indexes are biased by 1 (because 0 is reserved)
     assert(index > 0 && index <= oops_count(), "must be a valid non-zero index");
-    assert(!_oops_are_stale, "oops are stale");
     return &oops_begin()[index - 1];
   }
 
@@ -526,8 +492,6 @@ class nmethod : public CompiledMethod {
   void copy_values(GrowableArray<jobject>* oops);
   void copy_values(GrowableArray<Metadata*>* metadata);
 
-  void free_native_invokers();
-
   // Relocation support
 private:
   void fix_oop_relocations(address begin, address end, bool initialize_immediates);
@@ -537,10 +501,6 @@ public:
   void fix_oop_relocations(address begin, address end) { fix_oop_relocations(begin, end, false); }
   void fix_oop_relocations()                           { fix_oop_relocations(NULL, NULL, false); }
 
-  // Sweeper support
-  long  stack_traversal_mark()                    { return _stack_traversal_mark; }
-  void  set_stack_traversal_mark(long l)          { _stack_traversal_mark = l; }
-
   // On-stack replacement support
   int   osr_entry_bci() const                     { assert(is_osr_method(), "wrong kind of nmethod"); return _entry_bci; }
   address  osr_entry() const                      { assert(is_osr_method(), "wrong kind of nmethod"); return _osr_entry_point; }
@@ -551,22 +511,15 @@ public:
   // Verify calls to dead methods have been cleaned.
   void verify_clean_inline_caches();
 
-  // unlink and deallocate this nmethod
-  // Only NMethodSweeper class is expected to use this. NMethodSweeper is not
-  // expected to use any other private methods/data in this class.
+  // Unlink this nmethod from the system
+  void unlink();
 
- protected:
+  // Deallocate this nmethod - called by the GC
   void flush();
 
- public:
-  // When true is returned, it is unsafe to remove this nmethod even if
-  // it is a zombie, since the VM or the ServiceThread might still be
-  // using it.
-  bool is_locked_by_vm() const                    { return _lock_count >0; }
-
   // See comment at definition of _last_seen_on_stack
-  void mark_as_seen_on_stack();
-  bool can_convert_to_zombie();
+  void mark_as_maybe_on_stack();
+  bool is_maybe_on_stack();
 
   // Evolution support. We make old (discarded) compiled methods point to new Method*s.
   void set_method(Method* method) { _method = method; }
@@ -595,6 +548,9 @@ public:
   // nmethod.
   bool oops_do_try_claim();
 
+  // Loom support for following nmethods on the stack
+  void follow_nmethod(OopIterateClosure* cl);
+
   // Class containing callbacks for the oops_do_process_weak/strong() methods
   // below.
   class OopsDoProcessor {
@@ -621,9 +577,7 @@ public:
 
   address* orig_pc_addr(const frame* fr);
 
-  // used by jvmti to track if the load and unload events has been reported
-  bool  unload_reported() const                   { return _unload_reported; }
-  void  set_unload_reported()                     { _unload_reported = true; }
+  // used by jvmti to track if the load events has been reported
   bool  load_reported() const                     { return _load_reported; }
   void  set_load_reported()                       { _load_reported = true; }
 
@@ -632,9 +586,10 @@ public:
   void copy_scopes_pcs(PcDesc* pcs, int count);
   void copy_scopes_data(address buffer, int size);
 
-  // Accessor/mutator for the original pc of a frame before a frame was deopted.
-  address get_original_pc(const frame* fr) { return *orig_pc_addr(fr); }
-  void    set_original_pc(const frame* fr, address pc) { *orig_pc_addr(fr) = pc; }
+  int orig_pc_offset() { return _orig_pc_offset; }
+
+  // Post successful compilation
+  void post_compiled_method(CompileTask* task);
 
   // jvmti support:
   void post_compiled_method_load_event(JvmtiThreadState* state = NULL);
@@ -664,7 +619,6 @@ public:
   void print_scopes() { print_scopes_on(tty); }
   void print_scopes_on(outputStream* st)          PRODUCT_RETURN;
   void print_value_on(outputStream* st) const;
-  void print_native_invokers();
   void print_handler_table();
   void print_nul_chk_table();
   void print_recorded_oop(int log_n, int index);
@@ -681,7 +635,7 @@ public:
   void print_calls(outputStream* st)              PRODUCT_RETURN;
   static void print_statistics()                  PRODUCT_RETURN;
 
-  void maybe_print_nmethod(DirectiveSet* directive);
+  void maybe_print_nmethod(const DirectiveSet* directive);
   void print_nmethod(bool print_code);
 
   // need to re-define this from CodeBlob else the overload hides it
@@ -729,9 +683,6 @@ public:
   // corresponds to the given method as well.
   virtual bool is_dependent_on_method(Method* dependee);
 
-  // is it ok to patch at address?
-  bool is_patchable_at(address instr_address);
-
   // JVMTI's GetLocalInstance() support
   ByteSize native_receiver_sp_offset() {
     return _native_receiver_sp_offset;
@@ -754,52 +705,9 @@ public:
   virtual CompiledStaticCall* compiledStaticCall_at(Relocation* call_site) const;
   virtual CompiledStaticCall* compiledStaticCall_at(address addr) const;
   virtual CompiledStaticCall* compiledStaticCall_before(address addr) const;
-};
 
-// Locks an nmethod so its code will not get removed and it will not
-// be made into a zombie, even if it is a not_entrant method. After the
-// nmethod becomes a zombie, if CompiledMethodUnload event processing
-// needs to be done, then lock_nmethod() is used directly to keep the
-// generated code from being reused too early.
-class nmethodLocker : public StackObj {
-  CompiledMethod* _nm;
-
- public:
-
-  // note: nm can be NULL
-  // Only JvmtiDeferredEvent::compiled_method_unload_event()
-  // should pass zombie_ok == true.
-  static void lock_nmethod(CompiledMethod* nm, bool zombie_ok = false);
-  static void unlock_nmethod(CompiledMethod* nm); // (ditto)
-
-  nmethodLocker(address pc); // derive nm from pc
-  nmethodLocker(nmethod *nm) { _nm = nm; lock_nmethod(_nm); }
-  nmethodLocker(CompiledMethod *nm) {
-    _nm = nm;
-    lock(_nm);
-  }
-
-  static void lock(CompiledMethod* method, bool zombie_ok = false) {
-    if (method == NULL) return;
-    lock_nmethod(method, zombie_ok);
-  }
-
-  static void unlock(CompiledMethod* method) {
-    if (method == NULL) return;
-    unlock_nmethod(method);
-  }
-
-  nmethodLocker() { _nm = NULL; }
-  ~nmethodLocker() {
-    unlock(_nm);
-  }
-
-  CompiledMethod* code() { return _nm; }
-  void set_code(CompiledMethod* new_nm, bool zombie_ok = false) {
-    unlock(_nm);   // note:  This works even if _nm==new_nm.
-    _nm = new_nm;
-    lock(_nm, zombie_ok);
-  }
+  virtual void  make_deoptimized();
+  void finalize_relocations();
 };
 
 #endif // SHARE_CODE_NMETHOD_HPP
