@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,95 +39,109 @@
 #include "jfr/jfrEvents.hpp"
 #include "utilities/ticks.hpp"
 
-G1PostEvacuateCollectionSetCleanupTask1::G1PostEvacuateCollectionSetCleanupTask1(G1ParScanThreadStateSet* per_thread_states,
-                                                                                 G1EvacFailureRegions* evac_failure_regions) :
-  G1BatchedGangTask("Post Evacuate Cleanup 1", G1CollectedHeap::heap()->phase_times())
-{
-  add_serial_task(new MergePssTask(per_thread_states));
-  add_serial_task(new RecalculateUsedTask());
-  if (SampleCollectionSetCandidatesTask::should_execute()) {
-    add_serial_task(new SampleCollectionSetCandidatesTask());
-  }
-  if (RemoveSelfForwardPtrsTask::should_execute()) {
-    add_parallel_task(new RemoveSelfForwardPtrsTask(per_thread_states->rdcqs(), evac_failure_regions));
-  }
-  add_parallel_task(G1CollectedHeap::heap()->rem_set()->create_cleanup_after_scan_heap_roots_task());
-}
+class G1PostEvacuateCollectionSetCleanupTask1::MergePssTask : public G1AbstractSubTask {
+  G1ParScanThreadStateSet* _per_thread_states;
 
-G1PostEvacuateCollectionSetCleanupTask1::MergePssTask::MergePssTask(G1ParScanThreadStateSet* per_thread_states) :
-  G1AbstractSubTask(G1GCPhaseTimes::MergePSS),
-  _per_thread_states(per_thread_states) { }
-
-void G1PostEvacuateCollectionSetCleanupTask1::MergePssTask::do_work(uint worker_id) {
-  _per_thread_states->flush();
-}
-
-double G1PostEvacuateCollectionSetCleanupTask1::RecalculateUsedTask::worker_cost() const {
-  // If there is no evacuation failure, the work to perform is minimal.
-  return G1CollectedHeap::heap()->evacuation_failed() ? 1.0 : AlmostNoWork;
-}
-
-void G1PostEvacuateCollectionSetCleanupTask1::RecalculateUsedTask::do_work(uint worker_id) {
-  G1CollectedHeap::heap()->update_used_after_gc();
-}
-
-bool G1PostEvacuateCollectionSetCleanupTask1::SampleCollectionSetCandidatesTask::should_execute() {
-  return G1CollectedHeap::heap()->should_sample_collection_set_candidates();
-}
-
-double G1PostEvacuateCollectionSetCleanupTask1::SampleCollectionSetCandidatesTask::worker_cost() const {
-  return should_execute() ? 1.0 : AlmostNoWork;
-}
-
-class G1SampleCollectionSetCandidatesClosure : public HeapRegionClosure {
 public:
-  G1CardSetMemoryStats _total;
+  MergePssTask(G1ParScanThreadStateSet* per_thread_states) :
+    G1AbstractSubTask(G1GCPhaseTimes::MergePSS),
+    _per_thread_states(per_thread_states) { }
 
-  bool do_heap_region(HeapRegion* r) override {
-    _total.add(r->rem_set()->card_set_memory_stats());
-    return false;
+  double worker_cost() const override { return 1.0; }
+
+  void do_work(uint worker_id) override { _per_thread_states->flush_stats(); }
+};
+
+class G1PostEvacuateCollectionSetCleanupTask1::RecalculateUsedTask : public G1AbstractSubTask {
+  bool _evacuation_failed;
+
+public:
+  RecalculateUsedTask(bool evacuation_failed) : G1AbstractSubTask(G1GCPhaseTimes::RecalculateUsed), _evacuation_failed(evacuation_failed) { }
+
+  double worker_cost() const override {
+    // If there is no evacuation failure, the work to perform is minimal.
+    return _evacuation_failed ? 1.0 : AlmostNoWork;
+  }
+
+  void do_work(uint worker_id) override { G1CollectedHeap::heap()->update_used_after_gc(_evacuation_failed); }
+};
+
+class G1PostEvacuateCollectionSetCleanupTask1::SampleCollectionSetCandidatesTask : public G1AbstractSubTask {
+public:
+  SampleCollectionSetCandidatesTask() : G1AbstractSubTask(G1GCPhaseTimes::SampleCollectionSetCandidates) { }
+
+  static bool should_execute() {
+    return G1CollectedHeap::heap()->should_sample_collection_set_candidates();
+  }
+
+  double worker_cost() const override {
+    return should_execute() ? 1.0 : AlmostNoWork;
+  }
+
+  void do_work(uint worker_id) override {
+
+    class G1SampleCollectionSetCandidatesClosure : public HeapRegionClosure {
+    public:
+      G1SegmentedArrayMemoryStats _total;
+
+      bool do_heap_region(HeapRegion* r) override {
+        _total.add(r->rem_set()->card_set_memory_stats());
+        return false;
+      }
+    } cl;
+
+    G1CollectedHeap* g1h = G1CollectedHeap::heap();
+
+    g1h->collection_set()->candidates()->iterate(&cl);
+    g1h->set_collection_set_candidates_stats(cl._total);
   }
 };
 
-void G1PostEvacuateCollectionSetCleanupTask1::SampleCollectionSetCandidatesTask::do_work(uint worker_id) {
-  G1CollectedHeap* g1h = G1CollectedHeap::heap();
+class G1PostEvacuateCollectionSetCleanupTask1::RestoreRetainedRegionsTask : public G1AbstractSubTask {
+  G1RemoveSelfForwardsTask _task;
+  G1EvacFailureRegions* _evac_failure_regions;
 
-  G1SampleCollectionSetCandidatesClosure cl;
-  g1h->collection_set()->candidates()->iterate(&cl);
-  g1h->set_collection_set_candidates_stats(cl._total);
+public:
+  RestoreRetainedRegionsTask(G1EvacFailureRegions* evac_failure_regions) :
+    G1AbstractSubTask(G1GCPhaseTimes::RestoreRetainedRegions),
+    _task(evac_failure_regions),
+    _evac_failure_regions(evac_failure_regions) {
+  }
+
+  double worker_cost() const override {
+    assert(_evac_failure_regions->evacuation_failed(), "Should not call this if not executed");
+
+    double workers_per_region = (double)G1CollectedHeap::get_chunks_per_region() / G1RestoreRetainedRegionChunksPerWorker;
+    return workers_per_region * _evac_failure_regions->num_regions_failed_evacuation();
+  }
+
+  void do_work(uint worker_id) override {
+    _task.work(worker_id);
+  }
+};
+
+G1PostEvacuateCollectionSetCleanupTask1::G1PostEvacuateCollectionSetCleanupTask1(G1ParScanThreadStateSet* per_thread_states,
+                                                                                 G1EvacFailureRegions* evac_failure_regions) :
+  G1BatchedTask("Post Evacuate Cleanup 1", G1CollectedHeap::heap()->phase_times())
+{
+  bool evacuation_failed = evac_failure_regions->evacuation_failed();
+
+  add_serial_task(new MergePssTask(per_thread_states));
+  add_serial_task(new RecalculateUsedTask(evacuation_failed));
+  if (SampleCollectionSetCandidatesTask::should_execute()) {
+    add_serial_task(new SampleCollectionSetCandidatesTask());
+  }
+  add_parallel_task(G1CollectedHeap::heap()->rem_set()->create_cleanup_after_scan_heap_roots_task());
+  if (evacuation_failed) {
+    add_parallel_task(new RestoreRetainedRegionsTask(evac_failure_regions));
+  }
 }
 
-bool G1PostEvacuateCollectionSetCleanupTask1::RemoveSelfForwardPtrsTask::should_execute() {
-  return G1CollectedHeap::heap()->evacuation_failed();
-}
-
-G1PostEvacuateCollectionSetCleanupTask1::
-    RemoveSelfForwardPtrsTask::RemoveSelfForwardPtrsTask(G1RedirtyCardsQueueSet* rdcqs,
-                                                         G1EvacFailureRegions* evac_failure_regions) :
-  G1AbstractSubTask(G1GCPhaseTimes::RemoveSelfForwardingPtr),
-  _task(rdcqs, evac_failure_regions),
-  _evac_failure_regions(evac_failure_regions) { }
-
-G1PostEvacuateCollectionSetCleanupTask1::RemoveSelfForwardPtrsTask::~RemoveSelfForwardPtrsTask() {
-  G1CollectedHeap* g1h = G1CollectedHeap::heap();
-  assert(_task.num_failed_regions() == _evac_failure_regions->num_regions_failed_evacuation(),
-         "Removed regions %u inconsistent with expected %u",
-         _task.num_failed_regions(), _evac_failure_regions->num_regions_failed_evacuation());
-}
-
-double G1PostEvacuateCollectionSetCleanupTask1::RemoveSelfForwardPtrsTask::worker_cost() const {
-  assert(should_execute(), "Should not call this if not executed");
-  return _evac_failure_regions->num_regions_failed_evacuation();
-}
-
-void G1PostEvacuateCollectionSetCleanupTask1::RemoveSelfForwardPtrsTask::do_work(uint worker_id) {
-  _task.work(worker_id);
-}
-
-class G1FreeHumongousRegionClosure : public HeapRegionClosure {
+class G1FreeHumongousRegionClosure : public HeapRegionIndexClosure {
   uint _humongous_objects_reclaimed;
   uint _humongous_regions_reclaimed;
   size_t _freed_bytes;
+  G1CollectedHeap* _g1h;
 
   // Returns whether the given humongous object defined by the start region index
   // is reclaimable.
@@ -157,7 +171,7 @@ class G1FreeHumongousRegionClosure : public HeapRegionClosure {
   //
   // Other implementation considerations:
   // - never consider object arrays at this time because they would pose
-  // considerable effort for cleaning up the the remembered sets. This is
+  // considerable effort for cleaning up the remembered sets. This is
   // required because stale remembered sets might reference locations that
   // are currently allocated into.
   bool is_reclaimable(uint region_idx) const {
@@ -168,18 +182,16 @@ public:
   G1FreeHumongousRegionClosure() :
     _humongous_objects_reclaimed(0),
     _humongous_regions_reclaimed(0),
-    _freed_bytes(0) {
-  }
+    _freed_bytes(0),
+    _g1h(G1CollectedHeap::heap())
+  {}
 
-  virtual bool do_heap_region(HeapRegion* r) {
-    if (!r->is_starts_humongous()) {
+  bool do_heap_region_index(uint region_index) override {
+    if (!is_reclaimable(region_index)) {
       return false;
     }
 
-    uint region_idx = r->hrm_index();
-    if (!is_reclaimable(region_idx)) {
-      return false;
-    }
+    HeapRegion* r = _g1h->region_at(region_index);
 
     oop obj = cast_to_oop(r->bottom());
     guarantee(obj->is_typeArray(),
@@ -187,27 +199,25 @@ public:
               PTR_FORMAT " is not.", p2i(r->bottom()));
 
     log_debug(gc, humongous)("Reclaimed humongous region %u (object size " SIZE_FORMAT " @ " PTR_FORMAT ")",
-                             region_idx,
-                             (size_t)obj->size() * HeapWordSize,
+                             region_index,
+                             obj->size() * HeapWordSize,
                              p2i(r->bottom())
                             );
 
-    G1CollectedHeap* g1h = G1CollectedHeap::heap();
-    G1ConcurrentMark* const cm = g1h->concurrent_mark();
+    G1ConcurrentMark* const cm = _g1h->concurrent_mark();
     cm->humongous_object_eagerly_reclaimed(r);
-    assert(!cm->is_marked_in_prev_bitmap(obj) && !cm->is_marked_in_next_bitmap(obj),
-           "Eagerly reclaimed humongous region %u should not be marked at all but is in prev %s next %s",
-           region_idx,
-           BOOL_TO_STR(cm->is_marked_in_prev_bitmap(obj)),
-           BOOL_TO_STR(cm->is_marked_in_next_bitmap(obj)));
+    assert(!cm->is_marked_in_bitmap(obj),
+           "Eagerly reclaimed humongous region %u should not be marked at all but is in bitmap %s",
+           region_index,
+           BOOL_TO_STR(cm->is_marked_in_bitmap(obj)));
     _humongous_objects_reclaimed++;
     do {
-      HeapRegion* next = g1h->next_region_in_humongous(r);
+      HeapRegion* next = _g1h->next_region_in_humongous(r);
       _freed_bytes += r->used();
       r->set_containing_set(nullptr);
       _humongous_regions_reclaimed++;
-      g1h->free_humongous_region(r, nullptr);
-      g1h->hr_printer()->cleanup(r);
+      _g1h->free_humongous_region(r, nullptr);
+      _g1h->hr_printer()->cleanup(r);
       r = next;
     } while (r != nullptr);
 
@@ -227,72 +237,87 @@ public:
   }
 };
 
-void G1PostEvacuateCollectionSetCleanupTask2::ResetHotCardCacheTask::do_work(uint worker_id) {
-  G1CollectedHeap::heap()->reset_hot_card_cache();
-}
+class G1PostEvacuateCollectionSetCleanupTask2::ResetHotCardCacheTask : public G1AbstractSubTask {
+public:
+  ResetHotCardCacheTask() : G1AbstractSubTask(G1GCPhaseTimes::ResetHotCardCache) { }
 
-void G1PostEvacuateCollectionSetCleanupTask2::PurgeCodeRootsTask::do_work(uint worker_id) {
-  G1CollectedHeap::heap()->purge_code_root_memory();
-}
+  double worker_cost() const override { return 0.5; }
+  void do_work(uint worker_id) override { G1CollectedHeap::heap()->reset_hot_card_cache(); }
+};
+
+class G1PostEvacuateCollectionSetCleanupTask2::PurgeCodeRootsTask : public G1AbstractSubTask {
+public:
+  PurgeCodeRootsTask() : G1AbstractSubTask(G1GCPhaseTimes::PurgeCodeRoots) { }
+
+  double worker_cost() const override { return 1.0; }
+  void do_work(uint worker_id) override { G1CollectedHeap::heap()->purge_code_root_memory(); }
+};
 
 #if COMPILER2_OR_JVMCI
-void G1PostEvacuateCollectionSetCleanupTask2::UpdateDerivedPointersTask::do_work(uint worker_id) {
-  DerivedPointerTable::update_pointers();
-}
+class G1PostEvacuateCollectionSetCleanupTask2::UpdateDerivedPointersTask : public G1AbstractSubTask {
+public:
+  UpdateDerivedPointersTask() : G1AbstractSubTask(G1GCPhaseTimes::UpdateDerivedPointers) { }
+
+  double worker_cost() const override { return 1.0; }
+  void do_work(uint worker_id) override {   DerivedPointerTable::update_pointers(); }
+};
 #endif
 
-bool G1PostEvacuateCollectionSetCleanupTask2::EagerlyReclaimHumongousObjectsTask::should_execute() {
-  return G1CollectedHeap::heap()->should_do_eager_reclaim();
-}
+class G1PostEvacuateCollectionSetCleanupTask2::EagerlyReclaimHumongousObjectsTask : public G1AbstractSubTask {
+  uint _humongous_regions_reclaimed;
+  size_t _bytes_freed;
 
-G1PostEvacuateCollectionSetCleanupTask2::EagerlyReclaimHumongousObjectsTask::EagerlyReclaimHumongousObjectsTask() :
-  G1AbstractSubTask(G1GCPhaseTimes::EagerlyReclaimHumongousObjects),
-  _humongous_regions_reclaimed(0),
-  _bytes_freed(0) { }
+public:
+  EagerlyReclaimHumongousObjectsTask() :
+    G1AbstractSubTask(G1GCPhaseTimes::EagerlyReclaimHumongousObjects),
+    _humongous_regions_reclaimed(0),
+    _bytes_freed(0) { }
 
-void G1PostEvacuateCollectionSetCleanupTask2::EagerlyReclaimHumongousObjectsTask::do_work(uint worker_id) {
-  G1CollectedHeap* g1h = G1CollectedHeap::heap();
+  virtual ~EagerlyReclaimHumongousObjectsTask() {
+    G1CollectedHeap* g1h = G1CollectedHeap::heap();
 
-  G1FreeHumongousRegionClosure cl;
-  g1h->heap_region_iterate(&cl);
+    g1h->remove_from_old_gen_sets(0, 0, _humongous_regions_reclaimed);
+    g1h->decrement_summary_bytes(_bytes_freed);
+  }
 
-  record_work_item(worker_id, G1GCPhaseTimes::EagerlyReclaimNumTotal, g1h->num_humongous_objects());
-  record_work_item(worker_id, G1GCPhaseTimes::EagerlyReclaimNumCandidates, g1h->num_humongous_reclaim_candidates());
-  record_work_item(worker_id, G1GCPhaseTimes::EagerlyReclaimNumReclaimed, cl.humongous_objects_reclaimed());
+  double worker_cost() const override { return 1.0; }
+  void do_work(uint worker_id) override {
+    G1CollectedHeap* g1h = G1CollectedHeap::heap();
 
-  _humongous_regions_reclaimed = cl.humongous_regions_reclaimed();
-  _bytes_freed = cl.bytes_freed();
-}
+    G1FreeHumongousRegionClosure cl;
+    g1h->heap_region_iterate(&cl);
 
-G1PostEvacuateCollectionSetCleanupTask2::EagerlyReclaimHumongousObjectsTask::~EagerlyReclaimHumongousObjectsTask() {
-  G1CollectedHeap* g1h = G1CollectedHeap::heap();
+    record_work_item(worker_id, G1GCPhaseTimes::EagerlyReclaimNumTotal, g1h->num_humongous_objects());
+    record_work_item(worker_id, G1GCPhaseTimes::EagerlyReclaimNumCandidates, g1h->num_humongous_reclaim_candidates());
+    record_work_item(worker_id, G1GCPhaseTimes::EagerlyReclaimNumReclaimed, cl.humongous_objects_reclaimed());
 
-  g1h->remove_from_old_gen_sets(0, 0, _humongous_regions_reclaimed);
-  g1h->decrement_summary_bytes(_bytes_freed);
-}
+    _humongous_regions_reclaimed = cl.humongous_regions_reclaimed();
+    _bytes_freed = cl.bytes_freed();
+  }
+};
 
-bool G1PostEvacuateCollectionSetCleanupTask2::RestorePreservedMarksTask::should_execute() {
-  return G1CollectedHeap::heap()->evacuation_failed();
-}
+class G1PostEvacuateCollectionSetCleanupTask2::RestorePreservedMarksTask : public G1AbstractSubTask {
+  PreservedMarksSet* _preserved_marks;
+  WorkerTask* _task;
 
-G1PostEvacuateCollectionSetCleanupTask2::RestorePreservedMarksTask::RestorePreservedMarksTask(PreservedMarksSet* preserved_marks) :
-  G1AbstractSubTask(G1GCPhaseTimes::RestorePreservedMarks), _preserved_marks(preserved_marks), _task(preserved_marks->create_task()) { }
+public:
+  RestorePreservedMarksTask(PreservedMarksSet* preserved_marks) :
+    G1AbstractSubTask(G1GCPhaseTimes::RestorePreservedMarks),
+    _preserved_marks(preserved_marks),
+    _task(preserved_marks->create_task()) { }
 
-G1PostEvacuateCollectionSetCleanupTask2::RestorePreservedMarksTask::~RestorePreservedMarksTask() {
-  delete _task;
-}
+  virtual ~RestorePreservedMarksTask() {
+    delete _task;
+  }
 
-double G1PostEvacuateCollectionSetCleanupTask2::RestorePreservedMarksTask::worker_cost() const {
-  assert(should_execute(), "Should not call this if not executed");
-  return _preserved_marks->num();
-}
+  double worker_cost() const override {
+    return _preserved_marks->num();
+  }
 
-void G1PostEvacuateCollectionSetCleanupTask2::RestorePreservedMarksTask::do_work(uint worker_id) {
-  _task->work(worker_id);
-}
+  void do_work(uint worker_id) override { _task->work(worker_id); }
+};
 
 class RedirtyLoggedCardTableEntryClosure : public G1CardTableEntryClosure {
- private:
   size_t _num_dirtied;
   G1CollectedHeap* _g1h;
   G1CardTable* _g1_ct;
@@ -308,7 +333,7 @@ class RedirtyLoggedCardTableEntryClosure : public G1CardTableEntryClosure {
     return _g1h->is_in_cset(hr) && !_evac_failure_regions->contains(hr->hrm_index());
   }
 
- public:
+public:
   RedirtyLoggedCardTableEntryClosure(G1CollectedHeap* g1h, G1EvacFailureRegions* evac_failure_regions) :
     G1CardTableEntryClosure(),
     _num_dirtied(0),
@@ -329,43 +354,87 @@ class RedirtyLoggedCardTableEntryClosure : public G1CardTableEntryClosure {
   size_t num_dirtied()   const { return _num_dirtied; }
 };
 
-G1PostEvacuateCollectionSetCleanupTask2::
-      RedirtyLoggedCardsTask::RedirtyLoggedCardsTask(G1RedirtyCardsQueueSet* rdcqs,
-                                                     G1EvacFailureRegions* evac_failure_regions) :
-  G1AbstractSubTask(G1GCPhaseTimes::RedirtyCards),
-  _rdcqs(rdcqs),
-  _nodes(rdcqs->all_completed_buffers()),
-  _evac_failure_regions(evac_failure_regions) { }
+class G1PostEvacuateCollectionSetCleanupTask2::ClearRetainedRegionBitmaps : public G1AbstractSubTask {
+  G1EvacFailureRegions* _evac_failure_regions;
+  HeapRegionClaimer _claimer;
 
-G1PostEvacuateCollectionSetCleanupTask2::RedirtyLoggedCardsTask::~RedirtyLoggedCardsTask() {
-  G1DirtyCardQueueSet& dcq = G1BarrierSet::dirty_card_queue_set();
-  dcq.merge_bufferlists(_rdcqs);
-  _rdcqs->verify_empty();
-}
+  class ClearRetainedRegionBitmapsClosure : public HeapRegionClosure {
+  public:
 
-double G1PostEvacuateCollectionSetCleanupTask2::RedirtyLoggedCardsTask::worker_cost() const {
-  // Needs more investigation.
-  return G1CollectedHeap::heap()->workers()->active_workers();
-}
-
-void G1PostEvacuateCollectionSetCleanupTask2::RedirtyLoggedCardsTask::do_work(uint worker_id) {
-  RedirtyLoggedCardTableEntryClosure cl(G1CollectedHeap::heap(), _evac_failure_regions);
-  const size_t buffer_size = _rdcqs->buffer_size();
-  BufferNode* next = Atomic::load(&_nodes);
-  while (next != nullptr) {
-    BufferNode* node = next;
-    next = Atomic::cmpxchg(&_nodes, node, node->next());
-    if (next == node) {
-      cl.apply_to_buffer(node, buffer_size, worker_id);
-      next = node->next();
+    bool do_heap_region(HeapRegion* r) override {
+      assert(r->bottom() == r->top_at_mark_start(),
+             "TAMS should have been reset for region %u", r->hrm_index());
+      G1CollectedHeap::heap()->clear_bitmap_for_region(r);
+      return false;
     }
+  };
+
+public:
+
+  ClearRetainedRegionBitmaps(G1EvacFailureRegions* evac_failure_regions) :
+    G1AbstractSubTask(G1GCPhaseTimes::ClearRetainedRegionBitmaps),
+    _evac_failure_regions(evac_failure_regions),
+    _claimer(0) {
+    assert(!G1CollectedHeap::heap()->collector_state()->in_concurrent_start_gc(),
+           "Should not clear bitmaps of retained regions during concurrent start");
   }
-  record_work_item(worker_id, 0, cl.num_dirtied());
-}
+
+  void set_max_workers(uint max_workers) override {
+    _claimer.set_n_workers(max_workers);
+  }
+
+  double worker_cost() const override {
+    return _evac_failure_regions->num_regions_failed_evacuation();
+  }
+
+  void do_work(uint worker_id) override {
+    ClearRetainedRegionBitmapsClosure cl;
+    _evac_failure_regions->par_iterate(&cl, &_claimer, worker_id);
+  }
+};
+
+class G1PostEvacuateCollectionSetCleanupTask2::RedirtyLoggedCardsTask : public G1AbstractSubTask {
+  G1RedirtyCardsQueueSet* _rdcqs;
+  BufferNode* volatile _nodes;
+  G1EvacFailureRegions* _evac_failure_regions;
+
+public:
+  RedirtyLoggedCardsTask(G1RedirtyCardsQueueSet* rdcqs, G1EvacFailureRegions* evac_failure_regions) :
+    G1AbstractSubTask(G1GCPhaseTimes::RedirtyCards),
+    _rdcqs(rdcqs),
+    _nodes(rdcqs->all_completed_buffers()),
+    _evac_failure_regions(evac_failure_regions) { }
+
+  virtual ~RedirtyLoggedCardsTask() {
+    G1DirtyCardQueueSet& dcq = G1BarrierSet::dirty_card_queue_set();
+    dcq.merge_bufferlists(_rdcqs);
+    _rdcqs->verify_empty();
+  }
+
+  double worker_cost() const override {
+    // Needs more investigation.
+    return G1CollectedHeap::heap()->workers()->active_workers();
+  }
+
+  void do_work(uint worker_id) override {
+    RedirtyLoggedCardTableEntryClosure cl(G1CollectedHeap::heap(), _evac_failure_regions);
+    const size_t buffer_size = _rdcqs->buffer_size();
+    BufferNode* next = Atomic::load(&_nodes);
+    while (next != nullptr) {
+      BufferNode* node = next;
+      next = Atomic::cmpxchg(&_nodes, node, node->next());
+      if (next == node) {
+        cl.apply_to_buffer(node, buffer_size, worker_id);
+        next = node->next();
+      }
+    }
+    record_work_item(worker_id, 0, cl.num_dirtied());
+  }
+};
 
 // Helper class to keep statistics for the collection set freeing
 class FreeCSetStats {
-  size_t _before_used_bytes;   // Usage in regions successfully evacutate
+  size_t _before_used_bytes;   // Usage in regions successfully evacuate
   size_t _after_used_bytes;    // Usage in regions failing evacuation
   size_t _bytes_allocated_in_old_since_last_gc; // Size of young regions turned into old
   size_t _failure_used_words;  // Live size in failed regions
@@ -408,7 +477,7 @@ public:
   }
 
   void account_failed_region(HeapRegion* r) {
-    size_t used_words = r->marked_bytes() / HeapWordSize;
+    size_t used_words = r->live_bytes() / HeapWordSize;
     _failure_used_words += used_words;
     _failure_waste_words += HeapRegion::GrainWords - used_words;
     _after_used_bytes += r->used();
@@ -499,6 +568,15 @@ class FreeCSetClosure : public HeapRegionClosure {
     // gen statistics, but we need to update old gen statistics.
     stats()->account_failed_region(r);
 
+    G1GCPhaseTimes* p = _g1h->phase_times();
+    assert(!r->is_pinned(), "Unexpected pinned region at index %u", r->hrm_index());
+    assert(r->in_collection_set(), "bad CS");
+
+    p->record_or_add_thread_work_item(G1GCPhaseTimes::RestoreRetainedRegions,
+                                      _worker_id,
+                                      1,
+                                      G1GCPhaseTimes::RestoreRetainedRegionsNum);
+
     // Update the region state due to the failed evacuation.
     r->handle_evacuation_failure();
 
@@ -562,23 +640,32 @@ public:
   }
 };
 
-FreeCSetStats* G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask::worker_stats(uint worker) {
-  return &_worker_stats[worker];
-}
+class G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask : public G1AbstractSubTask {
+  G1CollectedHeap*  _g1h;
+  G1EvacInfo* _evacuation_info;
+  FreeCSetStats*    _worker_stats;
+  HeapRegionClaimer _claimer;
+  const size_t*     _surviving_young_words;
+  uint              _active_workers;
+  G1EvacFailureRegions* _evac_failure_regions;
 
-void G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask::report_statistics() {
-  // Merge the accounting
-  FreeCSetStats total_stats;
-  for (uint worker = 0; worker < _active_workers; worker++) {
-    total_stats.merge_stats(worker_stats(worker));
+  FreeCSetStats* worker_stats(uint worker) {
+    return &_worker_stats[worker];
   }
-  total_stats.report(_g1h, _evacuation_info);
-}
 
-G1PostEvacuateCollectionSetCleanupTask2::
-      FreeCollectionSetTask::FreeCollectionSetTask(G1EvacInfo* evacuation_info,
-                                                   const size_t* surviving_young_words,
-                                                   G1EvacFailureRegions* evac_failure_regions) :
+  void report_statistics() {
+    // Merge the accounting
+    FreeCSetStats total_stats;
+    for (uint worker = 0; worker < _active_workers; worker++) {
+      total_stats.merge_stats(worker_stats(worker));
+    }
+    total_stats.report(_g1h, _evacuation_info);
+  }
+
+public:
+  FreeCollectionSetTask(G1EvacInfo* evacuation_info,
+                        const size_t* surviving_young_words,
+                        G1EvacFailureRegions* evac_failure_regions) :
     G1AbstractSubTask(G1GCPhaseTimes::FreeCollectionSet),
     _g1h(G1CollectedHeap::heap()),
     _evacuation_info(evacuation_info),
@@ -587,59 +674,63 @@ G1PostEvacuateCollectionSetCleanupTask2::
     _surviving_young_words(surviving_young_words),
     _active_workers(0),
     _evac_failure_regions(evac_failure_regions) {
-  _g1h->clear_eden();
-}
 
-G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask::~FreeCollectionSetTask() {
-  Ticks serial_time = Ticks::now();
-  report_statistics();
-  for (uint worker = 0; worker < _active_workers; worker++) {
-    _worker_stats[worker].~FreeCSetStats();
+    _g1h->clear_eden();
   }
-  FREE_C_HEAP_ARRAY(FreeCSetStats, _worker_stats);
-  _g1h->phase_times()->record_serial_free_cset_time_ms((Ticks::now() - serial_time).seconds() * 1000.0);
-  _g1h->clear_collection_set();
-}
 
-double G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask::worker_cost() const {
-  return G1CollectedHeap::heap()->collection_set()->region_length();
-}
-
-void G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask::set_max_workers(uint max_workers) {
-  _active_workers = max_workers;
-  _worker_stats = NEW_C_HEAP_ARRAY(FreeCSetStats, max_workers, mtGC);
-  for (uint worker = 0; worker < _active_workers; worker++) {
-    ::new (&_worker_stats[worker]) FreeCSetStats();
+  virtual ~FreeCollectionSetTask() {
+    Ticks serial_time = Ticks::now();
+    report_statistics();
+    for (uint worker = 0; worker < _active_workers; worker++) {
+      _worker_stats[worker].~FreeCSetStats();
+    }
+    FREE_C_HEAP_ARRAY(FreeCSetStats, _worker_stats);
+    _g1h->phase_times()->record_serial_free_cset_time_ms((Ticks::now() - serial_time).seconds() * 1000.0);
+    _g1h->clear_collection_set();
   }
-  _claimer.set_n_workers(_active_workers);
-}
 
-void G1PostEvacuateCollectionSetCleanupTask2::FreeCollectionSetTask::do_work(uint worker_id) {
-  FreeCSetClosure cl(_surviving_young_words, worker_id, worker_stats(worker_id), _evac_failure_regions);
-  _g1h->collection_set_par_iterate_all(&cl, &_claimer, worker_id);
-  // Report per-region type timings.
-  cl.report_timing();
-}
+  double worker_cost() const override { return G1CollectedHeap::heap()->collection_set()->region_length(); }
+
+  void set_max_workers(uint max_workers) override {
+    _active_workers = max_workers;
+    _worker_stats = NEW_C_HEAP_ARRAY(FreeCSetStats, max_workers, mtGC);
+    for (uint worker = 0; worker < _active_workers; worker++) {
+      ::new (&_worker_stats[worker]) FreeCSetStats();
+    }
+    _claimer.set_n_workers(_active_workers);
+  }
+
+  void do_work(uint worker_id) override {
+    FreeCSetClosure cl(_surviving_young_words, worker_id, worker_stats(worker_id), _evac_failure_regions);
+    _g1h->collection_set_par_iterate_all(&cl, &_claimer, worker_id);
+    // Report per-region type timings.
+    cl.report_timing();
+  }
+};
 
 G1PostEvacuateCollectionSetCleanupTask2::G1PostEvacuateCollectionSetCleanupTask2(G1ParScanThreadStateSet* per_thread_states,
                                                                                  G1EvacInfo* evacuation_info,
                                                                                  G1EvacFailureRegions* evac_failure_regions) :
-  G1BatchedGangTask("Post Evacuate Cleanup 2", G1CollectedHeap::heap()->phase_times())
+  G1BatchedTask("Post Evacuate Cleanup 2", G1CollectedHeap::heap()->phase_times())
 {
   add_serial_task(new ResetHotCardCacheTask());
   add_serial_task(new PurgeCodeRootsTask());
 #if COMPILER2_OR_JVMCI
   add_serial_task(new UpdateDerivedPointersTask());
 #endif
-  if (EagerlyReclaimHumongousObjectsTask::should_execute()) {
+  if (G1CollectedHeap::heap()->has_humongous_reclaim_candidates()) {
     add_serial_task(new EagerlyReclaimHumongousObjectsTask());
   }
 
-  if (RestorePreservedMarksTask::should_execute()) {
+  if (evac_failure_regions->evacuation_failed()) {
     add_parallel_task(new RestorePreservedMarksTask(per_thread_states->preserved_marks_set()));
+    // Keep marks on bitmaps in retained regions during concurrent start - they will all be old.
+    if (!G1CollectedHeap::heap()->collector_state()->in_concurrent_start_gc()) {
+      add_parallel_task(new ClearRetainedRegionBitmaps(evac_failure_regions));
+    }
   }
   add_parallel_task(new RedirtyLoggedCardsTask(per_thread_states->rdcqs(), evac_failure_regions));
   add_parallel_task(new FreeCollectionSetTask(evacuation_info,
-                                                   per_thread_states->surviving_young_words(),
-                                                   evac_failure_regions));
+                                              per_thread_states->surviving_young_words(),
+                                              evac_failure_regions));
 }
