@@ -27,6 +27,68 @@
 
 #include "runtime/globals_shared.hpp"
 
+// Enable evacuation failure injector by default in non-product builds.
+
+#ifdef EVAC_FAILURE_INJECTOR
+#error "EVAC_FAILURE_INJECTOR already defined"
+#endif
+#ifndef PRODUCT
+#define EVAC_FAILURE_INJECTOR 1
+#else
+#define EVAC_FAILURE_INJECTOR 0
+#endif
+
+#if EVAC_FAILURE_INJECTOR
+#define GC_G1_EVACUATION_FAILURE_FLAGS(develop,                             \
+                                       develop_pd,                          \
+                                       product,                             \
+                                       product_pd,                          \
+                                       notproduct,                          \
+                                       range,                               \
+                                       constraint)                          \
+                                                                            \
+  product(bool, G1EvacuationFailureALot, false,                             \
+          "Force use of evacuation failure handling during certain "        \
+          "evacuation pauses")                                              \
+                                                                            \
+  product(uintx, G1EvacuationFailureALotCount, 1000,                        \
+          "Number of successful evacuations between evacuation failures "   \
+          "occurring at object copying per thread")                         \
+                                                                            \
+  product(uintx, G1EvacuationFailureALotInterval, 5,                        \
+          "Total collections between forced triggering of evacuation "      \
+          "failures")                                                       \
+                                                                            \
+  product(bool, G1EvacuationFailureALotDuringConcMark, true,                \
+          "Force use of evacuation failure handling during evacuation "     \
+          "pauses when marking is in progress")                             \
+                                                                            \
+  product(bool, G1EvacuationFailureALotDuringConcurrentStart, true,         \
+          "Force use of evacuation failure handling during concurrent "     \
+          "start evacuation pauses")                                        \
+                                                                            \
+  product(bool, G1EvacuationFailureALotDuringYoungGC, true,                 \
+          "Force use of evacuation failure handling during young "          \
+          "evacuation pauses")                                              \
+                                                                            \
+  product(bool, G1EvacuationFailureALotDuringMixedGC, true,                 \
+          "Force use of evacuation failure handling during mixed "          \
+          "evacuation pauses")                                              \
+                                                                            \
+  product(uint, G1EvacuationFailureALotCSetPercent, 100,                    \
+          "The percentage of regions in the collection set starting "       \
+          "from the beginning where the forced evacuation failure "         \
+          "injection will be applied.")                                     \
+          range(1, 100)
+#else
+#define GC_G1_EVACUATION_FAILURE_FLAGS(develop,                             \
+                                       develop_pd,                          \
+                                       product,                             \
+                                       product_pd,                          \
+                                       notproduct,                          \
+                                       range,                               \
+                                       constraint)
+#endif
 //
 // Defines all globals flags used by the garbage-first compiler.
 //
@@ -154,7 +216,7 @@
           "The threshold that defines (>=) a hot card.")                    \
           range(0, max_jubyte)                                              \
                                                                             \
-  develop(uint, G1RemSetArrayOfCardsEntriesBase, 4,                         \
+  develop(uint, G1RemSetArrayOfCardsEntriesBase, 8,                         \
           "Maximum number of entries per region in the Array of Cards "     \
           "card set container per MB of a heap region.")                    \
           range(1, 65536)                                                   \
@@ -201,7 +263,7 @@
                                                                             \
   product(size_t, G1HeapRegionSize, 0,                                      \
           "Size of the G1 regions.")                                        \
-          range(0, 32*M)                                                    \
+          range(0, NOT_LP64(32*M) LP64_ONLY(512*M))                         \
           constraint(G1HeapRegionSizeConstraintFunc,AfterMemoryInit)        \
                                                                             \
   product(uint, G1ConcRefinementThreads, 0,                                 \
@@ -248,13 +310,6 @@
           "The target number of mixed GCs after a marking cycle.")          \
           range(0, max_uintx)                                               \
                                                                             \
-  product(bool, G1EagerReclaimHumongousObjects, true, EXPERIMENTAL,         \
-          "Try to reclaim dead large objects at every young GC.")           \
-                                                                            \
-  product(bool, G1EagerReclaimHumongousObjectsWithStaleRefs, true, EXPERIMENTAL, \
-          "Try to reclaim dead large objects that have a few stale "        \
-          "references at every young GC.")                                  \
-                                                                            \
   product(uint, G1EagerReclaimRemSetThreshold, 0, EXPERIMENTAL,             \
           "Maximum number of remembered set entries a humongous region "    \
           "otherwise eligible for eager reclaim may have to be a candidate "\
@@ -269,34 +324,6 @@
           "as a percentage of the heap size.")                              \
           range(0, 100)                                                     \
                                                                             \
-  notproduct(bool, G1EvacuationFailureALot, false,                          \
-          "Force use of evacuation failure handling during certain "        \
-          "evacuation pauses")                                              \
-                                                                            \
-  develop(uintx, G1EvacuationFailureALotCount, 1000,                        \
-          "Number of successful evacuations between evacuation failures "   \
-          "occurring at object copying")                                    \
-                                                                            \
-  develop(uintx, G1EvacuationFailureALotInterval, 5,                        \
-          "Total collections between forced triggering of evacuation "      \
-          "failures")                                                       \
-                                                                            \
-  develop(bool, G1EvacuationFailureALotDuringConcMark, true,                \
-          "Force use of evacuation failure handling during evacuation "     \
-          "pauses when marking is in progress")                             \
-                                                                            \
-  develop(bool, G1EvacuationFailureALotDuringConcurrentStart, true,         \
-          "Force use of evacuation failure handling during concurrent "     \
-          "start evacuation pauses")                                        \
-                                                                            \
-  develop(bool, G1EvacuationFailureALotDuringYoungGC, true,                 \
-          "Force use of evacuation failure handling during young "          \
-          "evacuation pauses")                                              \
-                                                                            \
-  develop(bool, G1EvacuationFailureALotDuringMixedGC, true,                 \
-          "Force use of evacuation failure handling during mixed "          \
-          "evacuation pauses")                                              \
-                                                                            \
   product(bool, G1VerifyRSetsDuringFullGC, false, DIAGNOSTIC,               \
           "If true, perform verification of each heap region's "            \
           "remembered set when verifying the heap during a full GC.")       \
@@ -340,10 +367,23 @@
           "percentage of the currently used memory.")                       \
           range(0.0, 1.0)                                                   \
                                                                             \
-  product(bool, G1UsePreventiveGC, true, DIAGNOSTIC,                        \
+  product(bool, G1UsePreventiveGC, false, DIAGNOSTIC,                       \
           "Allows collections to be triggered proactively based on the      \
            number of free regions and the expected survival rates in each   \
-           section of the heap.")
+           section of the heap.")                                           \
+                                                                            \
+  product(uint, G1RestoreRetainedRegionChunksPerWorker, 16, DIAGNOSTIC,     \
+          "The number of chunks assigned per worker thread for "            \
+          "retained region restore purposes.")                              \
+          range(1, 256)                                                     \
+                                                                            \
+  GC_G1_EVACUATION_FAILURE_FLAGS(develop,                                   \
+                    develop_pd,                                             \
+                    product,                                                \
+                    product_pd,                                             \
+                    notproduct,                                             \
+                    range,                                                  \
+                    constraint)
 
 // end of GC_G1_FLAGS
 
