@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -103,12 +103,11 @@ void HeapRegion::setup_heap_region_size(size_t max_heap_size) {
 void HeapRegion::handle_evacuation_failure() {
   uninstall_surv_rate_group();
   clear_young_index_in_cset();
-  set_old();
-  _next_marked_bytes = 0;
-}
+  clear_index_in_opt_cset();
+  move_to_old();
 
-void HeapRegion::process_and_drop_evac_failure_objs(ObjectClosure* closure) {
-  _evac_failure_objs.process_and_drop(closure);
+  _rem_set->clean_code_roots(this);
+  _rem_set->clear_locked(true /* only_cardset */);
 }
 
 void HeapRegion::unlink_from_list() {
@@ -129,8 +128,6 @@ void HeapRegion::hr_clear(bool clear_space) {
 
   rem_set()->clear_locked();
 
-  zero_marked_bytes();
-
   init_top_at_mark_start();
   if (clear_space) clear(SpaceDecorator::Mangle);
 
@@ -230,7 +227,6 @@ HeapRegion::HeapRegion(uint hrm_index,
   _bottom(mr.start()),
   _end(mr.end()),
   _top(NULL),
-  _compaction_top(NULL),
   _bot_part(bot, this),
   _pre_dummy_top(NULL),
   _rem_set(NULL),
@@ -242,12 +238,12 @@ HeapRegion::HeapRegion(uint hrm_index,
 #ifdef ASSERT
   _containing_set(NULL),
 #endif
-  _prev_top_at_mark_start(NULL), _next_top_at_mark_start(NULL),
-  _prev_marked_bytes(0), _next_marked_bytes(0),
+  _top_at_mark_start(NULL),
+  _parsable_bottom(NULL),
+  _garbage_bytes(0),
   _young_index_in_cset(-1),
   _surv_rate_group(NULL), _age_index(G1SurvRateGroup::InvalidAgeIndex), _gc_efficiency(-1.0),
-  _node_index(G1NUMA::UnknownNodeIndex),
-  _evac_failure_objs(hrm_index, _bottom)
+  _node_index(G1NUMA::UnknownNodeIndex)
 {
   assert(Universe::on_page_boundary(mr.start()) && Universe::on_page_boundary(mr.end()),
          "invalid space boundaries");
@@ -264,8 +260,6 @@ void HeapRegion::initialize(bool clear_space, bool mangle_space) {
   }
 
   set_top(bottom());
-  set_compaction_top(bottom());
-  reset_bot();
 
   hr_clear(false /*clear_space*/);
 }
@@ -278,59 +272,52 @@ void HeapRegion::report_region_type_change(G1HeapRegionTraceType::Type to) {
                                             used());
 }
 
-void HeapRegion::note_self_forwarding_removal_start(bool during_concurrent_start,
-                                                    bool during_conc_mark) {
-  // We always recreate the prev marking info and we'll explicitly
-  // mark all objects we find to be self-forwarded on the prev
-  // bitmap. So all objects need to be below PTAMS.
-  _prev_marked_bytes = 0;
+void HeapRegion::note_evacuation_failure(bool during_concurrent_start) {
+  // PB must be bottom - we only evacuate old gen regions after scrubbing, and
+  // young gen regions never have their PB set to anything other than bottom.
+  assert(parsable_bottom_acquire() == bottom(), "must be");
+
+  _garbage_bytes = 0;
 
   if (during_concurrent_start) {
-    // During concurrent start, we'll also explicitly mark all objects
-    // we find to be self-forwarded on the next bitmap. So all
-    // objects need to be below NTAMS.
-    _next_top_at_mark_start = top();
-    _next_marked_bytes = 0;
-  } else if (during_conc_mark) {
-    // During concurrent mark, all objects in the CSet (including
-    // the ones we find to be self-forwarded) are implicitly live.
-    // So all objects need to be above NTAMS.
-    _next_top_at_mark_start = bottom();
-    _next_marked_bytes = 0;
+    // Self-forwarding marks all objects. Adjust TAMS so that these marks are
+    // below it.
+    set_top_at_mark_start(top());
+  } else {
+    // Outside of the mixed phase all regions that had an evacuation failure must
+    // be young regions, and their TAMS is always bottom. Similarly, before the
+    // start of the mixed phase, we scrubbed and reset TAMS to bottom.
+    assert(top_at_mark_start() == bottom(), "must be");
   }
 }
 
-void HeapRegion::note_self_forwarding_removal_end(size_t marked_bytes) {
-  assert(marked_bytes <= used(),
-         "marked: " SIZE_FORMAT " used: " SIZE_FORMAT, marked_bytes, used());
-  _prev_top_at_mark_start = top();
-  _prev_marked_bytes = marked_bytes;
+void HeapRegion::note_self_forward_chunk_done(size_t garbage_bytes) {
+  Atomic::add(&_garbage_bytes, garbage_bytes, memory_order_relaxed);
 }
 
 // Code roots support
-
-void HeapRegion::add_strong_code_root(nmethod* nm) {
+void HeapRegion::add_code_root(nmethod* nm) {
   HeapRegionRemSet* hrrs = rem_set();
-  hrrs->add_strong_code_root(nm);
+  hrrs->add_code_root(nm);
 }
 
-void HeapRegion::add_strong_code_root_locked(nmethod* nm) {
+void HeapRegion::add_code_root_locked(nmethod* nm) {
   assert_locked_or_safepoint(CodeCache_lock);
   HeapRegionRemSet* hrrs = rem_set();
-  hrrs->add_strong_code_root_locked(nm);
+  hrrs->add_code_root_locked(nm);
 }
 
-void HeapRegion::remove_strong_code_root(nmethod* nm) {
+void HeapRegion::remove_code_root(nmethod* nm) {
   HeapRegionRemSet* hrrs = rem_set();
-  hrrs->remove_strong_code_root(nm);
+  hrrs->remove_code_root(nm);
 }
 
-void HeapRegion::strong_code_roots_do(CodeBlobClosure* blk) const {
+void HeapRegion::code_roots_do(CodeBlobClosure* blk) const {
   HeapRegionRemSet* hrrs = rem_set();
-  hrrs->strong_code_roots_do(blk);
+  hrrs->code_roots_do(blk);
 }
 
-class VerifyStrongCodeRootOopClosure: public OopClosure {
+class VerifyCodeRootOopClosure: public OopClosure {
   const HeapRegion* _hr;
   bool _failures;
   bool _has_oops_in_region;
@@ -358,7 +345,7 @@ class VerifyStrongCodeRootOopClosure: public OopClosure {
   }
 
 public:
-  VerifyStrongCodeRootOopClosure(const HeapRegion* hr):
+  VerifyCodeRootOopClosure(const HeapRegion* hr):
     _hr(hr), _failures(false), _has_oops_in_region(false) {}
 
   void do_oop(narrowOop* p) { do_oop_work(p); }
@@ -368,33 +355,27 @@ public:
   bool has_oops_in_region() { return _has_oops_in_region; }
 };
 
-class VerifyStrongCodeRootCodeBlobClosure: public CodeBlobClosure {
+class VerifyCodeRootCodeBlobClosure: public CodeBlobClosure {
   const HeapRegion* _hr;
   bool _failures;
 public:
-  VerifyStrongCodeRootCodeBlobClosure(const HeapRegion* hr) :
+  VerifyCodeRootCodeBlobClosure(const HeapRegion* hr) :
     _hr(hr), _failures(false) {}
 
   void do_code_blob(CodeBlob* cb) {
     nmethod* nm = (cb == NULL) ? NULL : cb->as_compiled_method()->as_nmethod_or_null();
     if (nm != NULL) {
       // Verify that the nemthod is live
-      if (!nm->is_alive()) {
-        log_error(gc, verify)("region [" PTR_FORMAT "," PTR_FORMAT "] has dead nmethod " PTR_FORMAT " in its strong code roots",
+      VerifyCodeRootOopClosure oop_cl(_hr);
+      nm->oops_do(&oop_cl);
+      if (!oop_cl.has_oops_in_region()) {
+        log_error(gc, verify)("region [" PTR_FORMAT "," PTR_FORMAT "] has nmethod " PTR_FORMAT " in its code roots with no pointers into region",
+                              p2i(_hr->bottom()), p2i(_hr->end()), p2i(nm));
+        _failures = true;
+      } else if (oop_cl.failures()) {
+        log_error(gc, verify)("region [" PTR_FORMAT "," PTR_FORMAT "] has other failures for nmethod " PTR_FORMAT,
                               p2i(_hr->bottom()), p2i(_hr->end()), p2i(nm));
         _failures = true;
-      } else {
-        VerifyStrongCodeRootOopClosure oop_cl(_hr);
-        nm->oops_do(&oop_cl);
-        if (!oop_cl.has_oops_in_region()) {
-          log_error(gc, verify)("region [" PTR_FORMAT "," PTR_FORMAT "] has nmethod " PTR_FORMAT " in its strong code roots with no pointers into region",
-                                p2i(_hr->bottom()), p2i(_hr->end()), p2i(nm));
-          _failures = true;
-        } else if (oop_cl.failures()) {
-          log_error(gc, verify)("region [" PTR_FORMAT "," PTR_FORMAT "] has other failures for nmethod " PTR_FORMAT,
-                                p2i(_hr->bottom()), p2i(_hr->end()), p2i(nm));
-          _failures = true;
-        }
       }
     }
   }
@@ -402,47 +383,47 @@ public:
   bool failures()       { return _failures; }
 };
 
-void HeapRegion::verify_strong_code_roots(VerifyOption vo, bool* failures) const {
+void HeapRegion::verify_code_roots(VerifyOption vo, bool* failures) const {
   if (!G1VerifyHeapRegionCodeRoots) {
     // We're not verifying code roots.
     return;
   }
-  if (vo == VerifyOption_G1UseFullMarking) {
+  if (vo == VerifyOption::G1UseFullMarking) {
     // Marking verification during a full GC is performed after class
-    // unloading, code cache unloading, etc so the strong code roots
+    // unloading, code cache unloading, etc so the code roots
     // attached to each heap region are in an inconsistent state. They won't
-    // be consistent until the strong code roots are rebuilt after the
-    // actual GC. Skip verifying the strong code roots in this particular
+    // be consistent until the code roots are rebuilt after the
+    // actual GC. Skip verifying the code roots in this particular
     // time.
     assert(VerifyDuringGC, "only way to get here");
     return;
   }
 
   HeapRegionRemSet* hrrs = rem_set();
-  size_t strong_code_roots_length = hrrs->strong_code_roots_list_length();
+  size_t code_roots_length = hrrs->code_roots_list_length();
 
   // if this region is empty then there should be no entries
-  // on its strong code root list
+  // on its code root list
   if (is_empty()) {
-    if (strong_code_roots_length > 0) {
+    if (code_roots_length > 0) {
       log_error(gc, verify)("region " HR_FORMAT " is empty but has " SIZE_FORMAT " code root entries",
-                            HR_FORMAT_PARAMS(this), strong_code_roots_length);
+                            HR_FORMAT_PARAMS(this), code_roots_length);
       *failures = true;
     }
     return;
   }
 
   if (is_continues_humongous()) {
-    if (strong_code_roots_length > 0) {
+    if (code_roots_length > 0) {
       log_error(gc, verify)("region " HR_FORMAT " is a continuation of a humongous region but has " SIZE_FORMAT " code root entries",
-                            HR_FORMAT_PARAMS(this), strong_code_roots_length);
+                            HR_FORMAT_PARAMS(this), code_roots_length);
       *failures = true;
     }
     return;
   }
 
-  VerifyStrongCodeRootCodeBlobClosure cb_cl(this);
-  strong_code_roots_do(&cb_cl);
+  VerifyCodeRootCodeBlobClosure cb_cl(this);
+  code_roots_do(&cb_cl);
 
   if (cb_cl.failures()) {
     *failures = true;
@@ -462,8 +443,8 @@ void HeapRegion::print_on(outputStream* st) const {
   } else {
     st->print("|  ");
   }
-  st->print("|TAMS " PTR_FORMAT ", " PTR_FORMAT "| %s ",
-               p2i(prev_top_at_mark_start()), p2i(next_top_at_mark_start()), rem_set()->get_state_str());
+  st->print("|TAMS " PTR_FORMAT "| PB " PTR_FORMAT "| %s ",
+            p2i(top_at_mark_start()), p2i(parsable_bottom_acquire()), rem_set()->get_state_str());
   if (UseNUMA) {
     G1NUMA* numa = G1NUMA::numa();
     if (node_index() < numa->num_active_nodes()) {
@@ -483,10 +464,9 @@ protected:
   bool _failures;
   int _n_failures;
   VerifyOption _vo;
+
 public:
-  // _vo == UsePrevMarking -> use "prev" marking information,
-  // _vo == UseNextMarking -> use "next" marking information,
-  // _vo == UseFullMarking -> use "next" marking bitmap but no TAMS.
+
   G1VerificationClosure(G1CollectedHeap* g1h, VerifyOption vo) :
     _g1h(g1h), _ct(g1h->card_table()),
     _containing_obj(NULL), _failures(false), _n_failures(0), _vo(vo) {
@@ -531,16 +511,16 @@ public:
     if (!CompressedOops::is_null(heap_oop)) {
       oop obj = CompressedOops::decode_not_null(heap_oop);
       bool failed = false;
-      if (!_g1h->is_in(obj) || _g1h->is_obj_dead_cond(obj, _vo)) {
-        MutexLocker x(ParGCRareEvent_lock,
-          Mutex::_no_safepoint_check_flag);
+      bool is_in_heap = _g1h->is_in(obj);
+      if (!is_in_heap || _g1h->is_obj_dead_cond(obj, _vo)) {
+        MutexLocker x(ParGCRareEvent_lock, Mutex::_no_safepoint_check_flag);
 
         if (!_failures) {
           log.error("----------");
         }
         ResourceMark rm;
-        if (!_g1h->is_in(obj)) {
-          HeapRegion* from = _g1h->heap_region_containing((HeapWord*)p);
+        if (!is_in_heap) {
+          HeapRegion* from = _g1h->heap_region_containing(p);
           log.error("Field " PTR_FORMAT " of live obj " PTR_FORMAT " in region " HR_FORMAT,
                     p2i(p), p2i(_containing_obj), HR_FORMAT_PARAMS(from));
           LogStream ls(log.error());
@@ -549,7 +529,7 @@ public:
           log.error("points to obj " PTR_FORMAT " in region " HR_FORMAT " remset %s",
                     p2i(obj), HR_FORMAT_PARAMS(to), to->rem_set()->get_state_str());
         } else {
-          HeapRegion* from = _g1h->heap_region_containing((HeapWord*)p);
+          HeapRegion* from = _g1h->heap_region_containing(p);
           HeapRegion* to = _g1h->heap_region_containing(obj);
           log.error("Field " PTR_FORMAT " of live obj " PTR_FORMAT " in region " HR_FORMAT,
                     p2i(p), p2i(_containing_obj), HR_FORMAT_PARAMS(from));
@@ -588,7 +568,7 @@ public:
     Log(gc, verify) log;
     if (!CompressedOops::is_null(heap_oop)) {
       oop obj = CompressedOops::decode_not_null(heap_oop);
-      HeapRegion* from = _g1h->heap_region_containing((HeapWord*)p);
+      HeapRegion* from = _g1h->heap_region_containing(p);
       HeapRegion* to = _g1h->heap_region_containing(obj);
       if (from != NULL && to != NULL &&
         from != to &&
@@ -604,8 +584,7 @@ public:
                 cv_field == dirty :
                 cv_obj == dirty || cv_field == dirty));
         if (is_bad) {
-          MutexLocker x(ParGCRareEvent_lock,
-            Mutex::_no_safepoint_check_flag);
+          MutexLocker x(ParGCRareEvent_lock, Mutex::_no_safepoint_check_flag);
 
           if (!_failures) {
             log.error("----------");
@@ -655,6 +634,8 @@ void HeapRegion::verify(VerifyOption vo,
   VerifyLiveClosure vl_cl(g1h, vo);
   VerifyRemSetClosure vr_cl(g1h, vo);
   bool is_region_humongous = is_humongous();
+  // We cast p to an oop, so region-bottom must be an obj-start.
+  assert(!is_region_humongous || is_starts_humongous(), "invariant");
   size_t object_num = 0;
   while (p < top()) {
     oop obj = cast_to_oop(p);
@@ -733,12 +714,7 @@ void HeapRegion::verify(VerifyOption vo,
     return;
   }
 
-  verify_strong_code_roots(vo, failures);
-}
-
-void HeapRegion::verify() const {
-  bool dummy = false;
-  verify(VerifyOption_G1UsePrevMarking, /* failures */ &dummy);
+  verify_code_roots(vo, failures);
 }
 
 void HeapRegion::verify_rem_set(VerifyOption vo, bool* failures) const {
@@ -777,18 +753,16 @@ void HeapRegion::verify_rem_set(VerifyOption vo, bool* failures) const {
 
 void HeapRegion::verify_rem_set() const {
   bool failures = false;
-  verify_rem_set(VerifyOption_G1UsePrevMarking, &failures);
+  verify_rem_set(VerifyOption::G1UseConcMarking, &failures);
   guarantee(!failures, "HeapRegion RemSet verification failed");
 }
 
 void HeapRegion::clear(bool mangle_space) {
   set_top(bottom());
-  set_compaction_top(bottom());
 
   if (ZapUnusedHeapArea && mangle_space) {
     mangle_unused_area();
   }
-  reset_bot();
 }
 
 #ifndef PRODUCT
@@ -797,18 +771,14 @@ void HeapRegion::mangle_unused_area() {
 }
 #endif
 
-void HeapRegion::initialize_bot_threshold() {
-  _bot_part.initialize_threshold();
-}
-
-void HeapRegion::alloc_block_in_bot(HeapWord* start, HeapWord* end) {
-  _bot_part.alloc_block(start, end);
+void HeapRegion::update_bot_for_block(HeapWord* start, HeapWord* end) {
+  _bot_part.update_for_block(start, end);
 }
 
 void HeapRegion::object_iterate(ObjectClosure* blk) {
   HeapWord* p = bottom();
   while (p < top()) {
-    if (block_is_obj(p)) {
+    if (block_is_obj(p, parsable_bottom())) {
       blk->do_object(cast_to_oop(p));
     }
     p += block_size(p);
@@ -818,8 +788,26 @@ void HeapRegion::object_iterate(ObjectClosure* blk) {
 void HeapRegion::fill_with_dummy_object(HeapWord* address, size_t word_size, bool zap) {
   // Keep the BOT in sync for old generation regions.
   if (is_old()) {
-    update_bot_at(address, word_size);
+    update_bot_for_obj(address, word_size);
   }
   // Fill in the object.
   CollectedHeap::fill_with_object(address, word_size, zap);
 }
+
+void HeapRegion::fill_range_with_dead_objects(HeapWord* start, HeapWord* end) {
+  size_t range_size = pointer_delta(end, start);
+
+  // Fill the dead range with objects. G1 might need to create two objects if
+  // the range is larger than half a region, which is the max_fill_size().
+  CollectedHeap::fill_with_objects(start, range_size);
+  HeapWord* current = start;
+  do {
+    // Update the BOT if the a threshold is crossed.
+    size_t obj_size = cast_to_oop(current)->size();
+    update_bot_for_block(current, current + obj_size);
+
+    // Advance to the next object.
+    current += obj_size;
+    guarantee(current <= end, "Should never go past end");
+  } while (current != end);
+}
