@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,7 +25,6 @@
 #include "precompiled.hpp"
 #include "classfile/vmClasses.hpp"
 #include "classfile/vmSymbols.hpp"
-#include "gc/shared/blockOffsetTable.inline.hpp"
 #include "gc/shared/collectedHeap.inline.hpp"
 #include "gc/shared/genCollectedHeap.hpp"
 #include "gc/shared/genOopClosures.inline.hpp"
@@ -44,6 +43,7 @@
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/macros.hpp"
 #if INCLUDE_SERIALGC
+#include "gc/serial/serialBlockOffsetTable.inline.hpp"
 #include "gc/serial/defNewGeneration.hpp"
 #endif
 
@@ -289,6 +289,7 @@ bool ContiguousSpace::is_free_block(const HeapWord* p) const {
   return p >= _top;
 }
 
+#if INCLUDE_SERIALGC
 void OffsetTableContigSpace::clear(bool mangle_space) {
   ContiguousSpace::clear(mangle_space);
   _offsets.initialize_threshold();
@@ -305,6 +306,7 @@ void OffsetTableContigSpace::set_end(HeapWord* new_end) {
   _offsets.resize(pointer_delta(new_end, bottom()));
   Space::set_end(new_end);
 }
+#endif // INCLUDE_SERIALGC
 
 #ifndef PRODUCT
 
@@ -555,8 +557,12 @@ void CompactibleSpace::compact() {
       // copy object and reinit its mark
       assert(cur_obj != compaction_top, "everything in this pass should be moving");
       Copy::aligned_conjoint_words(cur_obj, compaction_top, size);
-      cast_to_oop(compaction_top)->init_mark();
-      assert(cast_to_oop(compaction_top)->klass() != NULL, "should have a class");
+      oop new_obj = cast_to_oop(compaction_top);
+
+      ContinuationGCSupport::transform_stack_chunk(new_obj);
+
+      new_obj->init_mark();
+      assert(new_obj->klass() != NULL, "should have a class");
 
       debug_only(prev_obj = cur_obj);
       cur_obj += size;
@@ -579,22 +585,24 @@ void Space::print() const { print_on(tty); }
 
 void Space::print_on(outputStream* st) const {
   print_short_on(st);
-  st->print_cr(" [" INTPTR_FORMAT ", " INTPTR_FORMAT ")",
+  st->print_cr(" [" PTR_FORMAT ", " PTR_FORMAT ")",
                 p2i(bottom()), p2i(end()));
 }
 
 void ContiguousSpace::print_on(outputStream* st) const {
   print_short_on(st);
-  st->print_cr(" [" INTPTR_FORMAT ", " INTPTR_FORMAT ", " INTPTR_FORMAT ")",
+  st->print_cr(" [" PTR_FORMAT ", " PTR_FORMAT ", " PTR_FORMAT ")",
                 p2i(bottom()), p2i(top()), p2i(end()));
 }
 
+#if INCLUDE_SERIALGC
 void OffsetTableContigSpace::print_on(outputStream* st) const {
   print_short_on(st);
-  st->print_cr(" [" INTPTR_FORMAT ", " INTPTR_FORMAT ", "
-                INTPTR_FORMAT ", " INTPTR_FORMAT ")",
+  st->print_cr(" [" PTR_FORMAT ", " PTR_FORMAT ", "
+                PTR_FORMAT ", " PTR_FORMAT ")",
               p2i(bottom()), p2i(top()), p2i(_offsets.threshold()), p2i(end()));
 }
+#endif
 
 void ContiguousSpace::verify() const {
   HeapWord* p = bottom();
@@ -730,38 +738,7 @@ HeapWord* ContiguousSpace::par_allocate(size_t size) {
   return par_allocate_impl(size);
 }
 
-void ContiguousSpace::allocate_temporary_filler(int factor) {
-  // allocate temporary type array decreasing free size with factor 'factor'
-  assert(factor >= 0, "just checking");
-  size_t size = pointer_delta(end(), top());
-
-  // if space is full, return
-  if (size == 0) return;
-
-  if (factor > 0) {
-    size -= size/factor;
-  }
-  size = align_object_size(size);
-
-  const size_t array_header_size = typeArrayOopDesc::header_size(T_INT);
-  if (size >= align_object_size(array_header_size)) {
-    size_t length = (size - array_header_size) * (HeapWordSize / sizeof(jint));
-    // allocate uninitialized int array
-    typeArrayOop t = (typeArrayOop) cast_to_oop(allocate(size));
-    assert(t != NULL, "allocation should succeed");
-    t->set_mark(markWord::prototype());
-    t->set_klass(Universe::intArrayKlassObj());
-    t->set_length((int)length);
-  } else {
-    assert(size == CollectedHeap::min_fill_size(),
-           "size for smallest fake object doesn't match");
-    instanceOop obj = (instanceOop) cast_to_oop(allocate(size));
-    obj->set_mark(markWord::prototype());
-    obj->set_klass_gap(0);
-    obj->set_klass(vmClasses::Object_klass());
-  }
-}
-
+#if INCLUDE_SERIALGC
 void OffsetTableContigSpace::initialize_threshold() {
   _offsets.initialize_threshold();
 }
@@ -820,3 +797,4 @@ void OffsetTableContigSpace::verify() const {
 size_t TenuredSpace::allowed_dead_ratio() const {
   return MarkSweepDeadRatio;
 }
+#endif // INCLUDE_SERIALGC
