@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -41,7 +41,6 @@
 #include "oops/methodData.hpp"
 #include "oops/method.inline.hpp"
 #include "oops/oop.inline.hpp"
-#include "prims/forte.hpp"
 #include "prims/jvmtiExport.hpp"
 #include "prims/methodHandles.hpp"
 #include "runtime/handles.inline.hpp"
@@ -97,7 +96,7 @@ address    AbstractInterpreter::_native_abi_to_tosca    [AbstractInterpreter::nu
 //------------------------------------------------------------------------------------------------------------------------
 // Generation of complete interpreter
 
-AbstractInterpreterGenerator::AbstractInterpreterGenerator(StubQueue* _code) {
+AbstractInterpreterGenerator::AbstractInterpreterGenerator() {
   _masm                      = NULL;
 }
 
@@ -134,6 +133,9 @@ AbstractInterpreter::MethodKind AbstractInterpreter::method_kind(const methodHan
       case vmIntrinsics::_floatToRawIntBits: return java_lang_Float_floatToRawIntBits;
       case vmIntrinsics::_longBitsToDouble:  return java_lang_Double_longBitsToDouble;
       case vmIntrinsics::_doubleToRawLongBits: return java_lang_Double_doubleToRawLongBits;
+#ifdef AMD64
+      case vmIntrinsics::_currentThread:     return java_lang_Thread_currentThread;
+#endif
 #endif // ZERO
       case vmIntrinsics::_dsin:              return java_lang_math_sin;
       case vmIntrinsics::_dcos:              return java_lang_math_cos;
@@ -161,6 +163,10 @@ AbstractInterpreter::MethodKind AbstractInterpreter::method_kind(const methodHan
 
   // Native method?
   if (m->is_native()) {
+    if (m->is_continuation_native_intrinsic()) {
+      // This entry will never be called.  The real entry gets generated later, like for MH intrinsics.
+      return abstract;
+    }
     assert(!m->is_method_handle_intrinsic(), "overlapping bits here, watch out");
     return m->is_synchronized() ? native_synchronized : native;
   }
@@ -316,7 +322,7 @@ address AbstractInterpreter::deopt_continue_after_entry(Method* method, address
       methodHandle mh(thread, method);
       type = Bytecode_invoke(mh, bci).result_type();
       // since the cache entry might not be initialized:
-      // (NOT needed for the old calling convension)
+      // (NOT needed for the old calling convention)
       if (!is_top_frame) {
         int index = Bytes::get_native_u2(bcp+1);
         method->constants()->cache()->entry_at(index)->set_parameter_size(callee_parameters);
@@ -330,7 +336,7 @@ address AbstractInterpreter::deopt_continue_after_entry(Method* method, address
       methodHandle mh(thread, method);
       type = Bytecode_invoke(mh, bci).result_type();
       // since the cache entry might not be initialized:
-      // (NOT needed for the old calling convension)
+      // (NOT needed for the old calling convention)
       if (!is_top_frame) {
         int index = Bytes::get_native_u4(bcp+1);
         method->constants()->invokedynamic_cp_cache_entry_at(index)->set_parameter_size(callee_parameters);
@@ -384,7 +390,7 @@ bool AbstractInterpreter::bytecode_should_reexecute(Bytecodes::Code code) {
     case Bytecodes::_tableswitch:
     case Bytecodes::_fast_binaryswitch:
     case Bytecodes::_fast_linearswitch:
-    // recompute condtional expression folded into _if<cond>
+    // recompute conditional expression folded into _if<cond>
     case Bytecodes::_lcmp      :
     case Bytecodes::_fcmpl     :
     case Bytecodes::_fcmpg     :
