@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -35,9 +35,9 @@
 #include "runtime/arguments.hpp"
 #include "runtime/atomic.hpp"
 #include "runtime/globals.hpp"
+#include "runtime/javaThread.hpp"
 #include "runtime/mutexLocker.hpp"
 #include "runtime/os.hpp"
-#include "runtime/thread.inline.hpp"
 #include "utilities/growableArray.hpp"
 #include "utilities/ostream.hpp"
 
@@ -49,6 +49,7 @@ static const size_t iso8601_len = 19; // "YYYY-MM-DDTHH:MM:SS" (note: we just us
 static fio_fd emergency_fd = invalid_fd;
 static const int64_t chunk_file_header_size = 68;
 static const size_t chunk_file_extension_length = sizeof chunk_file_jfr_ext - 1;
+static char _dump_path[JVM_MAXPATHLEN] = { 0 };
 
 /*
  * The emergency dump logic is restrictive when it comes to
@@ -66,12 +67,17 @@ static bool is_path_empty() {
 }
 
 // returns with an appended file separator (if successful)
-static size_t get_current_directory() {
-  if (os::get_current_directory(_path_buffer, sizeof(_path_buffer)) == NULL) {
-    return 0;
+static size_t get_dump_directory() {
+  const char* dump_path = JfrEmergencyDump::get_dump_path();
+  if (*dump_path == '\0') {
+    if (os::get_current_directory(_path_buffer, sizeof(_path_buffer)) == NULL) {
+      return 0;
+    }
+  } else {
+    strcpy(_path_buffer, dump_path);
   }
-  const size_t cwd_len = strlen(_path_buffer);
-  const int result = jio_snprintf(_path_buffer + cwd_len,
+  const size_t path_len = strlen(_path_buffer);
+  const int result = jio_snprintf(_path_buffer + path_len,
                                   sizeof(_path_buffer),
                                   "%s",
                                   os::file_separator());
@@ -98,14 +104,14 @@ static bool open_emergency_dump_fd(const char* path) {
 
 static void close_emergency_dump_file() {
   if (is_emergency_dump_file_open()) {
-    os::close(emergency_fd);
+    ::close(emergency_fd);
   }
 }
 
 static const char* create_emergency_dump_path() {
   assert(is_path_empty(), "invariant");
 
-  const size_t path_len = get_current_directory();
+  const size_t path_len = get_dump_directory();
   if (path_len == 0) {
     return NULL;
   }
@@ -130,7 +136,16 @@ static bool open_emergency_dump_file() {
     // opened already
     return true;
   }
-  return open_emergency_dump_fd(create_emergency_dump_path());
+
+  bool result = open_emergency_dump_fd(create_emergency_dump_path());
+  if (!result && *_dump_path != '\0') {
+    log_warning(jfr)("Unable to create an emergency dump file at the location set by dumppath=%s", _dump_path);
+    // Fallback. Try to create it in the current directory.
+    *_dump_path = '\0';
+    *_path_buffer = '\0';
+    result = open_emergency_dump_fd(create_emergency_dump_path());
+  }
+  return result;
 }
 
 static void report(outputStream* st, bool emergency_file_opened, const char* repository_path) {
@@ -150,6 +165,21 @@ static void report(outputStream* st, bool emergency_file_opened, const char* rep
   }
 }
 
+void JfrEmergencyDump::set_dump_path(const char* path) {
+  if (path == NULL || *path == '\0') {
+    os::get_current_directory(_dump_path, sizeof(_dump_path));
+  } else {
+    if (strlen(path) < JVM_MAXPATHLEN) {
+      strncpy(_dump_path, path, JVM_MAXPATHLEN);
+      _dump_path[JVM_MAXPATHLEN - 1] = '\0';
+    }
+  }
+}
+
+const char* JfrEmergencyDump::get_dump_path() {
+  return _dump_path;
+}
+
 void JfrEmergencyDump::on_vm_error_report(outputStream* st, const char* repository_path) {
   assert(st != NULL, "invariant");
   Thread* thread = Thread::current_or_null_safe();
@@ -267,7 +297,7 @@ const char* RepositoryIterator::filter(const char* file_name) const {
     return NULL;
   }
   const int64_t size = file_size(fd);
-  os::close(fd);
+  ::close(fd);
   if (size <= chunk_file_header_size) {
     return NULL;
   }
@@ -358,7 +388,7 @@ static void write_repository_files(const RepositoryIterator& iterator, char* con
         bytes_written += (int64_t)os::write(emergency_fd, copy_block, bytes_read - bytes_written);
         assert(bytes_read == bytes_written, "invariant");
       }
-      os::close(current_fd);
+      ::close(current_fd);
     }
   }
 }
