@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -201,7 +201,7 @@ void IdealGraphPrinter::end_head() {
 void IdealGraphPrinter::print_attr(const char *name, intptr_t val) {
   stringStream stream;
   stream.print(INTX_FORMAT, val);
-  print_attr(name, stream.as_string());
+  print_attr(name, stream.freeze());
 }
 
 void IdealGraphPrinter::print_attr(const char *name, const char *val) {
@@ -225,7 +225,7 @@ void IdealGraphPrinter::text(const char *s) {
 void IdealGraphPrinter::print_prop(const char *name, int val) {
   stringStream stream;
   stream.print("%d", val);
-  print_prop(name, stream.as_string());
+  print_prop(name, stream.freeze());
 }
 
 void IdealGraphPrinter::print_prop(const char *name, const char *val) {
@@ -245,8 +245,8 @@ void IdealGraphPrinter::print_method(ciMethod *method, int bci, InlineTree *tree
   stringStream shortStr;
   method->print_short_name(&shortStr);
 
-  print_attr(METHOD_NAME_PROPERTY, str.as_string());
-  print_attr(METHOD_SHORT_NAME_PROPERTY, shortStr.as_string());
+  print_attr(METHOD_NAME_PROPERTY, str.freeze());
+  print_attr(METHOD_SHORT_NAME_PROPERTY, shortStr.freeze());
   print_attr(METHOD_BCI_PROPERTY, bci);
 
   end_head();
@@ -305,7 +305,7 @@ void IdealGraphPrinter::begin_method() {
   // Add method name
   stringStream strStream;
   method->print_name(&strStream);
-  print_prop(METHOD_NAME_PROPERTY, strStream.as_string());
+  print_prop(METHOD_NAME_PROPERTY, strStream.freeze());
 
   if (method->flags().is_public()) {
     print_prop(METHOD_IS_PUBLIC_PROPERTY, TRUE_VALUE);
@@ -316,7 +316,9 @@ void IdealGraphPrinter::begin_method() {
   }
 
   if (C->is_osr_compilation()) {
-      print_prop(COMPILATION_OSR_PROPERTY, TRUE_VALUE);
+      stringStream ss;
+      ss.print("bci: %d, line: %d", C->entry_bci(), method->line_number_from_bci(C->entry_bci()));
+      print_prop(COMPILATION_OSR_PROPERTY, ss.freeze());
   }
 
   print_prop(COMPILATION_ID_PROPERTY, C->compile_id());
@@ -385,6 +387,12 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
         print_prop("block", C->cfg()->get_block(0)->_pre_order);
       } else {
         print_prop("block", block->_pre_order);
+        if (node == block->head()) {
+          if (block->_idom != NULL) {
+            print_prop("idom", block->_idom->_pre_order);
+          }
+          print_prop("dom_depth", block->_dom_depth);
+        }
         // Print estimated execution frequency, normalized within a [0,1] range.
         buffer[0] = 0;
         stringStream freq(buffer, sizeof(buffer) - 1);
@@ -418,6 +426,14 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
         break;
     }
 
+    Node_Notes* nn = C->node_notes_at(node->_idx);
+    if (nn != NULL && !nn->is_clear() && nn->jvms() != NULL) {
+      buffer[0] = 0;
+      stringStream ss(buffer, sizeof(buffer) - 1);
+      nn->jvms()->dump_spec(&ss);
+      print_prop("jvms", buffer);
+    }
+
     const jushort flags = node->flags();
     if (flags & Node::Flag_is_Copy) {
       print_prop("is_copy", "true");
@@ -446,6 +462,9 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
     if (flags & Node::Flag_has_call) {
       print_prop("has_call", "true");
     }
+    if (flags & Node::Flag_is_reduction) {
+      print_prop("is_reduction", "true");
+    }
 
     if (C->matcher() != NULL) {
       if (C->matcher()->is_shared(node)) {
@@ -472,19 +491,20 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
       print_prop("idealOpcode", (const char *)NodeClassNames[node->as_Mach()->ideal_Opcode()]);
     }
 
+    print_field(node);
+
     buffer[0] = 0;
     stringStream s2(buffer, sizeof(buffer) - 1);
 
     node->dump_spec(&s2);
-    if (t != NULL && (t->isa_instptr() || t->isa_klassptr())) {
+    if (t != NULL && (t->isa_instptr() || t->isa_instklassptr())) {
       const TypeInstPtr  *toop = t->isa_instptr();
-      const TypeKlassPtr *tkls = t->isa_klassptr();
-      ciKlass*           klass = toop ? toop->klass() : (tkls ? tkls->klass() : NULL );
-      if( klass && klass->is_loaded() && klass->is_interface() ) {
+      const TypeInstKlassPtr *tkls = t->isa_instklassptr();
+      if ((toop != NULL && toop->is_interface()) || (tkls != NULL && tkls->is_interface())) {
         s2.print("  Interface:");
-      } else if( toop ) {
+      } else if (toop) {
         s2.print("  Oop:");
-      } else if( tkls ) {
+      } else if (tkls) {
         s2.print("  Klass:");
       }
       t->dump_on(&s2);
@@ -587,7 +607,7 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
         bciStream.print("%d ", caller->bci());
         caller = caller->caller();
       }
-      print_prop("bci", bciStream.as_string());
+      print_prop("bci", bciStream.freeze());
       if (last != NULL && last->has_linenumber_table() && last_bci >= 0) {
         print_prop("line", last->line_number_from_bci(last_bci));
       }
@@ -597,7 +617,7 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
     if (node->debug_orig() != NULL) {
       stringStream dorigStream;
       node->dump_orig(&dorigStream, false);
-      print_prop("debug_orig", dorigStream.as_string());
+      print_prop("debug_orig", dorigStream.freeze());
     }
 #endif
 
@@ -620,9 +640,96 @@ void IdealGraphPrinter::visit_node(Node *n, bool edges, VectorSet* temp_set) {
   }
 }
 
-void IdealGraphPrinter::walk_nodes(Node *start, bool edges, VectorSet* temp_set) {
+void IdealGraphPrinter::print_field(const Node* node) {
+  buffer[0] = 0;
+  stringStream ss(buffer, sizeof(buffer) - 1);
+  ciField* field = get_field(node);
+  uint depth = 0;
+  if (field == NULL) {
+    depth++;
+    field = find_source_field_of_array_access(node, depth);
+  }
 
+  if (field != NULL) {
+    // Either direct field access or array access
+    field->print_name_on(&ss);
+    for (uint i = 0; i < depth; i++) {
+      // For arrays: Add [] for each dimension
+      ss.print("[]");
+    }
+    if (node->is_Store()) {
+      print_prop("destination", buffer);
+    } else {
+      print_prop("source", buffer);
+    }
+  }
+}
 
+ciField* IdealGraphPrinter::get_field(const Node* node) {
+  const TypePtr* adr_type = node->adr_type();
+  Compile::AliasType* atp = NULL;
+  if (C->have_alias_type(adr_type)) {
+    atp = C->alias_type(adr_type);
+  }
+  if (atp != NULL) {
+    ciField* field = atp->field();
+    if (field != NULL) {
+      // Found field associated with 'node'.
+      return field;
+    }
+  }
+  return NULL;
+}
+
+// Try to find the field that is associated with a memory node belonging to an array access.
+ciField* IdealGraphPrinter::find_source_field_of_array_access(const Node* node, uint& depth) {
+  if (!node->is_Mem()) {
+    // Not an array access
+    return NULL;
+  }
+
+  do {
+    if (node->adr_type() != NULL && node->adr_type()->isa_aryptr()) {
+      // Only process array accesses. Pattern match to find actual field source access.
+      node = get_load_node(node);
+      if (node != NULL) {
+        ciField* field = get_field(node);
+        if (field != NULL) {
+          return field;
+        }
+        // Could be a multi-dimensional array. Repeat loop.
+        depth++;
+        continue;
+      }
+    }
+    // Not an array access with a field source.
+    break;
+  } while (depth < 256); // Cannot have more than 255 dimensions
+
+  return NULL;
+}
+
+// Pattern match on the inputs of 'node' to find load node for the field access.
+Node* IdealGraphPrinter::get_load_node(const Node* node) {
+  Node* load = NULL;
+  Node* addr = node->as_Mem()->in(MemNode::Address);
+  if (addr != NULL && addr->is_AddP()) {
+    Node* base = addr->as_AddP()->base_node();
+    if (base != NULL) {
+      base = base->uncast();
+      if (base->is_Load()) {
+        // Mem(AddP([ConstraintCast*](LoadP))) for non-compressed oops.
+        load = base;
+      } else if (base->is_DecodeN() && base->in(1)->is_Load()) {
+        // Mem(AddP([ConstraintCast*](DecodeN(LoadN)))) for compressed oops.
+        load = base->in(1);
+      }
+    }
+  }
+  return load;
+}
+
+void IdealGraphPrinter::walk_nodes(Node* start, bool edges, VectorSet* temp_set) {
   VectorSet visited;
   GrowableArray<Node *> nodeStack(Thread::current()->resource_area(), 0, 0, NULL);
   nodeStack.push(start);
@@ -663,7 +770,7 @@ void IdealGraphPrinter::walk_nodes(Node *start, bool edges, VectorSet* temp_set)
 }
 
 void IdealGraphPrinter::print_method(const char *name, int level) {
-  if (C->should_print(level)) {
+  if (C->should_print_igv(level)) {
     print(name, (Node *) C->root());
   }
 }
