@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,7 +39,6 @@ class JVMState;
 class MachCallDynamicJavaNode;
 class MachCallJavaNode;
 class MachCallLeafNode;
-class MachCallNativeNode;
 class MachCallNode;
 class MachCallRuntimeNode;
 class MachCallStaticJavaNode;
@@ -136,6 +135,14 @@ public:
     return ::as_VectorSRegister(reg(ra_, node, idx));
   }
 #endif
+#if defined(AARCH64)
+  PRegister as_PRegister(PhaseRegAlloc* ra_, const Node* node) const {
+    return ::as_PRegister(reg(ra_, node));
+  }
+  PRegister as_PRegister(PhaseRegAlloc* ra_, const Node* node, int idx) const {
+    return ::as_PRegister(reg(ra_, node, idx));
+  }
+#endif
 
   virtual intptr_t  constant() const;
   virtual relocInfo::relocType constant_reloc() const;
@@ -204,9 +211,6 @@ public:
 // Base type for all machine specific nodes.  All node classes generated by the
 // ADLC inherit from this class.
 class MachNode : public Node {
-private:
-  bool _removed = false;
-
 public:
   MachNode() : Node((uint)0), _barrier(0), _num_opnds(0), _opnds(NULL) {
     init_class_id(Class_Mach);
@@ -263,10 +267,10 @@ public:
   virtual const RegMask *cisc_RegMask() const { return NULL; }
 
   // If this instruction is a 2-address instruction, then return the
-  // index of the input which must match the output.  Not nessecary
+  // index of the input which must match the output.  Not necessary
   // for instructions which bind the input and output register to the
-  // same singleton regiser (e.g., Intel IDIV which binds AX to be
-  // both an input and an output).  It is nessecary when the input and
+  // same singleton register (e.g., Intel IDIV which binds AX to be
+  // both an input and an output).  It is necessary when the input and
   // output have choices - but they must use the same choice.
   virtual uint two_adr( ) const { return 0; }
 
@@ -277,9 +281,9 @@ public:
   // more leafs.  Must be set by MachNode constructor to point to an
   // internal array of MachOpers.  The MachOper array is sized by
   // specific MachNodes described in the ADL.
-  uint _num_opnds;
+  uint16_t _num_opnds;
   MachOper **_opnds;
-  uint  num_opnds() const { return _num_opnds; }
+  uint16_t num_opnds() const { return _num_opnds; }
 
   // Emit bytes into cbuf
   virtual void  emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const;
@@ -375,8 +379,8 @@ public:
 
   // Returns true if this node is a check that can be implemented with a trap.
   virtual bool is_TrapBasedCheckNode() const { return false; }
-  void set_removed() { _removed = true; }
-  bool get_removed() { return _removed; }
+  void set_removed() { add_flag(Flag_is_removed_by_peephole); }
+  bool get_removed() { return (flags() & Flag_is_removed_by_peephole) != 0; }
 
 #ifndef PRODUCT
   virtual const char *Name() const = 0; // Machine-specific name
@@ -1019,25 +1023,6 @@ public:
   }
 };
 
-class MachCallNativeNode: public MachCallNode {
-  virtual bool cmp( const Node &n ) const;
-  virtual uint size_of() const;
-  void print_regs(const GrowableArray<VMReg>& regs, outputStream* st) const;
-public:
-  const char *_name;
-  GrowableArray<VMReg> _arg_regs;
-  GrowableArray<VMReg> _ret_regs;
-
-  MachCallNativeNode() : MachCallNode() {
-    init_class_id(Class_MachCallNative);
-  }
-
-  virtual int ret_addr_offset();
-#ifndef PRODUCT
-  virtual void dump_spec(outputStream *st) const;
-#endif
-};
-
 //------------------------------MachHaltNode-----------------------------------
 // Machine-specific versions of halt nodes
 class MachHaltNode : public MachReturnNode {
