@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -423,7 +423,7 @@ PhaseRemoveUseless::PhaseRemoveUseless(PhaseGVN* gvn, Unique_Node_List* worklist
   worklist->remove_useless_nodes(_useful.member_set());
 
   // Disconnect 'useless' nodes that are adjacent to useful nodes
-  C->remove_useless_nodes(_useful);
+  C->disconnect_useless_nodes(_useful, worklist);
 }
 
 //=============================================================================
@@ -1507,7 +1507,7 @@ void PhaseIterGVN::add_users_to_worklist0( Node *n ) {
 }
 
 // Return counted loop Phi if as a counted loop exit condition, cmp
-// compares the the induction variable with n
+// compares the induction variable with n
 static PhiNode* countedloop_phi_from_cmp(CmpNode* cmp, Node* n) {
   for (DUIterator_Fast imax, i = cmp->fast_outs(imax); i < imax; i++) {
     Node* bol = cmp->fast_out(i);
@@ -1691,7 +1691,7 @@ bool PhaseIterGVN::no_dependent_zero_check(Node* n) const {
     case Op_DivI:
     case Op_ModI: {
       // Type of divisor includes 0?
-      if (n->in(2)->is_top()) {
+      if (type(n->in(2)) == Type::TOP) {
         // 'n' is dead. Treat as if zero check is still there to avoid any further optimizations.
         return false;
       }
@@ -1701,7 +1701,7 @@ bool PhaseIterGVN::no_dependent_zero_check(Node* n) const {
     case Op_DivL:
     case Op_ModL: {
       // Type of divisor includes 0?
-      if (n->in(2)->is_top()) {
+      if (type(n->in(2)) == Type::TOP) {
         // 'n' is dead. Treat as if zero check is still there to avoid any further optimizations.
         return false;
       }
@@ -1753,119 +1753,193 @@ static bool ccp_type_widens(const Type* t, const Type* t0) {
 }
 #endif //ASSERT
 
-//------------------------------analyze----------------------------------------
+// In this analysis, all types are initially set to TOP. We iteratively call Value() on all nodes of the graph until
+// we reach a fixed-point (i.e. no types change anymore). We start with a list that only contains the root node. Each time
+// a new type is set, we push all uses of that node back to the worklist (in some cases, we also push grandchildren
+// or nodes even further down back to the worklist because their type could change as a result of the current type
+// change).
 void PhaseCCP::analyze() {
   // Initialize all types to TOP, optimistic analysis
-  for (int i = C->unique() - 1; i >= 0; i--)  {
-    _types.map(i,Type::TOP);
+  for (uint i = 0; i < C->unique(); i++)  {
+    _types.map(i, Type::TOP);
   }
 
   // Push root onto worklist
   Unique_Node_List worklist;
   worklist.push(C->root());
 
+  assert(_root_and_safepoints.size() == 0, "must be empty (unused)");
+  _root_and_safepoints.push(C->root());
+
   // Pull from worklist; compute new value; push changes out.
   // This loop is the meat of CCP.
-  while( worklist.size() ) {
-    Node* n; // Node to be examined in this iteration
-    if (StressCCP) {
-      n = worklist.remove(C->random() % worklist.size());
-    } else {
-      n = worklist.pop();
+  while (worklist.size() != 0) {
+    Node* n = fetch_next_node(worklist);
+    if (n->is_SafePoint()) {
+      // Make sure safepoints are processed by PhaseCCP::transform even if they are
+      // not reachable from the bottom. Otherwise, infinite loops would be removed.
+      _root_and_safepoints.push(n);
     }
-    const Type *t = n->Value(this);
-    if (t != type(n)) {
-      assert(ccp_type_widens(t, type(n)), "ccp type must widen");
+    const Type* new_type = n->Value(this);
+    if (new_type != type(n)) {
+      assert(ccp_type_widens(new_type, type(n)), "ccp type must widen");
+      dump_type_and_node(n, new_type);
+      set_type(n, new_type);
+      push_child_nodes_to_worklist(worklist, n);
+    }
+  }
+}
+
+// Fetch next node from worklist to be examined in this iteration.
+Node* PhaseCCP::fetch_next_node(Unique_Node_List& worklist) {
+  if (StressCCP) {
+    return worklist.remove(C->random() % worklist.size());
+  } else {
+    return worklist.pop();
+  }
+}
+
 #ifndef PRODUCT
-      if( TracePhaseCCP ) {
-        t->dump();
-        do { tty->print("\t"); } while (tty->position() < 16);
-        n->dump();
-      }
+void PhaseCCP::dump_type_and_node(const Node* n, const Type* t) {
+  if (TracePhaseCCP) {
+    t->dump();
+    do {
+      tty->print("\t");
+    } while (tty->position() < 16);
+    n->dump();
+  }
+}
 #endif
-      set_type(n, t);
-      for (DUIterator_Fast imax, i = n->fast_outs(imax); i < imax; i++) {
-        Node* m = n->fast_out(i);   // Get user
-        if (m->is_Region()) {  // New path to Region?  Must recheck Phis too
-          for (DUIterator_Fast i2max, i2 = m->fast_outs(i2max); i2 < i2max; i2++) {
-            Node* p = m->fast_out(i2); // Propagate changes to uses
-            if (p->bottom_type() != type(p)) { // If not already bottomed out
-              worklist.push(p); // Propagate change to user
-            }
-          }
-        }
-        // If we changed the receiver type to a call, we need to revisit
-        // the Catch following the call.  It's looking for a non-NULL
-        // receiver to know when to enable the regular fall-through path
-        // in addition to the NullPtrException path
-        if (m->is_Call()) {
-          for (DUIterator_Fast i2max, i2 = m->fast_outs(i2max); i2 < i2max; i2++) {
-            Node* p = m->fast_out(i2);  // Propagate changes to uses
-            if (p->is_Proj() && p->as_Proj()->_con == TypeFunc::Control) {
-              Node* catch_node = p->find_out_with(Op_Catch);
-              if (catch_node != NULL) {
-                worklist.push(catch_node);
-              }
-            }
-          }
-        }
-        if (m->bottom_type() != type(m)) { // If not already bottomed out
-          worklist.push(m);     // Propagate change to user
-        }
 
-        // CmpU nodes can get their type information from two nodes up in the
-        // graph (instead of from the nodes immediately above). Make sure they
-        // are added to the worklist if nodes they depend on are updated, since
-        // they could be missed and get wrong types otherwise.
-        uint m_op = m->Opcode();
-        if (m_op == Op_AddI || m_op == Op_SubI) {
-          for (DUIterator_Fast i2max, i2 = m->fast_outs(i2max); i2 < i2max; i2++) {
-            Node* p = m->fast_out(i2); // Propagate changes to uses
-            if (p->Opcode() == Op_CmpU) {
-              // Got a CmpU which might need the new type information from node n.
-              if(p->bottom_type() != type(p)) { // If not already bottomed out
-                worklist.push(p); // Propagate change to user
-              }
-            }
-          }
-        }
-        // If n is used in a counted loop exit condition then the type
-        // of the counted loop's Phi depends on the type of n. See
-        // PhiNode::Value().
-        if (m_op == Op_CmpI || m_op == Op_CmpL) {
-          PhiNode* phi = countedloop_phi_from_cmp(m->as_Cmp(), n);
-          if (phi != NULL) {
-            worklist.push(phi);
-          }
-        }
-        // Loading the java mirror from a Klass requires two loads and the type
-        // of the mirror load depends on the type of 'n'. See LoadNode::Value().
-        BarrierSetC2* bs = BarrierSet::barrier_set()->barrier_set_c2();
-        bool has_load_barrier_nodes = bs->has_load_barrier_nodes();
+// We need to propagate the type change of 'n' to all its uses. Depending on the kind of node, additional nodes
+// (grandchildren or even further down) need to be revisited as their types could also be improved as a result
+// of the new type of 'n'. Push these nodes to the worklist.
+void PhaseCCP::push_child_nodes_to_worklist(Unique_Node_List& worklist, Node* n) const {
+  for (DUIterator_Fast imax, i = n->fast_outs(imax); i < imax; i++) {
+    Node* use = n->fast_out(i);
+    push_if_not_bottom_type(worklist, use);
+    push_more_uses(worklist, n, use);
+  }
+}
 
-        if (m_op == Op_LoadP && m->bottom_type()->isa_rawptr()) {
-          for (DUIterator_Fast i2max, i2 = m->fast_outs(i2max); i2 < i2max; i2++) {
-            Node* u = m->fast_out(i2);
-            const Type* ut = u->bottom_type();
-            if (u->Opcode() == Op_LoadP && ut->isa_instptr() && ut != type(u)) {
-              if (has_load_barrier_nodes) {
-                // Search for load barriers behind the load
-                for (DUIterator_Fast i3max, i3 = u->fast_outs(i3max); i3 < i3max; i3++) {
-                  Node* b = u->fast_out(i3);
-                  if (bs->is_gc_barrier_node(b)) {
-                    worklist.push(b);
-                  }
-                }
-              }
-              worklist.push(u);
-            }
-          }
+void PhaseCCP::push_if_not_bottom_type(Unique_Node_List& worklist, Node* n) const {
+  if (n->bottom_type() != type(n)) {
+    worklist.push(n);
+  }
+}
+
+// For some nodes, we need to propagate the type change to grandchildren or even further down.
+// Add them back to the worklist.
+void PhaseCCP::push_more_uses(Unique_Node_List& worklist, Node* parent, const Node* use) const {
+  push_phis(worklist, use);
+  push_catch(worklist, use);
+  push_cmpu(worklist, use);
+  push_counted_loop_phi(worklist, parent, use);
+  push_loadp(worklist, use);
+  push_and(worklist, parent, use);
+}
+
+
+// We must recheck Phis too if use is a Region.
+void PhaseCCP::push_phis(Unique_Node_List& worklist, const Node* use) const {
+  if (use->is_Region()) {
+    for (DUIterator_Fast imax, i = use->fast_outs(imax); i < imax; i++) {
+      push_if_not_bottom_type(worklist, use->fast_out(i));
+    }
+  }
+}
+
+// If we changed the receiver type to a call, we need to revisit the Catch node following the call. It's looking for a
+// non-NULL receiver to know when to enable the regular fall-through path in addition to the NullPtrException path.
+void PhaseCCP::push_catch(Unique_Node_List& worklist, const Node* use) {
+  if (use->is_Call()) {
+    for (DUIterator_Fast imax, i = use->fast_outs(imax); i < imax; i++) {
+      Node* proj = use->fast_out(i);
+      if (proj->is_Proj() && proj->as_Proj()->_con == TypeFunc::Control) {
+        Node* catch_node = proj->find_out_with(Op_Catch);
+        if (catch_node != NULL) {
+          worklist.push(catch_node);
         }
       }
     }
   }
 }
 
+// CmpU nodes can get their type information from two nodes up in the graph (instead of from the nodes immediately
+// above). Make sure they are added to the worklist if nodes they depend on are updated since they could be missed
+// and get wrong types otherwise.
+void PhaseCCP::push_cmpu(Unique_Node_List& worklist, const Node* use) const {
+  uint use_op = use->Opcode();
+  if (use_op == Op_AddI || use_op == Op_SubI) {
+    for (DUIterator_Fast imax, i = use->fast_outs(imax); i < imax; i++) {
+      Node* cmpu = use->fast_out(i);
+      if (cmpu->Opcode() == Op_CmpU) {
+        // Got a CmpU which might need the new type information from node n.
+        push_if_not_bottom_type(worklist, cmpu);
+      }
+    }
+  }
+}
+
+// If n is used in a counted loop exit condition, then the type of the counted loop's Phi depends on the type of 'n'.
+// Seem PhiNode::Value().
+void PhaseCCP::push_counted_loop_phi(Unique_Node_List& worklist, Node* parent, const Node* use) {
+  uint use_op = use->Opcode();
+  if (use_op == Op_CmpI || use_op == Op_CmpL) {
+    PhiNode* phi = countedloop_phi_from_cmp(use->as_Cmp(), parent);
+    if (phi != NULL) {
+      worklist.push(phi);
+    }
+  }
+}
+
+// Loading the java mirror from a Klass requires two loads and the type of the mirror load depends on the type of 'n'.
+// See LoadNode::Value().
+void PhaseCCP::push_loadp(Unique_Node_List& worklist, const Node* use) const {
+  BarrierSetC2* barrier_set = BarrierSet::barrier_set()->barrier_set_c2();
+  bool has_load_barrier_nodes = barrier_set->has_load_barrier_nodes();
+
+  if (use->Opcode() == Op_LoadP && use->bottom_type()->isa_rawptr()) {
+    for (DUIterator_Fast imax, i = use->fast_outs(imax); i < imax; i++) {
+      Node* loadp = use->fast_out(i);
+      const Type* ut = loadp->bottom_type();
+      if (loadp->Opcode() == Op_LoadP && ut->isa_instptr() && ut != type(loadp)) {
+        if (has_load_barrier_nodes) {
+          // Search for load barriers behind the load
+          push_load_barrier(worklist, barrier_set, loadp);
+        }
+        worklist.push(loadp);
+      }
+    }
+  }
+}
+
+void PhaseCCP::push_load_barrier(Unique_Node_List& worklist, const BarrierSetC2* barrier_set, const Node* use) {
+  for (DUIterator_Fast imax, i = use->fast_outs(imax); i < imax; i++) {
+    Node* barrier_node = use->fast_out(i);
+    if (barrier_set->is_gc_barrier_node(barrier_node)) {
+      worklist.push(barrier_node);
+    }
+  }
+}
+
+// AndI/L::Value() optimizes patterns similar to (v << 2) & 3 to zero if they are bitwise disjoint.
+// Add the AndI/L nodes back to the worklist to re-apply Value() in case the shift value changed.
+void PhaseCCP::push_and(Unique_Node_List& worklist, const Node* parent, const Node* use) const {
+  uint use_op = use->Opcode();
+  if ((use_op == Op_LShiftI || use_op == Op_LShiftL)
+      && use->in(2) == parent) { // is shift value (right-hand side of LShift)
+    for (DUIterator_Fast imax, i = use->fast_outs(imax); i < imax; i++) {
+      Node* and_node = use->fast_out(i);
+      uint and_node_op = and_node->Opcode();
+      if (and_node_op == Op_AndI || and_node_op == Op_AndL) {
+        push_if_not_bottom_type(worklist, and_node);
+      }
+    }
+  }
+}
+
+
 //------------------------------do_transform-----------------------------------
 // Top level driver for the recursive transformer
 void PhaseCCP::do_transform() {
@@ -1882,15 +1956,34 @@ Node *PhaseCCP::transform( Node *n ) {
   Node *new_node = _nodes[n->_idx]; // Check for transformed node
   if( new_node != NULL )
     return new_node;                // Been there, done that, return old answer
-  new_node = transform_once(n);     // Check for constant
-  _nodes.map( n->_idx, new_node );  // Flag as having been cloned
+
+  assert(n->is_Root(), "traversal must start at root");
+  assert(_root_and_safepoints.member(n), "root (n) must be in list");
 
   // Allocate stack of size _nodes.Size()/2 to avoid frequent realloc
-  GrowableArray <Node *> trstack(C->live_nodes() >> 1);
+  GrowableArray <Node *> transform_stack(C->live_nodes() >> 1);
+  Unique_Node_List useful; // track all visited nodes, so that we can remove the complement
 
-  trstack.push(new_node);           // Process children of cloned node
-  while ( trstack.is_nonempty() ) {
-    Node *clone = trstack.pop();
+  // Initialize the traversal.
+  // This CCP pass may prove that no exit test for a loop ever succeeds (i.e. the loop is infinite). In that case,
+  // the logic below doesn't follow any path from Root to the loop body: there's at least one such path but it's proven
+  // never taken (its type is TOP). As a consequence the node on the exit path that's input to Root (let's call it n) is
+  // replaced by the top node and the inputs of that node n are not enqueued for further processing. If CCP only works
+  // through the graph from Root, this causes the loop body to never be processed here even when it's not dead (that
+  // is reachable from Root following its uses). To prevent that issue, transform() starts walking the graph from Root
+  // and all safepoints.
+  for (uint i = 0; i < _root_and_safepoints.size(); ++i) {
+    Node* nn = _root_and_safepoints.at(i);
+    Node* new_node = _nodes[nn->_idx];
+    assert(new_node == NULL, "");
+    new_node = transform_once(nn);  // Check for constant
+    _nodes.map(nn->_idx, new_node); // Flag as having been cloned
+    transform_stack.push(new_node); // Process children of cloned node
+    useful.push(new_node);
+  }
+
+  while (transform_stack.is_nonempty()) {
+    Node* clone = transform_stack.pop();
     uint cnt = clone->req();
     for( uint i = 0; i < cnt; i++ ) {          // For all inputs do
       Node *input = clone->in(i);
@@ -1899,13 +1992,33 @@ Node *PhaseCCP::transform( Node *n ) {
         if( new_input == NULL ) {
           new_input = transform_once(input);   // Check for constant
           _nodes.map( input->_idx, new_input );// Flag as having been cloned
-          trstack.push(new_input);
+          transform_stack.push(new_input);     // Process children of cloned node
+          useful.push(new_input);
         }
         assert( new_input == clone->in(i), "insanity check");
       }
     }
   }
-  return new_node;
+
+  // The above transformation might lead to subgraphs becoming unreachable from the
+  // bottom while still being reachable from the top. As a result, nodes in that
+  // subgraph are not transformed and their bottom types are not updated, leading to
+  // an inconsistency between bottom_type() and type(). In rare cases, LoadNodes in
+  // such a subgraph, might be re-enqueued for IGVN indefinitely by MemNode::Ideal_common
+  // because their address type is inconsistent. Therefore, we aggressively remove
+  // all useless nodes here even before PhaseIdealLoop::build_loop_late gets a chance
+  // to remove them anyway.
+  if (C->cached_top_node()) {
+    useful.push(C->cached_top_node());
+  }
+  C->update_dead_node_list(useful);
+  remove_useless_nodes(useful.member_set());
+  _worklist.remove_useless_nodes(useful.member_set());
+  C->disconnect_useless_nodes(useful, &_worklist);
+
+  Node* new_root = _nodes[n->_idx];
+  assert(new_root->is_Root(), "transformed root node must be a root node");
+  return new_root;
 }
 
 
