@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,8 +21,8 @@
  * questions.
  */
 
-#ifndef SHARE_VM_OPTO_SUPERWORD_HPP
-#define SHARE_VM_OPTO_SUPERWORD_HPP
+#ifndef SHARE_OPTO_SUPERWORD_HPP
+#define SHARE_OPTO_SUPERWORD_HPP
 
 #include "opto/loopnode.hpp"
 #include "opto/node.hpp"
@@ -244,6 +244,45 @@ class OrderedPair {
   static const OrderedPair initial;
 };
 
+// -----------------------VectorElementSizeStats-----------------------
+// Vector lane size statistics for loop vectorization with vector masks
+class VectorElementSizeStats {
+ private:
+  static const int NO_SIZE = -1;
+  static const int MIXED_SIZE = -2;
+  int* _stats;
+
+ public:
+  VectorElementSizeStats(Arena* a) : _stats(NEW_ARENA_ARRAY(a, int, 4)) {
+    memset(_stats, 0, sizeof(int) * 4);
+  }
+
+  void record_size(int size) {
+    assert(1 <= size && size <= 8 && is_power_of_2(size), "Illegal size");
+    _stats[exact_log2(size)]++;
+  }
+
+  int smallest_size() {
+    for (int i = 0; i <= 3; i++) {
+      if (_stats[i] > 0) return (1 << i);
+    }
+    return NO_SIZE;
+  }
+
+  int largest_size() {
+    for (int i = 3; i >= 0; i--) {
+      if (_stats[i] > 0) return (1 << i);
+    }
+    return NO_SIZE;
+  }
+
+  int unique_size() {
+    int small = smallest_size();
+    int large = largest_size();
+    return (small == large) ? small : MIXED_SIZE;
+  }
+};
+
 // -----------------------------SuperWord---------------------------------
 // Transforms scalar operations into packed (superword) operations.
 class SuperWord : public ResourceObj {
@@ -286,7 +325,9 @@ class SuperWord : public ResourceObj {
  public:
   SuperWord(PhaseIdealLoop* phase);
 
-  void transform_loop(IdealLoopTree* lpt, bool do_optimization);
+  bool transform_loop(IdealLoopTree* lpt, bool do_optimization);
+
+  int max_vector_size(BasicType bt);
 
   void unrolling_analysis(int &local_loop_unroll_factor);
 
@@ -367,6 +408,7 @@ class SuperWord : public ResourceObj {
     BasicType bt = velt_basic_type(n);
     return vector_width(n)*type2aelembytes(bt);
   }
+  int get_vw_bytes_special(MemNode* s);
   MemNode* align_to_ref()            { return _align_to_ref; }
   void  set_align_to_ref(MemNode* m) { _align_to_ref = m; }
 
@@ -378,17 +420,20 @@ class SuperWord : public ResourceObj {
   void set_bb_idx(Node* n, int i) { _bb_idx.at_put_grow(n->_idx, i); }
 
   // visited set accessors
-  void visited_clear()           { _visited.Clear(); }
+  void visited_clear()           { _visited.clear(); }
   void visited_set(Node* n)      { return _visited.set(bb_idx(n)); }
   int visited_test(Node* n)      { return _visited.test(bb_idx(n)); }
   int visited_test_set(Node* n)  { return _visited.test_set(bb_idx(n)); }
-  void post_visited_clear()      { _post_visited.Clear(); }
+  void post_visited_clear()      { _post_visited.clear(); }
   void post_visited_set(Node* n) { return _post_visited.set(bb_idx(n)); }
   int post_visited_test(Node* n) { return _post_visited.test(bb_idx(n)); }
 
   // Ensure node_info contains element "i"
   void grow_node_info(int i) { if (i >= _node_info.length()) _node_info.at_put_grow(i, SWNodeInfo::initial); }
 
+  // should we align vector memory references on this platform?
+  bool vectors_should_be_aligned() { return !Matcher::misaligned_vectors_ok() || AlignVector; }
+
   // memory alignment for a node
   int alignment(Node* n)                     { return _node_info.adr_at(bb_idx(n))->_alignment; }
   void set_alignment(Node* n, int a)         { int i = bb_idx(n); grow_node_info(i); _node_info.adr_at(i)->_alignment = a; }
@@ -418,7 +463,7 @@ class SuperWord : public ResourceObj {
   // methods
 
   // Extract the superword level parallelism
-  void SLP_extract();
+  bool SLP_extract();
   // Find the adjacent memory references and create pack pairs for them.
   void find_adjacent_refs();
   // Tracing support
@@ -432,7 +477,7 @@ class SuperWord : public ResourceObj {
   int get_iv_adjustment(MemNode* mem);
   // Can the preloop align the reference to position zero in the vector?
   bool ref_is_alignable(SWPointer& p);
-  // rebuild the graph so all loads in different iterations of cloned loop become dependant on phi node (in _do_vector_loop only)
+  // rebuild the graph so all loads in different iterations of cloned loop become dependent on phi node (in _do_vector_loop only)
   bool hoist_loads_in_graph();
   // Test whether MemNode::Memory dependency to the same load but in the first iteration of this loop is coming from memory phi
   // Return false if failed
@@ -473,6 +518,7 @@ class SuperWord : public ResourceObj {
   int data_size(Node* s);
   // Extend packset by following use->def and def->use links from pack members.
   void extend_packlist();
+  int adjust_alignment_for_type_conversion(Node* s, Node* t, int align);
   // Extend the packset by visiting operand definitions of nodes in pack p
   bool follow_use_defs(Node_List* p);
   // Extend the packset by visiting uses of nodes in pack p
@@ -502,10 +548,12 @@ class SuperWord : public ResourceObj {
   void co_locate_pack(Node_List* p);
   Node* pick_mem_state(Node_List* pk);
   Node* find_first_mem_state(Node_List* pk);
-  Node* find_last_mem_state(Node_List* pk, Node* first_mem);
+  Node* find_last_mem_state(Node_List* pk, Node* first_mem, bool &is_dependent);
 
   // Convert packs into vector node operations
-  void output();
+  bool output();
+  // Create vector mask for post loop vectorization
+  Node* create_post_loop_vmask();
   // Create a vector operand for the nodes in pack p for operand: in(opd_idx)
   Node* vector_opd(Node_List* p, int opd_idx);
   // Can code be generated for pack p?
@@ -524,6 +572,10 @@ class SuperWord : public ResourceObj {
   void bb_insert_after(Node* n, int pos);
   // Compute max depth for expressions from beginning of block
   void compute_max_depth();
+  // Return the longer type for vectorizable type-conversion node or illegal type for other nodes.
+  BasicType longer_type_for_conversion(Node* n);
+  // Find the longest type in def-use chain for packed nodes, and then compute the max vector size.
+  int max_vector_size_in_def_use_chain(Node* n);
   // Compute necessary vector element type for expressions
   void compute_vector_element_type();
   // Are s1 and s2 in a pack pair and ordered as s1,s2?
@@ -568,62 +620,19 @@ class SuperWord : public ResourceObj {
 
 //------------------------------SWPointer---------------------------
 // Information about an address for dependence checking and vector alignment
-//
-// We parse and represent pointers of the simple form:
-//
-//   pointer   = adr + offset + invar + scale * ConvI2L(iv)
-//
-// Where:
-//
-//   adr: the base address of an array (base = adr)
-//        OR
-//        some address to off-heap memory (base = TOP)
-//
-//   offset: a constant offset
-//   invar:  a runtime variable, which is invariant during the loop
-//   scale:  scaling factor
-//   iv:     loop induction variable
-//
-// But more precisely, we parse the composite-long-int form:
-//
-//   pointer   = adr + long_offset + long_invar + long_scale * ConvI2L(int_offset + inv_invar + int_scale * iv)
-//
-//   pointer   = adr + long_offset + long_invar + long_scale * ConvI2L(int_index)
-//   int_index =       int_offset  + int_invar  + int_scale  * iv
-//
-// However, for aliasing and adjacency checks (e.g. SWPointer::cmp()) we always use the simple form to make
-// decisions. Hence, we must make sure to only create a "valid" SWPointer if the optimisations based on the
-// simple form produce the same result as the compound-long-int form would. Intuitively, this depends on
-// if the int_index overflows, but the precise conditions are given in SWPointer::is_safe_to_use_as_simple_form().
-//
-//   ConvI2L(int_index) = ConvI2L(int_offset  + int_invar  + int_scale  * iv)
-//                      = Convi2L(int_offset) + ConvI2L(int_invar) + ConvI2L(int_scale) * ConvI2L(iv)
-//
-//   scale  = long_scale * ConvI2L(int_scale)
-//   offset = long_offset + long_scale * ConvI2L(int_offset)
-//   invar  = long_invar  + long_scale * ConvI2L(int_invar)
-//
-//   pointer   = adr + offset + invar + scale * ConvI2L(iv)
-//
-class SWPointer {
+class SWPointer : public ResourceObj {
  protected:
   MemNode*   _mem;           // My memory reference node
   SuperWord* _slp;           // SuperWord class
 
-  // Components of the simple form:
-  Node* _base;               // Base address of an array OR NULL if some off-heap memory.
-  Node* _adr;                // Same as _base if an array pointer OR some off-heap memory pointer.
-  jint  _scale;              // multiplier for iv (in bytes), 0 if no loop iv
-  jint  _offset;             // constant offset (in bytes)
+  Node* _base;               // NULL if unsafe nonheap reference
+  Node* _adr;                // address pointer
+  int   _scale;              // multiplier for iv (in bytes), 0 if no loop iv
+  int   _offset;             // constant offset (in bytes)
+
   Node* _invar;              // invariant offset (in bytes), NULL if none
   bool  _negate_invar;       // if true then use: (0 - _invar)
-
-  // The int_index components of the compound-long-int form. Used to decide if it is safe to use the
-  // simple form rather than the compound-long-int form that was parsed.
-  bool  _has_int_index_after_convI2L;
-  int   _int_index_after_convI2L_offset;
-  Node* _int_index_after_convI2L_invar;
-  int   _int_index_after_convI2L_scale;
+  Node* _invar_scale;        // multiplier for invariant
 
   Node_Stack* _nstack;       // stack used to record a swpointer trace of variants
   bool        _analyze_only; // Used in loop unrolling only for swpointer trace
@@ -633,7 +642,7 @@ class SWPointer {
   IdealLoopTree*  lpt() const   { return _slp->lpt(); }
   PhiNode*        iv() const    { return _slp->iv();  } // Induction var
 
-  bool is_main_loop_member(Node* n) const;
+  bool is_loop_member(Node* n) const;
   bool invariant(Node* n) const;
 
   // Match: k*iv + offset
@@ -643,8 +652,6 @@ class SWPointer {
   // Match: offset is (k [+/- invariant])
   bool offset_plus_k(Node* n, bool negate = false);
 
-  bool is_safe_to_use_as_simple_form(Node* base, Node* adr) const;
-
  public:
   enum CMP {
     Less          = 1,
@@ -668,50 +675,22 @@ class SWPointer {
   int   scale_in_bytes()   { return _scale; }
   Node* invar()            { return _invar; }
   bool  negate_invar()     { return _negate_invar; }
+  Node* invar_scale()      { return _invar_scale; }
   int   offset_in_bytes()  { return _offset; }
   int   memory_size()      { return _mem->memory_size(); }
   Node_Stack* node_stack() { return _nstack; }
 
   // Comparable?
-  // We compute if and how two SWPointers can alias at runtime, i.e. if the two addressed regions of memory can
-  // ever overlap. There are essentially 3 relevant return states:
-  //  - NotComparable:  Synonymous to "unknown aliasing".
-  //                    We have no information about how the two SWPointers can alias. They could overlap, refer
-  //                    to another location in the same memory object, or point to a completely different object.
-  //                    -> Memory edge required. Aliasing unlikely but possible.
-  //
-  //  - Less / Greater: Synonymous to "never aliasing".
-  //                    The two SWPointers may point into the same memory object, but be non-aliasing (i.e. we
-  //                    know both address regions inside the same memory object, but these regions are non-
-  //                    overlapping), or the SWPointers point to entirely different objects.
-  //                    -> No memory edge required. Aliasing impossible.
-  //
-  //  - Equal:          Synonymous to "overlap, or point to different memory objects".
-  //                    The two SWPointers either overlap on the same memory object, or point to two different
-  //                    memory objects.
-  //                    -> Memory edge required. Aliasing likely.
-  //
-  // In a future refactoring, we can simplify to two states:
-  //  - NeverAlias:     instead of Less / Greater
-  //  - MayAlias:       instead of Equal / NotComparable
-  //
-  // Two SWPointer are "comparable" (Less / Greater / Equal), iff all of these conditions apply:
-  //   1) Both are valid, i.e. expressible in the compound-long-int or simple form.
-  //   2) The adr are identical, or both are array bases of different arrays.
-  //   3) They have identical scale.
-  //   4) They have identical invar.
-  //   5) The difference in offsets is limited: abs(offset0 - offset1) < 2^31.
+  bool invar_equals(SWPointer& q) {
+      return (_invar        == q._invar   &&
+              _invar_scale  == q._invar_scale &&
+              _negate_invar == q._negate_invar);
+  }
+
   int cmp(SWPointer& q) {
     if (valid() && q.valid() &&
         (_adr == q._adr || (_base == _adr && q._base == q._adr)) &&
-        _scale == q._scale   &&
-        _invar == q._invar   &&
-        _negate_invar == q._negate_invar) {
-      jlong difference = abs(java_subtract((jlong)_offset, (jlong)q._offset));
-      jlong max_diff = (jlong)1 << 31;
-      if (difference >= max_diff) {
-        return NotComparable;
-      }
+        _scale == q._scale   && invar_equals(q)) {
       bool overlap = q._offset <   _offset +   memory_size() &&
                        _offset < q._offset + q.memory_size();
       return overlap ? Equal : (_offset < q._offset ? Less : Greater);
@@ -727,6 +706,8 @@ class SWPointer {
   static bool equal(int cmp)      { return cmp == Equal; }
   static bool comparable(int cmp) { return cmp < NotComparable; }
 
+  static bool has_potential_dependence(GrowableArray<SWPointer*> swptrs);
+
   void print();
 
 #ifndef PRODUCT
@@ -781,7 +762,7 @@ class SWPointer {
     void scaled_iv_6(Node* n, int scale);
     void scaled_iv_7(Node* n);
     void scaled_iv_8(Node* n, SWPointer* tmp);
-    void scaled_iv_9(Node* n, int _scale, int _offset, int mult);
+    void scaled_iv_9(Node* n, int _scale, int _offset, Node* _invar, bool _negate_invar);
     void scaled_iv_10(Node* n);
 
     void offset_plus_k_1(Node* n);
@@ -798,13 +779,6 @@ class SWPointer {
 
   } _tracer;//TRacer;
 #endif
-
-  static bool try_AddI_no_overflow(jint offset1, jint offset2, jint& result);
-  static bool try_SubI_no_overflow(jint offset1, jint offset2, jint& result);
-  static bool try_AddSubI_no_overflow(jint offset1, jint offset2, bool is_sub, jint& result);
-  static bool try_LShiftI_no_overflow(jint offset1, int offset2, jint& result);
-  static bool try_MulI_no_overflow(jint offset1, jint offset2, jint& result);
-
 };
 
-#endif // SHARE_VM_OPTO_SUPERWORD_HPP
+#endif // SHARE_OPTO_SUPERWORD_HPP
