@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -40,9 +40,9 @@
 #include "runtime/fieldDescriptor.inline.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
+#include "runtime/javaThread.hpp"
 #include "runtime/jfieldIDWorkaround.hpp"
 #include "runtime/jniHandles.inline.hpp"
-#include "runtime/thread.inline.hpp"
 #include "utilities/formatBuffer.hpp"
 #include "utilities/utf8.hpp"
 
@@ -59,7 +59,7 @@
 
 // Execute the given block of source code with the thread in VM state.
 // To do this, transition from the NATIVE state to the VM state, execute
-// the code, and transtition back.  The ThreadInVMfromNative constructor
+// the code, and transition back.  The ThreadInVMfromNative constructor
 // performs the transition to VM state, its destructor restores the
 // NATIVE state.
 
@@ -2014,7 +2014,16 @@ JNI_ENTRY_CHECKED(jobject,
   checked_jni_GetModule(JNIEnv *env,
                         jclass clazz))
     functionEnter(thr);
-    jobject result = UNCHECKED()->GetModule(env,clazz);
+    jobject result = UNCHECKED()->GetModule(env, clazz);
+    functionExit(thr);
+    return result;
+JNI_END
+
+JNI_ENTRY_CHECKED(jboolean,
+  checked_jni_IsVirtualThread(JNIEnv *env,
+                              jobject obj))
+    functionEnter(thr);
+    jboolean result = UNCHECKED()->IsVirtualThread(env, obj);
     functionExit(thr);
     return result;
 JNI_END
@@ -2304,7 +2313,11 @@ struct JNINativeInterface_  checked_jni_NativeInterface = {
 
     // Module Features
 
-    checked_jni_GetModule
+    checked_jni_GetModule,
+
+    // Virtual threads
+
+    checked_jni_IsVirtualThread
 };
 
 
