@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,19 +22,19 @@
  *
  */
 
-#ifndef SHARE_VM_PRIMS_JVMTIENVBASE_HPP
-#define SHARE_VM_PRIMS_JVMTIENVBASE_HPP
+#ifndef SHARE_PRIMS_JVMTIENVBASE_HPP
+#define SHARE_PRIMS_JVMTIENVBASE_HPP
 
-#include "classfile/classLoader.hpp"
 #include "prims/jvmtiEnvThreadState.hpp"
 #include "prims/jvmtiEventController.hpp"
 #include "prims/jvmtiThreadState.hpp"
 #include "oops/oopHandle.hpp"
+#include "runtime/atomic.hpp"
 #include "runtime/fieldDescriptor.hpp"
 #include "runtime/frame.hpp"
-#include "runtime/orderAccess.hpp"
-#include "runtime/thread.hpp"
-#include "runtime/vmOperations.hpp"
+#include "runtime/javaThread.hpp"
+#include "runtime/threads.hpp"
+#include "runtime/vmOperation.hpp"
 #include "utilities/growableArray.hpp"
 #include "utilities/macros.hpp"
 
@@ -83,6 +83,19 @@ class JvmtiEnvBase : public CHeapObj<mtInternal> {
   static void leaving_dying_thread_env_iteration()  { --_dying_thread_env_iteration_count; }
   static bool is_inside_dying_thread_env_iteration(){ return _dying_thread_env_iteration_count > 0; }
 
+  static jvmtiError suspend_thread(oop thread_oop, JavaThread* java_thread, bool single_suspend,
+                                   int* need_safepoint_p);
+  static jvmtiError resume_thread(oop thread_oop, JavaThread* java_thread, bool single_resume);
+  static jvmtiError check_thread_list(jint count, const jthread* list);
+  static bool is_in_thread_list(jint count, const jthread* list, oop jt_oop);
+
+  // check if thread_oop represents a passive carrier thread
+  static bool is_passive_carrier_thread(JavaThread* java_thread, oop thread_oop) {
+    return java_thread != NULL && java_thread->jvmti_vthread() != NULL
+                               && java_thread->jvmti_vthread() != thread_oop
+                               && java_thread->threadObj() == thread_oop;
+  }
+
  private:
 
   enum {
@@ -154,6 +167,37 @@ class JvmtiEnvBase : public CHeapObj<mtInternal> {
     return byte_offset_of(JvmtiEnvBase, _jvmti_external);
   };
 
+  // If (thread == NULL) then return current thread object.
+  // Otherwise return JNIHandles::resolve_external_guard(thread).
+  static oop current_thread_obj_or_resolve_external_guard(jthread thread);
+
+  static jvmtiError get_JavaThread(ThreadsList* tlist, jthread thread, JavaThread** jt_pp) {
+    jvmtiError err = JVMTI_ERROR_NONE;
+    if (thread == NULL) {
+      *jt_pp = JavaThread::current();
+    } else {
+      err = JvmtiExport::cv_external_thread_to_JavaThread(tlist, thread, jt_pp, NULL);
+    }
+    return err;
+  }
+
+  // If there is a virtual thread mounted on the JavaThread* then
+  // return virtual thread oop. Otherwise, return thread oop.
+  static oop get_vthread_or_thread_oop(JavaThread* jt) {
+    oop result = jt->threadObj();
+    if (jt->jvmti_vthread() != NULL) {
+      result = jt->jvmti_vthread();
+    }
+    return result;
+  }
+
+  static jvmtiError get_threadOop_and_JavaThread(ThreadsList* t_list, jthread thread,
+                                                 JavaThread** jt_pp, oop* thread_oop_p);
+
+  // Return true if java thread is a carrier thread with a mounted virtual thread.
+  static bool is_cthread_with_mounted_vthread(JavaThread* jt);
+  static bool is_cthread_with_continuation(JavaThread* jt);
+
   static JvmtiEnv* JvmtiEnv_from_jvmti_env(jvmtiEnv *env) {
     return (JvmtiEnv*)((intptr_t)env - in_bytes(jvmti_external_offset()));
   };
@@ -255,11 +299,11 @@ class JvmtiEnvBase : public CHeapObj<mtInternal> {
   }
 
   JvmtiTagMap* tag_map_acquire() {
-    return OrderAccess::load_acquire(&_tag_map);
+    return Atomic::load_acquire(&_tag_map);
   }
 
   void release_set_tag_map(JvmtiTagMap* tag_map) {
-    OrderAccess::release_store(&_tag_map, tag_map);
+    Atomic::release_store(&_tag_map, tag_map);
   }
 
   // return true if event is enabled globally or for any thread
@@ -286,29 +330,70 @@ class JvmtiEnvBase : public CHeapObj<mtInternal> {
                                    javaVFrame *jvf,
                                    GrowableArray<jvmtiMonitorStackDepthInfo*>* owned_monitors_list,
                                    jint depth);
-  vframe* vframeFor(JavaThread* java_thread, jint depth);
-
  public:
+  static javaVFrame* jvf_for_thread_and_depth(JavaThread* java_thread, jint depth);
+
   // get a field descriptor for the specified class and field
   static bool get_field_descriptor(Klass* k, jfieldID field, fieldDescriptor* fd);
 
-  // JVMTI API helper functions which are called at safepoint or thread is suspended.
-  jvmtiError get_frame_count(JvmtiThreadState *state, jint *count_ptr);
+  // check and skip frames hidden in mount/unmount transitions
+  static javaVFrame* check_and_skip_hidden_frames(bool is_in_VTMS_transition, javaVFrame* jvf);
+  static javaVFrame* check_and_skip_hidden_frames(JavaThread* jt, javaVFrame* jvf);
+  static javaVFrame* check_and_skip_hidden_frames(oop vthread, javaVFrame* jvf);
+
+  // check if virtual thread is not terminated (alive)
+  static bool is_vthread_alive(oop vt);
+
+  // return JavaThread if virtual thread is mounted, NULL otherwise
+  static JavaThread* get_JavaThread_or_null(oop vthread);
+
+  // get virtual thread last java vframe
+  static javaVFrame* get_vthread_jvf(oop vthread);
+
+  // get carrier thread last java vframe
+  static javaVFrame* get_cthread_last_java_vframe(JavaThread* jt, RegisterMap* reg_map);
+
+  // get ordinary thread thread state
+  static jint get_thread_state(oop thread_oop, JavaThread* jt);
+
+  // get virtual thread thread state
+  static jint get_vthread_state(oop thread_oop, JavaThread* jt);
+
+  // enumerates the live threads in the given thread group
+  static jvmtiError get_live_threads(JavaThread* current_thread, Handle group_hdl, jint *count_ptr, Handle **thread_objs_p);
+
+  // enumerates the subgroups in the given thread group
+  static jvmtiError get_subgroups(JavaThread* current_thread, Handle group_hdl, jint *count_ptr, Handle **group_objs_p);
+
+  // JVMTI API helper functions which are called when target thread is suspended
+  // or at safepoint / thread local handshake.
+  static jint get_frame_count(javaVFrame* jvf);
+  jvmtiError get_frame_count(JavaThread* java_thread, jint *count_ptr);
+  jvmtiError get_frame_count(oop frame_oop, jint *count_ptr);
+  jvmtiError get_frame_location(javaVFrame* jvf, jint depth,
+                                jmethodID* method_ptr, jlocation* location_ptr);
   jvmtiError get_frame_location(JavaThread* java_thread, jint depth,
-                                              jmethodID* method_ptr, jlocation* location_ptr);
-  jvmtiError get_object_monitor_usage(JavaThread *calling_thread,
-                                                    jobject object, jvmtiMonitorUsage* info_ptr);
-  jvmtiError get_stack_trace(JavaThread *java_thread,
-                                           jint stack_depth, jint max_count,
-                                           jvmtiFrameInfo* frame_buffer, jint* count_ptr);
-  jvmtiError get_current_contended_monitor(JavaThread *calling_thread,
-                                                         JavaThread *java_thread,
-                                                         jobject *monitor_ptr);
-  jvmtiError get_owned_monitors(JavaThread *calling_thread, JavaThread* java_thread,
-                          GrowableArray<jvmtiMonitorStackDepthInfo*> *owned_monitors_list);
-  jvmtiError check_top_frame(JavaThread* current_thread, JavaThread* java_thread,
-                             jvalue value, TosState tos, Handle* ret_ob_h);
-  jvmtiError force_early_return(JavaThread* java_thread, jvalue value, TosState tos);
+                                jmethodID* method_ptr, jlocation* location_ptr);
+  jvmtiError get_frame_location(oop vthread_oop, jint depth,
+                                jmethodID* method_ptr, jlocation* location_ptr);
+  jvmtiError set_frame_pop(JvmtiThreadState* state, javaVFrame* jvf, jint depth);
+  jvmtiError get_object_monitor_usage(JavaThread* calling_thread,
+                                      jobject object, jvmtiMonitorUsage* info_ptr);
+  jvmtiError get_stack_trace(javaVFrame* jvf,
+                             jint stack_depth, jint max_count,
+                             jvmtiFrameInfo* frame_buffer, jint* count_ptr);
+  jvmtiError get_stack_trace(JavaThread* java_thread,
+                             jint stack_depth, jint max_count,
+                             jvmtiFrameInfo* frame_buffer, jint* count_ptr);
+  jvmtiError get_current_contended_monitor(JavaThread* calling_thread, JavaThread* java_thread,
+                                           jobject* monitor_ptr, bool is_virtual);
+  jvmtiError get_owned_monitors(JavaThread* calling_thread, JavaThread* java_thread,
+                                GrowableArray<jvmtiMonitorStackDepthInfo*> *owned_monitors_list);
+  jvmtiError get_owned_monitors(JavaThread* calling_thread, JavaThread* java_thread, javaVFrame* jvf,
+                                GrowableArray<jvmtiMonitorStackDepthInfo*> *owned_monitors_list);
+  static jvmtiError check_top_frame(Thread* current_thread, JavaThread* java_thread,
+                                    jvalue value, TosState tos, Handle* ret_ob_h);
+  jvmtiError force_early_return(jthread thread, jvalue value, TosState tos);
 };
 
 // This class is the only safe means of iterating through environments.
@@ -336,71 +421,84 @@ class JvmtiEnvIterator : public StackObj {
   JvmtiEnv* next(JvmtiEnvBase* env) { return env->next_environment(); }
 };
 
-// VM operation to update for pop top frame.
-class VM_UpdateForPopTopFrame : public VM_Operation {
-private:
-  JvmtiThreadState* _state;
+class JvmtiHandshakeClosure : public HandshakeClosure {
+ protected:
   jvmtiError _result;
-
-public:
-  VM_UpdateForPopTopFrame(JvmtiThreadState* state) {
-    _state = state;
-    _result = JVMTI_ERROR_NONE;
-  }
-  VMOp_Type type() const { return VMOp_UpdateForPopTopFrame; }
+ public:
+  JvmtiHandshakeClosure(const char* name)
+    : HandshakeClosure(name),
+      _result(JVMTI_ERROR_THREAD_NOT_ALIVE) {}
   jvmtiError result() { return _result; }
-  void doit();
 };
 
-// VM operation to set frame pop.
-class VM_SetFramePop : public VM_Operation {
+class SetForceEarlyReturn : public JvmtiHandshakeClosure {
+private:
+  JvmtiThreadState* _state;
+  jvalue _value;
+  TosState _tos;
+public:
+  SetForceEarlyReturn(JvmtiThreadState* state, jvalue value, TosState tos)
+    : JvmtiHandshakeClosure("SetForceEarlyReturn"),
+     _state(state),
+     _value(value),
+     _tos(tos) {}
+  void do_thread(Thread *target) {
+    doit(target, false /* self */);
+  }
+  void doit(Thread *target, bool self);
+};
+
+// HandshakeClosure to update for pop top frame.
+class UpdateForPopTopFrameClosure : public JvmtiHandshakeClosure {
+private:
+  JvmtiThreadState* _state;
+
+public:
+  UpdateForPopTopFrameClosure(JvmtiThreadState* state)
+    : JvmtiHandshakeClosure("UpdateForPopTopFrame"),
+     _state(state) {}
+  void do_thread(Thread *target) {
+    doit(target, false /* self */);
+  }
+  void doit(Thread *target, bool self);
+};
+
+// HandshakeClosure to set frame pop.
+class SetFramePopClosure : public JvmtiHandshakeClosure {
 private:
   JvmtiEnv *_env;
   JvmtiThreadState* _state;
   jint _depth;
-  jvmtiError _result;
 
 public:
-  VM_SetFramePop(JvmtiEnv *env, JvmtiThreadState* state, jint depth) {
-    _env = env;
-    _state = state;
-    _depth = depth;
-    _result = JVMTI_ERROR_NONE;
+  SetFramePopClosure(JvmtiEnv *env, JvmtiThreadState* state, jint depth)
+    : JvmtiHandshakeClosure("SetFramePopClosure"),
+      _env(env),
+      _state(state),
+      _depth(depth) {}
+  void do_thread(Thread *target) {
+    doit(target, false /* self */);
   }
-  // Nested operation must be allowed for the VM_EnterInterpOnlyMode that is
-  // called from the JvmtiEventControllerPrivate::recompute_thread_enabled.
-  bool allow_nested_vm_operations() const { return true; }
-  VMOp_Type type() const { return VMOp_SetFramePop; }
-  jvmtiError result() { return _result; }
-  void doit();
+  void doit(Thread *target, bool self);
 };
 
-
-// VM operation to get monitor information with stack depth.
-class VM_GetOwnedMonitorInfo : public VM_Operation {
+// HandshakeClosure to get monitor information with stack depth.
+class GetOwnedMonitorInfoClosure : public JvmtiHandshakeClosure {
 private:
-  JvmtiEnv *_env;
   JavaThread* _calling_thread;
-  JavaThread *_java_thread;
-  jvmtiError _result;
+  JvmtiEnv *_env;
   GrowableArray<jvmtiMonitorStackDepthInfo*> *_owned_monitors_list;
 
 public:
-  VM_GetOwnedMonitorInfo(JvmtiEnv* env, JavaThread* calling_thread,
-                                   JavaThread* java_thread,
-                                   GrowableArray<jvmtiMonitorStackDepthInfo*>* owned_monitor_list) {
-    _env = env;
-    _calling_thread = calling_thread;
-    _java_thread = java_thread;
-    _owned_monitors_list = owned_monitor_list;
-    _result = JVMTI_ERROR_NONE;
-  }
-  VMOp_Type type() const { return VMOp_GetOwnedMonitorInfo; }
-  void doit();
-  jvmtiError result() { return _result; }
+  GetOwnedMonitorInfoClosure(JavaThread* calling_thread, JvmtiEnv* env,
+                             GrowableArray<jvmtiMonitorStackDepthInfo*>* owned_monitor_list)
+    : JvmtiHandshakeClosure("GetOwnedMonitorInfo"),
+      _calling_thread(calling_thread),
+      _env(env),
+      _owned_monitors_list(owned_monitor_list) {}
+  void do_thread(Thread *target);
 };
 
-
 // VM operation to get object monitor usage.
 class VM_GetObjectMonitorUsage : public VM_Operation {
 private:
@@ -425,60 +523,64 @@ public:
 
 };
 
-// VM operation to get current contended monitor.
-class VM_GetCurrentContendedMonitor : public VM_Operation {
+// HandshakeClosure to get current contended monitor. It is used for both platform and virtual threads.
+class GetCurrentContendedMonitorClosure : public JvmtiHandshakeClosure {
 private:
-  JvmtiEnv *_env;
   JavaThread *_calling_thread;
-  JavaThread *_java_thread;
+  JvmtiEnv *_env;
   jobject *_owned_monitor_ptr;
-  jvmtiError _result;
+  bool _is_virtual;
 
 public:
-  VM_GetCurrentContendedMonitor(JvmtiEnv *env, JavaThread *calling_thread, JavaThread *java_thread, jobject *mon_ptr) {
-    _env = env;
-    _calling_thread = calling_thread;
-    _java_thread = java_thread;
-    _owned_monitor_ptr = mon_ptr;
-  }
-  VMOp_Type type() const { return VMOp_GetCurrentContendedMonitor; }
-  jvmtiError result() { return _result; }
-  void doit();
+  GetCurrentContendedMonitorClosure(JavaThread* calling_thread, JvmtiEnv *env, jobject *mon_ptr, bool is_virtual)
+    : JvmtiHandshakeClosure("GetCurrentContendedMonitor"),
+      _calling_thread(calling_thread),
+      _env(env),
+      _owned_monitor_ptr(mon_ptr),
+      _is_virtual(is_virtual) {}
+  void do_thread(Thread *target);
 };
 
-// VM operation to get stack trace at safepoint.
-class VM_GetStackTrace : public VM_Operation {
+// HandshakeClosure to get stack trace.
+class GetStackTraceClosure : public JvmtiHandshakeClosure {
 private:
   JvmtiEnv *_env;
-  JavaThread *_java_thread;
   jint _start_depth;
   jint _max_count;
   jvmtiFrameInfo *_frame_buffer;
   jint *_count_ptr;
-  jvmtiError _result;
 
 public:
-  VM_GetStackTrace(JvmtiEnv *env, JavaThread *java_thread,
-                   jint start_depth, jint max_count,
-                   jvmtiFrameInfo* frame_buffer, jint* count_ptr) {
-    _env = env;
-    _java_thread = java_thread;
-    _start_depth = start_depth;
-    _max_count = max_count;
-    _frame_buffer = frame_buffer;
-    _count_ptr = count_ptr;
-  }
-  jvmtiError result() { return _result; }
-  VMOp_Type type() const { return VMOp_GetStackTrace; }
-  void doit();
+  GetStackTraceClosure(JvmtiEnv *env, jint start_depth, jint max_count,
+                       jvmtiFrameInfo* frame_buffer, jint* count_ptr)
+    : JvmtiHandshakeClosure("GetStackTrace"),
+      _env(env),
+      _start_depth(start_depth),
+      _max_count(max_count),
+      _frame_buffer(frame_buffer),
+      _count_ptr(count_ptr) {}
+  void do_thread(Thread *target);
 };
 
-// forward declaration
+#ifdef ASSERT
+// HandshakeClosure to print stack trace in JvmtiVTMSTransitionDisabler error handling.
+class PrintStackTraceClosure : public HandshakeClosure {
+ public:
+  static void do_thread_impl(Thread *target);
+
+  PrintStackTraceClosure()
+      : HandshakeClosure("PrintStackTraceClosure") {}
+  void do_thread(Thread *target);
+};
+#endif
+
+// Forward declaration.
 struct StackInfoNode;
 
-// VM operation to get stack trace at safepoint.
-class VM_GetMultipleStackTraces : public VM_Operation {
+// Get stack trace at safepoint or at direct handshake.
+class MultipleStackTracesCollector {
 private:
+  friend class VM_GetThreadListStackTraces;
   JvmtiEnv *_env;
   jint _max_frame_count;
   jvmtiStackInfo *_stack_info;
@@ -491,100 +593,315 @@ private:
   struct StackInfoNode *head()        { return _head; }
   void set_head(StackInfoNode *head)  { _head = head; }
 
-protected:
+public:
+  MultipleStackTracesCollector(JvmtiEnv *env, jint max_frame_count)
+    : _env(env),
+      _max_frame_count(max_frame_count),
+      _stack_info(NULL),
+      _result(JVMTI_ERROR_NONE),
+      _frame_count_total(0),
+      _head(NULL) {
+  }
   void set_result(jvmtiError result)  { _result = result; }
   void fill_frames(jthread jt, JavaThread *thr, oop thread_oop);
   void allocate_and_fill_stacks(jint thread_count);
-
-public:
-  VM_GetMultipleStackTraces(JvmtiEnv *env, jint max_frame_count) {
-    _env = env;
-    _max_frame_count = max_frame_count;
-    _frame_count_total = 0;
-    _head = NULL;
-    _result = JVMTI_ERROR_NONE;
-  }
-  VMOp_Type type() const             { return VMOp_GetMultipleStackTraces; }
   jvmtiStackInfo *stack_info()       { return _stack_info; }
   jvmtiError result()                { return _result; }
 };
 
 
 // VM operation to get stack trace at safepoint.
-class VM_GetAllStackTraces : public VM_GetMultipleStackTraces {
+class VM_GetAllStackTraces : public VM_Operation {
 private:
   JavaThread *_calling_thread;
   jint _final_thread_count;
+  MultipleStackTracesCollector _collector;
 
 public:
   VM_GetAllStackTraces(JvmtiEnv *env, JavaThread *calling_thread,
                        jint max_frame_count)
-      : VM_GetMultipleStackTraces(env, max_frame_count) {
-    _calling_thread = calling_thread;
+      : _calling_thread(calling_thread),
+        _final_thread_count(0),
+        _collector(env, max_frame_count) {
   }
   VMOp_Type type() const          { return VMOp_GetAllStackTraces; }
   void doit();
   jint final_thread_count()       { return _final_thread_count; }
+  jvmtiStackInfo *stack_info()    { return _collector.stack_info(); }
+  jvmtiError result()             { return _collector.result(); }
 };
 
 // VM operation to get stack trace at safepoint.
-class VM_GetThreadListStackTraces : public VM_GetMultipleStackTraces {
+class VM_GetThreadListStackTraces : public VM_Operation {
 private:
   jint _thread_count;
   const jthread* _thread_list;
+  MultipleStackTracesCollector _collector;
+
+  JvmtiEnvBase *env() { return _collector.env(); }
 
 public:
   VM_GetThreadListStackTraces(JvmtiEnv *env, jint thread_count, const jthread* thread_list, jint max_frame_count)
-      : VM_GetMultipleStackTraces(env, max_frame_count) {
-    _thread_count = thread_count;
-    _thread_list = thread_list;
+      : _thread_count(thread_count),
+        _thread_list(thread_list),
+        _collector(env, max_frame_count) {
   }
   VMOp_Type type() const { return VMOp_GetThreadListStackTraces; }
   void doit();
+  jvmtiStackInfo *stack_info()    { return _collector.stack_info(); }
+  jvmtiError result()             { return _collector.result(); }
 };
 
-
-// VM operation to count stack frames at safepoint.
-class VM_GetFrameCount : public VM_Operation {
+class VM_VirtualThreadGetStackTrace : public VM_Operation {
 private:
   JvmtiEnv *_env;
-  JvmtiThreadState *_state;
+  Handle _vthread_h;
+  jint _start_depth;
+  jint _max_count;
+  jvmtiFrameInfo* _frame_buffer;
+  jint* _count_ptr;
+  jvmtiError _result;
+
+public:
+  VM_VirtualThreadGetStackTrace(JvmtiEnv *env, Handle vthread_h,
+                                jint start_depth, jint max_count,
+                                jvmtiFrameInfo* frame_buffer, jint* count_ptr)
+    : _env(env),
+      _vthread_h(vthread_h),
+      _start_depth(start_depth),
+      _max_count(max_count),
+      _frame_buffer(frame_buffer),
+      _count_ptr(count_ptr),
+      _result(JVMTI_ERROR_NONE)
+  {}
+
+  VMOp_Type type() const { return VMOp_VirtualThreadGetStackTrace; }
+  void doit();
+  jvmtiError result() { return _result; }
+};
+
+class VM_VirtualThreadGetFrameCount : public VM_Operation {
+private:
+  JvmtiEnv *_env;
+  Handle _vthread_h;
+  jint* _count_ptr;
+  jvmtiError _result;
+
+public:
+  VM_VirtualThreadGetFrameCount(JvmtiEnv *env, Handle vthread_h, jint* count_ptr)
+    : _env(env),
+      _vthread_h(vthread_h),
+      _count_ptr(count_ptr),
+      _result(JVMTI_ERROR_NONE)
+  {}
+
+  VMOp_Type type() const { return VMOp_VirtualThreadGetFrameCount; }
+  void doit();
+  jvmtiError result() { return _result; }
+};
+
+// HandshakeClosure to get single stack trace.
+class GetSingleStackTraceClosure : public HandshakeClosure {
+private:
+  JavaThread *_calling_thread;
+  jthread _jthread;
+  MultipleStackTracesCollector _collector;
+
+public:
+  GetSingleStackTraceClosure(JvmtiEnv *env, JavaThread *calling_thread,
+                             jthread thread, jint max_frame_count)
+    : HandshakeClosure("GetSingleStackTrace"),
+      _calling_thread(calling_thread),
+      _jthread(thread),
+      _collector(env, max_frame_count) {
+  }
+  void do_thread(Thread *target);
+  jvmtiStackInfo *stack_info()    { return _collector.stack_info(); }
+  jvmtiError result()             { return _collector.result(); }
+};
+
+// HandshakeClosure to count stack frames.
+class GetFrameCountClosure : public JvmtiHandshakeClosure {
+private:
+  JvmtiEnv *_env;
+  jint *_count_ptr;
+
+public:
+  GetFrameCountClosure(JvmtiEnv *env, jint *count_ptr)
+    : JvmtiHandshakeClosure("GetFrameCount"),
+      _env(env),
+      _count_ptr(count_ptr) {}
+  void do_thread(Thread *target);
+};
+
+// HandshakeClosure to get frame location.
+class GetFrameLocationClosure : public JvmtiHandshakeClosure {
+private:
+  JvmtiEnv *_env;
+  jint _depth;
+  jmethodID* _method_ptr;
+  jlocation* _location_ptr;
+
+public:
+  GetFrameLocationClosure(JvmtiEnv *env, jint depth,
+                          jmethodID* method_ptr, jlocation* location_ptr)
+    : JvmtiHandshakeClosure("GetFrameLocation"),
+      _env(env),
+      _depth(depth),
+      _method_ptr(method_ptr),
+      _location_ptr(location_ptr) {}
+  void do_thread(Thread *target);
+};
+
+// HandshakeClosure to get virtual thread monitor information with stack depth.
+class VirtualThreadGetOwnedMonitorInfoClosure : public HandshakeClosure {
+private:
+  JvmtiEnv *_env;
+  Handle _vthread_h;
+  GrowableArray<jvmtiMonitorStackDepthInfo*> *_owned_monitors_list;
+  jvmtiError _result;
+
+public:
+  VirtualThreadGetOwnedMonitorInfoClosure(JvmtiEnv* env,
+                                          Handle vthread_h,
+                                          GrowableArray<jvmtiMonitorStackDepthInfo*>* owned_monitors_list)
+    : HandshakeClosure("VirtualThreadGetOwnedMonitorInfo"),
+      _env(env),
+      _vthread_h(vthread_h),
+      _owned_monitors_list(owned_monitors_list),
+      _result(JVMTI_ERROR_THREAD_NOT_ALIVE) {}
+
+  void do_thread(Thread *target);
+  jvmtiError result() { return _result; }
+};
+
+// HandshakeClosure to get virtual thread thread at safepoint.
+class VirtualThreadGetThreadClosure : public HandshakeClosure {
+private:
+  Handle _vthread_h;
+  jthread* _carrier_thread_ptr;
+  jvmtiError _result;
+
+public:
+  VirtualThreadGetThreadClosure(Handle vthread_h, jthread* carrier_thread_ptr)
+    : HandshakeClosure("VirtualThreadGetThread"),
+      _vthread_h(vthread_h),
+      _carrier_thread_ptr(carrier_thread_ptr),
+      _result(JVMTI_ERROR_NONE) {}
+
+  void do_thread(Thread *target);
+  jvmtiError result() { return _result; }
+};
+
+// HandshakeClosure to get virtual thread stack trace at safepoint.
+class VirtualThreadGetStackTraceClosure : public HandshakeClosure {
+private:
+  JvmtiEnv *_env;
+  Handle _vthread_h;
+  jint _start_depth;
+  jint _max_count;
+  jvmtiFrameInfo *_frame_buffer;
   jint *_count_ptr;
   jvmtiError _result;
 
 public:
-  VM_GetFrameCount(JvmtiEnv *env, JvmtiThreadState *state, jint *count_ptr) {
-    _env = env;
-    _state = state;
-    _count_ptr = count_ptr;
-  }
-  VMOp_Type type() const { return VMOp_GetFrameCount; }
-  jvmtiError result()    { return _result; }
-  void doit();
+  VirtualThreadGetStackTraceClosure(JvmtiEnv *env, Handle vthread_h,
+                                    jint start_depth, jint max_count,
+                                    jvmtiFrameInfo* frame_buffer, jint* count_ptr)
+    : HandshakeClosure("VirtualThreadGetStackTrace"),
+      _env(env),
+      _vthread_h(vthread_h),
+       _start_depth(start_depth),
+      _max_count(max_count),
+      _frame_buffer(frame_buffer),
+      _count_ptr(count_ptr),
+      _result(JVMTI_ERROR_NONE) {}
+
+  void do_thread(Thread *target);
+  jvmtiError result() { return _result; }
 };
 
-// VM operation to frame location at safepoint.
-class VM_GetFrameLocation : public VM_Operation {
+// HandshakeClosure to count virtual thread stack frames at safepoint.
+class VirtualThreadGetFrameCountClosure : public HandshakeClosure {
 private:
   JvmtiEnv *_env;
-  JavaThread* _java_thread;
+  Handle _vthread_h;
+  jint *_count_ptr;
+  jvmtiError _result;
+
+public:
+  VirtualThreadGetFrameCountClosure(JvmtiEnv *env, Handle vthread_h, jint *count_ptr)
+    : HandshakeClosure("VirtualThreadGetFrameCount"),
+      _env(env), _vthread_h(vthread_h), _count_ptr(count_ptr),
+      _result(JVMTI_ERROR_NONE) {}
+
+  void do_thread(Thread *target);
+  jvmtiError result() { return _result; }
+};
+
+// HandshakeClosure get to virtual thread frame location at safepoint.
+class VirtualThreadGetFrameLocationClosure : public HandshakeClosure {
+private:
+  JvmtiEnv *_env;
+  Handle _vthread_h;
   jint _depth;
   jmethodID* _method_ptr;
   jlocation* _location_ptr;
   jvmtiError _result;
 
 public:
-  VM_GetFrameLocation(JvmtiEnv *env, JavaThread* java_thread, jint depth,
-                      jmethodID* method_ptr, jlocation* location_ptr) {
-    _env = env;
-    _java_thread = java_thread;
-    _depth = depth;
-    _method_ptr = method_ptr;
-    _location_ptr = location_ptr;
+  VirtualThreadGetFrameLocationClosure(JvmtiEnv *env, Handle vthread_h, jint depth,
+                                       jmethodID* method_ptr, jlocation* location_ptr)
+    : HandshakeClosure("VirtualThreadGetFrameLocation"),
+      _env(env),
+      _vthread_h(vthread_h),
+      _depth(depth),
+      _method_ptr(method_ptr),
+      _location_ptr(location_ptr),
+      _result(JVMTI_ERROR_NONE) {}
+
+  void do_thread(Thread *target);
+  jvmtiError result() { return _result; }
+};
+
+// HandshakeClosure to get virtual thread state at safepoint.
+class VirtualThreadGetThreadStateClosure : public HandshakeClosure {
+private:
+  Handle _vthread_h;
+  jint *_state_ptr;
+  jvmtiError _result;
+
+public:
+  VirtualThreadGetThreadStateClosure(Handle vthread_h, jint *state_ptr)
+    : HandshakeClosure("VirtualThreadGetThreadState"),
+      _vthread_h(vthread_h),
+      _state_ptr(state_ptr),
+      _result(JVMTI_ERROR_NONE) {}
+
+  void do_thread(Thread *target);
+  jvmtiError result() { return _result; }
+};
+
+// HandshakeClosure to set frame pop for a virtual thread.
+class VirtualThreadSetFramePopClosure : public JvmtiHandshakeClosure {
+private:
+  JvmtiEnv *_env;
+  Handle _vthread_h;
+  JvmtiThreadState* _state;
+  jint _depth;
+
+public:
+  VirtualThreadSetFramePopClosure(JvmtiEnv *env, Handle vthread_h, JvmtiThreadState* state, jint depth)
+    : JvmtiHandshakeClosure("VirtualThreadSetFramePopClosure"),
+      _env(env),
+      _vthread_h(vthread_h),
+      _state(state),
+      _depth(depth) {}
+
+  void do_thread(Thread *target) {
+    doit(target, false /* self */);
   }
-  VMOp_Type type() const { return VMOp_GetFrameLocation; }
-  jvmtiError result()    { return _result; }
-  void doit();
+  void doit(Thread *target, bool self);
 };
 
 
@@ -621,17 +938,15 @@ class ResourceTracker : public StackObj {
 // Jvmti monitor closure to collect off stack monitors.
 class JvmtiMonitorClosure: public MonitorClosure {
  private:
-  JavaThread *_java_thread;
   JavaThread *_calling_thread;
   GrowableArray<jvmtiMonitorStackDepthInfo*> *_owned_monitors_list;
   jvmtiError _error;
   JvmtiEnvBase *_env;
 
  public:
-  JvmtiMonitorClosure(JavaThread* thread, JavaThread *calling_thread,
+  JvmtiMonitorClosure(JavaThread *calling_thread,
                       GrowableArray<jvmtiMonitorStackDepthInfo*> *owned_monitors,
                       JvmtiEnvBase *env) {
-    _java_thread = thread;
     _calling_thread = calling_thread;
     _owned_monitors_list = owned_monitors;
     _error = JVMTI_ERROR_NONE;
@@ -652,4 +967,4 @@ public:
   jvmtiError get_all_modules(JvmtiEnv* env, jint* module_count_ptr, jobject** modules_ptr);
 };
 
-#endif // SHARE_VM_PRIMS_JVMTIENVBASE_HPP
+#endif // SHARE_PRIMS_JVMTIENVBASE_HPP
