@@ -46,6 +46,7 @@ public:
 };
 
 class CodeBlob;
+enum class CodeBlobType;
 class CodeHeap;
 class JavaThread;
 
@@ -60,9 +61,9 @@ class WhiteBox : public AllStatic {
     Symbol* signature_symbol);
   static const char* lookup_jstring(const char* field_name, oop object);
   static bool lookup_bool(const char* field_name, oop object);
-  static int get_blob_type(const CodeBlob* code);
-  static CodeHeap* get_code_heap(int blob_type);
-  static CodeBlob* allocate_code_blob(int size, int blob_type);
+  static CodeBlobType get_blob_type(const CodeBlob* code);
+  static CodeHeap* get_code_heap(CodeBlobType blob_type);
+  static CodeBlob* allocate_code_blob(int size, CodeBlobType blob_type);
   static int array_bytes_to_length(size_t bytes);
   static void register_methods(JNIEnv* env, jclass wbclass, JavaThread* thread,
     JNINativeMethod* method_array, int method_count);
