@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -27,12 +27,10 @@
 
 #include "jvm_md.h"
 #include "metaprogramming/integralConstant.hpp"
+#include "runtime/osInfo.hpp"
 #include "utilities/exceptions.hpp"
 #include "utilities/ostream.hpp"
 #include "utilities/macros.hpp"
-#ifndef _WINDOWS
-# include <setjmp.h>
-#endif
 #ifdef __APPLE__
 # include <mach/mach_time.h>
 #endif
@@ -40,9 +38,73 @@
 class AgentLibrary;
 class frame;
 
-// os defines the interface to operating system; this includes traditional
-// OS services (time, I/O) as well as other functionality with system-
-// dependent code.
+// Rules for using and implementing methods declared in the "os" class
+// ===================================================================
+//
+// The "os" class defines a number of the interfaces for porting HotSpot
+// to different operating systems. For example, I/O, memory, timing, etc.
+// Note that additional classes such as Semaphore, Mutex, etc., are used for
+// porting specific groups of features.
+//
+// Structure of os*.{cpp, hpp} files
+//
+// - os.hpp
+//
+//   (This file) declares the entire API of the "os" class.
+//
+// - os.inline.hpp
+//
+//   To use any of the inline methods declared in the "os" class, this
+//   header file must be included.
+//
+// - src/hotspot/os/<os>/os_<os>.hpp
+// - src/hotspot/os/posix/os_posix.hpp
+//
+//   These headers declare APIs that should be used only within the
+//   platform-specific source files for that particular OS.
+//
+//   For example, os_linux.hpp declares the os::Linux class, which provides
+//   many methods that can be used by files under os/linux/ and os_cpu/linux_*/
+//
+//   os_posix.hpp can be used by platform-specific files for POSIX-like
+//   OSes such as aix, bsd and linux.
+//
+//   Platform-independent source files should not include these header files
+//   (although sadly there are some rare exceptions ...)
+//
+// - os.cpp
+//
+//   Platform-independent methods of the "os" class are defined
+//   in os.cpp. These are not part of the porting interface, but rather
+//   can be considered as convenience functions for accessing
+//   the porting interface. E.g., os::print_function_and_library_name().
+//
+// The methods declared in os.hpp but not implemented in os.cpp are
+// a part the HotSpot Porting APIs. They must be implemented in one of
+// the following four files:
+//
+// - src/hotspot/os/<os>/os_<os>.inline.hpp
+// - src/hotspot/os_cpu/<os>_<cpu>/os_<os>_<cpu>.inline.hpp
+// - src/hotspot/os/<os>/os_<os>.cpp
+// - src/hotspot/os_cpu/<os>_<cpu>/os_<os>_<cpu>.cpp
+//
+//   The Porting APIs declared as "inline" in os.hpp MUST be
+//   implemented in one of the two .inline.hpp files, depending on
+//   whether the feature is specific to a particular CPU architecture
+//   for this OS. These two files are automatically included by
+//   os.inline.hpp. Platform-independent source files must not include
+//   these two files directly.
+//
+//   If the full definition of an inline method is too complex to fit in a
+//   header file, the actual implementation can be deferred to another
+//   method defined in the .cpp files.
+//
+//   The Porting APIs that are *not* declared as "inline" in os.hpp MUST
+//   be implemented in one of the two .cpp files above. These files
+//   also implement OS-specific APIs such as os::Linux, os::Posix, etc.
+//
+// (Note: on the POSIX-like platforms, some of the Porting APIs are implemented
+// in os_posix.cpp instead).
 
 class Thread;
 class JavaThread;
@@ -186,8 +248,6 @@ class os: AllStatic {
   static jint init_2(void);                    // Called after command line parsing
                                                // and VM ergonomics processing
 
-  // unset environment variable
-  static bool unsetenv(const char* name);
   // Get environ pointer, platform independently
   static char** get_environ();
 
@@ -197,7 +257,7 @@ class os: AllStatic {
   static jlong  javaTimeNanos();
   static void   javaTimeNanos_info(jvmtiTimerInfo *info_ptr);
   static void   javaTimeSystemUTC(jlong &seconds, jlong &nanos);
-  static void   run_periodic_checks();
+  static void   run_periodic_checks(outputStream* st);
 
   // Returns the elapsed time in seconds since the vm started.
   static double elapsedTime();
@@ -248,7 +308,7 @@ class os: AllStatic {
     // which being declared MP when in fact not, is a problem - then
     // the bootstrap routine for the stub generator needs to check
     // the processor count directly and leave the bootstrap routine
-    // in place until called after initialization has ocurred.
+    // in place until called after initialization has occurred.
     return (_processor_count != 1);
   }
 
@@ -284,11 +344,27 @@ class os: AllStatic {
   // Interface for stack banging (predetect possible stack overflow for
   // exception processing)  There are guard pages, and above that shadow
   // pages for stack overflow checking.
-  static bool uses_stack_guard_pages();
-  static bool must_commit_stack_guard_pages();
-  static void map_stack_shadow_pages(address sp);
+  inline static bool uses_stack_guard_pages();
+  inline static bool must_commit_stack_guard_pages();
+  inline static void map_stack_shadow_pages(address sp);
   static bool stack_shadow_pages_available(Thread *thread, const methodHandle& method, address sp);
 
+ private:
+  // Minimum stack size a thread can be created with (allowing
+  // the VM to completely create the thread and enter user code).
+  // The initial values exclude any guard pages (by HotSpot or libc).
+  // set_minimum_stack_sizes() will add the size required for
+  // HotSpot guard pages depending on page size and flag settings.
+  // Libc guard pages are never considered by these values.
+  static size_t _compiler_thread_min_stack_allowed;
+  static size_t _java_thread_min_stack_allowed;
+  static size_t _vm_internal_thread_min_stack_allowed;
+  static size_t _os_min_stack_allowed;
+
+  // Check and sets minimum stack sizes
+  static jint set_minimum_stack_sizes();
+
+ public:
   // Find committed memory region within specified range (start, start + size),
   // return true if found any
   static bool committed_in_range(address start, size_t size, address& committed_start, size_t& committed_size);
@@ -296,7 +372,7 @@ class os: AllStatic {
   // OS interface to Virtual Memory
 
   // Return the default page size.
-  static int    vm_page_size();
+  static int vm_page_size() { return OSInfo::vm_page_size(); }
 
   // The set of page sizes which the VM is allowed to use (may be a subset of
   //  the page sizes actually available on the platform).
@@ -337,10 +413,12 @@ class os: AllStatic {
                                                   const char* base,
                                                   const size_t size);
 
-  static int    vm_allocation_granularity();
+  static int vm_allocation_granularity() { return OSInfo::vm_allocation_granularity(); }
+
+  inline static size_t cds_core_region_alignment();
 
   // Reserves virtual memory.
-  static char*  reserve_memory(size_t bytes, bool executable = false, MEMFLAGS flags = mtOther);
+  static char*  reserve_memory(size_t bytes, bool executable = false, MEMFLAGS flags = mtNone);
 
   // Reserves virtual memory that starts at an address that is aligned to 'alignment'.
   static char*  reserve_memory_aligned(size_t size, size_t alignment, bool executable = false);
@@ -367,10 +445,9 @@ class os: AllStatic {
   // Prints all mappings
   static void print_memory_mappings(outputStream* st);
 
-  // Touch memory pages that cover the memory range from start to end (exclusive)
-  // to make the OS back the memory range with actual memory.
-  // Current implementation may not touch the last page if unaligned addresses
-  // are passed.
+  // Touch memory pages that cover the memory range from start to end
+  // (exclusive) to make the OS back the memory range with actual memory.
+  // Other threads may use the memory range concurrently with pretouch.
   static void   pretouch_memory(void* start, void* end, size_t page_size = vm_page_size());
 
   enum ProtType { MEM_PROT_NONE, MEM_PROT_READ, MEM_PROT_RW, MEM_PROT_RWX };
@@ -442,7 +519,7 @@ class os: AllStatic {
   enum ThreadType {
     vm_thread,
     gc_thread,         // GC thread
-    java_thread,       // Java, CodeCacheSweeper, JVMTIAgent and Service threads.
+    java_thread,       // Java, JVMTIAgent and Service threads.
     compiler_thread,
     watcher_thread,
     asynclog_thread,   // dedicated to flushing logs
@@ -517,14 +594,15 @@ class os: AllStatic {
 
   // run cmd in a separate process and return its exit code; or -1 on failures.
   // Note: only safe to use in fatal error situations.
-  // The "prefer_vfork" argument is only used on POSIX platforms to
-  // indicate whether vfork should be used instead of fork to spawn the
-  // child process (ignored on AIX, which always uses vfork).
-  static int fork_and_exec(const char *cmd, bool prefer_vfork = false);
+  static int fork_and_exec(const char *cmd);
 
-  // Call ::exit() on all platforms but Windows
+  // Call ::exit() on all platforms
   static void exit(int num);
 
+  // Call ::_exit() on all platforms. Similar semantics to die() except we never
+  // want a core dump.
+  static void _exit(int num);
+
   // Terminate the VM, but don't exit the process
   static void shutdown();
 
@@ -542,9 +620,8 @@ class os: AllStatic {
 
   // File i/o operations
   static int open(const char *path, int oflag, int mode);
-  static FILE* open(int fd, const char* mode);
+  static FILE* fdopen(int fd, const char* mode);
   static FILE* fopen(const char* path, const char* mode);
-  static int close(int fd);
   static jlong lseek(int fd, jlong offset, int whence);
   static bool file_exists(const char* file);
   // This function, on Windows, canonicalizes a given path (see os_windows.cpp for details).
@@ -552,8 +629,6 @@ class os: AllStatic {
   // the input pointer.
   static char* native_path(char *path);
   static int ftruncate(int fd, jlong length);
-  static int fsync(int fd);
-  static int available(int fd, jlong *bytes);
   static int get_fileno(FILE* fp);
   static void flockfile(FILE* fp);
   static void funlockfile(FILE* fp);
@@ -564,9 +639,8 @@ class os: AllStatic {
 
   //File i/o operations
 
-  static ssize_t read(int fd, void *buf, unsigned int nBytes);
   static ssize_t read_at(int fd, void *buf, unsigned int nBytes, jlong offset);
-  static size_t write(int fd, const void *buf, unsigned int nBytes);
+  static ssize_t write(int fd, const void *buf, unsigned int nBytes);
 
   // Reading directories.
   static DIR*           opendir(const char* dirname);
@@ -625,6 +699,9 @@ class os: AllStatic {
                                               bool demangle = true,
                                               bool strip_arguments = false);
 
+  // Used only on PPC.
+  inline static void* resolve_function_descriptor(void* p);
+
   // Find out whether the pc is in the static code for jvm.dll/libjvm.so.
   static bool address_is_in_vm(address addr);
 
@@ -680,12 +757,17 @@ class os: AllStatic {
   static void print_dll_info(outputStream* st);
   static void print_environment_variables(outputStream* st, const char** env_list);
   static void print_context(outputStream* st, const void* context);
+  static void print_tos_pc(outputStream* st, const void* context);
+  static void print_tos(outputStream* st, address sp);
+  static void print_instructions(outputStream* st, address pc, int unitsize);
   static void print_register_info(outputStream* st, const void* context);
   static bool signal_sent_by_kill(const void* siginfo);
   static void print_siginfo(outputStream* st, const void* siginfo);
   static void print_signal_handlers(outputStream* st, char* buf, size_t buflen);
   static void print_date_and_time(outputStream* st, char* buf, size_t buflen);
-  static void print_instructions(outputStream* st, address pc, int unitsize);
+
+  static void print_user_info(outputStream* st);
+  static void print_active_locale(outputStream* st);
 
   // helper for output of seconds in days , hours and months
   static void print_dhm(outputStream* st, const char* startStr, long sec);
@@ -785,15 +867,7 @@ class os: AllStatic {
   // Like strdup, but exit VM when strdup() returns NULL
   static char* strdup_check_oom(const char*, MEMFLAGS flags = mtInternal);
 
-#ifndef PRODUCT
-  static julong num_mallocs;         // # of calls to malloc/realloc
-  static julong alloc_bytes;         // # of bytes allocated
-  static julong num_frees;           // # of calls to free
-  static julong free_bytes;          // # of bytes freed
-#endif
-
   // SocketInterface (ex HPI SocketInterface )
-  static int socket(int domain, int type, int protocol);
   static int socket_close(int fd);
   static int recv(int fd, char* buf, size_t nBytes, uint flags);
   static int send(int fd, char* buf, size_t nBytes, uint flags);
@@ -869,30 +943,59 @@ class os: AllStatic {
   static bool supports_map_sync();
 
  public:
-  class CrashProtectionCallback : public StackObj {
-  public:
-    virtual void call() = 0;
-  };
 
-  // Platform dependent stuff
+  // File conventions
+  static const char* file_separator();
+  static const char* line_separator();
+  static const char* path_separator();
+
+  // Information about the protection of the page at address '0' on this os.
+  inline static bool zero_page_read_protected();
+
+  static void setup_fpu();
+  static bool supports_sse();
+  static juint cpu_microcode_revision();
+
+  static inline jlong rdtsc();
+
+  // Used to register dynamic code cache area with the OS
+  // Note: Currently only used in 64 bit Windows implementations
+  inline static bool register_code_area(char *low, char *high);
+
+  // Platform-specific code for interacting with individual OSes.
+  // TODO: This is for compatibility only with current usage of os::Linux, etc.
+  // We can get rid of the following block if we rename such a class to something
+  // like ::LinuxUtils
+#if defined(AIX)
+  class Aix;
+#elif defined(BSD)
+  class Bsd;
+#elif defined(LINUX)
+  class Linux;
+#elif defined(_WINDOWS)
+  class win32;
+#endif
+
+  // Ditto - Posix-specific API. Ideally should be moved to something like ::PosixUtils.
 #ifndef _WINDOWS
-# include "os_posix.hpp"
+  class Posix;
+#endif
+
+  // FIXME - some random stuff that was in os_windows.hpp
+#ifdef _WINDOWS
+  // strtok_s is the Windows thread-safe equivalent of POSIX strtok_r
+# define strtok_r strtok_s
+# define S_ISCHR(mode)   (((mode) & _S_IFCHR) == _S_IFCHR)
+# define S_ISFIFO(mode)  (((mode) & _S_IFIFO) == _S_IFIFO)
 #endif
-#include OS_CPU_HEADER(os)
-#include OS_HEADER(os)
 
 #ifndef OS_NATIVE_THREAD_CREATION_FAILED_MSG
 #define OS_NATIVE_THREAD_CREATION_FAILED_MSG "unable to create native thread: possibly out of memory or process/resource limits reached"
 #endif
 
  public:
-#ifndef PLATFORM_PRINT_NATIVE_STACK
-  // No platform-specific code for printing the native stack.
-  static bool platform_print_native_stack(outputStream* st, const void* context,
-                                          char *buf, int buf_size) {
-    return false;
-  }
-#endif
+  inline static bool platform_print_native_stack(outputStream* st, const void* context,
+                                                 char *buf, int buf_size);
 
   // debugging support (mostly used by debug.cpp but also fatal error handler)
   static bool find(address pc, outputStream* st = tty); // OS specific function to make sense out of an address
@@ -921,104 +1024,11 @@ class os: AllStatic {
   static char*  build_agent_function_name(const char *sym, const char *cname,
                                           bool is_absolute_path);
 
-  class SuspendedThreadTaskContext {
-  public:
-    SuspendedThreadTaskContext(Thread* thread, void *ucontext) : _thread(thread), _ucontext(ucontext) {}
-    Thread* thread() const { return _thread; }
-    void* ucontext() const { return _ucontext; }
-  private:
-    Thread* _thread;
-    void* _ucontext;
-  };
-
-  class SuspendedThreadTask {
-  public:
-    SuspendedThreadTask(Thread* thread) : _thread(thread), _done(false) {}
-    void run();
-    bool is_done() { return _done; }
-    virtual void do_task(const SuspendedThreadTaskContext& context) = 0;
-  protected:
-    ~SuspendedThreadTask() {}
-  private:
-    void internal_do_task();
-    Thread* _thread;
-    bool _done;
-  };
-
 #if defined(__APPLE__) && defined(AARCH64)
   // Enables write or execute access to writeable and executable pages.
   static void current_thread_enable_wx(WXMode mode);
 #endif // __APPLE__ && AARCH64
 
-#ifndef _WINDOWS
-  // Suspend/resume support
-  // Protocol:
-  //
-  // a thread starts in SR_RUNNING
-  //
-  // SR_RUNNING can go to
-  //   * SR_SUSPEND_REQUEST when the WatcherThread wants to suspend it
-  // SR_SUSPEND_REQUEST can go to
-  //   * SR_RUNNING if WatcherThread decides it waited for SR_SUSPENDED too long (timeout)
-  //   * SR_SUSPENDED if the stopped thread receives the signal and switches state
-  // SR_SUSPENDED can go to
-  //   * SR_WAKEUP_REQUEST when the WatcherThread has done the work and wants to resume
-  // SR_WAKEUP_REQUEST can go to
-  //   * SR_RUNNING when the stopped thread receives the signal
-  //   * SR_WAKEUP_REQUEST on timeout (resend the signal and try again)
-  class SuspendResume {
-   public:
-    enum State {
-      SR_RUNNING,
-      SR_SUSPEND_REQUEST,
-      SR_SUSPENDED,
-      SR_WAKEUP_REQUEST
-    };
-
-  private:
-    volatile State _state;
-
-  private:
-    /* try to switch state from state "from" to state "to"
-     * returns the state set after the method is complete
-     */
-    State switch_state(State from, State to);
-
-  public:
-    SuspendResume() : _state(SR_RUNNING) { }
-
-    State state() const { return _state; }
-
-    State request_suspend() {
-      return switch_state(SR_RUNNING, SR_SUSPEND_REQUEST);
-    }
-
-    State cancel_suspend() {
-      return switch_state(SR_SUSPEND_REQUEST, SR_RUNNING);
-    }
-
-    State suspended() {
-      return switch_state(SR_SUSPEND_REQUEST, SR_SUSPENDED);
-    }
-
-    State request_wakeup() {
-      return switch_state(SR_SUSPENDED, SR_WAKEUP_REQUEST);
-    }
-
-    State running() {
-      return switch_state(SR_WAKEUP_REQUEST, SR_RUNNING);
-    }
-
-    bool is_running() const {
-      return _state == SR_RUNNING;
-    }
-
-    bool is_suspended() const {
-      return _state == SR_SUSPENDED;
-    }
-  };
-#endif // !WINDOWS
-
  protected:
   static volatile unsigned int _rand_seed;    // seed for random number generator
   static int _processor_count;                // number of processors
