@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -29,10 +29,9 @@
 #include "oops/markWord.hpp"
 #include "runtime/basicLock.hpp"
 #include "runtime/handles.hpp"
-#include "utilities/growableArray.hpp"
-#include "utilities/linkedlist.hpp"
 #include "utilities/resourceHash.hpp"
 
+template <typename T> class GrowableArray;
 class LogStream;
 class ObjectMonitor;
 class ThreadsList;
@@ -49,15 +48,13 @@ class ObjectMonitorsHashtable {
   }
 
  public:
-  typedef LinkedListImpl<ObjectMonitor*,
-                         ResourceObj::C_HEAP, mtThread,
-                         AllocFailStrategy::RETURN_NULL> PtrList;
+  class PtrList;
 
+ private:
   // ResourceHashtable SIZE is specified at compile time so we
   // use 1031 which is the first prime after 1024.
   typedef ResourceHashtable<void*, PtrList*, 1031, ResourceObj::C_HEAP, mtThread,
                             &ObjectMonitorsHashtable::ptr_hash> PtrTable;
- private:
   PtrTable* _ptrs;
   size_t _key_count;
   size_t _om_count;
@@ -66,7 +63,7 @@ class ObjectMonitorsHashtable {
   // ResourceHashtable is passed to various functions and populated in
   // different places so we allocate it using C_HEAP to make it immune
   // from any ResourceMarks that happen to be in the code paths.
-  ObjectMonitorsHashtable() : _ptrs(new (ResourceObj::C_HEAP, mtThread) PtrTable()), _key_count(0), _om_count(0) {}
+  ObjectMonitorsHashtable() : _ptrs(new (ResourceObj::C_HEAP, mtThread) PtrTable), _key_count(0), _om_count(0) {}
 
   ~ObjectMonitorsHashtable();
 
