@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -36,10 +36,14 @@
 #include "interpreter/oopMapCache.hpp"
 #include "memory/resourceArea.hpp"
 #include "oops/instanceKlass.hpp"
+#include "oops/method.inline.hpp"
 #include "oops/oop.inline.hpp"
+#include "oops/stackChunkOop.hpp"
 #include "prims/jvmtiExport.hpp"
 #include "runtime/frame.inline.hpp"
+#include "runtime/globals.hpp"
 #include "runtime/handles.inline.hpp"
+#include "runtime/javaThread.inline.hpp"
 #include "runtime/objectMonitor.hpp"
 #include "runtime/objectMonitor.inline.hpp"
 #include "runtime/osThread.hpp"
@@ -47,21 +51,26 @@
 #include "runtime/stackFrameStream.inline.hpp"
 #include "runtime/stubRoutines.hpp"
 #include "runtime/synchronizer.hpp"
-#include "runtime/thread.inline.hpp"
 #include "runtime/vframe.inline.hpp"
 #include "runtime/vframeArray.hpp"
 #include "runtime/vframe_hp.hpp"
 
 vframe::vframe(const frame* fr, const RegisterMap* reg_map, JavaThread* thread)
-: _reg_map(reg_map), _thread(thread) {
+: _reg_map(reg_map), _thread(thread),
+  _chunk(Thread::current(), reg_map->stack_chunk()()) {
   assert(fr != NULL, "must have frame");
   _fr = *fr;
 }
 
 vframe::vframe(const frame* fr, JavaThread* thread)
-: _reg_map(thread), _thread(thread) {
+: _reg_map(thread,
+           RegisterMap::UpdateMap::include,
+           RegisterMap::ProcessFrames::include,
+           RegisterMap::WalkContinuation::skip),
+  _thread(thread), _chunk() {
   assert(fr != NULL, "must have frame");
   _fr = *fr;
+  assert(!_reg_map.in_cont(), "");
 }
 
 vframe* vframe::new_vframe(StackFrameStream& fst, JavaThread* thread) {
@@ -106,12 +115,17 @@ vframe* vframe::new_vframe(const frame* f, const RegisterMap* reg_map, JavaThrea
 vframe* vframe::sender() const {
   RegisterMap temp_map = *register_map();
   assert(is_top(), "just checking");
+  if (_fr.is_empty()) return NULL;
   if (_fr.is_entry_frame() && _fr.is_first_frame()) return NULL;
   frame s = _fr.real_sender(&temp_map);
   if (s.is_first_frame()) return NULL;
   return vframe::new_vframe(&s, &temp_map, thread());
 }
 
+bool vframe::is_vthread_entry() const {
+  return _fr.is_first_vthread_frame(register_map()->thread());
+}
+
 vframe* vframe::top() const {
   vframe* vf = (vframe*) this;
   while (!vf->is_top()) vf = vf->sender();
@@ -122,7 +136,9 @@ vframe* vframe::top() const {
 javaVFrame* vframe::java_sender() const {
   vframe* f = sender();
   while (f != NULL) {
-    if (f->is_java_frame()) return javaVFrame::cast(f);
+    if (f->is_vthread_entry()) break;
+    if (f->is_java_frame() && !javaVFrame::cast(f)->method()->is_continuation_enter_intrinsic())
+      return javaVFrame::cast(f);
     f = f->sender();
   }
   return NULL;
@@ -215,9 +231,8 @@ void javaVFrame::print_lock_info_on(outputStream* st, int frame_count) {
       Klass* k = obj->klass();
       st->print_cr("\t- %s <" INTPTR_FORMAT "> (a %s)", "parking to wait for ", p2i(obj), k->external_name());
     }
-    else if (thread()->osthread()->get_state() == OBJECT_WAIT) {
-      // We are waiting on an Object monitor but Object.wait() isn't the
-      // top-frame, so we should be waiting on a Class initialization monitor.
+    else if (thread()->osthread()->get_state() == CONDVAR_WAIT) {
+      // We are waiting on the native class initialization monitor.
       InstanceKlass* k = thread()->class_to_be_initialized();
       if (k != NULL) {
         st->print_cr("\t- waiting on the Class initialization monitor for %s", k->external_name());
@@ -277,14 +292,16 @@ void javaVFrame::print_lock_info_on(outputStream* st, int frame_count) {
 // ------------- interpretedVFrame --------------
 
 u_char* interpretedVFrame::bcp() const {
-  return fr().interpreter_frame_bcp();
+  return stack_chunk() == NULL ? fr().interpreter_frame_bcp() : stack_chunk()->interpreter_frame_bcp(fr());
 }
 
 void interpretedVFrame::set_bcp(u_char* bcp) {
+  assert(stack_chunk() == NULL, "Not supported for heap frames"); // unsupported for now because seems to be unused
   fr().interpreter_frame_set_bcp(bcp);
 }
 
 intptr_t* interpretedVFrame::locals_addr_at(int offset) const {
+  assert(stack_chunk() == NULL, "Not supported for heap frames"); // unsupported for now because seems to be unused
   assert(fr().is_interpreted_frame(), "frame should be an interpreted frame");
   return fr().interpreter_frame_local_at(offset);
 }
@@ -292,10 +309,12 @@ intptr_t* interpretedVFrame::locals_addr_at(int offset) const {
 
 GrowableArray<MonitorInfo*>* interpretedVFrame::monitors() const {
   GrowableArray<MonitorInfo*>* result = new GrowableArray<MonitorInfo*>(5);
-  for (BasicObjectLock* current = (fr().previous_monitor_in_interpreter_frame(fr().interpreter_frame_monitor_begin()));
-       current >= fr().interpreter_frame_monitor_end();
-       current = fr().previous_monitor_in_interpreter_frame(current)) {
-    result->push(new MonitorInfo(current->obj(), current->lock(), false, false));
+  if (stack_chunk() == NULL) { // no monitors in continuations
+    for (BasicObjectLock* current = (fr().previous_monitor_in_interpreter_frame(fr().interpreter_frame_monitor_begin()));
+        current >= fr().interpreter_frame_monitor_end();
+        current = fr().previous_monitor_in_interpreter_frame(current)) {
+      result->push(new MonitorInfo(current->obj(), current->lock(), false, false));
+    }
   }
   return result;
 }
@@ -305,20 +324,29 @@ int interpretedVFrame::bci() const {
 }
 
 Method* interpretedVFrame::method() const {
-  return fr().interpreter_frame_method();
+  return stack_chunk() == NULL ? fr().interpreter_frame_method() : stack_chunk()->interpreter_frame_method(fr());
 }
 
 static StackValue* create_stack_value_from_oop_map(const InterpreterOopMap& oop_mask,
                                                    int index,
-                                                   const intptr_t* const addr) {
+                                                   const intptr_t* const addr,
+                                                   stackChunkOop chunk) {
 
   assert(index >= 0 &&
          index < oop_mask.number_of_entries(), "invariant");
 
   // categorize using oop_mask
   if (oop_mask.is_oop(index)) {
+    oop obj = NULL;
+    if (addr != NULL) {
+      if (chunk != NULL) {
+        obj = (chunk->has_bitmap() && UseCompressedOops) ? (oop)HeapAccess<>::oop_load((narrowOop*)addr) : HeapAccess<>::oop_load((oop*)addr);
+      } else {
+        obj = *(oop*)addr;
+      }
+    }
     // reference (oop) "r"
-    Handle h(Thread::current(), addr != NULL ? (*(oop*)addr) : (oop)NULL);
+    Handle h(Thread::current(), obj);
     return new StackValue(h);
   }
   // value (integer) "v"
@@ -328,7 +356,7 @@ static StackValue* create_stack_value_from_oop_map(const InterpreterOopMap& oop_
 static bool is_in_expression_stack(const frame& fr, const intptr_t* const addr) {
   assert(addr != NULL, "invariant");
 
-  // Ensure to be 'inside' the expresion stack (i.e., addr >= sp for Intel).
+  // Ensure to be 'inside' the expression stack (i.e., addr >= sp for Intel).
   // In case of exceptions, the expression stack is invalid and the sp
   // will be reset to express this condition.
   if (frame::interpreter_frame_expression_stack_direction() > 0) {
@@ -341,16 +369,22 @@ static bool is_in_expression_stack(const frame& fr, const intptr_t* const addr)
 static void stack_locals(StackValueCollection* result,
                          int length,
                          const InterpreterOopMap& oop_mask,
-                         const frame& fr) {
+                         const frame& fr,
+                         const stackChunkOop chunk) {
 
   assert(result != NULL, "invariant");
 
   for (int i = 0; i < length; ++i) {
-    const intptr_t* const addr = fr.interpreter_frame_local_at(i);
+    const intptr_t* addr;
+    if (chunk == NULL) {
+      addr = fr.interpreter_frame_local_at(i);
+      assert(addr >= fr.sp(), "must be inside the frame");
+    } else {
+      addr = chunk->interpreter_frame_local_at(fr, i);
+    }
     assert(addr != NULL, "invariant");
-    assert(addr >= fr.sp(), "must be inside the frame");
 
-    StackValue* const sv = create_stack_value_from_oop_map(oop_mask, i, addr);
+    StackValue* const sv = create_stack_value_from_oop_map(oop_mask, i, addr, chunk);
     assert(sv != NULL, "sanity check");
 
     result->add(sv);
@@ -361,21 +395,28 @@ static void stack_expressions(StackValueCollection* result,
                               int length,
                               int max_locals,
                               const InterpreterOopMap& oop_mask,
-                              const frame& fr) {
+                              const frame& fr,
+                              const stackChunkOop chunk) {
 
   assert(result != NULL, "invariant");
 
   for (int i = 0; i < length; ++i) {
-    const intptr_t* addr = fr.interpreter_frame_expression_stack_at(i);
-    assert(addr != NULL, "invariant");
-    if (!is_in_expression_stack(fr, addr)) {
-      // Need to ensure no bogus escapes.
-      addr = NULL;
+    const intptr_t* addr;
+    if (chunk == NULL) {
+      addr = fr.interpreter_frame_expression_stack_at(i);
+      assert(addr != NULL, "invariant");
+      if (!is_in_expression_stack(fr, addr)) {
+        // Need to ensure no bogus escapes.
+        addr = NULL;
+      }
+    } else {
+      addr = chunk->interpreter_frame_expression_stack_at(fr, i);
     }
 
     StackValue* const sv = create_stack_value_from_oop_map(oop_mask,
                                                            i + max_locals,
-                                                           addr);
+                                                           addr,
+                                                           chunk);
     assert(sv != NULL, "sanity check");
 
     result->add(sv);
@@ -424,9 +465,9 @@ StackValueCollection* interpretedVFrame::stack_data(bool expressions) const {
   }
 
   if (expressions) {
-    stack_expressions(result, length, max_locals, oop_mask, fr());
+    stack_expressions(result, length, max_locals, oop_mask, fr(), stack_chunk());
   } else {
-    stack_locals(result, length, oop_mask, fr());
+    stack_locals(result, length, oop_mask, fr(), stack_chunk());
   }
 
   assert(length == result->size(), "invariant");
@@ -492,15 +533,54 @@ void vframeStreamCommon::found_bad_method_frame() const {
 
 // top-frame will be skipped
 vframeStream::vframeStream(JavaThread* thread, frame top_frame,
-                           bool stop_at_java_call_stub) :
-    vframeStreamCommon(thread, true /* process_frames */) {
+                          bool stop_at_java_call_stub) :
+    vframeStreamCommon(RegisterMap(thread,
+                                   RegisterMap::UpdateMap::include,
+                                   RegisterMap::ProcessFrames::include,
+                                   RegisterMap::WalkContinuation::include)) {
   _stop_at_java_call_stub = stop_at_java_call_stub;
 
   // skip top frame, as it may not be at safepoint
-  _prev_frame = top_frame;
   _frame  = top_frame.sender(&_reg_map);
   while (!fill_from_frame()) {
-    _prev_frame = _frame;
+    _frame = _frame.sender(&_reg_map);
+  }
+}
+
+vframeStream::vframeStream(JavaThread* thread, Handle continuation_scope, bool stop_at_java_call_stub)
+ : vframeStreamCommon(RegisterMap(thread,
+                                  RegisterMap::UpdateMap::include,
+                                  RegisterMap::ProcessFrames::include,
+                                  RegisterMap::WalkContinuation::include)) {
+
+  _stop_at_java_call_stub = stop_at_java_call_stub;
+  _continuation_scope = continuation_scope;
+
+  if (!thread->has_last_Java_frame()) {
+    _mode = at_end_mode;
+    return;
+  }
+
+  _frame = _thread->last_frame();
+  _cont_entry = _thread->last_continuation();
+  while (!fill_from_frame()) {
+    _frame = _frame.sender(&_reg_map);
+  }
+}
+
+vframeStream::vframeStream(oop continuation, Handle continuation_scope)
+ : vframeStreamCommon(RegisterMap(continuation, RegisterMap::UpdateMap::include)) {
+
+  _stop_at_java_call_stub = false;
+  _continuation_scope = continuation_scope;
+
+  if (!Continuation::has_last_Java_frame(continuation, &_frame, &_reg_map)) {
+    _mode = at_end_mode;
+    return;
+  }
+
+  // _chunk = _reg_map.stack_chunk();
+  while (!fill_from_frame()) {
     _frame = _frame.sender(&_reg_map);
   }
 }
@@ -573,32 +653,20 @@ void vframeStreamCommon::skip_prefixed_method_and_wrappers() {
 
 javaVFrame* vframeStreamCommon::asJavaVFrame() {
   javaVFrame* result = NULL;
-  if (_mode == compiled_mode) {
-    compiledVFrame* cvf;
-    if (_frame.is_native_frame()) {
-      cvf = compiledVFrame::cast(vframe::new_vframe(&_frame, &_reg_map, _thread));
-      assert(cvf->cb() == cb(), "wrong code blob");
-    } else {
-      assert(_frame.is_compiled_frame(), "expected compiled Java frame");
+  // FIXME, need to re-do JDK-8271140 and check is_native_frame?
+  if (_mode == compiled_mode && _frame.is_compiled_frame()) {
+    assert(_frame.is_compiled_frame() || _frame.is_native_frame(), "expected compiled Java frame");
+    guarantee(_reg_map.update_map(), "");
 
-      // lazy update to register map
-      bool update_map = true;
-      RegisterMap map(_thread, update_map);
-      frame f = _prev_frame.sender(&map);
+    compiledVFrame* cvf = compiledVFrame::cast(vframe::new_vframe(&_frame, &_reg_map, _thread));
 
-      assert(f.is_compiled_frame(), "expected compiled Java frame");
+    guarantee(cvf->cb() == cb(), "wrong code blob");
 
-      cvf = compiledVFrame::cast(vframe::new_vframe(&f, &map, _thread));
+    cvf = cvf->at_scope(_decode_offset, _vframe_id); // get the same scope as this stream
 
-      assert(cvf->cb() == cb(), "wrong code blob");
-
-      // get the same scope as this stream
-      cvf = cvf->at_scope(_decode_offset, _vframe_id);
-
-      assert(cvf->scope()->decode_offset() == _decode_offset, "wrong scope");
-      assert(cvf->scope()->sender_decode_offset() == _sender_decode_offset, "wrong scope");
-    }
-    assert(cvf->vframe_id() == _vframe_id, "wrong vframe");
+    guarantee(cvf->scope()->decode_offset() == _decode_offset, "wrong scope");
+    guarantee(cvf->scope()->sender_decode_offset() == _sender_decode_offset, "wrong scope");
+    guarantee(cvf->vframe_id() == _vframe_id, "wrong vframe");
 
     result = cvf;
   } else {
@@ -608,13 +676,11 @@ javaVFrame* vframeStreamCommon::asJavaVFrame() {
   return result;
 }
 
-
 #ifndef PRODUCT
 void vframe::print() {
   if (WizardMode) _fr.print_value_on(tty,NULL);
 }
 
-
 void vframe::print_value() const {
   ((vframe*)this)->print();
 }
@@ -626,7 +692,7 @@ void entryVFrame::print_value() const {
 
 void entryVFrame::print() {
   vframe::print();
-  tty->print_cr("C Chunk inbetween Java");
+  tty->print_cr("C Chunk in between Java");
   tty->print_cr("C     link " INTPTR_FORMAT, p2i(_fr.link()));
 }
 
@@ -703,7 +769,7 @@ void javaVFrame::print_value() const {
   // Check frame size and print warning if it looks suspiciously large
   if (fr().sp() != NULL) {
     RegisterMap map = *register_map();
-    uint size = fr().frame_size(&map);
+    uint size = fr().frame_size();
 #ifdef _LP64
     if (size > 8*K) warning("SUSPICIOUSLY LARGE FRAME (%d)", size);
 #else
