@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -45,10 +45,11 @@
 #include "runtime/frame.inline.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/javaCalls.hpp"
+#include "runtime/javaThread.inline.hpp"
 #include "runtime/jniHandles.hpp"
 #include "runtime/os.hpp"
 #include "runtime/reflectionUtils.hpp"
-#include "runtime/thread.inline.hpp"
+#include "runtime/threads.hpp"
 #include "runtime/threadSMR.hpp"
 #include "runtime/vframe.hpp"
 #include "runtime/vmOperations.hpp"
@@ -409,6 +410,9 @@ class AbstractDumpWriter : public StackObj {
 
   // Returns true if we have enough room in the buffer for 'len' bytes.
   bool can_write_fast(size_t len);
+
+  void write_address(address a);
+
  public:
   AbstractDumpWriter() :
     _buffer(NULL),
@@ -428,6 +432,7 @@ class AbstractDumpWriter : public StackObj {
   void write_u4(u4 x);
   void write_u8(u8 x);
   void write_objectID(oop o);
+  void write_rootID(oop* p);
   void write_symbolID(Symbol* o);
   void write_classID(Klass* k);
   void write_id(u4 x);
@@ -510,8 +515,7 @@ void AbstractDumpWriter::write_u8(u8 x) {
   WRITE_KNOWN_TYPE(&v, 8);
 }
 
-void AbstractDumpWriter::write_objectID(oop o) {
-  address a = cast_from_oop<address>(o);
+void AbstractDumpWriter::write_address(address a) {
 #ifdef _LP64
   write_u8((u8)a);
 #else
@@ -519,13 +523,16 @@ void AbstractDumpWriter::write_objectID(oop o) {
 #endif
 }
 
+void AbstractDumpWriter::write_objectID(oop o) {
+  write_address(cast_from_oop<address>(o));
+}
+
+void AbstractDumpWriter::write_rootID(oop* p) {
+  write_address((address)p);
+}
+
 void AbstractDumpWriter::write_symbolID(Symbol* s) {
-  address a = (address)((uintptr_t)s);
-#ifdef _LP64
-  write_u8((u8)a);
-#else
-  write_u4((u4)a);
-#endif
+  write_address((address)((uintptr_t)s));
 }
 
 void AbstractDumpWriter::write_id(u4 x) {
@@ -576,12 +583,12 @@ void AbstractDumpWriter::start_sub_record(u1 tag, u4 len) {
     // Will be fixed up later if we add more sub-records.  If this is a huge sub-record,
     // this is already the correct length, since we don't add more sub-records.
     write_u4(len);
-    assert(Bytes::get_Java_u4((address)(buffer() + 5)) == len, "Inconsitent size!");
+    assert(Bytes::get_Java_u4((address)(buffer() + 5)) == len, "Inconsistent size!");
     _in_dump_segment = true;
     _is_huge_sub_record = len > buffer_size() - dump_segment_header_size;
   } else if (_is_huge_sub_record || (len > buffer_size() - position())) {
     // This object will not fit in completely or the last sub-record was huge.
-    // Finish the current segement and try again.
+    // Finish the current segment and try again.
     finish_dump_segment();
     start_sub_record(tag, len);
 
@@ -664,7 +671,7 @@ class ParWriterBufferQueue : public CHeapObj<mtInternal> {
       _tail = entry;
     }
     _length++;
-    assert(_tail->_next == NULL, "Bufer queue is polluted");
+    assert(_tail->_next == NULL, "Buffer queue is polluted");
   }
 
   ParWriterBufferQueueElem* dequeue() {
@@ -912,12 +919,10 @@ class DumperSupport : AllStatic {
   static void dump_instance_field_descriptors(AbstractDumpWriter* writer, Klass* k);
   // creates HPROF_GC_INSTANCE_DUMP record for the given object
   static void dump_instance(AbstractDumpWriter* writer, oop o);
-  // creates HPROF_GC_CLASS_DUMP record for the given class and each of its
-  // array classes
-  static void dump_class_and_array_classes(AbstractDumpWriter* writer, Klass* k);
-  // creates HPROF_GC_CLASS_DUMP record for a given primitive array
-  // class (and each multi-dimensional array class too)
-  static void dump_basic_type_array_class(AbstractDumpWriter* writer, Klass* k);
+  // creates HPROF_GC_CLASS_DUMP record for the given instance class
+  static void dump_instance_class(AbstractDumpWriter* writer, Klass* k);
+  // creates HPROF_GC_CLASS_DUMP record for a given array class
+  static void dump_array_class(AbstractDumpWriter* writer, Klass* k);
 
   // creates HPROF_GC_OBJ_ARRAY_DUMP record for the given object array
   static void dump_object_array(AbstractDumpWriter* writer, objArrayOop array);
@@ -1128,14 +1133,6 @@ u4 DumperSupport::get_static_fields_size(InstanceKlass* ik, u2& field_count) {
     }
   }
 
-  // Also provide a pointer to the init_lock if present, so there aren't unreferenced int[0]
-  // arrays.
-  oop init_lock = ik->init_lock();
-  if (init_lock != NULL) {
-    field_count++;
-    size += sizeof(address);
-  }
-
   // We write the value itself plus a name and a one byte type tag per field.
   return size + field_count * (sizeof(address) + 1);
 }
@@ -1173,14 +1170,6 @@ void DumperSupport::dump_static_fields(AbstractDumpWriter* writer, Klass* k) {
       prev = prev->previous_versions();
     }
   }
-
-  // Add init lock to the end if the class is not yet initialized
-  oop init_lock = ik->init_lock();
-  if (init_lock != NULL) {
-    writer->write_symbolID(vmSymbols::init_lock_name());         // name
-    writer->write_u1(sig2tag(vmSymbols::int_array_signature())); // type
-    writer->write_objectID(init_lock);
-  }
 }
 
 // dump the raw values of the instance fields of the given object
@@ -1243,9 +1232,8 @@ void DumperSupport::dump_instance(AbstractDumpWriter* writer, oop o) {
   writer->end_sub_record();
 }
 
-// creates HPROF_GC_CLASS_DUMP record for the given class and each of
-// its array classes
-void DumperSupport::dump_class_and_array_classes(AbstractDumpWriter* writer, Klass* k) {
+// creates HPROF_GC_CLASS_DUMP record for the given instance class
+void DumperSupport::dump_instance_class(AbstractDumpWriter* writer, Klass* k) {
   InstanceKlass* ik = InstanceKlass::cast(k);
 
   // We can safepoint and do a heap dump at a point where we have a Klass,
@@ -1298,73 +1286,42 @@ void DumperSupport::dump_class_and_array_classes(AbstractDumpWriter* writer, Kla
   dump_instance_field_descriptors(writer, ik);
 
   writer->end_sub_record();
-
-  // array classes
-  k = ik->array_klass_or_null();
-  while (k != NULL) {
-    assert(k->is_objArray_klass(), "not an ObjArrayKlass");
-
-    u4 size = 1 + sizeof(address) + 4 + 6 * sizeof(address) + 4 + 2 + 2 + 2;
-    writer->start_sub_record(HPROF_GC_CLASS_DUMP, size);
-    writer->write_classID(k);
-    writer->write_u4(STACK_TRACE_ID);
-
-    // super class of array classes is java.lang.Object
-    java_super = k->java_super();
-    assert(java_super != NULL, "checking");
-    writer->write_classID(java_super);
-
-    writer->write_objectID(ik->class_loader());
-    writer->write_objectID(ik->signers());
-    writer->write_objectID(ik->protection_domain());
-
-    writer->write_objectID(oop(NULL));    // reserved
-    writer->write_objectID(oop(NULL));
-    writer->write_u4(0);             // instance size
-    writer->write_u2(0);             // constant pool
-    writer->write_u2(0);             // static fields
-    writer->write_u2(0);             // instance fields
-
-    writer->end_sub_record();
-
-    // get the array class for the next rank
-    k = k->array_klass_or_null();
-  }
 }
 
-// creates HPROF_GC_CLASS_DUMP record for a given primitive array
-// class (and each multi-dimensional array class too)
-void DumperSupport::dump_basic_type_array_class(AbstractDumpWriter* writer, Klass* k) {
- // array classes
- while (k != NULL) {
-    Klass* klass = k;
-
-    u4 size = 1 + sizeof(address) + 4 + 6 * sizeof(address) + 4 + 2 + 2 + 2;
-    writer->start_sub_record(HPROF_GC_CLASS_DUMP, size);
-    writer->write_classID(klass);
-    writer->write_u4(STACK_TRACE_ID);
-
-    // super class of array classes is java.lang.Object
-    InstanceKlass* java_super = klass->java_super();
-    assert(java_super != NULL, "checking");
-    writer->write_classID(java_super);
-
-    writer->write_objectID(oop(NULL));    // loader
-    writer->write_objectID(oop(NULL));    // signers
-    writer->write_objectID(oop(NULL));    // protection domain
-
-    writer->write_objectID(oop(NULL));    // reserved
-    writer->write_objectID(oop(NULL));
-    writer->write_u4(0);             // instance size
-    writer->write_u2(0);             // constant pool
-    writer->write_u2(0);             // static fields
-    writer->write_u2(0);             // instance fields
-
-    writer->end_sub_record();
-
-    // get the array class for the next rank
-    k = klass->array_klass_or_null();
+// creates HPROF_GC_CLASS_DUMP record for the given array class
+void DumperSupport::dump_array_class(AbstractDumpWriter* writer, Klass* k) {
+  InstanceKlass* ik = NULL; // bottom class for object arrays, NULL for primitive type arrays
+  if (k->is_objArray_klass()) {
+    Klass *bk = ObjArrayKlass::cast(k)->bottom_klass();
+    assert(bk != NULL, "checking");
+    if (bk->is_instance_klass()) {
+      ik = InstanceKlass::cast(bk);
+    }
   }
+
+  u4 size = 1 + sizeof(address) + 4 + 6 * sizeof(address) + 4 + 2 + 2 + 2;
+  writer->start_sub_record(HPROF_GC_CLASS_DUMP, size);
+  writer->write_classID(k);
+  writer->write_u4(STACK_TRACE_ID);
+
+  // super class of array classes is java.lang.Object
+  InstanceKlass* java_super = k->java_super();
+  assert(java_super != NULL, "checking");
+  writer->write_classID(java_super);
+
+  writer->write_objectID(ik == NULL ? oop(NULL) : ik->class_loader());
+  writer->write_objectID(ik == NULL ? oop(NULL) : ik->signers());
+  writer->write_objectID(ik == NULL ? oop(NULL) : ik->protection_domain());
+
+  writer->write_objectID(oop(NULL));    // reserved
+  writer->write_objectID(oop(NULL));
+  writer->write_u4(0);             // instance size
+  writer->write_u2(0);             // constant pool
+  writer->write_u2(0);             // static fields
+  writer->write_u2(0);             // instance fields
+
+  writer->end_sub_record();
+
 }
 
 // Hprof uses an u4 as record length field,
@@ -1630,7 +1587,7 @@ void JNIGlobalsDumper::do_oop(oop* obj_p) {
     u4 size = 1 + 2 * sizeof(address);
     writer()->start_sub_record(HPROF_GC_ROOT_JNI_GLOBAL, size);
     writer()->write_objectID(o);
-    writer()->write_objectID((oopDesc*)obj_p);      // global ref ID
+    writer()->write_rootID(obj_p);      // global ref ID
     writer()->end_sub_record();
   }
 };
@@ -1936,17 +1893,11 @@ class VM_HeapDumper : public VM_GC_Operation, public WorkerTask {
   bool skip_operation() const;
 
   // writes a HPROF_LOAD_CLASS record
-  class ClassesDo;
   static void do_load_class(Klass* k);
 
   // writes a HPROF_GC_CLASS_DUMP record for the given class
-  // (and each array class too)
   static void do_class_dump(Klass* k);
 
-  // writes a HPROF_GC_CLASS_DUMP records for a given basic type
-  // array (and each multi-dimensional array too)
-  static void do_basic_type_array_class_dump(Klass* k);
-
   // HPROF_GC_ROOT_THREAD_OBJ records
   int do_thread(JavaThread* thread, u4 thread_serial_num);
   void do_threads();
@@ -2027,52 +1978,40 @@ void DumperSupport::end_of_dump(AbstractDumpWriter* writer) {
   writer->write_u4(0);
 }
 
-// writes a HPROF_LOAD_CLASS record for the class (and each of its
-// array classes)
+// writes a HPROF_LOAD_CLASS record for the class
 void VM_HeapDumper::do_load_class(Klass* k) {
   static u4 class_serial_num = 0;
 
   // len of HPROF_LOAD_CLASS record
   u4 remaining = 2*oopSize + 2*sizeof(u4);
 
-  // write a HPROF_LOAD_CLASS for the class and each array class
-  do {
-    DumperSupport::write_header(writer(), HPROF_LOAD_CLASS, remaining);
+  DumperSupport::write_header(writer(), HPROF_LOAD_CLASS, remaining);
 
-    // class serial number is just a number
-    writer()->write_u4(++class_serial_num);
+  // class serial number is just a number
+  writer()->write_u4(++class_serial_num);
 
-    // class ID
-    Klass* klass = k;
-    writer()->write_classID(klass);
+  // class ID
+  writer()->write_classID(k);
 
-    // add the Klass* and class serial number pair
-    dumper()->add_class_serial_number(klass, class_serial_num);
+  // add the Klass* and class serial number pair
+  dumper()->add_class_serial_number(k, class_serial_num);
 
-    writer()->write_u4(STACK_TRACE_ID);
+  writer()->write_u4(STACK_TRACE_ID);
 
-    // class name ID
-    Symbol* name = klass->name();
-    writer()->write_symbolID(name);
-
-    // write a LOAD_CLASS record for the array type (if it exists)
-    k = klass->array_klass_or_null();
-  } while (k != NULL);
+  // class name ID
+  Symbol* name = k->name();
+  writer()->write_symbolID(name);
 }
 
 // writes a HPROF_GC_CLASS_DUMP record for the given class
 void VM_HeapDumper::do_class_dump(Klass* k) {
   if (k->is_instance_klass()) {
-    DumperSupport::dump_class_and_array_classes(writer(), k);
+    DumperSupport::dump_instance_class(writer(), k);
+  } else {
+    DumperSupport::dump_array_class(writer(), k);
   }
 }
 
-// writes a HPROF_GC_CLASS_DUMP records for a given basic type
-// array (and each multi-dimensional array too)
-void VM_HeapDumper::do_basic_type_array_class_dump(Klass* k) {
-  DumperSupport::dump_basic_type_array_class(writer(), k);
-}
-
 // Walk the stack of the given thread.
 // Dumps a HPROF_GC_ROOT_JAVA_FRAME record for each local
 // Dumps a HPROF_GC_ROOT_JNI_LOCAL record for each JNI local
@@ -2092,7 +2031,10 @@ int VM_HeapDumper::do_thread(JavaThread* java_thread, u4 thread_serial_num) {
     ResourceMark rm(current_thread);
     HandleMark hm(current_thread);
 
-    RegisterMap reg_map(java_thread);
+    RegisterMap reg_map(java_thread,
+                        RegisterMap::UpdateMap::include,
+                        RegisterMap::ProcessFrames::include,
+                        RegisterMap::WalkContinuation::skip);
     frame f = java_thread->last_frame();
     vframe* vf = vframe::new_vframe(&f, &reg_map, java_thread);
     frame* last_entry_frame = NULL;
@@ -2291,7 +2233,6 @@ void VM_HeapDumper::work(uint worker_id) {
       LockedClassesDo locked_load_classes(&do_load_class);
       ClassLoaderDataGraph::classes_do(&locked_load_classes);
     }
-    Universe::basic_type_classes_do(&do_load_class);
 
     // write HPROF_FRAME and HPROF_TRACE records
     // this must be called after _klass_map is built when iterating the classes above.
@@ -2302,7 +2243,6 @@ void VM_HeapDumper::work(uint worker_id) {
       LockedClassesDo locked_dump_class(&do_class_dump);
       ClassLoaderDataGraph::classes_do(&locked_dump_class);
     }
-    Universe::basic_type_classes_do(&do_basic_type_array_class_dump);
 
     // HPROF_GC_ROOT_THREAD_OBJ + frames + jni locals
     do_threads();
@@ -2388,7 +2328,7 @@ void VM_HeapDumper::dump_stack_traces() {
       HandleMark hm(current_thread);
 
       ThreadStackTrace* stack_trace = new ThreadStackTrace(thread, false);
-      stack_trace->dump_stack_at_safepoint(-1, /* ObjectMonitorsHashtable is not needed here */ nullptr);
+      stack_trace->dump_stack_at_safepoint(-1, /* ObjectMonitorsHashtable is not needed here */ nullptr, true);
       _stack_traces[_num_threads++] = stack_trace;
 
       // write HPROF_FRAME records for this thread's stack trace
