@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -157,8 +157,8 @@ void MemSummaryReporter::report_summary_of_type(MEMFLAGS flag,
     }
   } else if (flag == mtNMT) {
     // Count malloc headers in "NMT" category
-    reserved_amount  += _malloc_snapshot->malloc_overhead()->size();
-    committed_amount += _malloc_snapshot->malloc_overhead()->size();
+    reserved_amount  += _malloc_snapshot->malloc_overhead();
+    committed_amount += _malloc_snapshot->malloc_overhead();
   }
 
   if (amount_in_current_scale(reserved_amount) > 0) {
@@ -210,9 +210,9 @@ void MemSummaryReporter::report_summary_of_type(MEMFLAGS flag,
     }
 
     if (flag == mtNMT &&
-      amount_in_current_scale(_malloc_snapshot->malloc_overhead()->size()) > 0) {
+      amount_in_current_scale(_malloc_snapshot->malloc_overhead()) > 0) {
       out->print_cr("%27s (tracking overhead=" SIZE_FORMAT "%s)", " ",
-        amount_in_current_scale(_malloc_snapshot->malloc_overhead()->size()), scale);
+        amount_in_current_scale(_malloc_snapshot->malloc_overhead()), scale);
     } else if (flag == mtClass) {
       // Metadata information
       report_metadata(Metaspace::NonClassType);
@@ -363,7 +363,7 @@ void MemDetailReporter::report_virtual_memory_region(const ReservedMemoryRegion*
     if (committed_rgn->size() == reserved_rgn->size() && committed_rgn->call_stack()->equals(*stack)) {
       // One region spanning the entire reserved region, with the same stack trace.
       // Don't print this regions because the "reserved and committed" line above
-      // already indicates that the region is comitted.
+      // already indicates that the region is committed.
       assert(itr.next() == NULL, "Unexpectedly more than one regions");
       return;
     }
@@ -731,13 +731,6 @@ void MemDetailDiffReporter::diff_virtual_memory_sites() const {
       } else if (compVal > 0) {
         old_virtual_memory_site(early_site);
         early_site = early_itr.next();
-      } else if (early_site->flag() != current_site->flag()) {
-        // This site was originally allocated with one flag, then released,
-        // then re-allocated at the same site (as far as we can tell) with a different flag.
-        old_virtual_memory_site(early_site);
-        early_site = early_itr.next();
-        new_virtual_memory_site(current_site);
-        current_site = current_itr.next();
       } else {
         diff_virtual_memory_site(early_site, current_site);
         early_site   = early_itr.next();
@@ -800,6 +793,8 @@ void MemDetailDiffReporter::old_virtual_memory_site(const VirtualMemoryAllocatio
 
 void MemDetailDiffReporter::diff_virtual_memory_site(const VirtualMemoryAllocationSite* early,
   const VirtualMemoryAllocationSite* current) const {
+  assert(early->flag() == current->flag() || early->flag() == mtNone,
+    "Expect the same flag, but %s != %s", NMTUtil::flag_to_name(early->flag()),NMTUtil::flag_to_name(current->flag()));
   diff_virtual_memory_site(current->call_stack(), current->reserved(), current->committed(),
     early->reserved(), early->committed(), current->flag());
 }
