@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,18 +22,18 @@
  *
  */
 
-#ifndef SHARE_VM_UTILITIES_BITMAP_HPP
-#define SHARE_VM_UTILITIES_BITMAP_HPP
+#ifndef SHARE_UTILITIES_BITMAP_HPP
+#define SHARE_UTILITIES_BITMAP_HPP
 
 #include "memory/allocation.hpp"
-#include "utilities/align.hpp"
+#include "runtime/atomic.hpp"
 #include "utilities/globalDefinitions.hpp"
 
 // Forward decl;
 class BitMapClosure;
 
 // Operations for bitmaps represented as arrays of unsigned integers.
-// Bit offsets are numbered from 0 to size-1.
+// Bits are numbered from 0 to size-1.
 
 // The "abstract" base BitMap class.
 //
@@ -50,8 +50,10 @@ class BitMap {
 
  public:
   typedef size_t idx_t;         // Type used for bit and word indices.
-  typedef uintptr_t bm_word_t;  // Element type of array that represents
-                                // the bitmap.
+  typedef uintptr_t bm_word_t;  // Element type of array that represents the
+                                // bitmap, with BitsPerWord bits per element.
+  // If this were to fail, there are lots of places that would need repair.
+  STATIC_ASSERT((sizeof(bm_word_t) * BitsPerByte) == BitsPerWord);
 
   // Hints for range sizes.
   typedef enum {
@@ -62,10 +64,52 @@ class BitMap {
   bm_word_t* _map;     // First word in bitmap
   idx_t      _size;    // Size of bitmap (in bits)
 
+  // The maximum allowable size of a bitmap, in words or bits.
+  // Limit max_size_in_bits so aligning up to a word boundary never overflows.
+  static idx_t max_size_in_words() { return raw_to_words_align_down(~idx_t(0)); }
+  static idx_t max_size_in_bits() { return max_size_in_words() * BitsPerWord; }
+
+  // Assumes relevant validity checking for bit has already been done.
+  static idx_t raw_to_words_align_up(idx_t bit) {
+    return raw_to_words_align_down(bit + (BitsPerWord - 1));
+  }
+
+  // Assumes relevant validity checking for bit has already been done.
+  static idx_t raw_to_words_align_down(idx_t bit) {
+    return bit >> LogBitsPerWord;
+  }
+
+  // Word-aligns bit and converts it to a word offset.
+  // precondition: bit <= size()
+  idx_t to_words_align_up(idx_t bit) const {
+    verify_limit(bit);
+    return raw_to_words_align_up(bit);
+  }
+
+  // Word-aligns bit and converts it to a word offset.
+  // precondition: bit <= size()
+  inline idx_t to_words_align_down(idx_t bit) const {
+    verify_limit(bit);
+    return raw_to_words_align_down(bit);
+  }
+
+  // Helper for get_next_{zero,one}_bit variants.
+  // - flip designates whether searching for 1s or 0s.  Must be one of
+  //   find_{zeros,ones}_flip.
+  // - aligned_right is true if r_index is a priori on a bm_word_t boundary.
+  template<bm_word_t flip, bool aligned_right>
+  inline idx_t get_next_bit_impl(idx_t l_index, idx_t r_index) const;
+
+  // Values for get_next_bit_impl flip parameter.
+  static const bm_word_t find_ones_flip = 0;
+  static const bm_word_t find_zeros_flip = ~(bm_word_t)0;
+
   // Threshold for performing small range operation, even when large range
   // operation was requested. Measured in words.
   static const size_t small_range_words = 32;
 
+  static bool is_small_range_of_words(idx_t beg_full_word, idx_t end_full_word);
+
  protected:
   // Return the position of bit within the word that contains it (e.g., if
   // bitmap words are 32 bits, return a number 0 <= n <= 31).
@@ -75,9 +119,6 @@ class BitMap {
   // containing the bit.
   static bm_word_t bit_mask(idx_t bit) { return (bm_word_t)1 << bit_in_word(bit); }
 
-  // Return the index of the word containing the specified bit.
-  static idx_t word_index(idx_t bit)  { return bit >> LogBitsPerWord; }
-
   // Return the bit number of the first bit in the specified word.
   static idx_t bit_index(idx_t word)  { return word << LogBitsPerWord; }
 
@@ -87,14 +128,20 @@ class BitMap {
   bm_word_t  map(idx_t word) const { return _map[word]; }
 
   // Return a pointer to the word containing the specified bit.
-  bm_word_t* word_addr(idx_t bit)             { return map() + word_index(bit); }
-  const bm_word_t* word_addr(idx_t bit) const { return map() + word_index(bit); }
+  bm_word_t* word_addr(idx_t bit) {
+    return map() + to_words_align_down(bit);
+  }
+  const bm_word_t* word_addr(idx_t bit) const {
+    return map() + to_words_align_down(bit);
+  }
 
   // Set a word to a specified value or to all ones; clear a word.
   void set_word  (idx_t word, bm_word_t val) { _map[word] = val; }
   void set_word  (idx_t word)            { set_word(word, ~(bm_word_t)0); }
   void clear_word(idx_t word)            { _map[word] = 0; }
 
+  static inline const bm_word_t load_word_ordered(const volatile bm_word_t* const addr, atomic_memory_order memory_order);
+
   // Utilities for ranges of bits.  Ranges are half-open [beg, end).
 
   // Ranges within a single word.
@@ -111,20 +158,8 @@ class BitMap {
 
   static void clear_range_of_words(bm_word_t* map, idx_t beg, idx_t end);
 
-  static bool is_small_range_of_words(idx_t beg_full_word, idx_t end_full_word);
-
-  // The index of the first full word in a range.
-  idx_t word_index_round_up(idx_t bit) const;
-
-  // Verification.
-  void verify_index(idx_t index) const NOT_DEBUG_RETURN;
-  void verify_range(idx_t beg_index, idx_t end_index) const NOT_DEBUG_RETURN;
-
-  // Statistics.
-  static const idx_t* _pop_count_table;
-  static void init_pop_count_table();
-  static idx_t num_set_bits(bm_word_t w);
-  static idx_t num_set_bits_from_table(unsigned char c);
+  idx_t count_one_bits_within_word(idx_t beg, idx_t end) const;
+  idx_t count_one_bits_in_range_of_words(idx_t beg_full_word, idx_t end_full_word) const;
 
   // Allocation Helpers.
 
@@ -144,23 +179,23 @@ class BitMap {
 
   // Resize the backing bitmap memory.
   //
-  // Old bits are transfered to the new memory
+  // Old bits are transferred to the new memory
   // and the extended memory is cleared.
   template <class Allocator>
-  void resize(const Allocator& allocator, idx_t new_size_in_bits);
+  void resize(const Allocator& allocator, idx_t new_size_in_bits, bool clear);
 
   // Set up and clear the bitmap memory.
   //
   // Precondition: The bitmap was default constructed and has
   // not yet had memory allocated via resize or (re)initialize.
   template <class Allocator>
-  void initialize(const Allocator& allocator, idx_t size_in_bits);
+  void initialize(const Allocator& allocator, idx_t size_in_bits, bool clear);
 
   // Set up and clear the bitmap memory.
   //
   // Can be called on previously initialized bitmaps.
   template <class Allocator>
-  void reinitialize(const Allocator& allocator, idx_t new_size_in_bits);
+  void reinitialize(const Allocator& allocator, idx_t new_size_in_bits, bool clear);
 
   // Set the map and size.
   void update(bm_word_t* map, idx_t size) {
@@ -169,7 +204,9 @@ class BitMap {
   }
 
   // Protected constructor and destructor.
-  BitMap(bm_word_t* map, idx_t size_in_bits) : _map(map), _size(size_in_bits) {}
+  BitMap(bm_word_t* map, idx_t size_in_bits) : _map(map), _size(size_in_bits) {
+    verify_size(size_in_bits);
+  }
   ~BitMap() {}
 
  public:
@@ -178,43 +215,36 @@ class BitMap {
 
   // Accessing
   static idx_t calc_size_in_words(size_t size_in_bits) {
-    return word_index(size_in_bits + BitsPerWord - 1);
-  }
-
-  static idx_t calc_size_in_bytes(size_t size_in_bits) {
-    return calc_size_in_words(size_in_bits) * BytesPerWord;
+    verify_size(size_in_bits);
+    return raw_to_words_align_up(size_in_bits);
   }
 
   idx_t size() const          { return _size; }
   idx_t size_in_words() const { return calc_size_in_words(size()); }
-  idx_t size_in_bytes() const { return calc_size_in_bytes(size()); }
+  idx_t size_in_bytes() const { return size_in_words() * BytesPerWord; }
 
   bool at(idx_t index) const {
     verify_index(index);
     return (*word_addr(index) & bit_mask(index)) != 0;
   }
 
-  // Align bit index up or down to the next bitmap word boundary, or check
-  // alignment.
-  static idx_t word_align_up(idx_t bit) {
-    return align_up(bit, BitsPerWord);
-  }
-  static idx_t word_align_down(idx_t bit) {
-    return align_down(bit, BitsPerWord);
-  }
-  static bool is_word_aligned(idx_t bit) {
-    return word_align_up(bit) == bit;
-  }
+  // memory_order must be memory_order_relaxed or memory_order_acquire.
+  bool par_at(idx_t index, atomic_memory_order memory_order = memory_order_acquire) const;
 
   // Set or clear the specified bit.
   inline void set_bit(idx_t bit);
   inline void clear_bit(idx_t bit);
 
-  // Atomically set or clear the specified bit.
-  inline bool par_set_bit(idx_t bit);
-  inline bool par_clear_bit(idx_t bit);
+  // Attempts to change a bit to a desired value. The operation returns true if
+  // this thread changed the value of the bit. It was changed with a RMW operation
+  // using the specified memory_order. The operation returns false if the change
+  // could not be set due to the bit already being observed in the desired state.
+  // The atomic access that observed the bit in the desired state has acquire
+  // semantics, unless memory_order is memory_order_relaxed or memory_order_release.
+  inline bool par_set_bit(idx_t bit, atomic_memory_order memory_order = memory_order_conservative);
+  inline bool par_clear_bit(idx_t bit, atomic_memory_order memory_order = memory_order_conservative);
 
-  // Put the given value at the given offset. The parallel version
+  // Put the given value at the given index. The parallel version
   // will CAS the value into the bitmap and is quite a bit slower.
   // The parallel version also returns a value indicating if the
   // calling thread was the one that changed the value of the bit.
@@ -243,14 +273,30 @@ class BitMap {
   void clear_large();
   inline void clear();
 
-  // Iteration support.  Returns "true" if the iteration completed, false
-  // if the iteration terminated early (because the closure "blk" returned
-  // false).
-  bool iterate(BitMapClosure* blk, idx_t leftIndex, idx_t rightIndex);
-  bool iterate(BitMapClosure* blk) {
-    // call the version that takes an interval
-    return iterate(blk, 0, size());
-  }
+  // Verification.
+
+  // Verify size_in_bits does not exceed max_size_in_bits().
+  static void verify_size(idx_t size_in_bits) NOT_DEBUG_RETURN;
+  // Verify bit is less than size().
+  void verify_index(idx_t bit) const NOT_DEBUG_RETURN;
+  // Verify bit is not greater than size().
+  void verify_limit(idx_t bit) const NOT_DEBUG_RETURN;
+  // Verify [beg,end) is a valid range, e.g. beg <= end <= size().
+  void verify_range(idx_t beg, idx_t end) const NOT_DEBUG_RETURN;
+
+  // Iteration support.  Applies the closure to the index for each set bit,
+  // starting from the least index in the range to the greatest, in order.
+  // The iteration terminates if the closure returns false.  Returns true if
+  // the iteration completed, false if terminated early because the closure
+  // returned false.  If the closure modifies the bitmap, modifications to
+  // bits at indices greater than the current index will affect which further
+  // indices the closure will be applied to.
+  // precondition: beg and end form a valid range.
+  template <class BitMapClosureType>
+  bool iterate(BitMapClosureType* cl, idx_t beg, idx_t end);
+
+  template <class BitMapClosureType>
+  bool iterate(BitMapClosureType* cl);
 
   // Looking for 1's and 0's at indices equal to or greater than "l_index",
   // stopping if none has been found before "r_index", and returning
@@ -272,6 +318,9 @@ class BitMap {
   // Returns the number of bits set in the bitmap.
   idx_t count_one_bits() const;
 
+  // Returns the number of bits set within  [beg, end).
+  idx_t count_one_bits(idx_t beg, idx_t end) const;
+
   // Set operations.
   void set_union(const BitMap& bits);
   void set_difference(const BitMap& bits);
@@ -305,7 +354,7 @@ class BitMap {
 #endif
 };
 
-// A concrete implementation of the the "abstract" BitMap class.
+// A concrete implementation of the "abstract" BitMap class.
 //
 // The BitMapView is used when the backing storage is managed externally.
 class BitMapView : public BitMap {
@@ -324,7 +373,7 @@ class ResourceBitMap : public BitMap {
 
   // Resize the backing bitmap memory.
   //
-  // Old bits are transfered to the new memory
+  // Old bits are transferred to the new memory
   // and the extended memory is cleared.
   void resize(idx_t new_size_in_bits);
 
@@ -369,20 +418,20 @@ class CHeapBitMap : public BitMap {
 
   // Resize the backing bitmap memory.
   //
-  // Old bits are transfered to the new memory
-  // and the extended memory is cleared.
-  void resize(idx_t new_size_in_bits);
+  // Old bits are transferred to the new memory
+  // and the extended memory is (optionally) cleared.
+  void resize(idx_t new_size_in_bits, bool clear = true);
 
-  // Set up and clear the bitmap memory.
+  // Set up and (optionally) clear the bitmap memory.
   //
   // Precondition: The bitmap was default constructed and has
   // not yet had memory allocated via resize or initialize.
-  void initialize(idx_t size_in_bits);
+  void initialize(idx_t size_in_bits, bool clear = true);
 
-  // Set up and clear the bitmap memory.
+  // Set up and (optionally) clear the bitmap memory.
   //
   // Can be called on previously initialized bitmaps.
-  void reinitialize(idx_t size_in_bits);
+  void reinitialize(idx_t size_in_bits, bool clear = true);
 };
 
 // Convenience class wrapping BitMap which provides multiple bits per slot.
@@ -416,12 +465,6 @@ class BitMap2D {
     return _map.size();
   }
 
-  // Returns number of full slots that have been allocated
-  idx_t size_in_slots() {
-    // Round down
-    return _map.size() / _bits_per_slot;
-  }
-
   bool is_valid_index(idx_t slot_index, idx_t bit_within_slot_index);
   bool at(idx_t slot_index, idx_t bit_within_slot_index) const;
   void set_bit(idx_t slot_index, idx_t bit_within_slot_index);
@@ -436,7 +479,7 @@ class BitMapClosure {
  public:
   // Callback when bit in map is set.  Should normally return "true";
   // return of false indicates that the bitmap iteration should terminate.
-  virtual bool do_bit(BitMap::idx_t offset) = 0;
+  virtual bool do_bit(BitMap::idx_t index) = 0;
 };
 
-#endif // SHARE_VM_UTILITIES_BITMAP_HPP
+#endif // SHARE_UTILITIES_BITMAP_HPP
