@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,27 +23,27 @@
  */
 
 #include "precompiled.hpp"
+#include "classfile/javaClasses.hpp"
 #include "classfile/systemDictionary.hpp"
+#include "classfile/vmClasses.hpp"
 #include "classfile/vmSymbols.hpp"
 #include "compiler/compileBroker.hpp"
 #include "logging/log.hpp"
 #include "logging/logStream.hpp"
 #include "memory/resourceArea.hpp"
+#include "memory/universe.hpp"
 #include "oops/oop.inline.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/init.hpp"
 #include "runtime/java.hpp"
 #include "runtime/javaCalls.hpp"
+#include "runtime/javaThread.hpp"
 #include "runtime/os.hpp"
-#include "runtime/thread.inline.hpp"
 #include "runtime/threadCritical.hpp"
 #include "runtime/atomic.hpp"
 #include "utilities/events.hpp"
 #include "utilities/exceptions.hpp"
 
-// Limit exception message components to 64K (the same max as Symbols)
-#define MAX_LEN 65535
-
 // Implementation of ThreadShadow
 void check_ThreadShadow() {
   const ByteSize offset1 = byte_offset_of(ThreadShadow, _pending_exception);
@@ -71,9 +71,18 @@ void ThreadShadow::clear_pending_exception() {
   _exception_file    = NULL;
   _exception_line    = 0;
 }
+
+void ThreadShadow::clear_pending_nonasync_exception() {
+  // Do not clear probable async exceptions.
+  if ((_pending_exception->klass() != vmClasses::InternalError_klass() ||
+       java_lang_InternalError::during_unsafe_access(_pending_exception) != JNI_TRUE)) {
+    clear_pending_exception();
+  }
+}
+
 // Implementation of Exceptions
 
-bool Exceptions::special_exception(Thread* thread, const char* file, int line, Handle h_exception) {
+bool Exceptions::special_exception(JavaThread* thread, const char* file, int line, Handle h_exception) {
   // bootstrapping check
   if (!Universe::is_fully_initialized()) {
    vm_exit_during_initialization(h_exception);
@@ -85,16 +94,15 @@ bool Exceptions::special_exception(Thread* thread, const char* file, int line, H
   // to prevent infinite recursion trying to initialize stack overflow without
   // adequate stack space.
   // This can happen with stress testing a large value of StackShadowPages
-  if (h_exception()->klass() == SystemDictionary::StackOverflowError_klass()) {
+  if (h_exception()->klass() == vmClasses::StackOverflowError_klass()) {
     InstanceKlass* ik = InstanceKlass::cast(h_exception->klass());
     assert(ik->is_initialized(),
            "need to increase java_thread_min_stack_allowed calculation");
   }
 #endif // ASSERT
 
-  if (thread->is_VM_thread()
-      || !thread->can_call_java()) {
-    // We do not care what kind of exception we get for the vm-thread or a thread which
+  if (!thread->can_call_java()) {
+    // We do not care what kind of exception we get for a thread which
     // is compiling.  We just install a dummy exception object
     thread->set_pending_exception(Universe::vm_exception(), file, line);
     return true;
@@ -103,11 +111,11 @@ bool Exceptions::special_exception(Thread* thread, const char* file, int line, H
   return false;
 }
 
-bool Exceptions::special_exception(Thread* thread, const char* file, int line, Symbol* h_name, const char* message) {
+bool Exceptions::special_exception(JavaThread* thread, const char* file, int line, Symbol* h_name, const char* message) {
   // bootstrapping check
   if (!Universe::is_fully_initialized()) {
     if (h_name == NULL) {
-      // atleast an informative message.
+      // at least an informative message.
       vm_exit_during_initialization("Exception", message);
     } else {
       vm_exit_during_initialization(h_name, message);
@@ -115,9 +123,8 @@ bool Exceptions::special_exception(Thread* thread, const char* file, int line, S
     ShouldNotReachHere();
   }
 
-  if (thread->is_VM_thread()
-      || !thread->can_call_java()) {
-    // We do not care what kind of exception we get for the vm-thread or a thread which
+  if (!thread->can_call_java()) {
+    // We do not care what kind of exception we get for a thread which
     // is compiling.  We just install a dummy exception object
     thread->set_pending_exception(Universe::vm_exception(), file, line);
     return true;
@@ -127,56 +134,53 @@ bool Exceptions::special_exception(Thread* thread, const char* file, int line, S
 
 // This method should only be called from generated code,
 // therefore the exception oop should be in the oopmap.
-void Exceptions::_throw_oop(Thread* thread, const char* file, int line, oop exception) {
+void Exceptions::_throw_oop(JavaThread* thread, const char* file, int line, oop exception) {
   assert(exception != NULL, "exception should not be NULL");
   Handle h_exception(thread, exception);
   _throw(thread, file, line, h_exception);
 }
 
-void Exceptions::_throw(Thread* thread, const char* file, int line, Handle h_exception, const char* message) {
-  ResourceMark rm;
+void Exceptions::_throw(JavaThread* thread, const char* file, int line, Handle h_exception, const char* message) {
+  ResourceMark rm(thread);
   assert(h_exception() != NULL, "exception should not be NULL");
 
   // tracing (do this up front - so it works during boot strapping)
-  log_info(exceptions)("Exception <%.*s%s%.*s> (" INTPTR_FORMAT ") \n"
-                       "thrown [%s, line %d]\nfor thread " INTPTR_FORMAT,
-                       MAX_LEN, h_exception->print_value_string(),
-                       message ? ": " : "",
-                       MAX_LEN, message ? message : "",
+  // Note, the print_value_string() argument is not called unless logging is enabled!
+  log_info(exceptions)("Exception <%s%s%s> (" PTR_FORMAT ") \n"
+                       "thrown [%s, line %d]\nfor thread " PTR_FORMAT,
+                       h_exception->print_value_string(),
+                       message ? ": " : "", message ? message : "",
                        p2i(h_exception()), file, line, p2i(thread));
+
   // for AbortVMOnException flag
   Exceptions::debug_check_abort(h_exception, message);
 
-  // Check for special boot-strapping/vm-thread handling
+  // Check for special boot-strapping/compiler-thread handling
   if (special_exception(thread, file, line, h_exception)) {
     return;
   }
 
-  if (h_exception->is_a(SystemDictionary::OutOfMemoryError_klass())) {
+  if (h_exception->is_a(vmClasses::OutOfMemoryError_klass())) {
     count_out_of_memory_exceptions(h_exception);
   }
 
-  if (h_exception->is_a(SystemDictionary::LinkageError_klass())) {
-    Atomic::inc(&_linkage_errors);
+  if (h_exception->is_a(vmClasses::LinkageError_klass())) {
+    Atomic::inc(&_linkage_errors, memory_order_relaxed);
   }
 
-  assert(h_exception->is_a(SystemDictionary::Throwable_klass()), "exception is not a subclass of java/lang/Throwable");
+  assert(h_exception->is_a(vmClasses::Throwable_klass()), "exception is not a subclass of java/lang/Throwable");
 
   // set the pending exception
   thread->set_pending_exception(h_exception(), file, line);
 
   // vm log
-  if (LogEvents){
-    Events::log_exception(thread, "Exception <%s%s%s> (" INTPTR_FORMAT ") thrown at [%s, line %d]",
-                          h_exception->print_value_string(), message ? ": " : "", message ? message : "",
-                          p2i(h_exception()), file, line);
-  }
+  Events::log_exception(thread, h_exception, message, file, line);
 }
 
 
-void Exceptions::_throw_msg(Thread* thread, const char* file, int line, Symbol* name, const char* message,
+void Exceptions::_throw_msg(JavaThread* thread, const char* file, int line, Symbol* name, const char* message,
                             Handle h_loader, Handle h_protection_domain) {
-  // Check for special boot-strapping/vm-thread handling
+  // Check for special boot-strapping/compiler-thread handling
   if (special_exception(thread, file, line, name, message)) return;
   // Create and throw exception
   Handle h_cause(thread, NULL);
@@ -184,26 +188,26 @@ void Exceptions::_throw_msg(Thread* thread, const char* file, int line, Symbol*
   _throw(thread, file, line, h_exception, message);
 }
 
-void Exceptions::_throw_msg_cause(Thread* thread, const char* file, int line, Symbol* name, const char* message, Handle h_cause,
+void Exceptions::_throw_msg_cause(JavaThread* thread, const char* file, int line, Symbol* name, const char* message, Handle h_cause,
                                   Handle h_loader, Handle h_protection_domain) {
-  // Check for special boot-strapping/vm-thread handling
+  // Check for special boot-strapping/compiler-thread handling
   if (special_exception(thread, file, line, name, message)) return;
   // Create and throw exception and init cause
   Handle h_exception = new_exception(thread, name, message, h_cause, h_loader, h_protection_domain);
   _throw(thread, file, line, h_exception, message);
 }
 
-void Exceptions::_throw_cause(Thread* thread, const char* file, int line, Symbol* name, Handle h_cause,
+void Exceptions::_throw_cause(JavaThread* thread, const char* file, int line, Symbol* name, Handle h_cause,
                               Handle h_loader, Handle h_protection_domain) {
-  // Check for special boot-strapping/vm-thread handling
+  // Check for special boot-strapping/compiler-thread handling
   if (special_exception(thread, file, line, h_cause)) return;
   // Create and throw exception
   Handle h_exception = new_exception(thread, name, h_cause, h_loader, h_protection_domain);
   _throw(thread, file, line, h_exception, NULL);
 }
 
-void Exceptions::_throw_args(Thread* thread, const char* file, int line, Symbol* name, Symbol* signature, JavaCallArguments *args) {
-  // Check for special boot-strapping/vm-thread handling
+void Exceptions::_throw_args(JavaThread* thread, const char* file, int line, Symbol* name, Symbol* signature, JavaCallArguments *args) {
+  // Check for special boot-strapping/compiler-thread handling
   if (special_exception(thread, file, line, name, NULL)) return;
   // Create and throw exception
   Handle h_loader(thread, NULL);
@@ -215,29 +219,29 @@ void Exceptions::_throw_args(Thread* thread, const char* file, int line, Symbol*
 
 // Methods for default parameters.
 // NOTE: These must be here (and not in the header file) because of include circularities.
-void Exceptions::_throw_msg_cause(Thread* thread, const char* file, int line, Symbol* name, const char* message, Handle h_cause) {
+void Exceptions::_throw_msg_cause(JavaThread* thread, const char* file, int line, Symbol* name, const char* message, Handle h_cause) {
   _throw_msg_cause(thread, file, line, name, message, h_cause, Handle(thread, NULL), Handle(thread, NULL));
 }
-void Exceptions::_throw_msg(Thread* thread, const char* file, int line, Symbol* name, const char* message) {
+void Exceptions::_throw_msg(JavaThread* thread, const char* file, int line, Symbol* name, const char* message) {
   _throw_msg(thread, file, line, name, message, Handle(thread, NULL), Handle(thread, NULL));
 }
-void Exceptions::_throw_cause(Thread* thread, const char* file, int line, Symbol* name, Handle h_cause) {
+void Exceptions::_throw_cause(JavaThread* thread, const char* file, int line, Symbol* name, Handle h_cause) {
   _throw_cause(thread, file, line, name, h_cause, Handle(thread, NULL), Handle(thread, NULL));
 }
 
 
-void Exceptions::throw_stack_overflow_exception(Thread* THREAD, const char* file, int line, const methodHandle& method) {
+void Exceptions::throw_stack_overflow_exception(JavaThread* THREAD, const char* file, int line, const methodHandle& method) {
   Handle exception;
   if (!THREAD->has_pending_exception()) {
-    InstanceKlass* k = SystemDictionary::StackOverflowError_klass();
+    InstanceKlass* k = vmClasses::StackOverflowError_klass();
     oop e = k->allocate_instance(CHECK);
     exception = Handle(THREAD, e);  // fill_in_stack trace does gc
     assert(k->is_initialized(), "need to increase java_thread_min_stack_allowed calculation");
     if (StackTraceInThrowable) {
-      java_lang_Throwable::fill_in_stack_trace(exception, method());
+      java_lang_Throwable::fill_in_stack_trace(exception, method);
     }
     // Increment counter for hs_err file reporting
-    Atomic::inc(&Exceptions::_stack_overflow_errors);
+    Atomic::inc(&Exceptions::_stack_overflow_errors, memory_order_relaxed);
   } else {
     // if prior exception, throw that one instead
     exception = Handle(THREAD, THREAD->pending_exception());
@@ -245,7 +249,7 @@ void Exceptions::throw_stack_overflow_exception(Thread* THREAD, const char* file
   _throw(THREAD, file, line, exception);
 }
 
-void Exceptions::fthrow(Thread* thread, const char* file, int line, Symbol* h_name, const char* format, ...) {
+void Exceptions::fthrow(JavaThread* thread, const char* file, int line, Symbol* h_name, const char* format, ...) {
   const int max_msg_size = 1024;
   va_list ap;
   va_start(ap, format);
@@ -258,12 +262,11 @@ void Exceptions::fthrow(Thread* thread, const char* file, int line, Symbol* h_na
 
 // Creates an exception oop, calls the <init> method with the given signature.
 // and returns a Handle
-Handle Exceptions::new_exception(Thread *thread, Symbol* name,
+Handle Exceptions::new_exception(JavaThread* thread, Symbol* name,
                                  Symbol* signature, JavaCallArguments *args,
                                  Handle h_loader, Handle h_protection_domain) {
   assert(Universe::is_fully_initialized(),
     "cannot be called during initialization");
-  assert(thread->is_Java_thread(), "can only be called by a Java thread");
   assert(!thread->has_pending_exception(), "already has exception");
 
   Handle h_exception;
@@ -290,7 +293,7 @@ Handle Exceptions::new_exception(Thread *thread, Symbol* name,
 // Creates an exception oop, calls the <init> method with the given signature.
 // and returns a Handle
 // Initializes the cause if cause non-null
-Handle Exceptions::new_exception(Thread *thread, Symbol* name,
+Handle Exceptions::new_exception(JavaThread* thread, Symbol* name,
                                  Symbol* signature, JavaCallArguments *args,
                                  Handle h_cause,
                                  Handle h_loader, Handle h_protection_domain) {
@@ -298,7 +301,7 @@ Handle Exceptions::new_exception(Thread *thread, Symbol* name,
 
   // Future: object initializer should take a cause argument
   if (h_cause.not_null()) {
-    assert(h_cause->is_a(SystemDictionary::Throwable_klass()),
+    assert(h_cause->is_a(vmClasses::Throwable_klass()),
         "exception cause is not a subclass of java/lang/Throwable");
     JavaValue result1(T_OBJECT);
     JavaCallArguments args1;
@@ -321,7 +324,7 @@ Handle Exceptions::new_exception(Thread *thread, Symbol* name,
 
 // Convenience method. Calls either the <init>() or <init>(Throwable) method when
 // creating a new exception
-Handle Exceptions::new_exception(Thread* thread, Symbol* name,
+Handle Exceptions::new_exception(JavaThread* thread, Symbol* name,
                                  Handle h_cause,
                                  Handle h_loader, Handle h_protection_domain,
                                  ExceptionMsgToUtf8Mode to_utf8_safe) {
@@ -338,7 +341,7 @@ Handle Exceptions::new_exception(Thread* thread, Symbol* name,
 
 // Convenience method. Calls either the <init>() or <init>(String) method when
 // creating a new exception
-Handle Exceptions::new_exception(Thread* thread, Symbol* name,
+Handle Exceptions::new_exception(JavaThread* thread, Symbol* name,
                                  const char* message, Handle h_cause,
                                  Handle h_loader, Handle h_protection_domain,
                                  ExceptionMsgToUtf8Mode to_utf8_safe) {
@@ -347,17 +350,14 @@ Handle Exceptions::new_exception(Thread* thread, Symbol* name,
   if (message == NULL) {
     signature = vmSymbols::void_method_signature();
   } else {
-    // We want to allocate storage, but we can't do that if there's
-    // a pending exception, so we preserve any pending exception
-    // around the allocation.
-    // If we get an exception from the allocation, prefer that to
-    // the exception we are trying to build, or the pending exception.
-    // This is sort of like what PRESERVE_EXCEPTION_MARK does, except
-    // for the preferencing and the early returns.
-    Handle incoming_exception(thread, NULL);
+    // There should be no pending exception. The caller is responsible for not calling
+    // this with a pending exception.
+    Handle incoming_exception;
     if (thread->has_pending_exception()) {
       incoming_exception = Handle(thread, thread->pending_exception());
       thread->clear_pending_exception();
+      ResourceMark rm(thread);
+      assert(incoming_exception.is_null(), "Pending exception while throwing %s %s", name->as_C_string(), message);
     }
     Handle msg;
     if (to_utf8_safe == safe_to_utf8) {
@@ -367,6 +367,8 @@ Handle Exceptions::new_exception(Thread* thread, Symbol* name,
       // Make a java string keeping the encoding scheme of the original string.
       msg = java_lang_String::create_from_platform_dependent_str(message, thread);
     }
+    // If we get an exception from the allocation, prefer that to
+    // the exception we are trying to build, or the pending exception (in product mode)
     if (thread->has_pending_exception()) {
       Handle exception(thread, thread->pending_exception());
       thread->clear_pending_exception();
@@ -389,7 +391,7 @@ Handle Exceptions::new_exception(Thread* thread, Symbol* name,
 // encoding scheme of the string into account. One thing we should do at some
 // point is to push this flag down to class java_lang_String since other
 // classes may need similar functionalities.
-Handle Exceptions::new_exception(Thread* thread, Symbol* name,
+Handle Exceptions::new_exception(JavaThread* thread, Symbol* name,
                                  const char* message,
                                  ExceptionMsgToUtf8Mode to_utf8_safe) {
 
@@ -406,25 +408,36 @@ Handle Exceptions::new_exception(Thread* thread, Symbol* name,
 // dynamically computed constant uses wrap_dynamic_exception for:
 //    - bootstrap method resolution
 //    - post call to MethodHandleNatives::linkDynamicConstant
-void Exceptions::wrap_dynamic_exception(Thread* THREAD) {
+void Exceptions::wrap_dynamic_exception(bool is_indy, JavaThread* THREAD) {
   if (THREAD->has_pending_exception()) {
+    bool log_indy = log_is_enabled(Debug, methodhandles, indy) && is_indy;
+    bool log_condy = log_is_enabled(Debug, methodhandles, condy) && !is_indy;
+    LogStreamHandle(Debug, methodhandles, indy) lsh_indy;
+    LogStreamHandle(Debug, methodhandles, condy) lsh_condy;
+    LogStream* ls = NULL;
+    if (log_indy) {
+      ls = &lsh_indy;
+    } else if (log_condy) {
+      ls = &lsh_condy;
+    }
     oop exception = THREAD->pending_exception();
+
     // See the "Linking Exceptions" section for the invokedynamic instruction
     // in JVMS 6.5.
-    if (exception->is_a(SystemDictionary::Error_klass())) {
+    if (exception->is_a(vmClasses::Error_klass())) {
       // Pass through an Error, including BootstrapMethodError, any other form
-      // of linkage error, or say ThreadDeath/OutOfMemoryError
-      if (TraceMethodHandles) {
-        tty->print_cr("[constant/invoke]dynamic passes through an Error for " INTPTR_FORMAT, p2i((void *)exception));
-        exception->print();
+      // of linkage error, or say OutOfMemoryError
+      if (ls != NULL) {
+        ls->print_cr("bootstrap method invocation wraps BSME around " PTR_FORMAT, p2i(exception));
+        exception->print_on(ls);
       }
       return;
     }
 
     // Otherwise wrap the exception in a BootstrapMethodError
-    if (TraceMethodHandles) {
-      tty->print_cr("[constant/invoke]dynamic throws BSME for " INTPTR_FORMAT, p2i((void *)exception));
-      exception->print();
+    if (ls != NULL) {
+      ls->print_cr("%s throws BSME for " PTR_FORMAT, is_indy ? "invokedynamic" : "dynamic constant", p2i(exception));
+      exception->print_on(ls);
     }
     Handle nested_exception(THREAD, exception);
     THREAD->clear_pending_exception();
@@ -440,13 +453,13 @@ volatile int Exceptions::_out_of_memory_error_metaspace_errors = 0;
 volatile int Exceptions::_out_of_memory_error_class_metaspace_errors = 0;
 
 void Exceptions::count_out_of_memory_exceptions(Handle exception) {
-  if (exception() == Universe::out_of_memory_error_metaspace()) {
-     Atomic::inc(&_out_of_memory_error_metaspace_errors);
-  } else if (exception() == Universe::out_of_memory_error_class_metaspace()) {
-     Atomic::inc(&_out_of_memory_error_class_metaspace_errors);
+  if (Universe::is_out_of_memory_error_metaspace(exception())) {
+     Atomic::inc(&_out_of_memory_error_metaspace_errors, memory_order_relaxed);
+  } else if (Universe::is_out_of_memory_error_class_metaspace(exception())) {
+     Atomic::inc(&_out_of_memory_error_class_metaspace_errors, memory_order_relaxed);
   } else {
      // everything else reported as java heap OOM
-     Atomic::inc(&_out_of_memory_error_java_heap_errors);
+     Atomic::inc(&_out_of_memory_error_java_heap_errors, memory_order_relaxed);
   }
 }
 
@@ -475,9 +488,18 @@ void Exceptions::print_exception_counts_on_error(outputStream* st) {
 
 // Implementation of ExceptionMark
 
-ExceptionMark::ExceptionMark(Thread*& thread) {
-  thread     = Thread::current();
-  _thread    = thread;
+ExceptionMark::ExceptionMark(JavaThread* thread) {
+  assert(thread == JavaThread::current(), "must be");
+  _thread  = thread;
+  check_no_pending_exception();
+}
+
+ExceptionMark::ExceptionMark() {
+  _thread = JavaThread::current();
+  check_no_pending_exception();
+}
+
+inline void ExceptionMark::check_no_pending_exception() {
   if (_thread->has_pending_exception()) {
     oop exception = _thread->pending_exception();
     _thread->clear_pending_exception(); // Needed to avoid infinite recursion
@@ -508,7 +530,11 @@ void Exceptions::debug_check_abort(const char *value_string, const char* message
       strstr(value_string, AbortVMOnException)) {
     if (AbortVMOnExceptionMessage == NULL || (message != NULL &&
         strstr(message, AbortVMOnExceptionMessage))) {
-      fatal("Saw %s, aborting", value_string);
+      if (message == NULL) {
+        fatal("Saw %s, aborting", value_string);
+      } else {
+        fatal("Saw %s: %s, aborting", value_string, message);
+      }
     }
   }
 }
@@ -521,7 +547,7 @@ void Exceptions::debug_check_abort(Handle exception, const char* message) {
 
 void Exceptions::debug_check_abort_helper(Handle exception, const char* message) {
   ResourceMark rm;
-  if (message == NULL && exception->is_a(SystemDictionary::Throwable_klass())) {
+  if (message == NULL && exception->is_a(vmClasses::Throwable_klass())) {
     oop msg = java_lang_Throwable::message(exception());
     if (msg != NULL) {
       message = java_lang_String::as_utf8_string(msg);
@@ -533,15 +559,15 @@ void Exceptions::debug_check_abort_helper(Handle exception, const char* message)
 // for logging exceptions
 void Exceptions::log_exception(Handle exception, const char* message) {
   ResourceMark rm;
-  const char* detail_message = java_lang_Throwable::message_as_utf8(exception());
+  Symbol* detail_message = java_lang_Throwable::detail_message(exception());
   if (detail_message != NULL) {
-    log_info(exceptions)("Exception <%.*s: %.*s>\n thrown in %.*s",
-                         MAX_LEN, exception->print_value_string(),
-                         MAX_LEN, detail_message,
-                         MAX_LEN, message);
+    log_info(exceptions)("Exception <%s: %s>\n thrown in %s",
+                         exception->print_value_string(),
+                         detail_message->as_C_string(),
+                         message);
   } else {
-    log_info(exceptions)("Exception <%.*s>\n thrown in %.*s",
-                         MAX_LEN, exception->print_value_string(),
-                         MAX_LEN, message);
+    log_info(exceptions)("Exception <%s>\n thrown in %s",
+                         exception->print_value_string(),
+                         message);
   }
 }
