@@ -320,12 +320,23 @@ public:
     return min;
   }
 
+  void truncate_to(int idx) {
+    for (int i = 0, j = idx; j < length(); i++, j++) {
+      at_put(i, at(j));
+    }
+    trunc_to(length() - idx);
+  }
+
+  void truncate_from(int idx) {
+    trunc_to(idx);
+  }
+
   size_t data_size_in_bytes() const {
     return _len * sizeof(E);
   }
 
   void print() const {
-    tty->print("Growable Array " INTPTR_FORMAT, p2i(this));
+    tty->print("Growable Array " PTR_FORMAT, p2i(this));
     tty->print(": length %d (_max %d) { ", _len, _max);
     for (int i = 0; i < _len; i++) {
       tty->print(INTPTR_FORMAT " ", *(intptr_t*)&(_data[i]));
