@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2022, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2019, Azul Systems, Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -36,7 +36,6 @@ import java.util.Optional;
 import java.util.StringTokenizer;
 
 import jdk.internal.access.SharedSecrets;
-import jdk.internal.loader.NativeLibrary;
 import jdk.internal.reflect.CallerSensitive;
 import jdk.internal.reflect.Reflection;
 
@@ -83,10 +82,11 @@ public class Runtime {
      * until they finish.  Once this is done the virtual machine
      * {@linkplain #halt halts}.
      *
-     * <p> If this method is invoked after all shutdown hooks have already
-     * been run and the status is nonzero then this method halts the
-     * virtual machine with the given status code. Otherwise, this method
-     * blocks indefinitely.
+     * <p> Invocations of this method are serialized such that only one
+     * invocation will actually proceed with the shutdown sequence and
+     * terminate the VM with the given status code. All other invocations
+     * will block indefinitely. If this method is invoked from a shutdown
+     * hook the system will deadlock.
      *
      * <p> The {@link System#exit(int) System.exit} method is the
      * conventional and convenient means of invoking this method.
@@ -290,6 +290,12 @@ public class Runtime {
      * behaves in exactly the same way as the invocation
      * {@link #exec(String, String[], File) exec}{@code (command, null, null)}.
      *
+     * @deprecated This method is error-prone and should not be used, the corresponding method
+     * {@link #exec(String[])} or {@link ProcessBuilder} should be used instead.
+     * The command string is broken into tokens using only whitespace characters.
+     * For an argument with an embedded space, such as a filename, this can cause problems
+     * as the token does not include the full filename.
+     *
      * @param   command   a specified system command.
      *
      * @return  A new {@link Process} object for managing the subprocess
@@ -311,6 +317,7 @@ public class Runtime {
      * @see     #exec(String[], String[], File)
      * @see     ProcessBuilder
      */
+    @Deprecated(since="18")
     public Process exec(String command) throws IOException {
         return exec(command, null, null);
     }
@@ -324,6 +331,12 @@ public class Runtime {
      * behaves in exactly the same way as the invocation
      * {@link #exec(String, String[], File) exec}{@code (command, envp, null)}.
      *
+     * @deprecated This method is error-prone and should not be used, the corresponding method
+     * {@link #exec(String[], String[])} or {@link ProcessBuilder} should be used instead.
+     * The command string is broken into tokens using only whitespace characters.
+     * For an argument with an embedded space, such as a filename, this can cause problems
+     * as the token does not include the full filename.
+     *
      * @param   command   a specified system command.
      *
      * @param   envp      array of strings, each element of which
@@ -352,6 +365,7 @@ public class Runtime {
      * @see     #exec(String[], String[], File)
      * @see     ProcessBuilder
      */
+    @Deprecated(since="18")
     public Process exec(String command, String[] envp) throws IOException {
         return exec(command, envp, null);
     }
@@ -374,6 +388,12 @@ public class Runtime {
      * produced by the tokenizer are then placed in the new string
      * array {@code cmdarray}, in the same order.
      *
+     * @deprecated This method is error-prone and should not be used, the corresponding method
+     * {@link #exec(String[], String[], File)} or {@link ProcessBuilder} should be used instead.
+     * The command string is broken into tokens using only whitespace characters.
+     * For an argument with an embedded space, such as a filename, this can cause problems
+     * as the token does not include the full filename.
+     *
      * @param   command   a specified system command.
      *
      * @param   envp      array of strings, each element of which
@@ -406,6 +426,7 @@ public class Runtime {
      * @see     ProcessBuilder
      * @since 1.3
      */
+    @Deprecated(since="18")
     public Process exec(String command, String[] envp, File dir)
         throws IOException {
         if (command.isEmpty())
@@ -686,8 +707,17 @@ public class Runtime {
      * The method {@link System#runFinalization()} is the conventional
      * and convenient means of invoking this method.
      *
+     * @deprecated Finalization has been deprecated for removal.  See
+     * {@link java.lang.Object#finalize} for background information and details
+     * about migration options.
+     * <p>
+     * When running in a JVM in which finalization has been disabled or removed,
+     * no objects will be pending finalization, so this method does nothing.
+     *
      * @see     java.lang.Object#finalize()
+     * @jls 12.6 Finalization of Class Instances
      */
+    @Deprecated(since="18", forRemoval=true)
     public void runFinalization() {
         SharedSecrets.getJavaLangRefAccess().runFinalization();
     }
@@ -1044,7 +1074,7 @@ public class Runtime {
                     m.group(VersionPattern.OPT_GROUP));
 
             // empty '+'
-            if (!build.isPresent()) {
+            if (build.isEmpty()) {
                 if (m.group(VersionPattern.PLUS_GROUP) != null) {
                     if (optional.isPresent()) {
                         if (pre.isPresent())
@@ -1056,7 +1086,7 @@ public class Runtime {
                             + " build or optional components: '" + s + "'");
                     }
                 } else {
-                    if (optional.isPresent() && !pre.isPresent()) {
+                    if (optional.isPresent() && pre.isEmpty()) {
                         throw new IllegalArgumentException("optional component"
                             + " must be preceded by a pre-release component"
                             + " or '+': '" + s + "'");
@@ -1322,11 +1352,11 @@ public class Runtime {
 
         private int comparePre(Version obj) {
             Optional<String> oPre = obj.pre();
-            if (!pre.isPresent()) {
+            if (pre.isEmpty()) {
                 if (oPre.isPresent())
                     return 1;
             } else {
-                if (!oPre.isPresent())
+                if (oPre.isEmpty())
                     return -1;
                 String val = pre.get();
                 String oVal = oPre.get();
@@ -1357,11 +1387,11 @@ public class Runtime {
 
         private int compareOptional(Version obj) {
             Optional<String> oOpt = obj.optional();
-            if (!optional.isPresent()) {
+            if (optional.isEmpty()) {
                 if (oOpt.isPresent())
                     return -1;
             } else {
-                if (!oOpt.isPresent())
+                if (oOpt.isEmpty())
                     return 1;
                 return optional.get().compareTo(oOpt.get());
             }
