@@ -101,6 +101,7 @@ public class Inflater {
     private byte[] inputArray;
     private int inputPos, inputLim;
     private boolean finished;
+    private boolean pendingOutput;
     private boolean needDict;
     private long bytesRead;
     private long bytesWritten;
@@ -415,11 +416,18 @@ public class Inflater {
             if ((result >>> 62 & 1) != 0) {
                 finished = true;
             }
+            if (written == len && !finished) {
+                pendingOutput = true;
+            } else {
+                pendingOutput = false;
+            }
             if ((result >>> 63 & 1) != 0) {
                 needDict = true;
             }
             if (input != null) {
-                input.position(inputPos + read);
+                if (read > 0) {
+                    input.position(inputPos + read);
+                }
             } else {
                 this.inputPos = inputPos + read;
             }
@@ -709,6 +717,10 @@ public class Inflater {
             throw new NullPointerException("Inflater has been closed");
     }
 
+    boolean hasPendingOutput() {
+        return pendingOutput;
+    }
+
     private static native void initIDs();
     private static native long init(boolean nowrap);
     private static native void setDictionary(long addr, byte[] b, int off,
