@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,6 +28,8 @@ package sun.nio.ch;
 import java.io.FileDescriptor;
 import java.io.IOException;
 import java.io.UncheckedIOException;
+import java.lang.foreign.MemorySegment;
+import java.lang.foreign.MemorySession;
 import java.lang.ref.Cleaner.Cleanable;
 import java.nio.ByteBuffer;
 import java.nio.MappedByteBuffer;
@@ -46,6 +48,10 @@ import java.util.Objects;
 
 import jdk.internal.access.JavaIOFileDescriptorAccess;
 import jdk.internal.access.SharedSecrets;
+import jdk.internal.foreign.AbstractMemorySegmentImpl;
+import jdk.internal.foreign.MappedMemorySegmentImpl;
+import jdk.internal.foreign.MemorySessionImpl;
+import jdk.internal.misc.Blocker;
 import jdk.internal.misc.ExtendedMapMode;
 import jdk.internal.misc.Unsafe;
 import jdk.internal.misc.VM;
@@ -58,18 +64,12 @@ import jdk.internal.access.foreign.UnmapperProxy;
 public class FileChannelImpl
     extends FileChannel
 {
-    // Memory allocation size for mapping buffers
-    private static final long allocationGranularity;
-
     // Access to FileDescriptor internals
     private static final JavaIOFileDescriptorAccess fdAccess =
         SharedSecrets.getJavaIOFileDescriptorAccess();
 
-    // Maximum direct transfer size
-    private static final int MAX_DIRECT_TRANSFER_SIZE;
-
     // Used to make native read and write calls
-    private final FileDispatcher nd;
+    private static final FileDispatcher nd = new FileDispatcherImpl();
 
     // File descriptor
     private final FileDescriptor fd;
@@ -124,12 +124,11 @@ public class FileChannelImpl
                             boolean writable, boolean direct, Object parent)
     {
         this.fd = fd;
+        this.path = path;
         this.readable = readable;
         this.writable = writable;
-        this.parent = parent;
-        this.path = path;
         this.direct = direct;
-        this.nd = new FileDispatcherImpl();
+        this.parent = parent;
         if (direct) {
             assert path != null;
             this.alignment = nd.setDirectIO(fd, path);
@@ -145,8 +144,9 @@ public class FileChannelImpl
             CleanerFactory.cleaner().register(this, new Closer(fd));
     }
 
-    // Used by FileInputStream.getChannel(), FileOutputStream.getChannel
-    // and RandomAccessFile.getChannel()
+
+    // Used by FileInputStream::getChannel, FileOutputStream::getChannel,
+    // and RandomAccessFile::getChannel
     public static FileChannel open(FileDescriptor fd, String path,
                                    boolean readable, boolean writable,
                                    boolean direct, Object parent)
@@ -229,7 +229,12 @@ public class FileChannelImpl
                 if (!isOpen())
                     return 0;
                 do {
-                    n = IOUtil.read(fd, dst, -1, direct, alignment, nd);
+                    long comp = Blocker.begin();
+                    try {
+                        n = IOUtil.read(fd, dst, -1, direct, alignment, nd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((n == IOStatus.INTERRUPTED) && isOpen());
                 return IOStatus.normalize(n);
             } finally {
@@ -258,8 +263,13 @@ public class FileChannelImpl
                 if (!isOpen())
                     return 0;
                 do {
-                    n = IOUtil.read(fd, dsts, offset, length,
-                            direct, alignment, nd);
+                    long comp = Blocker.begin();
+                    try {
+                        n = IOUtil.read(fd, dsts, offset, length, direct, alignment, nd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
+
                 } while ((n == IOStatus.INTERRUPTED) && isOpen());
                 return IOStatus.normalize(n);
             } finally {
@@ -285,7 +295,13 @@ public class FileChannelImpl
                 if (!isOpen())
                     return 0;
                 do {
-                    n = IOUtil.write(fd, src, -1, direct, alignment, nd);
+                    long comp = Blocker.begin();
+                    try {
+                        n = IOUtil.write(fd, src, -1, direct, alignment, nd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
+
                 } while ((n == IOStatus.INTERRUPTED) && isOpen());
                 return IOStatus.normalize(n);
             } finally {
@@ -314,8 +330,12 @@ public class FileChannelImpl
                 if (!isOpen())
                     return 0;
                 do {
-                    n = IOUtil.write(fd, srcs, offset, length,
-                            direct, alignment, nd);
+                    long comp = Blocker.begin();
+                    try {
+                        n = IOUtil.write(fd, srcs, offset, length, direct, alignment, nd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((n == IOStatus.INTERRUPTED) && isOpen());
                 return IOStatus.normalize(n);
             } finally {
@@ -340,8 +360,13 @@ public class FileChannelImpl
                     return 0;
                 boolean append = fdAccess.getAppend(fd);
                 do {
-                    // in append-mode then position is advanced to end before writing
-                    p = (append) ? nd.size(fd) : nd.seek(fd, -1);
+                    long comp = Blocker.begin();
+                    try {
+                        // in append-mode then position is advanced to end before writing
+                        p = (append) ? nd.size(fd) : nd.seek(fd, -1);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((p == IOStatus.INTERRUPTED) && isOpen());
                 return IOStatus.normalize(p);
             } finally {
@@ -365,7 +390,12 @@ public class FileChannelImpl
                 if (!isOpen())
                     return null;
                 do {
-                    p = nd.seek(fd, newPosition);
+                    long comp = Blocker.begin();
+                    try {
+                        p = nd.seek(fd, newPosition);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((p == IOStatus.INTERRUPTED) && isOpen());
                 return this;
             } finally {
@@ -387,7 +417,12 @@ public class FileChannelImpl
                 if (!isOpen())
                     return -1;
                 do {
-                    s = nd.size(fd);
+                    long comp = Blocker.begin();
+                    try {
+                        s = nd.size(fd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((s == IOStatus.INTERRUPTED) && isOpen());
                 return IOStatus.normalize(s);
             } finally {
@@ -418,14 +453,24 @@ public class FileChannelImpl
                 // get current size
                 long size;
                 do {
-                    size = nd.size(fd);
+                    long comp = Blocker.begin();
+                    try {
+                        size = nd.size(fd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((size == IOStatus.INTERRUPTED) && isOpen());
                 if (!isOpen())
                     return null;
 
                 // get current position
                 do {
-                    p = nd.seek(fd, -1);
+                    long comp = Blocker.begin();
+                    try {
+                        p = nd.seek(fd, -1);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((p == IOStatus.INTERRUPTED) && isOpen());
                 if (!isOpen())
                     return null;
@@ -434,7 +479,12 @@ public class FileChannelImpl
                 // truncate file if given size is less than the current size
                 if (newSize < size) {
                     do {
-                        rv = nd.truncate(fd, newSize);
+                        long comp = Blocker.begin();
+                        try {
+                            rv = nd.truncate(fd, newSize);
+                        } finally {
+                            Blocker.end(comp);
+                        }
                     } while ((rv == IOStatus.INTERRUPTED) && isOpen());
                     if (!isOpen())
                         return null;
@@ -444,7 +494,12 @@ public class FileChannelImpl
                 if (p > newSize)
                     p = newSize;
                 do {
-                    rp = nd.seek(fd, p);
+                    long comp = Blocker.begin();
+                    try {
+                        rp = nd.seek(fd, p);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((rp == IOStatus.INTERRUPTED) && isOpen());
                 return this;
             } finally {
@@ -465,7 +520,12 @@ public class FileChannelImpl
             if (!isOpen())
                 return;
             do {
-                rv = nd.force(fd, metaData);
+                long comp = Blocker.begin();
+                try {
+                    rv = nd.force(fd, metaData);
+                } finally {
+                    Blocker.end(comp);
+                }
             } while ((rv == IOStatus.INTERRUPTED) && isOpen());
         } finally {
             threads.remove(ti);
@@ -475,9 +535,9 @@ public class FileChannelImpl
     }
 
     // Assume at first that the underlying kernel supports sendfile();
-    // set this to false if we find out later that it doesn't
+    // set this to true if we find out later that it doesn't
     //
-    private static volatile boolean transferSupported = true;
+    private static volatile boolean transferToNotSupported;
 
     // Assume that the underlying kernel sendfile() will work if the target
     // fd is a pipe; set this to false if we find out later that it doesn't
@@ -504,8 +564,14 @@ public class FileChannelImpl
             ti = threads.add();
             if (!isOpen())
                 return -1;
+            boolean append = fdAccess.getAppend(targetFD);
             do {
-                n = transferTo0(fd, position, icount, targetFD);
+                long comp = Blocker.begin();
+                try {
+                    n = nd.transferTo(fd, position, icount, targetFD, append);
+                } finally {
+                    Blocker.end(comp);
+                }
             } while ((n == IOStatus.INTERRUPTED) && isOpen());
             if (n == IOStatus.UNSUPPORTED_CASE) {
                 if (target instanceof SinkChannelImpl)
@@ -516,7 +582,7 @@ public class FileChannelImpl
             }
             if (n == IOStatus.UNSUPPORTED) {
                 // Don't bother trying again
-                transferSupported = false;
+                transferToNotSupported = true;
                 return IOStatus.UNSUPPORTED;
             }
             return IOStatus.normalize(n);
@@ -530,7 +596,7 @@ public class FileChannelImpl
                                     WritableByteChannel target)
         throws IOException
     {
-        if (!transferSupported)
+        if (transferToNotSupported)
             return IOStatus.UNSUPPORTED;
 
         FileDescriptor targetFD = null;
@@ -574,6 +640,11 @@ public class FileChannelImpl
         }
     }
 
+    // Size threshold above which to use a mapped buffer;
+    // transferToArbitraryChannel() and transferFromArbitraryChannel()
+    // are faster for smaller transfers
+    private static final long MAPPED_TRANSFER_THRESHOLD = 16L*1024L;
+
     // Maximum size to map when using a mapped buffer
     private static final long MAPPED_TRANSFER_SIZE = 8L*1024L*1024L;
 
@@ -581,6 +652,9 @@ public class FileChannelImpl
                                           WritableByteChannel target)
         throws IOException
     {
+        if (count < MAPPED_TRANSFER_THRESHOLD)
+            return IOStatus.UNSUPPORTED_CASE;
+
         boolean isSelChImpl = (target instanceof SelChImpl);
         if (!((target instanceof FileChannelImpl) || isSelChImpl))
             return IOStatus.UNSUPPORTED;
@@ -681,34 +755,99 @@ public class FileChannelImpl
             throw new NonWritableChannelException();
         if ((position < 0) || (count < 0))
             throw new IllegalArgumentException();
-        long sz = size();
+        final long sz = size();
         if (position > sz)
             return 0;
 
-        if ((sz - position) < count)
-            count = sz - position;
+        // Now position <= sz so remaining >= 0 and
+        // remaining == 0 if and only if sz == 0
+        long remaining = sz - position;
 
-        // Attempt a direct transfer, if the kernel supports it, limiting
-        // the number of bytes according to which platform
-        int icount = (int)Math.min(count, MAX_DIRECT_TRANSFER_SIZE);
-        long n;
-        if ((n = transferToDirectly(position, icount, target)) >= 0)
-            return n;
+        // Adjust count only if remaining > 0, i.e.,
+        // sz > position which means sz > 0
+        if (remaining > 0 && remaining < count)
+            count = remaining;
 
-        // Attempt a mapped transfer, but only to trusted channel types
-        if ((n = transferToTrustedChannel(position, count, target)) >= 0)
-            return n;
+        // System calls supporting fast transfers might not work on files
+        // which advertise zero size such as those in Linux /proc
+        if (sz > 0) {
+            // Attempt a direct transfer, if the kernel supports it, limiting
+            // the number of bytes according to which platform
+            int icount = (int)Math.min(count, nd.maxDirectTransferSize());
+            long n;
+            if ((n = transferToDirectly(position, icount, target)) >= 0)
+                return n;
+
+            // Attempt a mapped transfer, but only to trusted channel types
+            if ((n = transferToTrustedChannel(position, count, target)) >= 0)
+                return n;
+        }
 
         // Slow path for untrusted targets
         return transferToArbitraryChannel(position, count, target);
     }
 
+    // Assume at first that the underlying kernel supports copy_file_range();
+    // set this to true if we find out later that it doesn't
+    //
+    private static volatile boolean transferFromNotSupported;
+
+    private long transferFromDirectlyInternal(FileDescriptor srcFD,
+                                              long position, long count)
+        throws IOException
+    {
+        long n = -1;
+        int ti = -1;
+        try {
+            beginBlocking();
+            ti = threads.add();
+            if (!isOpen())
+                return -1;
+            do {
+                long comp = Blocker.begin();
+                try {
+                    boolean append = fdAccess.getAppend(fd);
+                    n = nd.transferFrom(srcFD, fd, position, count, append);
+                } finally {
+                    Blocker.end(comp);
+                }
+            } while ((n == IOStatus.INTERRUPTED) && isOpen());
+            if (n == IOStatus.UNSUPPORTED) {
+                // Don't bother trying again
+                transferFromNotSupported = true;
+                return IOStatus.UNSUPPORTED;
+            }
+            return IOStatus.normalize(n);
+        } finally {
+            threads.remove(ti);
+            end (n > -1);
+        }
+    }
+
+    private long transferFromDirectly(FileChannelImpl src,
+                                      long position, long count)
+        throws IOException
+    {
+        if (!src.readable)
+            throw new NonReadableChannelException();
+        if (transferFromNotSupported)
+            return IOStatus.UNSUPPORTED;
+        FileDescriptor srcFD = src.fd;
+        if (srcFD == null)
+            return IOStatus.UNSUPPORTED_CASE;
+
+        return transferFromDirectlyInternal(srcFD, position, count);
+    }
+
     private long transferFromFileChannel(FileChannelImpl src,
                                          long position, long count)
         throws IOException
     {
         if (!src.readable)
             throw new NonReadableChannelException();
+        if (count < MAPPED_TRANSFER_THRESHOLD)
+            return IOStatus.UNSUPPORTED_CASE;
+
         synchronized (src.positionLock) {
             long pos = src.position();
             long max = Math.min(count, src.size() - pos);
@@ -790,6 +929,8 @@ public class FileChannelImpl
         ensureOpen();
         if (!src.isOpen())
             throw new ClosedChannelException();
+        if (src instanceof FileChannelImpl fci && !fci.readable)
+            throw new NonReadableChannelException();
         if (!writable)
             throw new NonWritableChannelException();
         if ((position < 0) || (count < 0))
@@ -797,9 +938,13 @@ public class FileChannelImpl
         if (position > size())
             return 0;
 
-        if (src instanceof FileChannelImpl fci) {
-            long n = transferFromFileChannel(fci, position, count);
-            if (n >= 0)
+        // System calls supporting fast transfers might not work on files
+        // which advertise zero size such as those in Linux /proc
+        if (src instanceof FileChannelImpl fci && fci.size() > 0) {
+            long n;
+            if ((n = transferFromDirectly(fci, position, count)) >= 0)
+                return n;
+            if ((n = transferFromFileChannel(fci, position, count)) >= 0)
                 return n;
         }
 
@@ -836,7 +981,12 @@ public class FileChannelImpl
             if (!isOpen())
                 return -1;
             do {
-                n = IOUtil.read(fd, dst, position, direct, alignment, nd);
+                long comp = Blocker.begin();
+                try {
+                    n = IOUtil.read(fd, dst, position, direct, alignment, nd);
+                } finally {
+                    Blocker.end(comp);
+                }
             } while ((n == IOStatus.INTERRUPTED) && isOpen());
             return IOStatus.normalize(n);
         } finally {
@@ -875,7 +1025,12 @@ public class FileChannelImpl
             if (!isOpen())
                 return -1;
             do {
-                n = IOUtil.write(fd, src, position, direct, alignment, nd);
+                long comp = Blocker.begin();
+                try {
+                    n = IOUtil.write(fd, src, position, direct, alignment, nd);
+                } finally {
+                    Blocker.end(comp);
+                }
             } while ((n == IOStatus.INTERRUPTED) && isOpen());
             return IOStatus.normalize(n);
         } finally {
@@ -888,12 +1043,9 @@ public class FileChannelImpl
 
     // -- Memory-mapped buffers --
 
-    private static abstract class Unmapper
+    private abstract static class Unmapper
         implements Runnable, UnmapperProxy
     {
-        // may be required to close file
-        private static final NativeDispatcher nd = new FileDispatcherImpl();
-
         private volatile long address;
         protected final long size;
         protected final long cap;
@@ -926,10 +1078,14 @@ public class FileChannelImpl
             unmap();
         }
 
+        public long capacity() {
+            return cap;
+        }
+
         public void unmap() {
             if (address == 0)
                 return;
-            unmap0(address, size);
+            nd.unmap(address, size);
             address = 0;
 
             // if this mapping has a valid file descriptor then we close it
@@ -1038,22 +1194,55 @@ public class FileChannelImpl
             else
                 return Util.newMappedByteBuffer(0, 0, dummy, null, isSync);
         } else if ((!writable) || (prot == MAP_RO)) {
-            return Util.newMappedByteBufferR((int)unmapper.cap,
-                    unmapper.address + unmapper.pagePosition,
-                    unmapper.fd,
-                    unmapper, isSync);
+            return Util.newMappedByteBufferR((int)unmapper.capacity(),
+                    unmapper.address(),
+                    unmapper.fileDescriptor(),
+                    unmapper, unmapper.isSync());
         } else {
-            return Util.newMappedByteBuffer((int)unmapper.cap,
-                    unmapper.address + unmapper.pagePosition,
-                    unmapper.fd,
-                    unmapper, isSync);
+            return Util.newMappedByteBuffer((int)unmapper.capacity(),
+                    unmapper.address(),
+                    unmapper.fileDescriptor(),
+                    unmapper, unmapper.isSync());
         }
     }
 
-    public Unmapper mapInternal(MapMode mode, long position, long size) throws IOException {
-        boolean isSync = isSync(Objects.requireNonNull(mode, "Mode is null"));
+    @Override
+    public MemorySegment map(MapMode mode, long offset, long size,
+                             MemorySession session)
+            throws IOException
+    {
+        Objects.requireNonNull(mode,"Mode is null");
+        Objects.requireNonNull(session, "Session is null");
+        MemorySessionImpl sessionImpl = MemorySessionImpl.toSessionImpl(session);
+        sessionImpl.checkValidState();
+        if (offset < 0)
+            throw new IllegalArgumentException("Requested bytes offset must be >= 0.");
+        if (size < 0)
+            throw new IllegalArgumentException("Requested bytes size must be >= 0.");
+
+        boolean isSync = isSync(mode);
         int prot = toProt(mode);
-        return mapInternal(mode, position, size, prot, isSync);
+        Unmapper unmapper = mapInternal(mode, offset, size, prot, isSync);
+        boolean readOnly = false;
+        if (mode == MapMode.READ_ONLY) {
+            readOnly = true;
+        }
+        if (unmapper != null) {
+            AbstractMemorySegmentImpl segment =
+                new MappedMemorySegmentImpl(unmapper.address(), unmapper, size,
+                                            readOnly, session);
+            MemorySessionImpl.ResourceList.ResourceCleanup resource =
+                new MemorySessionImpl.ResourceList.ResourceCleanup() {
+                    @Override
+                    public void cleanup() {
+                        unmapper.unmap();
+                    }
+                };
+            sessionImpl.addOrCleanupIfFail(resource);
+            return segment;
+        } else {
+            return new MappedMemorySegmentImpl.EmptyMappedMemorySegmentImpl(readOnly, sessionImpl);
+        }
     }
 
     private Unmapper mapInternal(MapMode mode, long position, long size, int prot, boolean isSync)
@@ -1083,7 +1272,12 @@ public class FileChannelImpl
             synchronized (positionLock) {
                 long filesize;
                 do {
-                    filesize = nd.size(fd);
+                    long comp = Blocker.begin();
+                    try {
+                        filesize = nd.size(fd);
+                    } finally {
+                        Blocker.end(comp);
+                    }
                 } while ((filesize == IOStatus.INTERRUPTED) && isOpen());
                 if (!isOpen())
                     return null;
@@ -1095,7 +1289,12 @@ public class FileChannelImpl
                     }
                     int rv;
                     do {
-                        rv = nd.truncate(fd, position + size);
+                        long comp = Blocker.begin();
+                        try {
+                            rv = nd.truncate(fd, position + size);
+                        } finally {
+                            Blocker.end(comp);
+                        }
                     } while ((rv == IOStatus.INTERRUPTED) && isOpen());
                     if (!isOpen())
                         return null;
@@ -1105,12 +1304,12 @@ public class FileChannelImpl
                     return null;
                 }
 
-                pagePosition = (int)(position % allocationGranularity);
+                pagePosition = (int)(position % nd.allocationGranularity());
                 long mapPosition = position - pagePosition;
                 mapSize = size + pagePosition;
                 try {
-                    // If map0 did not throw an exception, the address is valid
-                    addr = map0(prot, mapPosition, mapSize, isSync);
+                    // If map did not throw an exception, the address is valid
+                    addr = nd.map(fd, prot, mapPosition, mapSize, isSync);
                 } catch (OutOfMemoryError x) {
                     // An OutOfMemoryError may indicate that we've exhausted
                     // memory so force gc and re-attempt map
@@ -1121,7 +1320,7 @@ public class FileChannelImpl
                         Thread.currentThread().interrupt();
                     }
                     try {
-                        addr = map0(prot, mapPosition, mapSize, isSync);
+                        addr = nd.map(fd, prot, mapPosition, mapSize, isSync);
                     } catch (OutOfMemoryError y) {
                         // After a second OOME, fail
                         throw new IOException("Map failed", y);
@@ -1135,15 +1334,15 @@ public class FileChannelImpl
             try {
                 mfd = nd.duplicateForMapping(fd);
             } catch (IOException ioe) {
-                unmap0(addr, mapSize);
+                nd.unmap(addr, mapSize);
                 throw ioe;
             }
 
             assert (IOStatus.checkAll(addr));
-            assert (addr % allocationGranularity == 0);
+            assert (addr % nd.allocationGranularity() == 0);
             Unmapper um = (isSync
-                           ? new SyncUnmapper(addr, mapSize, size, mfd, pagePosition)
-                           : new DefaultUnmapper(addr, mapSize, size, mfd, pagePosition));
+                ? new SyncUnmapper(addr, mapSize, size, mfd, pagePosition)
+                : new DefaultUnmapper(addr, mapSize, size, mfd, pagePosition));
             return um;
         } finally {
             threads.remove(ti);
@@ -1271,6 +1470,8 @@ public class FileChannelImpl
             throw new NonReadableChannelException();
         if (!shared && !writable)
             throw new NonWritableChannelException();
+        if (size == 0)
+            size = Long.MAX_VALUE - Math.max(0, position);
         FileLockImpl fli = new FileLockImpl(this, position, size, shared);
         FileLockTable flt = fileLockTable();
         flt.add(fli);
@@ -1283,7 +1484,12 @@ public class FileChannelImpl
                 return null;
             int n;
             do {
-                n = nd.lock(fd, true, position, size, shared);
+                long comp = Blocker.begin();
+                try {
+                    n = nd.lock(fd, true, position, size, shared);
+                } finally {
+                    Blocker.end(comp);
+                }
             } while ((n == FileDispatcher.INTERRUPTED) && isOpen());
             if (isOpen()) {
                 if (n == FileDispatcher.RET_EX_LOCK) {
@@ -1316,6 +1522,8 @@ public class FileChannelImpl
             throw new NonReadableChannelException();
         if (!shared && !writable)
             throw new NonWritableChannelException();
+        if (size == 0)
+            size = Long.MAX_VALUE - Math.max(0, position);
         FileLockImpl fli = new FileLockImpl(this, position, size, shared);
         FileLockTable flt = fileLockTable();
         flt.add(fli);
@@ -1358,29 +1566,4 @@ public class FileChannelImpl
         assert fileLockTable != null;
         fileLockTable.remove(fli);
     }
-
-    // -- Native methods --
-
-    // Creates a new mapping
-    private native long map0(int prot, long position, long length, boolean isSync)
-        throws IOException;
-
-    // Removes an existing mapping
-    private static native int unmap0(long address, long length);
-
-    // Transfers from src to dst, or returns -2 if kernel can't do that
-    private native long transferTo0(FileDescriptor src, long position,
-                                    long count, FileDescriptor dst);
-
-    // Retrieves the maximum size of a transfer
-    private static native int maxDirectTransferSize0();
-
-    // Caches fieldIDs
-    private static native long initIDs();
-
-    static {
-        IOUtil.load();
-        allocationGranularity = initIDs();
-        MAX_DIRECT_TRANSFER_SIZE = maxDirectTransferSize0();
-    }
 }
