@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -120,7 +120,7 @@ public class DerValue {
     /** Tag value indicating an ASN.1 "GeneralizedTime" value. */
     public static final byte    tag_GeneralizedTime = 0x18;
 
-    /** Tag value indicating an ASN.1 "GenerallString" value. */
+    /** Tag value indicating an ASN.1 "GeneralString" value. */
     public static final byte    tag_GeneralString = 0x1B;
 
     /** Tag value indicating an ASN.1 "UniversalString" value. */
@@ -299,7 +299,7 @@ public class DerValue {
      *
      * @param tag the tag
      * @param out the DerOutputStream
-     * @returns a new DerValue using out as its content
+     * @return a new DerValue using out as its content
      */
     public static DerValue wrap(byte tag, DerOutputStream out) {
         return new DerValue(tag, out.buf(), 0, out.size(), false);
@@ -311,7 +311,7 @@ public class DerValue {
      * Attention: no cloning is made.
      *
      * @param buf the byte array containing the DER-encoded datum
-     * @returns a new DerValue
+     * @return a new DerValue
      */
     public static DerValue wrap(byte[] buf)
             throws IOException {
@@ -326,7 +326,7 @@ public class DerValue {
      * @param buf the byte array containing the DER-encoded datum
      * @param offset where the encoded datum starts inside {@code buf}
      * @param len length of bytes to parse inside {@code buf}
-     * @returns a new DerValue
+     * @return a new DerValue
      */
     public static DerValue wrap(byte[] buf, int offset, int len)
             throws IOException {
@@ -692,6 +692,28 @@ public class DerValue {
         };
     }
 
+    // check the number of pad bits, validate the pad bits in the bytes
+    // if enforcing DER (i.e. allowBER == false), and return the number of
+    // bits of the resulting BitString
+    private static int checkPaddedBits(int numOfPadBits, byte[] data,
+            int start, int end, boolean allowBER) throws IOException {
+        // number of pad bits should be from 0(min) to 7(max).
+        if ((numOfPadBits < 0) || (numOfPadBits > 7)) {
+            throw new IOException("Invalid number of padding bits");
+        }
+        int lenInBits = ((end - start) << 3) - numOfPadBits;
+        if (lenInBits < 0) {
+            throw new IOException("Not enough bytes in BitString");
+        }
+
+        // padding bits should be all zeros for DER
+        if (!allowBER && numOfPadBits != 0 &&
+                (data[end - 1] & (0xff >>> (8 - numOfPadBits))) != 0) {
+            throw new IOException("Invalid value of padding bits");
+        }
+        return lenInBits;
+    }
+
     /**
      * Returns an ASN.1 BIT STRING value, with the tag assumed implicit
      * based on the parameter.  The bit string must be byte-aligned.
@@ -708,18 +730,17 @@ public class DerValue {
         }
         if (end == start) {
             throw new IOException("Invalid encoding: zero length bit string");
-        }
-        int numOfPadBits = buffer[start];
-        if ((numOfPadBits < 0) || (numOfPadBits > 7)) {
-            throw new IOException("Invalid number of padding bits");
-        }
-        // minus the first byte which indicates the number of padding bits
-        byte[] retval = Arrays.copyOfRange(buffer, start + 1, end);
-        if (numOfPadBits != 0) {
-            // get rid of the padding bits
-            retval[end - start - 2] &= (0xff << numOfPadBits);
+
         }
         data.pos = data.end; // Compatibility. Reach end.
+
+        int numOfPadBits = buffer[start];
+        checkPaddedBits(numOfPadBits, buffer, start + 1, end, allowBER);
+        byte[] retval = Arrays.copyOfRange(buffer, start + 1, end);
+        if (allowBER && numOfPadBits != 0) {
+            // fix the potential non-zero padding bits
+            retval[retval.length - 1] &= (byte)((0xff << numOfPadBits));
+        }
         return retval;
     }
 
@@ -742,16 +763,11 @@ public class DerValue {
             throw new IOException("Invalid encoding: zero length bit string");
         }
         data.pos = data.end; // Compatibility. Reach end.
+
         int numOfPadBits = buffer[start];
-        if ((numOfPadBits < 0) || (numOfPadBits > 7)) {
-            throw new IOException("Invalid number of padding bits");
-        }
-        if (end == start + 1) {
-            return new BitArray(0);
-        } else {
-            return new BitArray(((end - start - 1) << 3) - numOfPadBits,
-                    Arrays.copyOfRange(buffer, start + 1, end));
-        }
+        int len = checkPaddedBits(numOfPadBits, buffer, start + 1, end,
+                allowBER);
+        return new BitArray(len, buffer, start + 1);
     }
 
     /**
@@ -1000,7 +1016,7 @@ public class DerValue {
                     throw new IOException("Parse " + type + " time, +hhmm");
                 }
 
-                time -= ((hr * 60) + min) * 60 * 1000;
+                time -= ((hr * 60L) + min) * 60 * 1000;
                 break;
 
             case '-':
@@ -1016,7 +1032,7 @@ public class DerValue {
                     throw new IOException("Parse " + type + " time, -hhmm");
                 }
 
-                time += ((hr * 60) + min) * 60 * 1000;
+                time += ((hr * 60L) + min) * 60 * 1000;
                 break;
 
             case 'Z':
@@ -1088,10 +1104,9 @@ public class DerValue {
         if (this == o) {
             return true;
         }
-        if (!(o instanceof DerValue)) {
+        if (!(o instanceof DerValue other)) {
             return false;
         }
-        DerValue other = (DerValue) o;
         if (tag != other.tag) {
             return false;
         }
