@@ -224,7 +224,7 @@ public class AquaButtonUI extends BasicButtonUI implements Sizeable {
             b.removeAncestorListener(listener);
         }
         uninstallHierListener(b);
-        AquaUtilControlSize.addSizePropertyListener(b);
+        AquaUtilControlSize.removeSizePropertyListener(b);
     }
 
     protected void uninstallDefaults(final AbstractButton b) {
@@ -305,7 +305,15 @@ public class AquaButtonUI extends BasicButtonUI implements Sizeable {
         }
 
         // performs icon and text rect calculations
-        final String text = layoutAndGetText(g, b, aquaBorder, i, viewRect, iconRect, textRect);
+        final String text;
+        final View v = (View)c.getClientProperty(BasicHTML.propertyKey);
+        if (v != null) {
+            // use zero insets for view since layout only handles text calculations
+            text = layoutAndGetText(g, b, aquaBorder, new Insets(0,0,0,0),
+                    viewRect, iconRect, textRect);
+        } else {
+            text = layoutAndGetText(g, b, aquaBorder, i, viewRect, iconRect, textRect);
+        }
 
         // Paint the Icon
         if (b.getIcon() != null) {
@@ -316,8 +324,7 @@ public class AquaButtonUI extends BasicButtonUI implements Sizeable {
             textRect.width = 50;
         }
 
-        if (text != null && !text.equals("")) {
-            final View v = (View)c.getClientProperty(BasicHTML.propertyKey);
+        if (text != null && !text.isEmpty()) {
             if (v != null) {
                 v.paint(g, textRect);
             } else {
@@ -393,10 +400,23 @@ public class AquaButtonUI extends BasicButtonUI implements Sizeable {
 
         if (icon == null) return;
 
+        Icon selectedIcon = null;
+
+        // the fallback icon should be based on the selected state
+        if (model.isSelected()) {
+            selectedIcon = b.getSelectedIcon();
+            if (selectedIcon != null) {
+                icon = selectedIcon;
+            }
+        }
         if (!model.isEnabled()) {
             if (model.isSelected()) {
                 tmpIcon = b.getDisabledSelectedIcon();
-            } else {
+               if (tmpIcon == null) {
+                   tmpIcon = selectedIcon;
+               }
+            }
+            if (tmpIcon == null) {
                 tmpIcon = b.getDisabledIcon();
             }
         } else if (model.isPressed() && model.isArmed()) {
@@ -409,7 +429,11 @@ public class AquaButtonUI extends BasicButtonUI implements Sizeable {
         } else if (b.isRolloverEnabled() && model.isRollover()) {
             if (model.isSelected()) {
                 tmpIcon = b.getRolloverSelectedIcon();
-            } else {
+                if (tmpIcon == null) {
+                    tmpIcon = selectedIcon;
+                }
+            }
+            if (tmpIcon == null) {
                 tmpIcon = b.getRolloverIcon();
             }
         } else if (model.isSelected()) {
@@ -559,6 +583,7 @@ public class AquaButtonUI extends BasicButtonUI implements Sizeable {
             // If focusLost arrives while the button has been left-clicked this would disarm the button,
             // causing actionPerformed not to fire on mouse release!
             //b.getModel().setArmed(false);
+            b.getModel().setPressed(false);
             ((Component)e.getSource()).repaint();
         }
 
