@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -312,7 +312,7 @@ public class JEditorPane extends JTextComponent {
      * Creates a <code>JEditorPane</code> based on a specified URL for input.
      *
      * @param initialPage the URL
-     * @exception IOException if the URL is <code>null</code>
+     * @throws IOException if the URL is <code>null</code>
      *          or cannot be accessed
      */
     public JEditorPane(URL initialPage) throws IOException {
@@ -325,7 +325,7 @@ public class JEditorPane extends JTextComponent {
      * a URL specification.
      *
      * @param url the URL
-     * @exception IOException if the URL is <code>null</code> or
+     * @throws IOException if the URL is <code>null</code> or
      *          cannot be accessed
      */
     public JEditorPane(String url) throws IOException {
@@ -340,7 +340,7 @@ public class JEditorPane extends JTextComponent {
      *
      * @param type mime type of the given text
      * @param text the text to initialize with; may be <code>null</code>
-     * @exception NullPointerException if the <code>type</code> parameter
+     * @throws NullPointerException if the <code>type</code> parameter
      *          is <code>null</code>
      */
     public JEditorPane(String type, String text) {
@@ -462,7 +462,7 @@ public class JEditorPane extends JTextComponent {
      * thread is done whether the load was successful or not.
      *
      * @param page the URL of the page
-     * @exception IOException for a <code>null</code> or invalid
+     * @throws IOException for a <code>null</code> or invalid
      *          page specification, or exception from the stream being read
      * @see #getPage
      */
@@ -578,7 +578,7 @@ public class JEditorPane extends JTextComponent {
      *
      * @param in the stream from which to read
      * @param desc an object describing the stream
-     * @exception IOException as thrown by the stream being
+     * @throws IOException as thrown by the stream being
      *          used to initialize
      * @see JTextComponent#read
      * @see #setDocument
@@ -619,37 +619,35 @@ public class JEditorPane extends JTextComponent {
         String charset = (String) getClientProperty("charset");
         try(Reader r = (charset != null) ? new InputStreamReader(in, charset) :
                 new InputStreamReader(in)) {
-            try {
-                kit.read(r, doc, 0);
-            } catch (BadLocationException e) {
-                throw new IOException(e.getMessage());
-            } catch (ChangedCharSetException changedCharSetException) {
-                String charSetSpec = changedCharSetException.getCharSetSpec();
-                if (changedCharSetException.keyEqualsCharSet()) {
-                    putClientProperty("charset", charSetSpec);
-                } else {
-                    setCharsetFromContentTypeParameters(charSetSpec);
-                }
-                try {
-                    in.reset();
-                } catch (IOException exception) {
-                    //mark was invalidated
-                    in.close();
-                    URL url = (URL)doc.getProperty(Document.StreamDescriptionProperty);
-                    if (url != null) {
-                        URLConnection conn = url.openConnection();
-                        in = conn.getInputStream();
-                    } else {
-                        //there is nothing we can do to recover stream
-                        throw changedCharSetException;
-                    }
-                }
-                try {
-                    doc.remove(0, doc.getLength());
-                } catch (BadLocationException e) {}
-                doc.putProperty("IgnoreCharsetDirective", Boolean.valueOf(true));
-                read(in, doc);
+            kit.read(r, doc, 0);
+        } catch (BadLocationException e) {
+            throw new IOException(e.getMessage());
+        } catch (ChangedCharSetException changedCharSetException) {
+            String charSetSpec = changedCharSetException.getCharSetSpec();
+            if (changedCharSetException.keyEqualsCharSet()) {
+                putClientProperty("charset", charSetSpec);
+            } else {
+                setCharsetFromContentTypeParameters(charSetSpec);
             }
+            try {
+                in.reset();
+            } catch (IOException exception) {
+                //mark was invalidated
+                in.close();
+                URL url = (URL)doc.getProperty(Document.StreamDescriptionProperty);
+                if (url != null) {
+                    URLConnection conn = url.openConnection();
+                    in = conn.getInputStream();
+                } else {
+                    //there is nothing we can do to recover stream
+                    throw changedCharSetException;
+                }
+            }
+            try {
+                doc.remove(0, doc.getLength());
+            } catch (BadLocationException e) {}
+            doc.putProperty("IgnoreCharsetDirective", Boolean.valueOf(true));
+            read(in, doc);
         }
     }
 
@@ -695,11 +693,7 @@ public class JEditorPane extends JTextComponent {
                                 setDocument(doc);
                             }
                         });
-                    } catch (InvocationTargetException ex) {
-                        UIManager.getLookAndFeel().provideErrorFeedback(
-                                                            JEditorPane.this);
-                        return old;
-                    } catch (InterruptedException ex) {
+                    } catch (InvocationTargetException | InterruptedException ex) {
                         UIManager.getLookAndFeel().provideErrorFeedback(
                                                             JEditorPane.this);
                         return old;
@@ -820,9 +814,7 @@ public class JEditorPane extends JTextComponent {
                         handleConnectionProperties(conn);
                     }
                 });
-            } catch (InterruptedException e) {
-                throw new RuntimeException(e);
-            } catch (InvocationTargetException e) {
+            } catch (InterruptedException | InvocationTargetException e) {
                 throw new RuntimeException(e);
             }
         }
@@ -927,7 +919,7 @@ public class JEditorPane extends JTextComponent {
      * Sets the current URL being displayed.
      *
      * @param url the URL for display
-     * @exception IOException for a <code>null</code> or invalid URL
+     * @throws IOException for a <code>null</code> or invalid URL
      *          specification
      */
     public void setPage(String url) throws IOException {
@@ -1063,14 +1055,9 @@ public class JEditorPane extends JTextComponent {
                     putClientProperty("charset", charset);
                 }
             }
-        }
-        catch (IndexOutOfBoundsException e) {
+        } catch (IndexOutOfBoundsException | NullPointerException e) {
             // malformed parameter list, use charset we have
-        }
-        catch (NullPointerException e) {
-            // malformed parameter list, use charset we have
-        }
-        catch (Exception e) {
+        } catch (Exception e) {
             // malformed parameter list, use charset we have; but complain
             System.err.println("JEditorPane.getCharsetFromContentTypeParameters failed on: " + paramlist);
             e.printStackTrace();
@@ -1481,9 +1468,7 @@ public class JEditorPane extends JTextComponent {
             Reader r = new StringReader(t);
             EditorKit kit = getEditorKit();
             kit.read(r, doc, 0);
-        } catch (IOException ioe) {
-            UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
-        } catch (BadLocationException ble) {
+        } catch (IOException | BadLocationException e) {
             UIManager.getLookAndFeel().provideErrorFeedback(JEditorPane.this);
         }
     }
