@@ -219,10 +219,14 @@ public class BasicSplitPaneDivider extends Container
      * Sets the size of the divider to {@code newSize}. That is
      * the width if the splitpane is {@code HORIZONTAL_SPLIT}, or
      * the height of {@code VERTICAL_SPLIT}.
+     * Divider sizes {@code newSize < 0} are ignored.
      *
      * @param newSize a new size
      */
     public void setDividerSize(int newSize) {
+        if (newSize < 0) {
+            return;
+        }
         dividerSize = newSize;
     }
 
@@ -376,9 +380,10 @@ public class BasicSplitPaneDivider extends Container
 
     /**
      * Messaged when the oneTouchExpandable value of the JSplitPane the
-     * receiver is contained in changes. Will create the
-     * <code>leftButton</code> and <code>rightButton</code> if they
-     * are null. invalidates the receiver as well.
+     * divider is contained in changes. Will create the
+     * <code>leftButton</code> and <code>rightButton</code> if they are null
+     * and corresponding JSplitPane supports oneTouchExpandable property.
+     * Invalidates the corresponding JSplitPane as well.
      */
     protected void oneTouchExpandableChanged() {
         if (!DefaultLookup.getBoolean(splitPane, splitPaneUI,
