@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -1214,8 +1214,6 @@ public class FilePane extends JPanel implements PropertyChangeListener {
 
             setText(text);
 
-            putClientProperty("html.disable", getFileChooser().getClientProperty("html.disable"));
-
             return this;
         }
     }
@@ -1604,8 +1602,6 @@ public class FilePane extends JPanel implements PropertyChangeListener {
                 }
             }
 
-            putClientProperty("html.disable", getFileChooser().getClientProperty("html.disable"));
-
             return this;
         }
     }
@@ -1755,11 +1751,12 @@ public class FilePane extends JPanel implements PropertyChangeListener {
     }
 
     private void doMultiSelectionChanged(PropertyChangeEvent e) {
+        clearSelection();
         if (getFileChooser().isMultiSelectionEnabled()) {
             listSelectionModel.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
+            getFileChooser().setSelectedFile(null);
         } else {
             listSelectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
-            clearSelection();
             getFileChooser().setSelectedFiles(null);
         }
     }
