@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,7 +39,6 @@ import java.util.List;
 import java.util.concurrent.ConcurrentLinkedDeque;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Flow;
-import java.util.concurrent.Flow.Subscription;
 
 import jdk.internal.net.http.common.Demand;
 import jdk.internal.net.http.common.HttpBodySubscriberWrapper;
@@ -209,13 +208,12 @@ class Http1Exchange<T> extends ExchangeImpl<T> {
         }
 
         @Override
-        protected void register() {
-            exchange.registerResponseSubscriber(this);
-        }
-
-        @Override
-        protected void unregister() {
-            exchange.unregisterResponseSubscriber(this);
+        protected void complete(Throwable t) {
+            try {
+                exchange.responseSubscriberCompleted(this);
+            } finally {
+                super.complete(t);
+            }
         }
     }
 
@@ -266,7 +264,7 @@ class Http1Exchange<T> extends ExchangeImpl<T> {
     // The Http1ResponseBodySubscriber is registered with the HttpClient
     // to ensure that it gets completed if the SelectorManager aborts due
     // to unexpected exceptions.
-    private boolean registerResponseSubscriber(Http1ResponseBodySubscriber<T> subscriber) {
+    void registerResponseSubscriber(Http1ResponseBodySubscriber<T> subscriber) {
         Throwable failed = null;
         synchronized (lock) {
             failed = this.failed;
@@ -276,14 +274,13 @@ class Http1Exchange<T> extends ExchangeImpl<T> {
         }
         if (failed != null) {
             subscriber.onError(failed);
-            return false;
         } else {
-            return client.registerSubscriber(subscriber);
+            client.registerSubscriber(subscriber);
         }
     }
 
-    private boolean unregisterResponseSubscriber(Http1ResponseBodySubscriber<T> subscriber) {
-        return client.unregisterSubscriber(subscriber);
+    void responseSubscriberCompleted(HttpBodySubscriberWrapper<T> subscriber) {
+        client.subscriberCompleted(subscriber);
     }
 
     @Override
@@ -453,6 +450,7 @@ class Http1Exchange<T> extends ExchangeImpl<T> {
         BodySubscriber<T> subscriber = handler.apply(response);
         Http1ResponseBodySubscriber<T> bs =
                 new Http1ResponseBodySubscriber<T>(subscriber, this);
+        registerResponseSubscriber(bs);
         return bs;
     }
 
