@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -36,12 +36,15 @@ import java.lang.ref.WeakReference;
 import java.net.Authenticator;
 import java.net.ConnectException;
 import java.net.CookieHandler;
+import java.net.InetAddress;
+import java.net.ProtocolException;
 import java.net.ProxySelector;
 import java.net.http.HttpConnectTimeoutException;
 import java.net.http.HttpTimeoutException;
 import java.nio.ByteBuffer;
 import java.nio.channels.CancelledKeyException;
 import java.nio.channels.ClosedChannelException;
+import java.nio.channels.ClosedSelectorException;
 import java.nio.channels.SelectableChannel;
 import java.nio.channels.SelectionKey;
 import java.nio.channels.Selector;
@@ -62,13 +65,18 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.TreeSet;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.ThreadFactory;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.BiConsumer;
 import java.util.function.BooleanSupplier;
 import java.util.stream.Stream;
 import java.net.http.HttpClient;
@@ -77,9 +85,12 @@ import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodyHandler;
 import java.net.http.HttpResponse.PushPromiseHandler;
 import java.net.http.WebSocket;
+
 import jdk.internal.net.http.common.BufferSupplier;
+import jdk.internal.net.http.common.HttpBodySubscriberWrapper;
 import jdk.internal.net.http.common.Log;
 import jdk.internal.net.http.common.Logger;
+import jdk.internal.net.http.common.MinimalFuture;
 import jdk.internal.net.http.common.Pair;
 import jdk.internal.net.http.common.Utils;
 import jdk.internal.net.http.common.OperationTrackers.Trackable;
@@ -100,6 +111,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     final Logger debugelapsed = Utils.getDebugLogger(this::dbgString, DEBUGELAPSED);
     final Logger debugtimeout = Utils.getDebugLogger(this::dbgString, DEBUGTIMEOUT);
     static final AtomicLong CLIENT_IDS = new AtomicLong();
+    private final AtomicLong CONNECTION_IDS = new AtomicLong();
 
     // Define the default factory as a static inner class
     // that embeds all the necessary logic to avoid
@@ -135,27 +147,42 @@ final class HttpClientImpl extends HttpClient implements Trackable {
      * is the SelectorManager thread. If the current thread is not
      * the selector manager thread the given task is executed inline.
      */
-    final static class DelegatingExecutor implements Executor {
+    static final class DelegatingExecutor implements Executor {
         private final BooleanSupplier isInSelectorThread;
         private final Executor delegate;
-        DelegatingExecutor(BooleanSupplier isInSelectorThread, Executor delegate) {
+        private final BiConsumer<Runnable, Throwable> errorHandler;
+        DelegatingExecutor(BooleanSupplier isInSelectorThread,
+                           Executor delegate,
+                           BiConsumer<Runnable, Throwable> errorHandler) {
             this.isInSelectorThread = isInSelectorThread;
             this.delegate = delegate;
+            this.errorHandler = errorHandler;
         }
 
         Executor delegate() {
             return delegate;
         }
 
+
+
         @Override
         public void execute(Runnable command) {
             if (isInSelectorThread.getAsBoolean()) {
-                delegate.execute(command);
+                ensureExecutedAsync(command);
             } else {
                 command.run();
             }
         }
 
+        public void ensureExecutedAsync(Runnable command) {
+            try {
+                delegate.execute(command);
+            } catch (Throwable t) {
+                errorHandler.accept(command, t);
+                ASYNC_POOL.execute(command);
+            }
+        }
+
         @SuppressWarnings("removal")
         private void shutdown() {
             if (delegate instanceof ExecutorService service) {
@@ -167,7 +194,129 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                         new RuntimePermission("modifyThread"));
             }
         }
+    }
 
+    // We maintain a list of pending requests that will be aborted if ever
+    // the selector manager thread exists abnormally.
+    // The request, its id, and its completable future, are stored in a record-like
+    // PendingRequest object added to the pending requests set (pendingRequests).
+    //
+    // When the request's cf completes, either normally or abnormally, a dependent action
+    // will remove the PendingRequest object from the pending requests set.
+    // If the SelectorManager threads exits abnormally, all pending requests in the
+    // pending requests set will be completed exceptionally, in the ASYNC_POOL.
+    //
+    // HttpClientImpl::registerPending(id, req, cf, client) is called from sendAsync
+    // to register the pending request in the pending requests set before returning
+    // the completable future to the caller.
+    //
+    // HttpClientImpl::abortPendingRequests(client, Throwable reason) is called from
+    // the SelectorManager when a throwable is caught just before exiting.
+    //
+    // A dependent action is registered with the pending request's cf to make sure
+    // that the pending request will be removed from the pending requests set if,
+    // or after, the cf is completed.
+    //
+    private final Set<PendingRequest> pendingRequests;
+    // an id to ensure total order of pending request objects
+    private final AtomicLong pendingRequestId = new AtomicLong();
+    private static final class PendingRequest implements Comparable<PendingRequest> {
+        final long id;
+        final HttpRequest request;
+        final CompletableFuture<?> cf;
+        final HttpClientImpl client;
+        final MultiExchange<?> mex;
+        Object ref;
+        private PendingRequest(long id,
+                               HttpRequest request,
+                               CompletableFuture<?> cf,
+                               MultiExchange<?> mex,
+                               HttpClientImpl client) {
+            this.id = id;
+            this.request = request;
+            this.cf = cf;
+            this.mex = mex;
+            this.client = client;
+        }
+
+        public void abort(Throwable t) {
+            try {
+                if (client.isSelectorThread()) {
+                    var done = cf.exceptionally((e) -> null);
+                    ASYNC_POOL.execute(() -> completeExceptionally(t));
+                    // special case for when this method is called in the SelectorManager thread:
+                    // we want to wait until all futures are completed before proceeding to
+                    // shutdown. This ensures that the caller receive the actual `reason`
+                    // and not something like "HTTP/2 client closed"...
+                    done.join();
+                } else {
+                    cf.completeExceptionally(t);
+                }
+            } finally {
+                mex.cancel(Utils.getIOException(t));
+            }
+        }
+
+        private void completeExceptionally(Throwable t) {
+            if (client.debug.on()) {
+                client.debug.log("aborting %s with %s", this, t);
+            }
+            try { cf.completeExceptionally(t); } catch (Throwable e) {
+                client.debug.log("Failed to complete cf for [%s]: %s", this, e);
+            }
+        }
+
+
+        @Override
+        public int compareTo(PendingRequest o) {
+            if (o == null) return 1;
+            return Long.compare(id, o.id);
+        }
+
+        public String toString() {
+            return id + ": " + request.toString();
+        }
+    }
+
+    static void registerPending(PendingRequest pending) {
+        // shortcut if cf is already completed: no need to go through the trouble of
+        //    registering it
+        if (pending.cf.isDone()) return;
+
+        var client = pending.client;
+        var cf = pending.cf;
+        var id = pending.id;
+        boolean added = client.pendingRequests.add(pending);
+        // this may immediately remove `pending` from the set is the cf is already completed
+        pending.ref = cf.whenComplete((r,t) -> client.pendingRequests.remove(pending));
+        assert added : "request %d was already added".formatted(id);
+        // should not happen, unless the selector manager has already
+        // exited abnormally
+        if (client.selmgr.isClosed()) {
+            pending.abort(client.selmgr.selectorClosedException());
+        }
+    }
+
+    static void abortPendingRequests(HttpClientImpl client, Throwable reason) {
+        reason = Utils.getCompletionCause(reason);
+        if (client.debug.on()) {
+            var msg = reason instanceof RejectedExecutionException
+                    ? reason.getClass() : reason;
+            client.debug.log("aborting pending requests due to: %s", msg);
+        }
+        closeSubscribers(client, reason);
+        var pendingRequests = client.pendingRequests;
+        while (!pendingRequests.isEmpty()) {
+            var pendings = pendingRequests.iterator();
+            while (pendings.hasNext()) {
+                var pending = pendings.next();
+                try {
+                    pending.abort(reason);
+                } finally {
+                    pendings.remove();
+                }
+            }
+        }
     }
 
     private final CookieHandler cookieHandler;
@@ -188,6 +337,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     private final Http2ClientImpl client2;
     private final long id;
     private final String dbgTag;
+    private final InetAddress localAddr;
 
     // The SSL DirectBuffer Supplier provides the ability to recycle
     // buffers used between the socket reader and the SSLEngine, or
@@ -206,6 +356,11 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     // have completed.
     private final WeakReference<HttpClientFacade> facadeRef;
 
+    private final ConcurrentSkipListSet<PlainHttpConnection> openedConnections
+            = new ConcurrentSkipListSet<>(HttpConnection.COMPARE_BY_ID);
+    private final ConcurrentSkipListSet<HttpBodySubscriberWrapper<?>> subscribers
+            = new ConcurrentSkipListSet<>(HttpBodySubscriberWrapper.COMPARE_BY_ID);
+
     // This counter keeps track of the number of operations pending
     // on the HttpClient. The SelectorManager thread will wait
     // until there are no longer any pending operations and the
@@ -237,8 +392,11 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     // the response has been fully received or the web socket is closed.
     private final AtomicLong pendingOperationCount = new AtomicLong();
     private final AtomicLong pendingWebSocketCount = new AtomicLong();
+    private final AtomicLong pendingHttpOperationsCount = new AtomicLong();
     private final AtomicLong pendingHttpRequestCount = new AtomicLong();
     private final AtomicLong pendingHttp2StreamCount = new AtomicLong();
+    private final AtomicLong pendingTCPConnectionCount = new AtomicLong();
+    private final AtomicBoolean isAlive = new AtomicBoolean();
 
     /** A Set of, deadline first, ordered timeout events. */
     private final TreeSet<TimeoutEvent> timeouts;
@@ -276,6 +434,17 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                            SingleFacadeFactory facadeFactory) {
         id = CLIENT_IDS.incrementAndGet();
         dbgTag = "HttpClientImpl(" + id +")";
+        @SuppressWarnings("removal")
+        var sm = System.getSecurityManager();
+        if (sm != null && builder.localAddr != null) {
+            // when a specific local address is configured, it will eventually
+            // lead to the SocketChannel.bind(...) call with an InetSocketAddress
+            // whose InetAddress is the local address and the port is 0. That ultimately
+            // leads to a SecurityManager.checkListen permission check for that port.
+            // so we do that security manager check here with port 0.
+            sm.checkListen(0);
+        }
+        localAddr = builder.localAddr;
         if (builder.sslContext == null) {
             try {
                 sslContext = SSLContext.getDefault();
@@ -292,7 +461,9 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         } else {
             isDefaultExecutor = false;
         }
-        delegatingExecutor = new DelegatingExecutor(this::isSelectorThread, ex);
+        pendingRequests = new ConcurrentSkipListSet<>();
+        delegatingExecutor = new DelegatingExecutor(this::isSelectorThread, ex,
+                this::onSubmitFailure);
         facadeRef = new WeakReference<>(facadeFactory.createFacade(this));
         client2 = new Http2ClientImpl(this);
         cookieHandler = builder.cookieHandler;
@@ -331,6 +502,17 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         assert facadeRef.get() != null;
     }
 
+    // called when the facade is GC'ed.
+    // Just wakes up the selector to cleanup...
+    void facadeCleanup() {
+        SelectorManager selmgr = this.selmgr;
+        if (selmgr != null) selmgr.wakeupSelector();
+    }
+
+    void onSubmitFailure(Runnable command, Throwable failure) {
+        selmgr.abort(failure);
+    }
+
     private void start() {
         selmgr.start();
     }
@@ -344,10 +526,48 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         connections.stop();
         // Clears HTTP/2 cache and close its connections.
         client2.stop();
+        // make sure all subscribers are completed
+        closeSubscribers();
+        // close TCP connection if any are still opened
+        openedConnections.forEach(this::closeConnection);
         // shutdown the executor if needed
         if (isDefaultExecutor) delegatingExecutor.shutdown();
     }
 
+    private void closeSubscribers() {
+        if (subscribers.isEmpty()) return;
+        IOException io = selmgr.selectorClosedException();
+        closeSubscribers(this, io);
+    }
+
+    private static void closeSubscribers(HttpClientImpl client, Throwable t) {
+        client.subscribers.forEach(s -> s.onError(t));
+    }
+
+    public void registerSubscriber(HttpBodySubscriberWrapper<?> subscriber) {
+        if (!selmgr.isClosed()) {
+            synchronized (selmgr) {
+                if (!selmgr.isClosed()) {
+                    subscribers.add(subscriber);
+                    return;
+                }
+            }
+        }
+        subscriber.onError(selmgr.selectorClosedException());
+    }
+
+    public void subscriberCompleted(HttpBodySubscriberWrapper<?> subscriber) {
+        subscribers.remove(subscriber);
+    }
+
+    private void closeConnection(HttpConnection conn) {
+        try { conn.close(); } catch (Throwable e) {
+            if (Log.channel()) {
+                Log.logChannel("Failed to close connection: " + e);
+            }
+        }
+    }
+
     private static SSLParameters getDefaultParams(SSLContext ctx) {
         SSLParameters params = ctx.getDefaultSSLParameters();
         return params;
@@ -365,16 +585,46 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return facadeRef.get();
     }
 
-    // Increments the pendingOperationCount.
-    final long reference() {
+    public long newConnectionId() {
+        return CONNECTION_IDS.incrementAndGet();
+    }
+
+    // Increments the pendingTCPConnectionCount
+    public void connectionOpened(PlainHttpConnection plainHttpConnection) {
+        if (openedConnections.add(plainHttpConnection)) {
+            pendingTCPConnectionCount.incrementAndGet();
+        }
+    }
+
+    // Decrements the pendingTCPConnectionCount
+    public void connectionClosed(PlainHttpConnection plainHttpConnection) {
+        if (openedConnections.remove(plainHttpConnection)) {
+            pendingTCPConnectionCount.decrementAndGet();
+        }
+    }
+
+    // Increments the pendingHttpRequestCount and pendingOperationCount.
+    final long requestReference() {
         pendingHttpRequestCount.incrementAndGet();
+        return reference();
+    }
+
+    // Decrements the pendingHttpRequestCount and pendingOperationCount.
+    final long requestUnreference() {
+        pendingHttpRequestCount.decrementAndGet();
+        return unreference();
+    }
+
+    // Increments the pendingHttpOperationsCount and pendingOperationCount.
+    final long reference() {
+        pendingHttpOperationsCount.incrementAndGet();
         return pendingOperationCount.incrementAndGet();
     }
 
-    // Decrements the pendingOperationCount.
+    // Decrements the pendingHttpOperationsCount and pendingOperationCount.
     final long unreference() {
         final long count = pendingOperationCount.decrementAndGet();
-        final long httpCount = pendingHttpRequestCount.decrementAndGet();
+        final long httpCount = pendingHttpOperationsCount.decrementAndGet();
         final long http2Count = pendingHttp2StreamCount.get();
         final long webSocketCount = pendingWebSocketCount.get();
         if (count == 0 && facade() == null) {
@@ -387,17 +637,17 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return count;
     }
 
-    // Increments the pendingOperationCount.
+    // Increments the pendingHttp2StreamCount and pendingOperationCount.
     final long streamReference() {
         pendingHttp2StreamCount.incrementAndGet();
         return pendingOperationCount.incrementAndGet();
     }
 
-    // Decrements the pendingOperationCount.
+    // Decrements the pendingHttp2StreamCount and pendingOperationCount.
     final long streamUnreference() {
         final long count = pendingOperationCount.decrementAndGet();
         final long http2Count = pendingHttp2StreamCount.decrementAndGet();
-        final long httpCount = pendingHttpRequestCount.get();
+        final long httpCount = pendingHttpOperationsCount.get();
         final long webSocketCount = pendingWebSocketCount.get();
         if (count == 0 && facade() == null) {
             selmgr.wakeupSelector();
@@ -409,17 +659,17 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return count;
     }
 
-    // Increments the pendingOperationCount.
+    // Increments the pendingWebSocketCount and pendingOperationCount.
     final long webSocketOpen() {
         pendingWebSocketCount.incrementAndGet();
         return pendingOperationCount.incrementAndGet();
     }
 
-    // Decrements the pendingOperationCount.
+    // Decrements the pendingWebSocketCount and pendingOperationCount.
     final long webSocketClose() {
         final long count = pendingOperationCount.decrementAndGet();
         final long webSocketCount = pendingWebSocketCount.decrementAndGet();
-        final long httpCount = pendingHttpRequestCount.get();
+        final long httpCount = pendingHttpOperationsCount.get();
         final long http2Count = pendingHttp2StreamCount.get();
         if (count == 0 && facade() == null) {
             selmgr.wakeupSelector();
@@ -432,28 +682,41 @@ final class HttpClientImpl extends HttpClient implements Trackable {
     }
 
     // Returns the pendingOperationCount.
+    // Incremented with any operation, whether it's HTTP/1.1, HTTP/2, or WebSocket
     final long referenceCount() {
         return pendingOperationCount.get();
     }
 
-    final static class HttpClientTracker implements Tracker {
+    // Trackers are used in test to verify that an instance of
+    // HttpClient has shutdown correctly, and that all operations
+    // have terminated.
+    static final class HttpClientTracker implements Tracker {
+        final AtomicLong requestCount;
         final AtomicLong httpCount;
         final AtomicLong http2Count;
         final AtomicLong websocketCount;
         final AtomicLong operationsCount;
+        final AtomicLong connnectionsCount;
         final Reference<?> reference;
+        final AtomicBoolean isAlive;
         final String name;
-        HttpClientTracker(AtomicLong http,
+        HttpClientTracker(AtomicLong request,
+                          AtomicLong http,
                           AtomicLong http2,
                           AtomicLong ws,
                           AtomicLong ops,
+                          AtomicLong conns,
                           Reference<?> ref,
+                          AtomicBoolean isAlive,
                           String name) {
+            this.requestCount = request;
             this.httpCount = http;
             this.http2Count = http2;
             this.websocketCount = ws;
             this.operationsCount = ops;
+            this.connnectionsCount = conns;
             this.reference = ref;
+            this.isAlive = isAlive;
             this.name = name;
         }
         @Override
@@ -461,6 +724,12 @@ final class HttpClientImpl extends HttpClient implements Trackable {
             return operationsCount.get();
         }
         @Override
+        public long getOutstandingHttpRequests() {
+            return requestCount.get();
+        }
+        @Override
+        public long getOutstandingTcpConnections() { return connnectionsCount.get();}
+        @Override
         public long getOutstandingHttpOperations() {
             return httpCount.get();
         }
@@ -472,28 +741,33 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         }
         @Override
         public boolean isFacadeReferenced() {
-            return reference.get() != null;
+            return !reference.refersTo(null);
         }
         @Override
+        public boolean isSelectorAlive() { return isAlive.get(); }
+        @Override
         public String getName() {
             return name;
         }
     }
 
     public Tracker getOperationsTracker() {
-        return new HttpClientTracker(pendingHttpRequestCount,
+        return new HttpClientTracker(
+                pendingHttpRequestCount,
+                pendingHttpOperationsCount,
                 pendingHttp2StreamCount,
                 pendingWebSocketCount,
                 pendingOperationCount,
+                pendingTCPConnectionCount,
                 facadeRef,
+                isAlive,
                 dbgTag);
     }
 
     // Called by the SelectorManager thread to figure out whether it's time
     // to terminate.
-    final boolean isReferenced() {
-        HttpClient facade = facade();
-        return facade != null || referenceCount() > 0;
+    boolean isReferenced() {
+        return !facadeRef.refersTo(null) || referenceCount() > 0;
     }
 
     /**
@@ -523,6 +797,14 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return Thread.currentThread() == selmgr;
     }
 
+    boolean isSelectorClosed() {
+        return selmgr.isClosed();
+    }
+
+    IOException selectorClosedException() {
+        return selmgr.selectorClosedException();
+    }
+
     Http2ClientImpl client2() {
         return client2;
     }
@@ -573,13 +855,13 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                 throw ce;
             } else if (throwable instanceof SSLHandshakeException) {
                 // special case for SSLHandshakeException
-                SSLHandshakeException he = new SSLHandshakeException(msg);
-                he.initCause(throwable);
-                throw he;
+                throw new SSLHandshakeException(msg, throwable);
             } else if (throwable instanceof SSLException) {
                 // any other SSLException is wrapped in a plain
                 // SSLException
                 throw new SSLException(msg, throwable);
+            } else if (throwable instanceof ProtocolException) {
+                throw new ProtocolException(msg);
             } else if (throwable instanceof IOException) {
                 throw new IOException(msg, throwable);
             } else {
@@ -615,6 +897,12 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         Objects.requireNonNull(userRequest);
         Objects.requireNonNull(responseHandler);
 
+        // should not happen, unless the selector manager has
+        // exited abnormally
+        if (selmgr.isClosed()) {
+            return MinimalFuture.failedFuture(selmgr.selectorClosedException());
+        }
+
         AccessControlContext acc = null;
         if (System.getSecurityManager() != null)
             acc = AccessController.getContext();
@@ -624,8 +912,9 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         if (requestImpl.method().equals("CONNECT"))
             throw new IllegalArgumentException("Unsupported method CONNECT");
 
+        long id = pendingRequestId.incrementAndGet();
         long start = DEBUGELAPSED ? System.nanoTime() : 0;
-        reference();
+        requestReference();
         try {
             if (debugelapsed.on())
                 debugelapsed.log("ClientImpl (async) send %s", userRequest);
@@ -648,8 +937,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                                                             responseHandler,
                                                             pushPromiseHandler,
                                                             acc);
-            CompletableFuture<HttpResponse<T>> res =
-                    mex.responseAsync(executor).whenComplete((b,t) -> unreference());
+            CompletableFuture<HttpResponse<T>> mexCf = mex.responseAsync(executor);
+            CompletableFuture<HttpResponse<T>> res = mexCf.whenComplete((b,t) -> requestUnreference());
             if (DEBUGELAPSED) {
                 res = res.whenComplete(
                         (b,t) -> debugCompleted("ClientImpl (async)", start, userRequest));
@@ -661,16 +950,21 @@ final class HttpClientImpl extends HttpClient implements Trackable {
             if (exchangeExecutor != null) {
                 res = res.whenCompleteAsync((r, t) -> { /* do nothing */}, ASYNC_POOL);
             }
+
+            // The mexCf is the Cf we need to abort if the SelectorManager thread
+            // is aborted.
+            PendingRequest pending = new PendingRequest(id, requestImpl, mexCf, mex, this);
+            registerPending(pending);
             return res;
         } catch(Throwable t) {
-            unreference();
+            requestUnreference();
             debugCompleted("ClientImpl (async)", start, userRequest);
             throw t;
         }
     }
 
     // Main loop for this client's selector
-    private final static class SelectorManager extends Thread {
+    private static final class SelectorManager extends Thread {
 
         // For testing purposes we have an internal System property that
         // can control the frequency at which the selector manager will wake
@@ -703,8 +997,9 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         private final List<AsyncTriggerEvent> deregistrations;
         private final Logger debug;
         private final Logger debugtimeout;
-        HttpClientImpl owner;
-        ConnectionPool pool;
+        private final HttpClientImpl owner;
+        private final ConnectionPool pool;
+        private final AtomicReference<Throwable> errorRef = new AtomicReference<>();
 
         SelectorManager(HttpClientImpl ref) throws IOException {
             super(null, null,
@@ -719,13 +1014,22 @@ final class HttpClientImpl extends HttpClient implements Trackable {
             selector = Selector.open();
         }
 
+        IOException selectorClosedException() {
+            var io = new IOException("selector manager closed");
+            var cause = errorRef.get();
+            if (cause != null) {
+                io.initCause(cause);
+            }
+            return io;
+        }
+
         void eventUpdated(AsyncEvent e) throws ClosedChannelException {
             if (Thread.currentThread() == this) {
                 SelectionKey key = e.channel().keyFor(selector);
                 if (key != null && key.isValid()) {
                     SelectorAttachment sa = (SelectorAttachment) key.attachment();
                     sa.register(e);
-                } else if (e.interestOps() != 0){
+                } else if (e.interestOps() != 0) {
                     // We don't care about paused events.
                     // These are actually handled by
                     // SelectorAttachment::resetInterestOps later on.
@@ -741,43 +1045,97 @@ final class HttpClientImpl extends HttpClient implements Trackable {
 
         // This returns immediately. So caller not allowed to send/receive
         // on connection.
-        synchronized void register(AsyncEvent e) {
-            registrations.add(e);
-            selector.wakeup();
-        }
-
-        synchronized void cancel(SocketChannel e) {
-            SelectionKey key = e.keyFor(selector);
-            if (key != null) {
-                key.cancel();
+        void register(AsyncEvent e) {
+            var closed = this.closed;
+            if (!closed) {
+                synchronized (this) {
+                    closed = this.closed;
+                    if (!closed) {
+                        registrations.add(e);
+                    }
+                }
+            }
+            if (closed) {
+                e.abort(selectorClosedException());
+            } else {
+                selector.wakeup();
             }
-            selector.wakeup();
         }
 
         void wakeupSelector() {
             selector.wakeup();
         }
 
-        synchronized void shutdown() {
-            Log.logTrace("{0}: shutting down", getName());
-            if (debug.on()) debug.log("SelectorManager shutting down");
-            closed = true;
+        void abort(Throwable t) {
+            boolean closed = this.closed;
+            errorRef.compareAndSet(null, t);
+            if (debug.on()) {
+                debug.log("aborting selector manager(closed=%s): " + t, closed);
+            }
+            t = errorRef.get();
+            boolean inSelectorThread = owner.isSelectorThread();
+            if (!inSelectorThread) {
+                // abort anything pending, then close
+                abortPendingRequests(owner, t);
+            }
+            Set<SelectionKey> keys = new HashSet<>();
+            Set<AsyncEvent> toAbort = new HashSet<>();
+            synchronized (this) {
+                if (closed = this.closed) return;
+                this.closed = true;
+                try {
+                    keys.addAll(selector.keys());
+                } catch (ClosedSelectorException ce) {
+                    // OK - nothing to do...
+                }
+                toAbort.addAll(this.registrations);
+                toAbort.addAll(this.deregistrations);
+                this.registrations.clear();
+                this.deregistrations.clear();
+            }
+            // double check after closing
+            abortPendingRequests(owner, t);
+
+            IOException io = toAbort.isEmpty()
+                    ? null : selectorClosedException();
+            for (AsyncEvent e : toAbort) {
+                try {
+                    e.abort(io);
+                } catch (Throwable x) {
+                    debug.log("Failed to abort event: " + x);
+                }
+            }
+            if (!inSelectorThread) selector.wakeup();
+        }
+
+        // Only called by the selector manager thread
+        private void shutdown() {
             try {
-                selector.close();
+                synchronized (this) {
+                    Log.logTrace("{0}: shutting down", getName());
+                    if (debug.on()) debug.log("SelectorManager shutting down");
+                    closed = true;
+                    selector.close();
+                }
             } catch (IOException ignored) {
             } finally {
                 owner.stop();
             }
         }
 
+        boolean isClosed() {
+            return closed;
+        }
+
         @Override
         public void run() {
             List<Pair<AsyncEvent,IOException>> errorList = new ArrayList<>();
             List<AsyncEvent> readyList = new ArrayList<>();
             List<Runnable> resetList = new ArrayList<>();
+            owner.isAlive.set(true);
             try {
                 if (Log.channel()) Log.logChannel(getName() + ": starting");
-                while (!Thread.currentThread().isInterrupted()) {
+                while (!Thread.currentThread().isInterrupted() && !closed) {
                     synchronized (this) {
                         assert errorList.isEmpty();
                         assert readyList.isEmpty();
@@ -810,7 +1168,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                                 // may throw IOE if channel closed: that's OK
                                 sa.register(event);
                                 if (!chan.isOpen()) {
-                                    throw new IOException("Channel closed");
+                                    throw new ClosedChannelException();
                                 }
                             } catch (IOException e) {
                                 Log.logTrace("{0}: {1}", getName(), e);
@@ -929,7 +1287,8 @@ final class HttpClientImpl extends HttpClient implements Trackable {
                     selector.selectedKeys().clear();
 
                     // handle selected events
-                    readyList.forEach((e) -> handleEvent(e, null));
+                    IOException ioe = closed ? selectorClosedException() : null;
+                    readyList.forEach((e) -> handleEvent(e, ioe));
                     readyList.clear();
 
                     // handle errors (closed channels etc...)
@@ -942,19 +1301,26 @@ final class HttpClientImpl extends HttpClient implements Trackable {
 
                 }
             } catch (Throwable e) {
+                errorRef.compareAndSet(null, e);
                 if (!closed) {
+                    closed = true; // set closed early so that new requests are rejected
                     // This terminates thread. So, better just print stack trace
                     String err = Utils.stackTrace(e);
                     Log.logError("{0}: {1}: {2}", getName(),
                             "HttpClientImpl shutting down due to fatal error", err);
                 }
+                abortPendingRequests(owner, selectorClosedException());
                 if (debug.on()) debug.log("shutting down", e);
                 if (Utils.ASSERTIONSENABLED && !debug.on()) {
                     e.printStackTrace(System.err); // always print the stack
                 }
             } finally {
                 if (Log.channel()) Log.logChannel(getName() + ": stopping");
-                shutdown();
+                try {
+                    shutdown();
+                } finally {
+                    owner.isAlive.set(false);
+                }
             }
         }
 
@@ -971,7 +1337,10 @@ final class HttpClientImpl extends HttpClient implements Trackable {
 
         /** Handles the given event. The given ioe may be null. */
         void handleEvent(AsyncEvent event, IOException ioe) {
-            if (closed || ioe != null) {
+            if (ioe == null && closed) {
+                ioe = selectorClosedException();
+            }
+            if (ioe != null) {
                 event.abort(ioe);
             } else {
                 event.handle();
@@ -1007,7 +1376,7 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         private final SelectableChannel chan;
         private final Selector selector;
         private final Set<AsyncEvent> pending;
-        private final static Logger debug =
+        private static final Logger debug =
                 Utils.getDebugLogger("SelectorAttachment"::toString, Utils.DEBUG);
         private int interestOps;
 
@@ -1181,6 +1550,10 @@ final class HttpClientImpl extends HttpClient implements Trackable {
         return version;
     }
 
+    InetAddress localAddress() {
+        return localAddr;
+    }
+
     String dbgString() {
         return dbgTag;
     }
