@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,8 +30,9 @@ import java.io.IOException;
 import java.io.UncheckedIOException;
 import java.lang.invoke.MethodHandles;
 import java.lang.invoke.VarHandle;
-import java.net.ProtocolException;
 import java.net.URI;
+import java.net.http.HttpResponse.BodyHandler;
+import java.net.http.HttpResponse.ResponseInfo;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -42,7 +43,6 @@ import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Flow;
 import java.util.concurrent.Flow.Subscription;
-import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.BiPredicate;
 import java.net.http.HttpClient;
@@ -50,13 +50,10 @@ import java.net.http.HttpHeaders;
 import java.net.http.HttpRequest;
 import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodySubscriber;
-
 import jdk.internal.net.http.common.*;
 import jdk.internal.net.http.frame.*;
 import jdk.internal.net.http.hpack.DecodingCallback;
 
-import static jdk.internal.net.http.Exchange.MAX_NON_FINAL_RESPONSES;
-
 /**
  * Http/2 Stream handling.
  *
@@ -140,10 +137,6 @@ class Stream<T> extends ExchangeImpl<T> {
     private volatile boolean remotelyClosed;
     private volatile boolean closed;
     private volatile boolean endStreamSent;
-    private volatile boolean finalResponseCodeReceived;
-    private volatile boolean trailerReceived;
-    private AtomicInteger nonFinalResponseCount = new AtomicInteger();
-
     // Indicates the first reason that was invoked when sending a ResetFrame
     // to the server. A streamState of 0 indicates that no reset was sent.
     // (see markStream(int code)
@@ -308,6 +301,16 @@ class Stream<T> extends ExchangeImpl<T> {
         return endStream;
     }
 
+    @Override
+    void expectContinueFailed(int rcode) {
+        // Have to mark request as sent, due to no request body being sent in the
+        // event of a 417 Expectation Failed or some other non 100 response code
+        requestSent();
+    }
+
+    // This method is called by Http2Connection::decrementStreamCount in order
+    // to make sure that the stream count is decremented only once for
+    // a given stream.
     boolean deRegister() {
         return DEREGISTERED.compareAndSet(this, false, true);
     }
@@ -320,7 +323,8 @@ class Stream<T> extends ExchangeImpl<T> {
         try {
             Log.logTrace("Reading body on stream {0}", streamid);
             debug.log("Getting BodySubscriber for: " + response);
-            BodySubscriber<T> bodySubscriber = handler.apply(new ResponseInfoImpl(response));
+            Http2StreamResponseSubscriber<T> bodySubscriber =
+                    createResponseSubscriber(handler, new ResponseInfoImpl(response));
             CompletableFuture<T> cf = receiveData(bodySubscriber, executor);
 
             PushGroup<?> pg = exchange.getPushGroup();
@@ -336,6 +340,25 @@ class Stream<T> extends ExchangeImpl<T> {
         }
     }
 
+    @Override
+    Http2StreamResponseSubscriber<T> createResponseSubscriber(BodyHandler<T> handler, ResponseInfo response) {
+        Http2StreamResponseSubscriber<T> subscriber =
+                new Http2StreamResponseSubscriber<>(handler.apply(response));
+        registerResponseSubscriber(subscriber);
+        return subscriber;
+    }
+
+    // The Http2StreamResponseSubscriber is registered with the HttpClient
+    // to ensure that it gets completed if the SelectorManager aborts due
+    // to unexpected exceptions.
+    private void registerResponseSubscriber(Http2StreamResponseSubscriber<?> subscriber) {
+        client().registerSubscriber(subscriber);
+    }
+
+    private void subscriberCompleted(Http2StreamResponseSubscriber<?> subscriber) {
+        client().subscriberCompleted(subscriber);
+    }
+
     @Override
     public String toString() {
         return "streamid: " + streamid;
@@ -393,10 +416,13 @@ class Stream<T> extends ExchangeImpl<T> {
         if (isCanceled()) {
             Throwable t = getCancelCause();
             responseBodyCF.completeExceptionally(t);
-        } else {
-            pendingResponseSubscriber = bodySubscriber;
-            sched.runOrSchedule(); // in case data waiting already to be processed
         }
+
+        // ensure that the body subscriber will be subsribed and onError() is
+        // invoked
+        pendingResponseSubscriber = bodySubscriber;
+        sched.runOrSchedule(); // in case data waiting already to be processed, or error
+
         return responseBodyCF;
     }
 
@@ -471,89 +497,32 @@ class Stream<T> extends ExchangeImpl<T> {
         return rspHeadersConsumer;
     }
 
-    String checkInterimResponseCountExceeded() {
-        // this is also checked by Exchange - but tracking it here too provides
-        // a more informative message.
-        int count = nonFinalResponseCount.incrementAndGet();
-        if (MAX_NON_FINAL_RESPONSES > 0 && (count < 0 || count > MAX_NON_FINAL_RESPONSES)) {
-            return String.format(
-                    "Stream %s PROTOCOL_ERROR: too many interim responses received: %s > %s",
-                    streamid, count, MAX_NON_FINAL_RESPONSES);
-        }
-        return null;
-    }
-
     protected void handleResponse() throws IOException {
         HttpHeaders responseHeaders = responseHeadersBuilder.build();
+        responseCode = (int)responseHeaders
+                .firstValueAsLong(":status")
+                .orElseThrow(() -> new IOException("no statuscode in response"));
 
-        if (!finalResponseCodeReceived) {
-            try {
-                responseCode = (int) responseHeaders
-                        .firstValueAsLong(":status")
-                        .orElseThrow(() -> new ProtocolException(String.format(
-                                "Stream %s PROTOCOL_ERROR: no status code in response",
-                                streamid)));
-            } catch (ProtocolException cause) {
-                cancelImpl(cause, ResetFrame.PROTOCOL_ERROR);
-                rspHeadersConsumer.reset();
-                return;
-            }
+        response = new Response(
+                request, exchange, responseHeaders, connection(),
+                responseCode, HttpClient.Version.HTTP_2);
 
-            String protocolErrorMsg = null;
-            // If informational code, response is partially complete
-            if (responseCode < 100 || responseCode > 199) {
-                this.finalResponseCodeReceived = true;
-            } else {
-                protocolErrorMsg = checkInterimResponseCountExceeded();
-            }
+        /* TODO: review if needs to be removed
+           the value is not used, but in case `content-length` doesn't parse as
+           long, there will be NumberFormatException. If left as is, make sure
+           code up the stack handles NFE correctly. */
+        responseHeaders.firstValueAsLong("content-length");
 
-            if (protocolErrorMsg != null) {
-                if (debug.on()) {
-                    debug.log(protocolErrorMsg);
-                }
-                cancelImpl(new ProtocolException(protocolErrorMsg), ResetFrame.PROTOCOL_ERROR);
-                rspHeadersConsumer.reset();
-                return;
-            }
-
-            response = new Response(
-                    request, exchange, responseHeaders, connection(),
-                    responseCode, HttpClient.Version.HTTP_2);
-
-            /* TODO: review if needs to be removed
-               the value is not used, but in case `content-length` doesn't parse as
-               long, there will be NumberFormatException. If left as is, make sure
-               code up the stack handles NFE correctly. */
-            responseHeaders.firstValueAsLong("content-length");
-
-            if (Log.headers()) {
-                StringBuilder sb = new StringBuilder("RESPONSE HEADERS:\n");
-                Log.dumpHeaders(sb, "    ", responseHeaders);
-                Log.logHeaders(sb.toString());
-            }
-
-            // this will clear the response headers
-            rspHeadersConsumer.reset();
-
-            completeResponse(response);
-        } else {
-            if (Log.headers()) {
-                StringBuilder sb = new StringBuilder("TRAILING HEADERS:\n");
-                Log.dumpHeaders(sb, "    ", responseHeaders);
-                Log.logHeaders(sb.toString());
-            }
-            if (trailerReceived) {
-                String protocolErrorMsg = String.format(
-                        "Stream %s PROTOCOL_ERROR: trailers already received", streamid);
-                if (debug.on()) {
-                    debug.log(protocolErrorMsg);
-                }
-                cancelImpl(new ProtocolException(protocolErrorMsg), ResetFrame.PROTOCOL_ERROR);
-            }
-            trailerReceived = true;
-            rspHeadersConsumer.reset();
+        if (Log.headers()) {
+            StringBuilder sb = new StringBuilder("RESPONSE HEADERS:\n");
+            Log.dumpHeaders(sb, "    ", responseHeaders);
+            Log.logHeaders(sb.toString());
         }
 
+        // this will clear the response headers
+        rspHeadersConsumer.reset();
+
+        completeResponse(response);
     }
 
     void incoming_reset(ResetFrame frame) {
@@ -1108,7 +1077,7 @@ class Stream<T> extends ExchangeImpl<T> {
 
     /**
      * A List of responses relating to this stream. Normally there is only
-     * one response, but interim responses like 100 are allowed
+     * one response, but intermediate responses like 100 are allowed
      * and must be passed up to higher level before continuing. Deals with races
      * such as if responses are returned before the CFs get created by
      * getResponseAsync()
@@ -1293,7 +1262,7 @@ class Stream<T> extends ExchangeImpl<T> {
         cancelImpl(e, ResetFrame.CANCEL);
     }
 
-    void cancelImpl(final Throwable e, final int resetFrameErrCode) {
+    private void cancelImpl(final Throwable e, final int resetFrameErrCode) {
         errorRef.compareAndSet(null, e);
         if (debug.on()) {
             if (streamid == 0) debug.log("cancelling stream: %s", (Object)e);
@@ -1362,7 +1331,6 @@ class Stream<T> extends ExchangeImpl<T> {
     }
 
     static class PushedStream<T> extends Stream<T> {
-        final Stream<T> parent;
         final PushGroup<T> pushGroup;
         // push streams need the response CF allocated up front as it is
         // given directly to user via the multi handler callback function.
@@ -1370,19 +1338,17 @@ class Stream<T> extends ExchangeImpl<T> {
         CompletableFuture<HttpResponse<T>> responseCF;
         final HttpRequestImpl pushReq;
         HttpResponse.BodyHandler<T> pushHandler;
-        private volatile boolean finalPushResponseCodeReceived;
 
-        PushedStream(Stream<T> parent,
-                     PushGroup<T> pushGroup,
+        PushedStream(PushGroup<T> pushGroup,
                      Http2Connection connection,
                      Exchange<T> pushReq) {
             // ## no request body possible, null window controller
             super(connection, pushReq, null);
-            this.parent = parent;
             this.pushGroup = pushGroup;
             this.pushReq = pushReq.request();
             this.pushCF = new MinimalFuture<>();
             this.responseCF = new MinimalFuture<>();
+
         }
 
         CompletableFuture<HttpResponse<T>> responseCF() {
@@ -1468,57 +1434,35 @@ class Stream<T> extends ExchangeImpl<T> {
         @Override
         protected void handleResponse() {
             HttpHeaders responseHeaders = responseHeadersBuilder.build();
+            responseCode = (int)responseHeaders
+                .firstValueAsLong(":status")
+                .orElse(-1);
 
-            if (!finalPushResponseCodeReceived) {
-                responseCode = (int)responseHeaders
-                    .firstValueAsLong(":status")
-                    .orElse(-1);
-
-                if (responseCode == -1) {
-                    cancelImpl(new ProtocolException("No status code"), ResetFrame.PROTOCOL_ERROR);
-                    rspHeadersConsumer.reset();
-                    return;
-                } else if (responseCode >= 100 && responseCode < 200) {
-                    String protocolErrorMsg = checkInterimResponseCountExceeded();
-                    if (protocolErrorMsg != null) {
-                        cancelImpl(new ProtocolException(protocolErrorMsg), ResetFrame.PROTOCOL_ERROR);
-                        rspHeadersConsumer.reset();
-                        return;
-                    }
-                }
-
-                this.finalPushResponseCodeReceived = true;
-
-                this.response = new Response(
-                        pushReq, exchange, responseHeaders, connection(),
-                        responseCode, HttpClient.Version.HTTP_2);
-
-                /* TODO: review if needs to be removed
-                   the value is not used, but in case `content-length` doesn't parse
-                   as long, there will be NumberFormatException. If left as is, make
-                   sure code up the stack handles NFE correctly. */
-                responseHeaders.firstValueAsLong("content-length");
-
-                if (Log.headers()) {
-                    StringBuilder sb = new StringBuilder("RESPONSE HEADERS");
-                    sb.append(" (streamid=").append(streamid).append("):\n");
-                    Log.dumpHeaders(sb, "    ", responseHeaders);
-                    Log.logHeaders(sb.toString());
-                }
-
-                rspHeadersConsumer.reset();
-
-                // different implementations for normal streams and pushed streams
-                completeResponse(response);
-            } else {
-                if (Log.headers()) {
-                    StringBuilder sb = new StringBuilder("TRAILING HEADERS");
-                    sb.append(" (streamid=").append(streamid).append("):\n");
-                    Log.dumpHeaders(sb, "    ", responseHeaders);
-                    Log.logHeaders(sb.toString());
-                }
-                rspHeadersConsumer.reset();
+            if (responseCode == -1) {
+                completeResponseExceptionally(new IOException("No status code"));
             }
+
+            this.response = new Response(
+                pushReq, exchange, responseHeaders, connection(),
+                responseCode, HttpClient.Version.HTTP_2);
+
+            /* TODO: review if needs to be removed
+               the value is not used, but in case `content-length` doesn't parse
+               as long, there will be NumberFormatException. If left as is, make
+               sure code up the stack handles NFE correctly. */
+            responseHeaders.firstValueAsLong("content-length");
+
+            if (Log.headers()) {
+                StringBuilder sb = new StringBuilder("RESPONSE HEADERS");
+                sb.append(" (streamid=").append(streamid).append("):\n");
+                Log.dumpHeaders(sb, "    ", responseHeaders);
+                Log.logHeaders(sb.toString());
+            }
+
+            rspHeadersConsumer.reset();
+
+            // different implementations for normal streams and pushed streams
+            completeResponse(response);
         }
     }
 
@@ -1566,12 +1510,9 @@ class Stream<T> extends ExchangeImpl<T> {
         return connection.dbgString() + "/Stream("+streamid+")";
     }
 
-    private class HeadersConsumer extends ValidatingHeadersConsumer implements DecodingCallback {
+    private class HeadersConsumer extends Http2Connection.ValidatingHeadersConsumer {
 
-        boolean maxHeaderListSizeReached;
-
-        @Override
-        public void reset() {
+        void reset() {
             super.reset();
             responseHeadersBuilder.clear();
             debug.log("Response builder cleared, ready to receive new headers.");
@@ -1581,46 +1522,28 @@ class Stream<T> extends ExchangeImpl<T> {
         public void onDecoded(CharSequence name, CharSequence value)
             throws UncheckedIOException
         {
-            if (maxHeaderListSizeReached) {
-                return;
-            }
-            try {
-                String n = name.toString();
-                String v = value.toString();
-                super.onDecoded(n, v);
-                responseHeadersBuilder.addHeader(n, v);
-                if (Log.headers() && Log.trace()) {
-                    Log.logTrace("RECEIVED HEADER (streamid={0}): {1}: {2}",
-                            streamid, n, v);
-                }
-            } catch (UncheckedIOException uio) {
-                // reset stream: From RFC 9113, section 8.1
-                // Malformed requests or responses that are detected MUST be
-                // treated as a stream error (Section 5.4.2) of type
-                // PROTOCOL_ERROR.
-                onProtocolError(uio.getCause());
+            String n = name.toString();
+            String v = value.toString();
+            super.onDecoded(n, v);
+            responseHeadersBuilder.addHeader(n, v);
+            if (Log.headers() && Log.trace()) {
+                Log.logTrace("RECEIVED HEADER (streamid={0}): {1}: {2}",
+                             streamid, n, v);
             }
         }
+    }
+
+    final class Http2StreamResponseSubscriber<U> extends HttpBodySubscriberWrapper<U> {
+        Http2StreamResponseSubscriber(BodySubscriber<U> subscriber) {
+            super(subscriber);
+        }
 
         @Override
-        protected String formatMessage(String message, String header) {
-            return "malformed response: " + super.formatMessage(message, header);
-        }
-
-        @Override
-        public void onMaxHeaderListSizeReached(long size, int maxHeaderListSize) throws ProtocolException {
-            if (maxHeaderListSizeReached) return;
+        protected void complete(Throwable t) {
             try {
-                DecodingCallback.super.onMaxHeaderListSizeReached(size, maxHeaderListSize);
-            } catch (ProtocolException cause) {
-                maxHeaderListSizeReached = true;
-                // If this is a push stream: cancel the parent.
-                if (Stream.this instanceof Stream.PushedStream<?> ps) {
-                    ps.parent.onProtocolError(cause);
-                }
-                // cancel the stream, continue processing
-                onProtocolError(cause);
-                reset();
+                Stream.this.subscriberCompleted(this);
+            } finally {
+                super.complete(t);
             }
         }
     }
