@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -607,12 +607,7 @@ public class KeyStoreLoginModule implements LoginModule {
                                 ("Incorrect keyStoreURL option");
             le.initCause(e);
             throw le;
-        } catch (GeneralSecurityException e) {
-            LoginException le = new LoginException
-                                ("Error initializing keystore");
-            le.initCause(e);
-            throw le;
-        } catch (IOException e) {
+        } catch (GeneralSecurityException | IOException e) {
             LoginException le = new LoginException
                                 ("Error initializing keystore");
             le.initCause(e);
@@ -667,21 +662,15 @@ public class KeyStoreLoginModule implements LoginModule {
             principal = certificate.getSubjectX500Principal();
 
             // if token, privateKeyPassword will be null
-            Key privateKey = keyStore.getKey(keyStoreAlias, privateKeyPassword);
-            if (privateKey == null
-                || !(privateKey instanceof PrivateKey))
-            {
+            Key key = keyStore.getKey(keyStoreAlias, privateKeyPassword);
+            if (!(key instanceof PrivateKey privateKey)) {
                 throw new FailedLoginException(
                     "Unable to recover key from keystore");
             }
 
             privateCredential = new X500PrivateCredential(
-                certificate, (PrivateKey) privateKey, keyStoreAlias);
-        } catch (KeyStoreException e) {
-            LoginException le = new LoginException("Error using keystore");
-            le.initCause(e);
-            throw le;
-        } catch (NoSuchAlgorithmException e) {
+                certificate, privateKey, keyStoreAlias);
+        } catch (KeyStoreException | NoSuchAlgorithmException e) {
             LoginException le = new LoginException("Error using keystore");
             le.initCause(e);
             throw le;
@@ -712,7 +701,7 @@ public class KeyStoreLoginModule implements LoginModule {
      * {@code login} method), then this method associates a
      * {@code X500Principal} for the subject distinguished name of the
      * first certificate in the alias's credentials in the subject's
-     * principals,the alias's certificate path in the subject's public
+     * principals, the alias's certificate path in the subject's public
      * credentials, and a {@code X500PrivateCredential} whose certificate
      * is the first  certificate in the alias's certificate path and whose
      * private key is the alias's private key in the subject's private
