@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,25 +23,25 @@
  */
 package com.sun.hotspot.igv.view;
 
-import com.sun.hotspot.igv.data.ChangedListener;
-import com.sun.hotspot.igv.data.ControllableChangedListener;
-import com.sun.hotspot.igv.data.InputBlock;
-import com.sun.hotspot.igv.data.InputNode;
-import com.sun.hotspot.igv.data.Pair;
 import com.sun.hotspot.igv.data.Properties;
-import com.sun.hotspot.igv.data.services.Scheduler;
+import com.sun.hotspot.igv.data.*;
 import com.sun.hotspot.igv.graph.*;
+import com.sun.hotspot.igv.hierarchicallayout.HierarchicalCFGLayoutManager;
 import com.sun.hotspot.igv.hierarchicallayout.HierarchicalClusterLayoutManager;
 import com.sun.hotspot.igv.hierarchicallayout.HierarchicalLayoutManager;
+import com.sun.hotspot.igv.hierarchicallayout.LinearLayoutManager;
 import com.sun.hotspot.igv.layout.LayoutGraph;
 import com.sun.hotspot.igv.selectioncoordinator.SelectionCoordinator;
 import com.sun.hotspot.igv.util.ColorIcon;
 import com.sun.hotspot.igv.util.DoubleClickAction;
 import com.sun.hotspot.igv.util.PropertiesSheet;
+import com.sun.hotspot.igv.view.actions.CustomSelectAction;
 import com.sun.hotspot.igv.view.actions.CustomizablePanAction;
 import com.sun.hotspot.igv.view.widgets.*;
 import java.awt.*;
-import java.awt.event.*;
+import java.awt.event.ActionEvent;
+import java.awt.event.KeyEvent;
+import java.awt.event.MouseEvent;
 import java.util.List;
 import java.util.*;
 import javax.swing.*;
@@ -60,6 +60,7 @@ import org.openide.nodes.AbstractNode;
 import org.openide.nodes.Children;
 import org.openide.nodes.Sheet;
 import org.openide.util.Lookup;
+import org.openide.util.Utilities;
 import org.openide.util.lookup.AbstractLookup;
 import org.openide.util.lookup.InstanceContent;
 
@@ -85,7 +86,6 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
     private Widget bottomRight;
     private DiagramViewModel model;
     private DiagramViewModel modelCopy;
-    private WidgetAction zoomAction;
     private boolean rebuilding;
 
     /**
@@ -261,16 +261,6 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
         }
     };
 
-    private MouseWheelListener mouseWheelListener = new MouseWheelListener() {
-
-        @Override
-        public void mouseWheelMoved(MouseWheelEvent e) {
-            if (e.isControlDown()) {
-                DiagramScene.this.relayoutWithoutLayout(null);
-            }
-        }
-    };
-
     public Point getScrollPosition() {
         return getScrollPane().getViewport().getViewPosition();
     }
@@ -400,10 +390,36 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
         // This panAction handles the event only when the left mouse button is
         // pressed without any modifier keys, otherwise it will not consume it
         // and the selection action (below) will handle the event
-        panAction = new CustomizablePanAction(~0, MouseEvent.BUTTON1_DOWN_MASK);
+        panAction = new CustomizablePanAction(MouseEvent.BUTTON1_DOWN_MASK);
         this.getActions().addAction(panAction);
 
-        selectAction = createSelectAction();
+        selectAction = new CustomSelectAction(new SelectProvider() {
+            public boolean isAimingAllowed(Widget widget, Point localLocation, boolean invertSelection) {
+                return false;
+            }
+
+            public boolean isSelectionAllowed(Widget widget, Point localLocation, boolean invertSelection) {
+                return findObject(widget) != null;
+            }
+
+            public void select(Widget widget, Point localLocation, boolean invertSelection) {
+                EditorTopComponent editor = EditorTopComponent.getActive();
+                if (editor != null) {
+                    editor.requestActive();
+                }
+                Object object = findObject(widget);
+                setFocusedObject(object);
+                if (object != null) {
+                    if (!invertSelection && getSelectedObjects().contains(object)) {
+                        return;
+                    }
+                    userSelectionSuggested(Collections.singleton(object), invertSelection);
+                } else {
+                    userSelectionSuggested(Collections.emptySet(), invertSelection);
+                }
+            }
+        });
+
         this.getActions().addAction(selectAction);
 
         blockLayer = new LayerWidget(this);
@@ -423,17 +439,10 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
         bottomRight.setPreferredLocation(new Point(-BORDER_SIZE, -BORDER_SIZE));
         this.addChild(bottomRight);
 
-        LayerWidget selectionLayer = new LayerWidget(this);
-        this.addChild(selectionLayer);
-
         this.setLayout(LayoutFactory.createAbsoluteLayout());
-
-        this.getInputBindings().setZoomActionModifiers(KeyEvent.CTRL_MASK);
-        zoomAction = ActionFactory.createMouseCenteredZoomAction(1.2);
-        this.getActions().addAction(zoomAction);
-        this.getView().addMouseWheelListener(mouseWheelListener);
+        this.getInputBindings().setZoomActionModifiers(Utilities.isMac() ? KeyEvent.META_MASK : KeyEvent.CTRL_MASK);
+        this.getActions().addAction(ActionFactory.createMouseCenteredZoomAction(1.1));
         this.getActions().addAction(ActionFactory.createPopupMenuAction(popupMenuProvider));
-
         this.getActions().addAction(ActionFactory.createWheelPanAction());
 
         LayerWidget selectLayer = new LayerWidget(this);
@@ -493,6 +502,19 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
         return a;
     }
 
+    public Action createGotoAction(final Block b) {
+        final DiagramScene diagramScene = this;
+        String name = "B" + b.getInputBlock().getName();
+        Action a = new AbstractAction(name) {
+            @Override
+            public void actionPerformed(ActionEvent e) {
+                diagramScene.gotoBlock(b);
+            }
+        };
+        a.setEnabled(true);
+        return a;
+    }
+
     public void setNewModel(DiagramViewModel model) {
         assert this.model == null : "can set model only once!";
         this.model = model;
@@ -518,15 +540,26 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
 
         Diagram d = getModel().getDiagramToView();
 
-        if (d.getGraph().getBlocks().isEmpty()) {
-            Scheduler s = Lookup.getDefault().lookup(Scheduler.class);
-            d.getGraph().clearBlocks();
-            s.schedule(d.getGraph());
-            d.getGraph().ensureNodesInBlocks();
-            d.updateBlocks();
+        Map<InputBlock, Integer> maxWidth = new HashMap<>();
+        for (InputBlock b : d.getGraph().getBlocks()) {
+            maxWidth.put(b, 10);
+        }
+        for (Figure f : d.getFigures()) {
+            // Update node text, since it might differ across views.
+            f.updateLines();
+            // Compute max node width in each block.
+            if (f.getWidth() > maxWidth.get(f.getBlock())) {
+                maxWidth.put(f.getBlock(), f.getWidth());
+            }
         }
 
         for (Figure f : d.getFigures()) {
+
+            // Set all nodes' width to the maximum width in the blocks?
+            if (getModel().getShowCFG()) {
+                f.setWidth(maxWidth.get(f.getBlock()));
+            }
+
             FigureWidget w = new FigureWidget(f, hoverAction, selectAction, this, mainLayer);
             w.getActions().addAction(ActionFactory.createPopupMenuAction(w));
             w.getActions().addAction(selectAction);
@@ -552,7 +585,7 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        if (getModel().getShowBlocks()) {
+        if (getModel().getShowBlocks() || getModel().getShowCFG()) {
             for (InputBlock bn : d.getGraph().getBlocks()) {
                 BlockWidget w = new BlockWidget(this, d, bn);
                 w.setVisible(false);
@@ -578,14 +611,22 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
     }
 
     private boolean isVisible(Connection c) {
-        FigureWidget w1 = getWidget(c.getInputSlot().getFigure());
-        FigureWidget w2 = getWidget(c.getOutputSlot().getFigure());
-
-        if (w1.isVisible() && w2.isVisible()) {
-            return true;
+        // Generally, a connection is visible if its source and destination
+        // widgets are visible. An exception is Figure connections in the CFG
+        // view, which are never shown.
+        if (getModel().getShowCFG() && c instanceof FigureConnection) {
+            return false;
         }
-
-        return false;
+        Widget w1, w2;
+        if (c instanceof BlockConnection) {
+            w1 = getWidget(((Block)c.getFromCluster()).getInputBlock());
+            w2 = getWidget(((Block)c.getToCluster()).getInputBlock());
+        } else {
+            assert (c instanceof FigureConnection);
+            w1 = getWidget(c.getFrom().getVertex());
+            w2 = getWidget(c.getTo().getVertex());
+        }
+        return w1.isVisible() && w2.isVisible();
     }
 
     private void relayout(Set<Widget> oldVisibleWidgets) {
@@ -608,22 +649,83 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        if (getModel().getShowBlocks()) {
-            HierarchicalClusterLayoutManager m = new HierarchicalClusterLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
-            HierarchicalLayoutManager manager = new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
-            manager.setMaxLayerLength(9);
-            manager.setMinLayerDifference(3);
-            m.setManager(manager);
-            m.setSubManager(new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS));
-            m.doLayout(new LayoutGraph(edges, figures));
-        } else {
-            HierarchicalLayoutManager manager = new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
-            manager.setMaxLayerLength(10);
-            manager.doLayout(new LayoutGraph(edges, figures));
+        if (getModel().getShowSea()) {
+            doSeaLayout(figures, edges);
+        } else if (getModel().getShowBlocks()) {
+            doClusteredLayout(figures, edges);
+        } else if (getModel().getShowCFG()) {
+            doCFGLayout(figures, edges);
         }
 
         relayoutWithoutLayout(oldVisibleWidgets);
     }
+
+    private void doSeaLayout(HashSet<Figure> figures, HashSet<Connection> edges) {
+        HierarchicalLayoutManager manager = new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
+        manager.setMaxLayerLength(10);
+        manager.doLayout(new LayoutGraph(edges, figures));
+    }
+
+    private void doClusteredLayout(HashSet<Figure> figures, HashSet<Connection> edges) {
+        HierarchicalClusterLayoutManager m = new HierarchicalClusterLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
+        HierarchicalLayoutManager manager = new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
+        manager.setMaxLayerLength(9);
+        manager.setMinLayerDifference(3);
+        m.setManager(manager);
+        m.setSubManager(new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS));
+        m.doLayout(new LayoutGraph(edges, figures));
+    }
+
+    private void doCFGLayout(HashSet<Figure> figures, HashSet<Connection> edges) {
+        Diagram diagram = getModel().getDiagramToView();
+        HierarchicalCFGLayoutManager m = new HierarchicalCFGLayoutManager();
+        HierarchicalLayoutManager manager = new HierarchicalLayoutManager(HierarchicalLayoutManager.Combine.SAME_OUTPUTS);
+        manager.setMaxLayerLength(9);
+        manager.setMinLayerDifference(1);
+        manager.setLayoutSelfEdges(true);
+        manager.setXOffset(25);
+        manager.setLayerOffset(25);
+        m.setManager(manager);
+        Map<InputNode, Figure> nodeFig = new HashMap<InputNode, Figure>();
+        for (Figure f : figures) {
+            InputNode n = f.getFirstSourceNode();
+            if (n != null) {
+                nodeFig.put(n, f);
+            }
+        }
+        // Compute global ranking among figures given by in-block order. If
+        // needed, this could be cached as long as it is computed for all the
+        // figures in the model, not just the visible ones.
+        Map<Figure, Integer> figureRank =
+            new HashMap<Figure, Integer>(figures.size());
+        int r = 0;
+        for (InputBlock b : getModel().getGraphToView().getBlocks()) {
+            for (InputNode n : b.getNodes()) {
+                Figure f = nodeFig.get(n);
+                if (f != null) {
+                    figureRank.put(f, r);
+                    r++;
+                }
+            }
+        }
+        // Add visible connections for CFG edges.
+        for (BlockConnection c : diagram.getBlockConnections()) {
+            if (isVisible(c)) {
+                edges.add(c);
+            }
+        }
+        m.setSubManager(new LinearLayoutManager(figureRank));
+        Set<Block> visibleBlocks = new HashSet<>();
+        for (Block b : diagram.getBlocks()) {
+            BlockWidget w = getWidget(b.getInputBlock());
+            if (w.isVisible()) {
+                visibleBlocks.add(b);
+            }
+        }
+        m.setClusters(new HashSet<>(visibleBlocks));
+        m.doLayout(new LayoutGraph(edges, figures));
+    }
+
     private Set<Pair<Point, Point>> lineCache = new HashSet<>();
 
     private void relayoutWithoutLayout(Set<Widget> oldVisibleWidgets) {
@@ -642,7 +744,7 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        for (Connection c : diagram.getConnections()) {
+        for (FigureConnection c : diagram.getConnections()) {
             List<Point> points = c.getControlPoints();
             FigureWidget w1 = getWidget((Figure) c.getTo().getVertex());
             FigureWidget w2 = getWidget((Figure) c.getFrom().getVertex());
@@ -656,7 +758,7 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        if (getModel().getShowBlocks()) {
+        if (getModel().getShowBlocks() || getModel().getShowCFG()) {
             for (Block b : diagram.getBlocks()) {
                 BlockWidget w = getWidget(b.getInputBlock());
                 if (w != null && w.isVisible()) {
@@ -705,7 +807,19 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
                 if (visibleFigureCount > ANIMATION_LIMIT || oldVisibleWidgets == null) {
                     anim = null;
                 }
-                processOutputSlot(lastLineCache, s, s.getConnections(), 0, null, null, offx2, offy2, anim);
+                List<Connection> cl = new ArrayList<>(s.getConnections().size());
+                for (FigureConnection c : s.getConnections()) {
+                    cl.add((Connection) c);
+                }
+                processOutputSlot(lastLineCache, s, cl, 0, null, null, offx2, offy2, anim);
+            }
+        }
+
+        if (getModel().getShowCFG()) {
+            for (BlockConnection c : diagram.getBlockConnections()) {
+                if (isVisible(c)) {
+                    processOutputSlot(lastLineCache, null, Collections.singletonList(c), 0, null, null, offx2, offy2, animator);
+                }
             }
         }
 
@@ -723,7 +837,7 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        if (getModel().getShowBlocks()) {
+        if (getModel().getShowBlocks() || getModel().getShowCFG()) {
             for (Block b : diagram.getBlocks()) {
                 BlockWidget w = getWidget(b.getInputBlock());
                 if (w != null && w.isVisible()) {
@@ -759,12 +873,15 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
 
             Point cur = controlPoints.get(controlPointIndex);
-            if (cur == null) {
+            if (cur == null) { // Long connection, has been cut vertically.
                 cur = specialNullPoint;
-            } else if (controlPointIndex == 0 && !s.shouldShowName()) {
-                cur = new Point(cur.x, cur.y - SLOT_OFFSET);
-            } else if (controlPointIndex == controlPoints.size() - 1 && !c.getInputSlot().shouldShowName()) {
-                cur = new Point(cur.x, cur.y + SLOT_OFFSET);
+            } else if (c.hasSlots()) {
+                if (controlPointIndex == 0 && !s.shouldShowName()) {
+                    cur = new Point(cur.x, cur.y - SLOT_OFFSET);
+                } else if (controlPointIndex == controlPoints.size() - 1 &&
+                           !((Slot)c.getTo()).shouldShowName()) {
+                    cur = new Point(cur.x, cur.y + SLOT_OFFSET);
+                }
             }
 
             if (pointMap.containsKey(cur)) {
@@ -883,6 +1000,13 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
         }
     }
 
+    public void gotoBlock(final Block block) {
+        BlockWidget bw = getWidget(block.getInputBlock());
+        if (bw != null) {
+            centerRectangle(bw.getBounds());
+        }
+    }
+
     private Set<Object> idSetToObjectSet(Set<Object> ids) {
 
         Set<Object> result = new HashSet<>();
@@ -1042,7 +1166,7 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        if (getModel().getShowBlocks()) {
+        if (getModel().getShowBlocks() || getModel().getShowCFG()) {
             for (InputBlock b : diagram.getGraph().getBlocks()) {
                 BlockWidget w = getWidget(b);
                 if (w.isVisible()) {
@@ -1102,10 +1226,27 @@ public class DiagramScene extends ObjectScene implements DiagramViewer {
             }
         }
 
-        if (getModel().getShowBlocks()) {
+        if (getModel().getShowCFG()) {
+            // Blockless figures and artificial blocks are hidden in this view.
+            for (Figure f : diagram.getFigures()) {
+                if (f.getBlock().isArtificial()) {
+                    FigureWidget w = getWidget(f);
+                    w.setVisible(false);
+                }
+            }
+            if (getModel().getShowEmptyBlocks()) {
+                // Add remaining blocks.
+                visibleBlocks.addAll(diagram.getGraph().getBlocks());
+            }
+        }
+
+        if (getModel().getShowBlocks() || getModel().getShowCFG()) {
             for (InputBlock b : diagram.getGraph().getBlocks()) {
 
-                boolean visibleAfter = visibleBlocks.contains(b);
+                // A block is visible if it is marked as such, except for
+                // artificial or null blocks in the CFG view.
+                boolean visibleAfter = visibleBlocks.contains(b) &&
+                    !(getModel().getShowCFG() && (b.isArtificial() || b.getNodes().isEmpty()));
 
                 BlockWidget w = getWidget(b);
                 if (visibleAfter) {
