@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,10 +23,10 @@
 
 package compiler.lib.ir_framework;
 
-import compiler.lib.ir_framework.driver.IRMatcher;
+import compiler.lib.ir_framework.driver.irmatching.IRMatcher;
 import compiler.lib.ir_framework.shared.*;
 import jdk.test.lib.Platform;
-import sun.hotspot.WhiteBox;
+import jdk.test.whitebox.WhiteBox;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -59,7 +59,7 @@ public class IRNode {
     public static final String ALLOC_ARRAY_OF = COMPOSITE_PREFIX + "(.*precise \\[.*" + IS_REPLACED + ":.*\\R((.*(?i:mov|xorl|nop|spill).*|\\s*|.*LGHI.*)\\R)*.*(?i:call,static).*wrapper for: _new_array_Java" + END;
 
     public static final String CHECKCAST_ARRAY = "(((?i:cmp|CLFI|CLR).*precise \\[.*:|.*(?i:mov|or).*precise \\[.*:.*\\R.*(cmp|CMP|CLR))" + END;
-    public static final String CHECKCAST_ARRAY_OF = COMPOSITE_PREFIX + "(((?i:cmp|CLFI|CLR).*precise \\[.*" + IS_REPLACED + ":|.*(?i:mov|or).*precise klass \\[.*" + IS_REPLACED + ";:.*\\R.*(cmp|CMP|CLR))" + END;
+    public static final String CHECKCAST_ARRAY_OF = COMPOSITE_PREFIX + "(((?i:cmp|CLFI|CLR).*precise \\[.*" + IS_REPLACED + ":|.*(?i:mov|or).*precise \\[.*" + IS_REPLACED + ":.*\\R.*(cmp|CMP|CLR))" + END;
     // Does not work on s390 (a rule containing this regex will be skipped on s390).
     public static final String CHECKCAST_ARRAYCOPY = "(.*((?i:call_leaf_nofp,runtime)|CALL,\\s?runtime leaf nofp|BCTRL.*.leaf call).*checkcast_arraycopy.*" + END;
 
@@ -74,6 +74,7 @@ public class IRNode {
     public static final String STORE_D = START + "StoreD" + MID + END;
     public static final String STORE_P = START + "StoreP" + MID + END;
     public static final String STORE_N = START + "StoreN" + MID + END;
+    public static final String STORE_VECTOR = START + "StoreVector" + MID + END;
     public static final String STORE_OF_CLASS = COMPOSITE_PREFIX + START + "Store(B|C|S|I|L|F|D|P|N)" + MID + "@\\S*" + IS_REPLACED + STORE_OF_CLASS_POSTFIX;
     public static final String STORE_B_OF_CLASS = COMPOSITE_PREFIX + START + "StoreB" + MID + "@\\S*" + IS_REPLACED + STORE_OF_CLASS_POSTFIX;
     public static final String STORE_C_OF_CLASS = COMPOSITE_PREFIX + START + "StoreC" + MID + "@\\S*" + IS_REPLACED + STORE_OF_CLASS_POSTFIX;
@@ -96,6 +97,7 @@ public class IRNode {
     public static final String LOAD_D = START + "LoadD" + MID + END;
     public static final String LOAD_P = START + "LoadP" + MID + END;
     public static final String LOAD_N = START + "LoadN" + MID + END;
+    public static final String LOAD_VECTOR = START + "LoadVector" + MID + END;
     public static final String LOAD_OF_CLASS = COMPOSITE_PREFIX + START + "Load(B|UB|S|US|I|L|F|D|P|N)" + MID + "@\\S*"+  IS_REPLACED + LOAD_OF_CLASS_POSTFIX;
     public static final String LOAD_B_OF_CLASS = COMPOSITE_PREFIX + START + "LoadB" + MID + "@\\S*" + IS_REPLACED + LOAD_OF_CLASS_POSTFIX;
     public static final String LOAD_UB_OF_CLASS = COMPOSITE_PREFIX + START + "LoadUB" + MID + "@\\S*" + IS_REPLACED + LOAD_OF_CLASS_POSTFIX;
@@ -113,10 +115,14 @@ public class IRNode {
     public static final String LOOP   = START + "Loop" + MID + END;
     public static final String COUNTEDLOOP = START + "CountedLoop\\b" + MID + END;
     public static final String COUNTEDLOOP_MAIN = START + "CountedLoop\\b" + MID + "main" + END;
+    public static final String OUTERSTRIPMINEDLOOP = START + "OuterStripMinedLoop\\b" + MID + END;
+    public static final String LONGCOUNTEDLOOP = START + "LongCountedLoop\\b" + MID + END;
+    public static final String IF = START + "If\\b" + MID + END;
 
-    public static final String CALL = START + "CallStaticJava" + MID + END;
-    public static final String DYNAMIC_CALL_OF_METHOD = COMPOSITE_PREFIX + START + "CallDynamicJava" + MID + IS_REPLACED + END;
-    public static final String STATIC_CALL_OF_METHOD = COMPOSITE_PREFIX + START + "CallStaticJava" + MID + IS_REPLACED + END;
+    public static final String CALL = START + "Call.*Java" + MID + END;
+    public static final String CALL_OF_METHOD = COMPOSITE_PREFIX + START + "Call.*Java" + MID + IS_REPLACED + " " +  END;
+    public static final String DYNAMIC_CALL_OF_METHOD = COMPOSITE_PREFIX + START + "CallDynamicJava" + MID + IS_REPLACED + " " + END;
+    public static final String STATIC_CALL_OF_METHOD = COMPOSITE_PREFIX + START + "CallStaticJava" + MID + IS_REPLACED + " " +  END;
     public static final String TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*reason" + END;
     public static final String PREDICATE_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*predicate" + END;
     public static final String UNSTABLE_IF_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*unstable_if" + END;
@@ -125,11 +131,104 @@ public class IRNode {
     public static final String NULL_ASSERT_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*null_assert" + END;
     public static final String RANGE_CHECK_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*range_check" + END;
     public static final String UNHANDLED_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*unhandled" + END;
+    public static final String INTRINSIC_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*intrinsic" + END;
+    public static final String DIV_BY_ZERO_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*div0_check" + END;
     // Does not work for VM builds without JVMCI like x86_32 (a rule containing this regex will be skipped without having JVMCI built).
     public static final String INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP = START + "CallStaticJava" + MID + "uncommon_trap.*intrinsic_or_type_checked_inlining" + END;
 
     public static final String SCOPE_OBJECT = "(.*# ScObj.*" + END;
     public static final String MEMBAR = START + "MemBar" + MID + END;
+    public static final String MEMBAR_STORESTORE = START + "MemBarStoreStore" + MID + END;
+    public static final String SAFEPOINT = START + "SafePoint" + MID + END;
+
+    public static final String CMOVEI = START + "CMoveI" + MID + END;
+    public static final String CMOVEVF = START + "CMoveVF" + MID + END;
+    public static final String CMOVEVD = START + "CMoveVD" + MID + END;
+    public static final String ABS_I = START + "AbsI" + MID + END;
+    public static final String ABS_L = START + "AbsL" + MID + END;
+    public static final String ABS_F = START + "AbsF" + MID + END;
+    public static final String ABS_D = START + "AbsD" + MID + END;
+    public static final String AND = START + "And(I|L)" + MID + END;
+    public static final String AND_I = START + "AndI" + MID + END;
+    public static final String AND_L = START + "AndL" + MID + END;
+    public static final String XOR_I = START + "XorI" + MID + END;
+    public static final String XOR_L = START + "XorL" + MID + END;
+    public static final String LSHIFT = START + "LShift(I|L)" + MID + END;
+    public static final String LSHIFT_I = START + "LShiftI" + MID + END;
+    public static final String LSHIFT_L = START + "LShiftL" + MID + END;
+    public static final String RSHIFT = START + "RShift(I|L)" + MID + END;
+    public static final String RSHIFT_I = START + "RShiftI" + MID + END;
+    public static final String RSHIFT_L = START + "RShiftL" + MID + END;
+    public static final String RSHIFT_VB = START + "RShiftVB" + MID + END;
+    public static final String RSHIFT_VS = START + "RShiftVS" + MID + END;
+    public static final String URSHIFT = START + "URShift(B|S|I|L)" + MID + END;
+    public static final String URSHIFT_I = START + "URShiftI" + MID + END;
+    public static final String URSHIFT_L = START + "URShiftL" + MID + END;
+    public static final String ADD = START + "Add(I|L|F|D|P)" + MID + END;
+    public static final String ADD_I = START + "AddI" + MID + END;
+    public static final String ADD_L = START + "AddL" + MID + END;
+    public static final String ADD_VD = START + "AddVD" + MID + END;
+    public static final String ADD_VI = START + "AddVI" + MID + END;
+    public static final String SUB = START + "Sub(I|L|F|D)" + MID + END;
+    public static final String SUB_I = START + "SubI" + MID + END;
+    public static final String SUB_L = START + "SubL" + MID + END;
+    public static final String SUB_F = START + "SubF" + MID + END;
+    public static final String SUB_D = START + "SubD" + MID + END;
+    public static final String CMP_U = START + "CmpU" + MID + END;
+    public static final String CMP_UL = START + "CmpUL" + MID + END;
+    public static final String CMP_U3 = START + "CmpU3" + MID + END;
+    public static final String CMP_UL3 = START + "CmpUL3" + MID + END;
+    public static final String CMP_I = START + "CmpI" + MID + END;
+    public static final String MUL = START + "Mul(I|L|F|D)" + MID + END;
+    public static final String MUL_I = START + "MulI" + MID + END;
+    public static final String MUL_L = START + "MulL" + MID + END;
+    public static final String MUL_F = START + "MulF" + MID + END;
+    public static final String DIV = START + "Div(I|L|F|D)" + MID + END;
+    public static final String DIV_L = START + "DivL" + MID + END;
+    public static final String CON_I = START + "ConI" + MID + END;
+    public static final String CON_L = START + "ConL" + MID + END;
+    public static final String CONV_I2L = START + "ConvI2L" + MID + END;
+    public static final String CONV_L2I = START + "ConvL2I" + MID + END;
+    public static final String CAST_II = START + "CastII" + MID + END;
+    public static final String CAST_LL = START + "CastLL" + MID + END;
+    public static final String POPCOUNT_L = START + "PopCountL" + MID + END;
+    public static final String PHI = START + "Phi" + MID + END;
+
+    public static final String AND_V = START + "AndV" + MID + END;
+    public static final String OR_V = START + "OrV" + MID + END;
+    public static final String XOR_V = START + "XorV" + MID + END;
+    public static final String AND_V_MASK = START + "AndVMask" + MID + END;
+    public static final String OR_V_MASK = START + "OrVMask" + MID + END;
+    public static final String XOR_V_MASK = START + "XorVMask" + MID + END;
+
+    public static final String VECTOR_CAST_B2X = START + "VectorCastB2X" + MID + END;
+    public static final String VECTOR_CAST_S2X = START + "VectorCastS2X" + MID + END;
+    public static final String VECTOR_CAST_I2X = START + "VectorCastI2X" + MID + END;
+    public static final String VECTOR_CAST_L2X = START + "VectorCastL2X" + MID + END;
+    public static final String VECTOR_CAST_F2X = START + "VectorCastF2X" + MID + END;
+    public static final String VECTOR_CAST_D2X = START + "VectorCastD2X" + MID + END;
+    public static final String VECTOR_UCAST_B2X = START + "VectorUCastB2X" + MID + END;
+    public static final String VECTOR_UCAST_S2X = START + "VectorUCastS2X" + MID + END;
+    public static final String VECTOR_UCAST_I2X = START + "VectorUCastI2X" + MID + END;
+    public static final String VECTOR_REINTERPRET = START + "VectorReinterpret" + MID + END;
+    public static final String VECTOR_BLEND = START + "VectorBlend" + MID + END;
+    public static final String REVERSE_BYTES_V = START + "ReverseBytesV" + MID + END;
+
+    public static final String Min_I = START + "MinI" + MID + END;
+    public static final String Max_I = START + "MaxI" + MID + END;
+    public static final String Min_V = START + "MinV" + MID + END;
+    public static final String Max_V = START + "MaxV" + MID + END;
+    public static final String MUL_VL = START + "MulVL" + MID + END;
+
+    public static final String ADD_REDUCTION_VF = START + "AddReductionVF" + MID + END;
+    public static final String ADD_REDUCTION_VD = START + "AddReductionVD" + MID + END;
+    public static final String MUL_REDUCTION_VF = START + "MulReductionVF" + MID + END;
+    public static final String MUL_REDUCTION_VD = START + "MulReductionVD" + MID + END;
+
+    public static final String FAST_LOCK   = START + "FastLock" + MID + END;
+    public static final String FAST_UNLOCK = START + "FastUnlock" + MID + END;
+
+    public static final String POPULATE_INDEX = START + "PopulateIndex" + MID + END;
 
     /**
      * Called by {@link IRMatcher} to merge special composite nodes together with additional user-defined input.
