@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,17 +23,25 @@
 
 package compiler.lib.ir_framework;
 
-import compiler.lib.ir_framework.driver.*;
+import compiler.lib.ir_framework.driver.FlagVMProcess;
+import compiler.lib.ir_framework.driver.TestVMException;
+import compiler.lib.ir_framework.driver.TestVMProcess;
+import compiler.lib.ir_framework.driver.irmatching.IRMatcher;
+import compiler.lib.ir_framework.driver.irmatching.IRViolationException;
 import compiler.lib.ir_framework.shared.*;
-import compiler.lib.ir_framework.test.*;
+import compiler.lib.ir_framework.test.TestVM;
 import jdk.test.lib.Platform;
 import jdk.test.lib.Utils;
 import jdk.test.lib.helpers.ClassFileInstaller;
-import sun.hotspot.WhiteBox;
+import jdk.test.whitebox.WhiteBox;
 
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.lang.reflect.Method;
+import java.net.MalformedURLException;
+import java.net.URL;
+import java.net.URLClassLoader;
+import java.nio.file.Path;
 import java.util.*;
 import java.util.stream.Collectors;
 
@@ -125,6 +133,9 @@ public class TestFramework {
                     "CompileThreshold",
                     "Xmixed",
                     "server",
+                    "UseAVX",
+                    "UseSSE",
+                    "UseSVE",
                     "Xlog",
                     "LogCompilation"
             )
@@ -135,7 +146,6 @@ public class TestFramework {
     public static final boolean EXCLUDELIST = !System.getProperty("Exclude", "").isEmpty();
     private static final boolean REPORT_STDOUT = Boolean.getBoolean("ReportStdout");
     // Only used for internal testing and should not be used for normal user testing.
-    private static final boolean SKIP_WHITEBOX_INSTALL = Boolean.getBoolean("SkipWhiteBoxInstall");
 
     private static final String RERUN_HINT = """
                                                #############################################################
@@ -312,7 +322,7 @@ public class TestFramework {
      * set test class.
      */
     public void start() {
-        if (!SKIP_WHITEBOX_INSTALL) {
+        if (shouldInstallWhiteBox()) {
             installWhiteBox();
         }
         disableIRVerificationIfNotFeasible();
@@ -334,6 +344,28 @@ public class TestFramework {
         }
     }
 
+    /**
+     * Try to load the Whitebox class from the user directory with a custom class loader. If the user has already built the
+     * Whitebox, we can load it. Otherwise, the framework needs to install it.
+     *
+     * @return true if the framework needs to install the Whitebox
+     */
+    private boolean shouldInstallWhiteBox() {
+        try {
+            URL url = Path.of(System.getProperty("user.dir")).toUri().toURL();
+            URLClassLoader userDirClassLoader =
+                    URLClassLoader.newInstance(new URL[] {url}, TestFramework.class.getClassLoader().getParent());
+            Class.forName(WhiteBox.class.getName(), false, userDirClassLoader);
+        } catch (MalformedURLException e) {
+            throw new TestFrameworkException("corrupted user.dir property", e);
+        } catch (ClassNotFoundException e) {
+            // We need to manually install the WhiteBox if we cannot load the WhiteBox class from the user directory.
+            // This happens when the user test does not explicitly install the WhiteBox as part of the test.
+            return true;
+        }
+        return false;
+    }
+
     /**
      * Set a new default warm-up (overriding the framework default of 2000 at
      * {@link TestVM#WARMUP_ITERATIONS}) to be applied for all tests that do not specify an explicit
@@ -603,11 +635,11 @@ public class TestFramework {
                 // Print stack trace otherwise
                 StringWriter errors = new StringWriter();
                 e.printStackTrace(new PrintWriter(errors));
-                builder.append(errors.toString());
+                builder.append(errors);
             }
             builder.append(System.lineSeparator());
         }
-        System.err.println(builder.toString());
+        System.err.println(builder);
         if (!VERBOSE && !REPORT_STDOUT && !TESTLIST && !EXCLUDELIST) {
             // Provide a hint to the user how to get additional output/debugging information.
             System.err.println(RERUN_HINT);
