@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -32,8 +32,8 @@ package gc.g1;
  * @library /test/lib
  * @modules java.base/jdk.internal.misc
  *          java.management
- * @build sun.hotspot.WhiteBox
- * @run driver jdk.test.lib.helpers.ClassFileInstaller sun.hotspot.WhiteBox
+ * @build jdk.test.whitebox.WhiteBox
+ * @run driver jdk.test.lib.helpers.ClassFileInstaller jdk.test.whitebox.WhiteBox
  * @run main/othervm -Xbootclasspath/a:. -XX:+UnlockDiagnosticVMOptions -XX:+WhiteBoxAPI
  *                   gc.g1.TestGCLogMessages
  */
@@ -41,7 +41,7 @@ package gc.g1;
 import jdk.test.lib.process.OutputAnalyzer;
 import jdk.test.lib.Platform;
 import jdk.test.lib.process.ProcessTools;
-import sun.hotspot.code.Compiler;
+import jdk.test.whitebox.code.Compiler;
 
 public class TestGCLogMessages {
 
@@ -90,6 +90,17 @@ public class TestGCLogMessages {
         }
     }
 
+    private class LogMessageWithJFROnly extends LogMessageWithLevel {
+        public LogMessageWithJFROnly(String message, Level level) {
+            super(message, level);
+        }
+
+        public boolean isAvailable() {
+            jdk.test.whitebox.WhiteBox WB = jdk.test.whitebox.WhiteBox.getWhiteBox();
+            return WB.isJFRIncluded();
+        }
+    }
+
     private LogMessageWithLevel allLogMessages[] = new LogMessageWithLevel[] {
         new LogMessageWithLevel("Pre Evacuate Collection Set", Level.INFO),
         new LogMessageWithLevel("Evacuate Collection Set", Level.INFO),
@@ -131,6 +142,7 @@ public class TestGCLogMessages {
         new LogMessageWithLevel("Scanned Cards", Level.DEBUG),
         new LogMessageWithLevel("Scanned Blocks", Level.DEBUG),
         new LogMessageWithLevel("Claimed Chunks", Level.DEBUG),
+        new LogMessageWithLevel("Found Roots", Level.DEBUG),
         // Code Roots Scan
         new LogMessageWithLevel("Code Root Scan", Level.DEBUG),
         // Object Copy
@@ -153,7 +165,7 @@ public class TestGCLogMessages {
         new LogMessageWithLevel("JVMTI Tag Weak OopStorage", Level.DEBUG),
         new LogMessageWithLevel("StringTable Weak", Level.DEBUG),
         new LogMessageWithLevel("ResolvedMethodTable Weak", Level.DEBUG),
-        new LogMessageWithLevel("Weak JFR Old Object Samples", Level.DEBUG),
+        new LogMessageWithJFROnly("Weak JFR Old Object Samples", Level.DEBUG),
         new LogMessageWithLevel("JNI Weak", Level.DEBUG),
 
         // Post Evacuate Cleanup 1
@@ -253,7 +265,8 @@ public class TestGCLogMessages {
     LogMessageWithLevel exhFailureMessages[] = new LogMessageWithLevel[] {
         new LogMessageWithLevel("Recalculate Used Memory", Level.DEBUG),
         new LogMessageWithLevel("Restore Preserved Marks", Level.DEBUG),
-        new LogMessageWithLevel("Remove Self Forwards", Level.DEBUG),
+        new LogMessageWithLevel("Restore Retained Regions", Level.DEBUG),
+        new LogMessageWithLevel("Evacuation Failure Regions", Level.DEBUG),
     };
 
     private void testWithEvacuationFailureLogs() throws Exception {
@@ -354,7 +367,7 @@ public class TestGCLogMessages {
 
     static class GCTestWithConcurrentStart {
         public static void main(String [] args) {
-            sun.hotspot.WhiteBox WB = sun.hotspot.WhiteBox.getWhiteBox();
+            jdk.test.whitebox.WhiteBox WB = jdk.test.whitebox.WhiteBox.getWhiteBox();
             WB.g1StartConcMarkCycle();
         }
     }
