@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,9 +28,9 @@
  * @summary SharedArchiveConsistency
  * @requires vm.cds
  * @library /test/lib
- * @build sun.hotspot.WhiteBox
+ * @build jdk.test.whitebox.WhiteBox
  * @compile test-classes/Hello.java
- * @run driver jdk.test.lib.helpers.ClassFileInstaller sun.hotspot.WhiteBox
+ * @run driver jdk.test.lib.helpers.ClassFileInstaller jdk.test.whitebox.WhiteBox
  * @run main/othervm -Xbootclasspath/a:. -XX:+UnlockDiagnosticVMOptions -XX:+WhiteBoxAPI SharedArchiveConsistency on
  * @run main/othervm -Xbootclasspath/a:. -XX:+UnlockDiagnosticVMOptions -XX:+WhiteBoxAPI SharedArchiveConsistency auto
  */
@@ -43,6 +43,8 @@ public class SharedArchiveConsistency {
     public static boolean shareAuto;       // true  == -Xshare:auto
                                            // false == -Xshare:on
 
+    private static int genericHeaderMinVersion;  // minimum supported CDS version
+    private static int currentCDSArchiveVersion; // current CDS version in java process
     // The following should be consistent with the enum in the C++ MetaspaceShared class
     public static String[] shared_region_name = {
         "rw",          // ReadWrite
@@ -74,10 +76,21 @@ public class SharedArchiveConsistency {
         return newArchiveName;
     }
 
-    public static void testAndCheck(String[] execArgs) throws Exception {
+    public static void testAndCheck(String[] execArgs, String... expectedMessages) throws Exception {
         OutputAnalyzer output = shareAuto ? TestCommon.execAuto(execArgs) : TestCommon.execCommon(execArgs);
         String stdtxt = output.getOutput();
         System.out.println("Note: this test may fail in very rare occasions due to CRC32 checksum collision");
+        for (String opt : execArgs) {
+          if (opt.equals("-XX:+VerifySharedSpaces")) {
+            // If VerifySharedSpaces is enabled, the VM should never crash even if the archive
+            // is corrupted (unless if we are so lucky that the corrupted archive ends up
+            // have the same checksum as recoreded in the header)
+            output.shouldNotContain("A fatal error has been detected by the Java Runtime Environment");
+          }
+        }
+        for (int i = 0; i < expectedMessages.length; i++) {
+            output.shouldContain(expectedMessages[i]);
+        }
         for (String message : matchMessages) {
             if (stdtxt.contains(message)) {
                 // match any to return
@@ -87,6 +100,10 @@ public class SharedArchiveConsistency {
         TestCommon.checkExec(output);
     }
 
+    private static String hex(int version) {
+        return String.format("0x%x", version);
+    }
+
     // dump with hello.jsa, then
     // read the jsa file
     //   1) run normal
@@ -104,6 +121,9 @@ public class SharedArchiveConsistency {
             throw new RuntimeException("Arg must be 'on' or 'auto'");
         }
         shareAuto = args[0].equals("auto");
+        genericHeaderMinVersion = CDSArchiveUtils.getGenericHeaderMinVersion();
+        currentCDSArchiveVersion = CDSArchiveUtils.getCurrentCDSArchiveVersion();
+
         String jarFile = JarBuilder.getOrCreateHelloJar();
 
         // dump (appcds.jsa created)
@@ -158,7 +178,7 @@ public class SharedArchiveConsistency {
         }
 
         // modify _magic, test should fail
-        System.out.println("\n2c. Corrupt _magic, should fail\n");
+        System.out.println("\n2b. Corrupt _magic, should fail\n");
         String modMagic = startNewArchive("modify-magic");
         copiedJsa = CDSArchiveUtils.copyArchiveFile(orgJsaFile, modMagic);
         CDSArchiveUtils.modifyHeaderIntField(copiedJsa, CDSArchiveUtils.offsetMagic(), -1);
@@ -170,23 +190,24 @@ public class SharedArchiveConsistency {
         }
 
         // modify _version, test should fail
-        System.out.println("\n2d. Corrupt _version, should fail\n");
+        System.out.println("\n2c. Corrupt _version, should fail\n");
         String modVersion = startNewArchive("modify-version");
+        int version = currentCDSArchiveVersion + 1;
         copiedJsa = CDSArchiveUtils.copyArchiveFile(orgJsaFile, modVersion);
-        CDSArchiveUtils.modifyHeaderIntField(copiedJsa, CDSArchiveUtils.offsetVersion(), 0x3FFFFFFF);
+        CDSArchiveUtils.modifyHeaderIntField(copiedJsa, CDSArchiveUtils.offsetVersion(), version);
         output = shareAuto ? TestCommon.execAuto(execArgs) : TestCommon.execCommon(execArgs);
-        output.shouldContain("The shared archive file has the wrong version");
-        output.shouldNotContain("Checksum verification failed");
+        output.shouldContain("The shared archive file version " + hex(version) + " does not match the required version " + hex(currentCDSArchiveVersion));
         if (shareAuto) {
             output.shouldContain(HELLO_WORLD);
         }
 
-        System.out.println("\n2e. Corrupt _version, should fail\n");
+        System.out.println("\n2d. Corrupt _version, should fail\n");
         String modVersion2 = startNewArchive("modify-version2");
         copiedJsa = CDSArchiveUtils.copyArchiveFile(orgJsaFile, modVersion2);
-        CDSArchiveUtils.modifyHeaderIntField(copiedJsa, CDSArchiveUtils.offsetVersion(), 0x00000000);
+        version = genericHeaderMinVersion - 1;
+        CDSArchiveUtils.modifyHeaderIntField(copiedJsa, CDSArchiveUtils.offsetVersion(), version);
         output = shareAuto ? TestCommon.execAuto(execArgs) : TestCommon.execCommon(execArgs);
-        output.shouldContain("Cannot handle shared archive file version 0. Must be at least 12");
+        output.shouldContain("Cannot handle shared archive file version " + hex(version) + ". Must be at least " + hex(genericHeaderMinVersion));
         output.shouldNotContain("Checksum verification failed");
         if (shareAuto) {
             output.shouldContain(HELLO_WORLD);
@@ -218,15 +239,21 @@ public class SharedArchiveConsistency {
         // insert  bytes in data section
         System.out.println("\n5. Insert bytes at beginning of data section, should fail\n");
         String insertBytes = startNewArchive("insert-bytes");
-        CDSArchiveUtils.insertBytesRandomlyAfterHeader(orgJsaFile, insertBytes, new byte[4096]);
+        CDSArchiveUtils.insertBytesRandomlyAfterHeader(orgJsaFile, insertBytes);
         testAndCheck(verifyExecArgs);
 
         // delete bytes in data section forward
-        System.out.println("\n6. Delete bytes at beginning of data section, should fail\n");
+        System.out.println("\n6a. Delete bytes at beginning of data section, should fail\n");
         String deleteBytes = startNewArchive("delete-bytes");
         CDSArchiveUtils.deleteBytesAtRandomPositionAfterHeader(orgJsaFile, deleteBytes, 4096 /*bytes*/);
         testAndCheck(verifyExecArgs);
 
+        // delete bytes at the end
+        System.out.println("\n6b. Delete bytes at the end, should fail\n");
+        deleteBytes = startNewArchive("delete-bytes-end");
+        CDSArchiveUtils.deleteBytesAtTheEnd(orgJsaFile, deleteBytes);
+        testAndCheck(verifyExecArgs, "The shared archive file has been truncated.");
+
         // modify contents in random area
         System.out.println("\n7. modify Content in random areas, should fail\n");
         String randomAreas = startNewArchive("random-areas");
