@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -24,49 +24,57 @@
 package ir_framework.tests;
 
 import compiler.lib.ir_framework.*;
-import compiler.lib.ir_framework.driver.IRViolationException;
+import compiler.lib.ir_framework.driver.irmatching.IRViolationException;
 import jdk.test.lib.Asserts;
 import jdk.test.lib.Platform;
-import sun.hotspot.WhiteBox;
+import jdk.test.whitebox.WhiteBox;
 
 import java.io.ByteArrayOutputStream;
 import java.io.PrintStream;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.List;
+import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 /*
  * @test
- * @requires vm.debug == true & vm.compMode != "Xint" & vm.compiler2.enabled & vm.flagless
+ * @requires vm.debug == true & vm.compMode != "Xint" & vm.compiler1.enabled & vm.compiler2.enabled & vm.flagless
  * @summary Test IR matcher with different default IR node regexes. Use -DPrintIREncoding.
  *          Normally, the framework should be called with driver.
  * @library /test/lib /testlibrary_tests /
- * @build sun.hotspot.WhiteBox
- * @run driver jdk.test.lib.helpers.ClassFileInstaller sun.hotspot.WhiteBox
- * @run main/othervm/timeout=240 -Xbootclasspath/a:. -DSkipWhiteBoxInstall=true -XX:+IgnoreUnrecognizedVMOptions -XX:+UnlockDiagnosticVMOptions
+ * @build jdk.test.whitebox.WhiteBox
+ * @run driver jdk.test.lib.helpers.ClassFileInstaller jdk.test.whitebox.WhiteBox
+ * @run main/othervm/timeout=240 -Xbootclasspath/a:. -XX:+IgnoreUnrecognizedVMOptions -XX:+UnlockDiagnosticVMOptions
  *                               -XX:+WhiteBoxAPI -DPrintIREncoding=true  ir_framework.tests.TestIRMatching
  */
 
 public class TestIRMatching {
 
-    private static final List<Exception> exceptions = new ArrayList<>();
+    private static final Map<Exception, String> exceptions = new LinkedHashMap<>();
+    private static final ByteArrayOutputStream baos = new ByteArrayOutputStream();
+    private static final ByteArrayOutputStream baosErr = new ByteArrayOutputStream();
+    private static final PrintStream ps = new PrintStream(baos);
+    private static final PrintStream psErr = new PrintStream(baosErr);
+    private static final PrintStream oldOut = System.out;
+    private static final PrintStream oldErr = System.err;
 
     private static void addException(Exception e) {
-        System.out.println(TestFramework.getLastTestVMOutput());
-        exceptions.add(e);
+        System.out.flush();
+        System.err.flush();
+        exceptions.put(e, baos + System.lineSeparator() + baosErr);
     }
 
     public static void main(String[] args) {
-        runFailOnTestsArgs(BadFailOnConstraint.create(AndOr1.class, "test1(int)", 1, "CallStaticJava"), "-XX:TLABRefillWasteFraction=50", "-XX:+UsePerfData", "-XX:+UseTLAB");
-        runFailOnTestsArgs(BadFailOnConstraint.create(AndOr1.class, "test2()", 1, "CallStaticJava"), "-XX:TLABRefillWasteFraction=50", "-XX:-UsePerfData", "-XX:+UseTLAB");
-
+        // Redirect System.out and System.err to reduce noise.
+        System.setOut(ps);
+        System.setErr(psErr);
         runWithArguments(AndOr1.class, "-XX:TLABRefillWasteFraction=52", "-XX:+UsePerfData", "-XX:+UseTLAB");
         runWithArguments(CountComparisons.class, "-XX:TLABRefillWasteFraction=50");
         runWithArguments(GoodCount.class, "-XX:TLABRefillWasteFraction=50");
         runWithArguments(MultipleFailOnGood.class, "-XX:TLABRefillWasteFraction=50");
 
+        runCheck(new String[] {"-XX:TLABRefillWasteFraction=50", "-XX:+UsePerfData", "-XX:+UseTLAB"}, BadFailOnConstraint.create(AndOr1.class, "test1(int)", 1, "CallStaticJava"));
+        runCheck(new String[] {"-XX:TLABRefillWasteFraction=50", "-XX:-UsePerfData", "-XX:+UseTLAB"}, BadFailOnConstraint.create(AndOr1.class, "test2()", 1, "CallStaticJava"));
+
         String[] allocMatches = { "MyClass", "wrapper for: _new_instance_Java" };
         runCheck(BadFailOnConstraint.create(MultipleFailOnBad.class, "fail1()", 1, 1, "Store"),
                  BadFailOnConstraint.create(MultipleFailOnBad.class, "fail1()", 1,  3, "Store"),
@@ -100,6 +108,12 @@ public class TestIRMatching {
                  BadCountsConstraint.create(BadCount.class, "bad3()", 2,  1, "Store")
         );
 
+        runCheck(GoodRuleConstraint.create(Calls.class, "calls()", 1),
+                 BadFailOnConstraint.create(Calls.class, "calls()", 2, 1, "CallStaticJava", "dontInline"),
+                 BadFailOnConstraint.create(Calls.class, "calls()", 2, 2, "CallStaticJava", "dontInline"),
+                 GoodRuleConstraint.create(Calls.class, "calls()", 3)
+        );
+
         String[] allocArrayMatches = { "MyClass", "wrapper for: _new_array_Java"};
         runCheck(BadFailOnConstraint.create(AllocArray.class, "allocArray()", 1, allocArrayMatches),
                  BadFailOnConstraint.create(AllocArray.class, "allocArray()", 2,  allocArrayMatches),
@@ -185,8 +199,9 @@ public class TestIRMatching {
                  WhiteBox.getWhiteBox().isJVMCISupportedByGC() ?
                     BadFailOnConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 2, "CallStaticJava", "uncommon_trap", "intrinsic_or_type_checked_inlining")
                     : GoodRuleConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 2),
-                 BadFailOnConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 3, "CallStaticJava", "uncommon_trap", "null_check"),
-                 GoodRuleConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 4)
+                 BadFailOnConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 3, "CallStaticJava", "uncommon_trap", "intrinsic"),
+                 BadFailOnConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 4, "CallStaticJava", "uncommon_trap", "null_check"),
+                 GoodRuleConstraint.create(Traps.class, "instrinsicOrTypeCheckedInlining()", 5)
         );
 
 
@@ -216,18 +231,12 @@ public class TestIRMatching {
                      : BadFailOnConstraint.create(CheckCastArray.class, "arrayCopy(java.lang.Object[],java.lang.Class)", 1, "checkcast_arraycopy")
         );
 
-        // Redirect stdout to stream and then check if we find required IR encoding read from socket.
-        ByteArrayOutputStream baos = new ByteArrayOutputStream();
-        PrintStream ps = new PrintStream(baos);
-        PrintStream old = System.out;
-        System.setOut(ps);
-
         try {
             runWithArgumentsFail(CompilationOutputOfFails.class);
-            Utils.shouldHaveThrownException();
+            Utils.shouldHaveThrownException(baos.toString());
         } catch (IRViolationException e) {
             try {
-                boolean failed = false;
+                StringBuilder failures = new StringBuilder();
                 System.out.flush();
                 String output = baos.toString();
                 baos.reset();
@@ -235,36 +244,36 @@ public class TestIRMatching {
                 Matcher matcher = pattern.matcher(output);
                 long bothCount = matcher.results().count();
                 if (bothCount != 7L) {
-                    exceptions.add(new RuntimeException("Could not find all both() methods, expected 7 but found " + bothCount));
-                    failed = true;
+                    failures.append("- Could not find all both() methods, expected 7 but found ").append(bothCount).append(System.lineSeparator());
                 }
                 pattern = Pattern.compile(">>> Compilation.*ideal\\d.*\\RPrintIdeal:(?:(?!>>> Compilation)[\\S\\s])+");
                 matcher = pattern.matcher(output);
                 int count = 0;
                 while (matcher.find()) {
                     String match = matcher.group();
-                    Asserts.assertFalse(match.contains("PrintOptoAssembly"), "Cannot contain opto assembly: " + output);
+                    if (match.contains("PrintOptoAssembly")) {
+                        failures.append("Cannot contain opto assembly: ").append(System.lineSeparator()).append(match);
+                    }
                     count++;
                 }
                 if (count != 7) {
-                    exceptions.add(new RuntimeException("Could not find all ideal() methods, expected 7 but found " + count));
-                    failed = true;
+                    failures.append("- Could not find all ideal() methods, expected 7 but found ").append(count).append(System.lineSeparator());
                 }
                 pattern = Pattern.compile(">>> Compilation.*opto\\d.*\\RPrintOptoAssembly:(?:(?!>>> Compilation)[\\S\\s])+");
                 matcher = pattern.matcher(output);
                 count = 0;
                 while (matcher.find()) {
                     String match = matcher.group();
-                    Asserts.assertFalse(match.contains("PrintIdeal"), "Cannot contain opto assembly: " + output);
+                    if (match.contains("PrintIdeal")) {
+                        failures.append("Cannot contain print assembly: ").append(System.lineSeparator()).append(match);
+                    }
                     count++;
                 }
                 if (count != 7) {
-                    exceptions.add(new RuntimeException("Could not find all opto() methods, expected 7 but found " + count));
-                    failed = true;
+                    failures.append("- Could not find all opto() methods, expected 7 but found ").append(count).append(System.lineSeparator());
                 }
-                if (failed) {
-                    System.err.println(TestFramework.getLastTestVMOutput());
-                    System.err.println(output);
+                if (!failures.isEmpty()) {
+                    addException(new RuntimeException(failures.toString()));
                 }
             } catch (Exception e1) {
                 addException(e1);
@@ -276,52 +285,80 @@ public class TestIRMatching {
         runWithArguments(FlagComparisons.class, "-XX:TLABRefillWasteFraction=50");
         System.out.flush();
         String output = baos.toString();
-        baos.reset();
-        findIrIds(output, "testMatchAllIf50", 0, 21);
+        findIrIds(output, "testMatchAllIf50", 1, 22);
         findIrIds(output, "testMatchNoneIf50", -1, -1);
 
         runWithArguments(FlagComparisons.class, "-XX:TLABRefillWasteFraction=49");
         System.out.flush();
         output = baos.toString();
-        baos.reset();
-        findIrIds(output, "testMatchAllIf50", 4, 6, 13, 18);
-        findIrIds(output, "testMatchNoneIf50", 0, 3, 8, 10, 17, 22);
+        findIrIds(output, "testMatchAllIf50", 5, 7, 14, 19);
+        findIrIds(output, "testMatchNoneIf50", 1, 4, 9, 11, 18, 23);
 
         runWithArguments(FlagComparisons.class, "-XX:TLABRefillWasteFraction=51");
         System.out.flush();
         output = baos.toString();
-        baos.reset();
-        findIrIds(output, "testMatchAllIf50", 7, 12, 19, 21);
-        findIrIds(output, "testMatchNoneIf50", 4, 7, 11, 16, 20, 22);
-        System.setOut(old);
+        findIrIds(output, "testMatchAllIf50", 8, 13, 20, 22);
+        findIrIds(output, "testMatchNoneIf50", 5, 8, 12, 17, 21, 23);
+        System.setOut(oldOut);
+        System.setErr(oldErr);
 
         if (!exceptions.isEmpty()) {
-            System.err.println("TestIRMatching failed with one or more exceptions:");
-            for (Exception e : exceptions) {
-                System.err.println(e.getMessage());
+            System.err.println("TestIRMatching failed with " + exceptions.size() + " exception(s):");
+            int i = 1;
+            System.err.println("************************");
+            for (Map.Entry<Exception, String> entry : exceptions.entrySet()) {
+                System.err.println("***** Exception " + String.format("%02d", i++) +" *****");
+                System.err.println("************************");
+
+                Exception e = entry.getKey();
                 e.printStackTrace(System.err);
-                System.err.println("---------");
+                System.err.println();
+                System.err.println("===== OUTPUT ======");
+                System.err.println(entry.getValue());
+                System.err.println("MESSAGE: " + e.getMessage());
+                System.err.println("************************");
             }
-            throw new RuntimeException("TestIRMatching failed with one or more exceptions - check stderr and stdout");
+            i = 1;
+            System.err.println("====================================");
+            System.err.println("********************");
+            System.err.println("***** OVERVIEW *****");
+            System.err.println("********************");
+            for (Map.Entry<Exception, String> entry : exceptions.entrySet()) {
+                Exception e = entry.getKey();
+                System.err.print((i++) + ") ");
+                entry.getKey().printStackTrace(System.err);
+                System.err.println("********************");
+            }
+            throw new RuntimeException("TestIRMatching failed with " + exceptions.size() + " exception(s) - check stderr and stdout");
         }
     }
 
+    private static void runFramework(TestFramework framework) {
+        baos.reset();
+        baosErr.reset();
+        framework.start();
+    }
+
     private static void runWithArguments(Class<?> clazz, String... args) {
         try {
-            new TestFramework(clazz).addFlags(args).start();
+            runFramework(new TestFramework(clazz).addFlags(args));
         } catch (Exception e) {
             addException(e);
         }
     }
 
     private static void runWithArgumentsFail(Class<?> clazz, String... args) {
-        new TestFramework(clazz).addFlags(args).start();
+        runFramework(new TestFramework(clazz).addFlags(args));
     }
 
     private static void runCheck(String[] args , Constraint... constraints) {
         try {
-            new TestFramework(constraints[0].getKlass()).addFlags(args).start(); // All constraints have the same class.
-            Utils.shouldHaveThrownException();
+            TestFramework framework = new TestFramework(constraints[0].getKlass()); // All constraints have the same class.
+            if (args != null) {
+                framework.addFlags(args);
+            }
+            runFramework(framework);
+            Utils.shouldHaveThrownException(baos.toString());
         } catch (IRViolationException e) {
             checkConstraints(e, constraints);
         } catch (Exception e) {
@@ -330,14 +367,7 @@ public class TestIRMatching {
     }
 
     private static void runCheck(Constraint... constraints) {
-        try {
-            TestFramework.run(constraints[0].getKlass()); // All constraints have the same class.
-            Utils.shouldHaveThrownException();
-        } catch (IRViolationException e) {
-            checkConstraints(e, constraints);
-        } catch (Exception e) {
-            addException(e);
-        }
+        runCheck(null, constraints);
     }
 
     private static void checkConstraints(IRViolationException e, Constraint[] constraints) {
@@ -347,25 +377,9 @@ public class TestIRMatching {
                 constraint.checkConstraint(e);
             }
         } catch (Exception e1) {
-            System.out.println(TestFramework.getLastTestVMOutput());
+            System.out.println(e.getCompilations());
             System.out.println(message);
-            exceptions.add(e1);
-        }
-    }
-
-    // Single constraint
-    private static void runFailOnTestsArgs(Constraint constraint, String... args) {
-        try {
-            new TestFramework(constraint.getKlass()).addFlags(args).start(); // All constraints have the same class.
-            Utils.shouldHaveThrownException();
-        } catch (IRViolationException e) {
-            try {
-                constraint.checkConstraint(e);
-            } catch (Exception e1) {
-                addException(e);
-            }
-        } catch (Exception e) {
-            addException(e);
+            addException(e1);
         }
     }
 
@@ -380,8 +394,9 @@ public class TestIRMatching {
                 builder.append(j);
             }
         }
-        Asserts.assertTrue(output.contains(builder.toString()), "Could not find encoding: \"" + builder.toString()
-                                                                + System.lineSeparator());
+        if (!output.contains(builder.toString())) {
+            addException(new RuntimeException("Could not find encoding: \"" + builder + System.lineSeparator()));
+        }
     }
 }
 
@@ -535,20 +550,20 @@ class MultipleFailOnBad {
 class FlagComparisons {
     // Applies all IR rules if TLABRefillWasteFraction=50
     @Test
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "50"}) // Index 0
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "50"}) // Index 1
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "=50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "= 50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " =   50"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<=50"}) // Index 4
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<=50"}) // Index 5
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<= 50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " <=  50"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">=50"}) // Index 7
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">=50"}) // Index 8
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">= 50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " >=  50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">49"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "> 49"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " >  49"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<51"}) // Index 13
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<51"}) // Index 14
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "< 51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " <  51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "!=51"})
@@ -556,34 +571,34 @@ class FlagComparisons {
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " !=  51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "!=49"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "!= 49"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " !=  49"}) // Index 21
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " !=  49"}) // Index 22
     public void testMatchAllIf50() {}
 
     // Applies no IR rules if TLABRefillWasteFraction=50
     @Test
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "49"}) // Index 0
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "49"}) // Index 1
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "=49"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "= 49"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " =  49"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "51"}) // Index 4
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "51"}) // Index 5
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "=51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "= 51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " =  51"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<=49"}) // Index 8
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<=49"}) // Index 9
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<= 49"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " <=  49"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">=51"}) // Index 11
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">=51"}) // Index 12
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">= 51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " >=  51"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", ">50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "> 50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " >  50"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<50"}) // Index 17
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "<50"}) // Index 18
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "< 50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " <  50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "!=50"})
     @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", "!= 50"})
-    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " !=  50"}) // Index 22
+    @IR(failOn = IRNode.CALL, applyIf = {"TLABRefillWasteFraction", " !=  50"}) // Index 23
     public void testMatchNoneIf50() {}
 }
 
@@ -799,14 +814,14 @@ class BadCount {
     int iFld;
     int result;
     @Test
-    @IR(counts = {IRNode.LOAD, "!= 1"})
+    @IR(counts = {IRNode.LOAD, "!= 1"}) // fail
     @IR(counts = {IRNode.STORE, "> 0"})
     public void bad1() {
         result = iFld;
     }
 
     @Test
-    @IR(counts = {IRNode.LOAD, "1"})
+    @IR(counts = {IRNode.LOAD, "1"}) // fail
     @IR(counts = {IRNode.STORE, "< 1"})
     public void bad2() {
         result = iFld;
@@ -814,8 +829,8 @@ class BadCount {
 
 
     @Test
-    @IR(counts = {IRNode.LOAD, "0"})
-    @IR(counts = {IRNode.STORE, " <= 0"})
+    @IR(counts = {IRNode.LOAD, "0"}) // fail
+    @IR(counts = {IRNode.STORE, " <= 0"}) // fail
     public void bad3() {
         result = iFld;
     }
@@ -867,6 +882,29 @@ class RunTests {
     }
 }
 
+class Calls {
+
+    @Test
+    @IR(counts = {IRNode.CALL, "1"})
+    @IR(failOn = {IRNode.CALL_OF_METHOD, "dontInline",  // Fails
+                  IRNode.STATIC_CALL_OF_METHOD, "dontInline"}) // Fails
+    @IR(failOn = {IRNode.CALL_OF_METHOD, "forceInline",
+                  IRNode.STATIC_CALL_OF_METHOD, "forceInline",
+                  IRNode.CALL_OF_METHOD, "dontInlines",
+                  IRNode.STATIC_CALL_OF_METHOD, "dontInlines",
+                  IRNode.CALL_OF_METHOD, "dont",
+                  IRNode.STATIC_CALL_OF_METHOD, "dont"})
+    public void calls() {
+        dontInline();
+        forceInline();
+    }
+
+    @DontInline
+    public void dontInline() {}
+
+    @ForceInline
+    public void forceInline() {}
+}
 
 class AllocArray {
     MyClass[] myClassArray;
@@ -994,6 +1032,7 @@ class Traps {
                   IRNode.NULL_ASSERT_TRAP,
                   IRNode.RANGE_CHECK_TRAP,
                   IRNode.CLASS_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public void noTraps() {
@@ -1014,6 +1053,7 @@ class Traps {
                   IRNode.NULL_ASSERT_TRAP,
                   IRNode.RANGE_CHECK_TRAP,
                   IRNode.CLASS_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public void predicateTrap() {
@@ -1033,6 +1073,7 @@ class Traps {
                   IRNode.NULL_ASSERT_TRAP,
                   IRNode.RANGE_CHECK_TRAP,
                   IRNode.CLASS_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public void nullCheck() {
@@ -1049,6 +1090,7 @@ class Traps {
                   IRNode.UNSTABLE_IF_TRAP,
                   IRNode.RANGE_CHECK_TRAP,
                   IRNode.CLASS_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public Object nullAssert() {
@@ -1064,6 +1106,7 @@ class Traps {
                   IRNode.NULL_ASSERT_TRAP,
                   IRNode.RANGE_CHECK_TRAP,
                   IRNode.CLASS_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public void unstableIf(boolean flag) {
@@ -1082,6 +1125,7 @@ class Traps {
                   IRNode.UNSTABLE_IF_TRAP,
                   IRNode.NULL_ASSERT_TRAP,
                   IRNode.RANGE_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public void classCheck() {
@@ -1100,6 +1144,7 @@ class Traps {
                   IRNode.UNSTABLE_IF_TRAP,
                   IRNode.NULL_ASSERT_TRAP,
                   IRNode.CLASS_CHECK_TRAP,
+                  IRNode.INTRINSIC_TRAP,
                   IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP,
                   IRNode.UNHANDLED_TRAP})
     public void rangeCheck() {
@@ -1110,6 +1155,7 @@ class Traps {
     @Test
     @IR(failOn = IRNode.TRAP) // fails
     @IR(failOn = IRNode.INTRINSIC_OR_TYPE_CHECKED_INLINING_TRAP) // fails
+    @IR(failOn = IRNode.INTRINSIC_TRAP) // fails
     @IR(failOn = IRNode.NULL_CHECK_TRAP) // fails
     @IR(failOn = {IRNode.PREDICATE_TRAP,
                   IRNode.UNSTABLE_IF_TRAP,
@@ -1468,7 +1514,7 @@ abstract class Constraint {
                 }
             }
         }
-        Asserts.assertTrue(matched, toString() + " should have been matched");
+        Asserts.assertTrue(matched, this + " should have been matched");
     }
 
     abstract protected void checkIRRule(String irRule);
@@ -1686,7 +1732,7 @@ class BadCountsConstraint extends RegexConstraint {
 
     private static List<String> getMatchesList(int foundCount, String[] matches, List<String> strings) {
         List<String> matchesList = new ArrayList<>();
-        matchesList.add("but found " + foundCount);
+        matchesList.add("Failed comparison: [found] " + foundCount);
         if (matches != null) {
             matchesList.addAll(strings);
         }
