@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,10 +25,11 @@
  * @test
  * @bug 4028006 4044013 4096694 4107276 4107570 4112869 4130885 7039469 7126465 7158483
  *      8008577 8077685 8098547 8133321 8138716 8148446 8151876 8159684 8166875 8181157
- *      8228469 8274407 8305113
+ *      8228469 8274407 8285844
  * @modules java.base/sun.util.resources
+ * @library /java/text/testlib
  * @summary test TimeZone
- * @run junit TimeZoneTest
+ * @run main TimeZoneTest -verbose
  */
 
 import java.io.*;
@@ -36,14 +37,14 @@ import java.text.*;
 import java.util.*;
 import sun.util.resources.LocaleData;
 
-import org.junit.jupiter.api.Test;
-
-import static org.junit.jupiter.api.Assertions.fail;
-
-public class TimeZoneTest
+public class TimeZoneTest extends IntlTest
 {
     static final int millisPerHour = 3600000;
 
+    public static void main(String[] args) throws Exception {
+        new TimeZoneTest().run(args);
+    }
+
     /**
      * Bug 4130885
      * Certain short zone IDs, used since 1.1.x, are incorrect.
@@ -99,7 +100,6 @@ public class TimeZoneTest
      * else. E.g., EST usually indicates the US Eastern zone, so it cannot be
      * used for Brazil (BET).
      */
-    @Test
     public void TestShortZoneIDs() throws Exception {
 
         ZoneDescriptor[] JDK_116_REFERENCE_LIST = {
@@ -122,7 +122,7 @@ public class TimeZoneTest
             new ZoneDescriptor("GMT", 0, false),
             new ZoneDescriptor("UTC", 0, false),
             new ZoneDescriptor("ECT", 60, true),
-            new ZoneDescriptor("ART", 120, true),
+            new ZoneDescriptor("ART", 120, false),
             new ZoneDescriptor("EET", 120, true),
             new ZoneDescriptor("EAT", 180, false),
             new ZoneDescriptor("MET", 60, true),
@@ -154,10 +154,10 @@ public class TimeZoneTest
             ZoneDescriptor referenceZone = JDK_116_REFERENCE_LIST[i];
             ZoneDescriptor currentZone = hash.get(referenceZone.getID());
             if (referenceZone.equals(currentZone)) {
-                System.out.println("ok " + referenceZone);
+                logln("ok " + referenceZone);
             }
             else {
-                fail("Fail: Expected " + referenceZone +
+                errln("Fail: Expected " + referenceZone +
                       "; got " + currentZone);
             }
         }
@@ -221,12 +221,14 @@ public class TimeZoneTest
         }
     }
 
-    static final String formatMinutes(int min) {
+    static final String formatSeconds(int sec) {
         char sign = '+';
-        if (min < 0) { sign = '-'; min = -min; }
-        int h = min/60;
-        min = min%60;
-        return "" + sign + h + ":" + ((min<10) ? "0" : "") + min;
+        if (sec < 0) { sign = '-'; sec = -sec; }
+        int h = sec / 3_600;
+        int m = sec % 3_600 / 60;
+        sec = sec % 60;
+        return "" + sign + h + ":" + ((m<10) ? "0" : "") + m +
+                (sec > 0 ? ":" + ((sec < 10) ? "0" : "") + sec : "");
     }
     /**
      * As part of the VM fix (see CCC approved RFE 4028006, bug
@@ -239,50 +241,56 @@ public class TimeZoneTest
      * ID "Custom" is no longer used for TimeZone objects created with
      * a custom time zone ID, such as "GMT-8". See 4322313.
      */
-    @Test
     public void TestCustomParse() throws Exception {
         Object[] DATA = {
-            // ID        Expected offset in minutes
-            "GMT",       null,
-            "GMT+0",     new Integer(0),
-            "GMT+1",     new Integer(60),
-            "GMT-0030",  new Integer(-30),
-            "GMT+15:99", null,
-            "GMT+",      null,
-            "GMT-",      null,
-            "GMT+0:",    null,
-            "GMT-:",     null,
-            "GMT+0010",  new Integer(10), // Interpret this as 00:10
-            "GMT-10",    new Integer(-10*60),
-            "GMT+30",    null,
-            "GMT-3:30",  new Integer(-(3*60+30)),
-            "GMT-230",   new Integer(-(2*60+30)),
+            // ID               Expected offset in seconds
+            "GMT",              null,
+            "GMT+0",            0,
+            "GMT+1",            60 * 60,
+            "GMT-0030",         -30 * 60,
+            "GMT+15:99",        null,
+            "GMT+",             null,
+            "GMT-",             null,
+            "GMT+0:",           null,
+            "GMT-:",            null,
+            "GMT+0010",         10 * 60, // Interpret this as 00:10
+            "GMT-10",           -10 * 60 * 60,
+            "GMT+30",           null,
+            "GMT-3:30",         -(3 * 60 + 30) * 60,
+            "GMT-230",          -(2 * 60 + 30) * 60,
+            "GMT+00:00:01",     1,
+            "GMT-00:00:01",     -1,
+            "GMT+00000",        null,
+            "GMT+00:00:01:",    null,
+            "GMT+00:00:012",    null,
+            "GMT+00:00:0",      null,
+            "GMT+00:00:",       null,
         };
         for (int i=0; i<DATA.length; i+=2) {
             String id = (String)DATA[i];
             Integer exp = (Integer)DATA[i+1];
             TimeZone zone = TimeZone.getTimeZone(id);
             if (zone.getID().equals("GMT")) {
-                System.out.println(id + " -> generic GMT");
+                logln(id + " -> generic GMT");
                 // When TimeZone.getTimeZone() can't parse the id, it
                 // returns GMT -- a dubious practice, but required for
                 // backward compatibility.
                 if (exp != null) {
-                    throw new Exception("Expected offset of " + formatMinutes(exp.intValue()) +
+                    throw new Exception("Expected offset of " + formatSeconds(exp.intValue()) +
                                         " for " + id + ", got parse failure");
                 }
             }
             else {
-                int ioffset = zone.getRawOffset()/60000;
-                String offset = formatMinutes(ioffset);
-                System.out.println(id + " -> " + zone.getID() + " GMT" + offset);
+                int ioffset = zone.getRawOffset() / 1_000;
+                String offset = formatSeconds(ioffset);
+                logln(id + " -> " + zone.getID() + " GMT" + offset);
                 if (exp == null) {
                     throw new Exception("Expected parse failure for " + id +
                                         ", got offset of " + offset +
                                         ", id " + zone.getID());
                 }
                 else if (ioffset != exp.intValue()) {
-                    throw new Exception("Expected offset of " + formatMinutes(exp.intValue()) +
+                    throw new Exception("Expected offset of " + formatSeconds(exp.intValue()) +
                                         ", id Custom, for " + id +
                                         ", got offset of " + offset +
                                         ", id " + zone.getID());
@@ -302,13 +310,12 @@ public class TimeZoneTest
      * 4/21/98 - make smarter, so the test works if the ext resources
      * are present or not.
      */
-    @Test
     public void TestDisplayName() {
         TimeZone zone = TimeZone.getTimeZone("PST");
         String name = zone.getDisplayName(Locale.ENGLISH);
-        System.out.println("PST->" + name);
+        logln("PST->" + name);
         if (!name.equals("Pacific Standard Time"))
-            fail("Fail: Expected \"Pacific Standard Time\"");
+            errln("Fail: Expected \"Pacific Standard Time\"");
 
         //*****************************************************************
         // THE FOLLOWING LINES MUST BE UPDATED IF THE LOCALE DATA CHANGES
@@ -327,7 +334,7 @@ public class TimeZoneTest
                                        ((Integer)DATA[i+1]).intValue(),
                                        Locale.ENGLISH);
             if (!name.equals(DATA[i+2]))
-                fail("Fail: Expected " + DATA[i+2] + "; got " + name);
+                errln("Fail: Expected " + DATA[i+2] + "; got " + name);
         }
 
         // Make sure that we don't display the DST name by constructing a fake
@@ -335,11 +342,11 @@ public class TimeZoneTest
         SimpleTimeZone zone2 = new SimpleTimeZone(0, "PST");
         zone2.setStartRule(Calendar.JANUARY, 1, 0);
         zone2.setEndRule(Calendar.DECEMBER, 31, 0);
-        System.out.println("Modified PST inDaylightTime->" + zone2.inDaylightTime(new Date()));
+        logln("Modified PST inDaylightTime->" + zone2.inDaylightTime(new Date()));
         name = zone2.getDisplayName(Locale.ENGLISH);
-        System.out.println("Modified PST->" + name);
+        logln("Modified PST->" + name);
         if (!name.equals("Pacific Standard Time"))
-            fail("Fail: Expected \"Pacific Standard Time\"");
+            errln("Fail: Expected \"Pacific Standard Time\"");
 
         // Make sure we get the default display format for Locales
         // with no display name data.
@@ -350,7 +357,7 @@ public class TimeZoneTest
         // THE FOLLOWING LINE MUST BE UPDATED IF THE LOCALE DATA CHANGES
         // THE FOLLOWING LINE MUST BE UPDATED IF THE LOCALE DATA CHANGES
         //*****************************************************************
-        System.out.println("PST(zh_CN)->" + name);
+        logln("PST(zh_CN)->" + name);
 
         // Now be smart -- check to see if zh resource is even present.
         // If not, we expect the en fallback behavior.
@@ -362,9 +369,9 @@ public class TimeZoneTest
         boolean noZH = enRB == zhRB;
 
         if (noZH) {
-            System.out.println("Warning: Not testing the zh_CN behavior because resource is absent");
+            logln("Warning: Not testing the zh_CN behavior because resource is absent");
             if (!name.equals("Pacific Standard Time"))
-                fail("Fail: Expected Pacific Standard Time");
+                errln("Fail: Expected Pacific Standard Time");
         }
         else if (!name.equals("Pacific Standard Time") &&
                  !name.equals("\u592a\u5e73\u6d0b\u6807\u51c6\u65f6\u95f4") &&
@@ -373,49 +380,48 @@ public class TimeZoneTest
                  !name.equals("GMT-8:00") &&
                  !name.equals("GMT-0800") &&
                  !name.equals("GMT-800")) {
-            fail("Fail: Expected GMT-08:00 or something similar\n"
-            + "************************************************************\n"
-            + "THE ABOVE FAILURE MAY JUST MEAN THE LOCALE DATA HAS CHANGED\n"
-            + "************************************************************\n");
+            errln("Fail: Expected GMT-08:00 or something similar");
+            errln("************************************************************");
+            errln("THE ABOVE FAILURE MAY JUST MEAN THE LOCALE DATA HAS CHANGED");
+            errln("************************************************************");
         }
 
         // Now try a non-existent zone
         zone2 = new SimpleTimeZone(90*60*1000, "xyzzy");
         name = zone2.getDisplayName(Locale.ENGLISH);
-        System.out.println("GMT+90min->" + name);
+        logln("GMT+90min->" + name);
         if (!name.equals("GMT+01:30") &&
             !name.equals("GMT+1:30") &&
             !name.equals("GMT+0130") &&
             !name.equals("GMT+130"))
-            fail("Fail: Expected GMT+01:30 or something similar");
+            errln("Fail: Expected GMT+01:30 or something similar");
     }
 
-    @Test
     public void TestGenericAPI() {
         String id = "NewGMT";
         int offset = 12345;
 
         SimpleTimeZone zone = new SimpleTimeZone(offset, id);
         if (zone.useDaylightTime()) {
-            fail("FAIL: useDaylightTime should return false");
+            errln("FAIL: useDaylightTime should return false");
         }
 
         TimeZone zoneclone = (TimeZone)zone.clone();
         if (!zoneclone.equals(zone)) {
-            fail("FAIL: clone or operator== failed");
+            errln("FAIL: clone or operator== failed");
         }
         zoneclone.setID("abc");
         if (zoneclone.equals(zone)) {
-            fail("FAIL: clone or operator!= failed");
+            errln("FAIL: clone or operator!= failed");
         }
 
         zoneclone = (TimeZone)zone.clone();
         if (!zoneclone.equals(zone)) {
-            fail("FAIL: clone or operator== failed");
+            errln("FAIL: clone or operator== failed");
         }
         zoneclone.setRawOffset(45678);
         if (zoneclone.equals(zone)) {
-            fail("FAIL: clone or operator!= failed");
+            errln("FAIL: clone or operator!= failed");
         }
 
         TimeZone saveDefault = TimeZone.getDefault();
@@ -423,10 +429,10 @@ public class TimeZoneTest
             TimeZone.setDefault(zone);
             TimeZone defaultzone = TimeZone.getDefault();
             if (defaultzone == zone) {
-                fail("FAIL: Default object is identical, not clone");
+                errln("FAIL: Default object is identical, not clone");
             }
             if (!defaultzone.equals(zone)) {
-                fail("FAIL: Default object is not equal");
+                errln("FAIL: Default object is not equal");
             }
         }
         finally {
@@ -435,14 +441,13 @@ public class TimeZoneTest
     }
 
     @SuppressWarnings("deprecation")
-    @Test
     public void TestRuleAPI()
     {
         // ErrorCode status = ZERO_ERROR;
 
         int offset = (int)(60*60*1000*1.75); // Pick a weird offset
         SimpleTimeZone zone = new SimpleTimeZone(offset, "TestZone");
-        if (zone.useDaylightTime()) fail("FAIL: useDaylightTime should return false");
+        if (zone.useDaylightTime()) errln("FAIL: useDaylightTime should return false");
 
         // Establish our expected transition times.  Do this with a non-DST
         // calendar with the (above) declared local offset.
@@ -470,18 +475,18 @@ public class TimeZoneTest
         long expMarchOne = 636251400000L;
         if (marchOne != expMarchOne)
         {
-            fail("FAIL: Expected start computed as " + marchOne +
+            errln("FAIL: Expected start computed as " + marchOne +
                   " = " + new Date(marchOne));
-            System.out.println("      Should be                  " + expMarchOne +
+            logln("      Should be                  " + expMarchOne +
                   " = " + new Date(expMarchOne));
         }
 
         long expJulyOne = 646793100000L;
         if (julyOne != expJulyOne)
         {
-            fail("FAIL: Expected start computed as " + julyOne +
+            errln("FAIL: Expected start computed as " + julyOne +
                   " = " + new Date(julyOne));
-            System.out.println("      Should be                  " + expJulyOne +
+            logln("      Should be                  " + expJulyOne +
                   " = " + new Date(expJulyOne));
         }
 
@@ -492,15 +497,15 @@ public class TimeZoneTest
 
         if (zone.inDaylightTime(new Date(marchOne - 1000)) ||
             !zone.inDaylightTime(new Date(marchOne)))
-            fail("FAIL: Start rule broken");
+            errln("FAIL: Start rule broken");
         if (!zone.inDaylightTime(new Date(julyOne - 1000)) ||
             zone.inDaylightTime(new Date(julyOne)))
-            fail("FAIL: End rule broken");
+            errln("FAIL: End rule broken");
 
         zone.setStartYear(1991);
         if (zone.inDaylightTime(new Date(marchOne)) ||
             zone.inDaylightTime(new Date(julyOne - 1000)))
-            fail("FAIL: Start year broken");
+            errln("FAIL: Start year broken");
 
         // failure(status, "TestRuleAPI");
         // delete gc;
@@ -513,7 +518,7 @@ public class TimeZoneTest
         boolean startsInDST = tz.inDaylightTime(new Date(min));
         // if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
         if (tz.inDaylightTime(new Date(max)) == startsInDST) {
-            System.out.println("Error: inDaylightTime(" + new Date(max) + ") != " + (!startsInDST));
+            logln("Error: inDaylightTime(" + new Date(max) + ") != " + (!startsInDST));
             return;
         }
         // if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
@@ -527,54 +532,52 @@ public class TimeZoneTest
             }
             // if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
         }
-        System.out.println("Binary Search Before: " + min + " = " + new Date(min));
-        System.out.println("Binary Search After:  " + max + " = " + new Date(max));
+        logln("Binary Search Before: " + min + " = " + new Date(min));
+        logln("Binary Search After:  " + max + " = " + new Date(max));
         long mindelta = expectedBoundary - min;
         long maxdelta = max - expectedBoundary;
         if (mindelta >= 0 &&
             mindelta <= INTERVAL &&
             mindelta >= 0 &&
             mindelta <= INTERVAL)
-            System.out.println("PASS: Expected bdry:  " + expectedBoundary + " = " + new Date(expectedBoundary));
+            logln("PASS: Expected bdry:  " + expectedBoundary + " = " + new Date(expectedBoundary));
         else
-            fail("FAIL: Expected bdry:  " + expectedBoundary + " = " + new Date(expectedBoundary));
+            errln("FAIL: Expected bdry:  " + expectedBoundary + " = " + new Date(expectedBoundary));
     }
 
     static final int INTERVAL = 100;
 
     // Bug 006; verify the offset for a specific zone.
-    @Test
     public void TestPRTOffset()
     {
         TimeZone tz = TimeZone.getTimeZone( "PRT" );
         if( tz == null ) {
-            fail( "FAIL: TimeZone(PRT) is null" );
+            errln( "FAIL: TimeZone(PRT) is null" );
         }
         else{
             if (tz.getRawOffset() != (-4*millisPerHour))
-                fail("FAIL: Offset for PRT should be -4");
+                errln("FAIL: Offset for PRT should be -4");
         }
 
     }
 
     // Test various calls
     @SuppressWarnings("deprecation")
-    @Test
     public void TestVariousAPI518()
     {
         TimeZone time_zone = TimeZone.getTimeZone("PST");
         Date d = new Date(97, Calendar.APRIL, 30);
 
-        System.out.println("The timezone is " + time_zone.getID());
+        logln("The timezone is " + time_zone.getID());
 
         if (time_zone.inDaylightTime(d) != true)
-            fail("FAIL: inDaylightTime returned false");
+            errln("FAIL: inDaylightTime returned false");
 
         if (time_zone.useDaylightTime() != true)
-            fail("FAIL: useDaylightTime returned false");
+            errln("FAIL: useDaylightTime returned false");
 
         if (time_zone.getRawOffset() != -8*millisPerHour)
-            fail( "FAIL: getRawOffset returned wrong value");
+            errln( "FAIL: getRawOffset returned wrong value");
 
         GregorianCalendar gc = new GregorianCalendar();
         gc.setTime(d);
@@ -582,11 +585,10 @@ public class TimeZoneTest
                                 gc.get(gc.DAY_OF_MONTH),
                                 gc.get(gc.DAY_OF_WEEK), 0)
             != -7*millisPerHour)
-            fail("FAIL: getOffset returned wrong value");
+            errln("FAIL: getOffset returned wrong value");
     }
 
     // Test getAvailableID API
-    @Test
     public void TestGetAvailableIDs913()
     {
         StringBuffer buf = new StringBuffer("TimeZone.getAvailableIDs() = { ");
@@ -597,7 +599,7 @@ public class TimeZoneTest
             buf.append(s[i]);
         }
         buf.append(" };");
-        System.out.println(buf.toString());
+        logln(buf.toString());
 
         buf.setLength(0);
         buf.append("TimeZone.getAvailableIDs(GMT+02:00) = { ");
@@ -608,32 +610,31 @@ public class TimeZoneTest
             buf.append(s[i]);
         }
         buf.append(" };");
-        System.out.println(buf.toString());
+        logln(buf.toString());
 
         TimeZone tz = TimeZone.getTimeZone("PST");
         if (tz != null)
-            System.out.println("getTimeZone(PST) = " + tz.getID());
+            logln("getTimeZone(PST) = " + tz.getID());
         else
-            fail("FAIL: getTimeZone(PST) = null");
+            errln("FAIL: getTimeZone(PST) = null");
 
         tz = TimeZone.getTimeZone("America/Los_Angeles");
         if (tz != null)
-            System.out.println("getTimeZone(America/Los_Angeles) = " + tz.getID());
+            logln("getTimeZone(America/Los_Angeles) = " + tz.getID());
         else
-            fail("FAIL: getTimeZone(PST) = null");
+            errln("FAIL: getTimeZone(PST) = null");
 
         // Bug 4096694
         tz = TimeZone.getTimeZone("NON_EXISTENT");
         if (tz == null)
-            fail("FAIL: getTimeZone(NON_EXISTENT) = null");
+            errln("FAIL: getTimeZone(NON_EXISTENT) = null");
         else if (!tz.getID().equals("GMT"))
-            fail("FAIL: getTimeZone(NON_EXISTENT) = " + tz.getID());
+            errln("FAIL: getTimeZone(NON_EXISTENT) = " + tz.getID());
     }
 
     /**
      * Bug 4107276
      */
-    @Test
     public void TestDSTSavings() {
         // It might be better to find a way to integrate this test into the main TimeZone
         // tests above, but I don't have time to figure out how to do this (or if it's
@@ -643,44 +644,43 @@ public class TimeZoneTest
                                                (int)(0.5 * millisPerHour));
 
         if (tz.getRawOffset() != -5 * millisPerHour)
-            fail("Got back a raw offset of " + (tz.getRawOffset() / millisPerHour) +
+            errln("Got back a raw offset of " + (tz.getRawOffset() / millisPerHour) +
                   " hours instead of -5 hours.");
         if (!tz.useDaylightTime())
-            fail("Test time zone should use DST but claims it doesn't.");
+            errln("Test time zone should use DST but claims it doesn't.");
         if (tz.getDSTSavings() != 0.5 * millisPerHour)
-            fail("Set DST offset to 0.5 hour, but got back " + (tz.getDSTSavings() /
+            errln("Set DST offset to 0.5 hour, but got back " + (tz.getDSTSavings() /
                                                                  millisPerHour) + " hours instead.");
 
         int offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JANUARY, 1,
                                   Calendar.THURSDAY, 10 * millisPerHour);
         if (offset != -5 * millisPerHour)
-            fail("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got "
+            errln("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JUNE, 1, Calendar.MONDAY,
                               10 * millisPerHour);
         if (offset != -4.5 * millisPerHour)
-            fail("The offset for 10 AM, 6/1/98 should have been -4.5 hours, but we got "
+            errln("The offset for 10 AM, 6/1/98 should have been -4.5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         tz.setDSTSavings(millisPerHour);
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JANUARY, 1,
                               Calendar.THURSDAY, 10 * millisPerHour);
         if (offset != -5 * millisPerHour)
-            fail("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got "
+            errln("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.JUNE, 1, Calendar.MONDAY,
                               10 * millisPerHour);
         if (offset != -4 * millisPerHour)
-            fail("The offset for 10 AM, 6/1/98 (with a 1-hour DST offset) should have been -4 hours, but we got "
+            errln("The offset for 10 AM, 6/1/98 (with a 1-hour DST offset) should have been -4 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
     }
 
     /**
      * Bug 4107570
      */
-    @Test
     public void TestAlternateRules() {
         // Like TestDSTSavings, this test should probably be integrated somehow with the main
         // test at the top of this class, but I didn't have time to figure out how to do that.
@@ -695,25 +695,25 @@ public class TimeZoneTest
         int offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 5,
                                   Calendar.THURSDAY, 10 * millisPerHour);
         if (offset != -5 * millisPerHour)
-            fail("The offset for 10AM, 3/5/98 should have been -5 hours, but we got "
+            errln("The offset for 10AM, 3/5/98 should have been -5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 15,
                               Calendar.SUNDAY, 10 * millisPerHour);
         if (offset != -4 * millisPerHour)
-            fail("The offset for 10AM, 3/15/98 should have been -4 hours, but we got "
+            errln("The offset for 10AM, 3/15/98 should have been -4 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 15,
                               Calendar.THURSDAY, 10 * millisPerHour);
         if (offset != -4 * millisPerHour)
-            fail("The offset for 10AM, 10/15/98 should have been -4 hours, but we got "
+            errln("The offset for 10AM, 10/15/98 should have been -4 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 25,
                               Calendar.SUNDAY, 10 * millisPerHour);
         if (offset != -5 * millisPerHour)
-            fail("The offset for 10AM, 10/25/98 should have been -5 hours, but we got "
+            errln("The offset for 10AM, 10/25/98 should have been -5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         // test the day-of-week-after-day-in-month API
@@ -723,25 +723,25 @@ public class TimeZoneTest
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 11,
                               Calendar.WEDNESDAY, 10 * millisPerHour);
         if (offset != -5 * millisPerHour)
-            fail("The offset for 10AM, 3/11/98 should have been -5 hours, but we got "
+            errln("The offset for 10AM, 3/11/98 should have been -5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.MARCH, 14,
                               Calendar.SATURDAY, 10 * millisPerHour);
         if (offset != -4 * millisPerHour)
-            fail("The offset for 10AM, 3/14/98 should have been -4 hours, but we got "
+            errln("The offset for 10AM, 3/14/98 should have been -4 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 15,
                               Calendar.THURSDAY, 10 * millisPerHour);
         if (offset != -4 * millisPerHour)
-            fail("The offset for 10AM, 10/15/98 should have been -4 hours, but we got "
+            errln("The offset for 10AM, 10/15/98 should have been -4 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
 
         offset = tz.getOffset(GregorianCalendar.AD, 1998, Calendar.OCTOBER, 17,
                               Calendar.SATURDAY, 10 * millisPerHour);
         if (offset != -5 * millisPerHour)
-            fail("The offset for 10AM, 10/17/98 should have been -5 hours, but we got "
+            errln("The offset for 10AM, 10/17/98 should have been -5 hours, but we got "
                   + (offset / millisPerHour) + " hours.");
     }
 }
