@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,15 +23,15 @@
 
 /*
  * @test
- * @bug 8217375
+ * @bug 8217375 8260286 8267319
  * @summary This test is used to verify the compatibility of jarsigner across
  *     different JDK releases. It also can be used to check jar signing (w/
  *     and w/o TSA) and to verify some specific signing and digest algorithms.
  *     Note that this is a manual test. For more details about the test and
  *     its usages, please look through the README.
  *
- * @library /test/lib /lib/testlibrary ../warnings
- * @compile -source 1.6 -target 1.6 JdkUtils.java
+ * @library /test/lib ../warnings
+ * @compile -source 1.7 -target 1.7 JdkUtils.java
  * @run main/manual/othervm Compatibility
  */
 
@@ -67,6 +67,7 @@ import java.util.jar.Manifest;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
+import jdk.test.lib.Platform;
 import jdk.test.lib.process.OutputAnalyzer;
 import jdk.test.lib.process.ProcessTools;
 import jdk.test.lib.util.JarUtils;
@@ -714,12 +715,13 @@ public class Compatibility {
         Status verifyingStatus = verifyingStatus(signItem, verifyItem, verifyOA);
 
         try {
-            String match = "^  (("
+            String match = "^  ("
                     + "  Signature algorithm: " + signItem.certInfo.
                             expectedSigalg() + ", " + signItem.certInfo.
                             expectedKeySize() + "-bit key"
                     + ")|("
                     + "  Digest algorithm: " + signItem.expectedDigestAlg()
+                    + (isWeakAlg(signItem.expectedDigestAlg()) ? " \\(weak\\)" : "")
                     + (signItem.tsaIndex < 0 ? "" :
                       ")|("
                     + "Timestamped by \".+\" on .*"
@@ -729,7 +731,7 @@ public class Compatibility {
                     + ")|("
                     + "  Timestamp signature algorithm: .*"
                       )
-                    + "))$";
+                    + ")$";
             verifyOA.stdoutShouldMatchByLine(
                     "^- Signed by \"CN=" +  signItem.certInfo.toString()
                             .replaceAll("[.]", "[.]") + "\"$",
@@ -805,7 +807,12 @@ public class Compatibility {
         boolean warning = false;
         for (String line : outputAnalyzer.getOutput().lines()
                 .toArray(String[]::new)) {
-            if (line.isBlank()) continue;
+            if (line.isBlank()) {
+                // If line is blank and warning flag is true, it is the end of warnings section
+                // This is needed when some info is added after warnings, such as timestamp expiration date
+                if (warning) warning = false;
+                continue;
+            }
             if (Test.JAR_VERIFIED.equals(line)) continue;
             if (line.matches(Test.ERROR + " ?") && expectedExitCode == 0) {
                 System.out.println("verifyingStatus: error: line.matches(" + Test.ERROR + "\" ?\"): " + line);
@@ -835,6 +842,9 @@ public class Compatibility {
                     + "not be able to validate this jar after the signer "
                     + "certificate's expiration date \\([^\\)]+\\) or after "
                     + "any future revocation date[.]") && !tsa) continue;
+
+            if (isWeakAlg(signItem.expectedDigestAlg())
+                    && line.contains(Test.WEAK_ALGORITHM_WARNING)) continue;
             if (Test.CERTIFICATE_SELF_SIGNED.equals(line)) continue;
             if (Test.HAS_EXPIRED_CERT_VERIFYING_WARNING.equals(line)
                     && signItem.certInfo.expired) continue;
@@ -844,6 +854,10 @@ public class Compatibility {
         return warning ? Status.WARNING : Status.NORMAL;
     }
 
+    private static boolean isWeakAlg(String alg) {
+        return SHA1.equals(alg);
+    }
+
     // Using specified jarsigner to sign the pre-created jar with specified
     // algorithms.
     private static OutputAnalyzer signJar(String jarsignerPath, String sigalg,
@@ -1021,13 +1035,19 @@ public class Compatibility {
             throws Throwable {
         long start = System.currentTimeMillis();
         try {
+            String[] cmd;
+            if (Platform.isWindows()) {
+                cmd = new String[args.length + 3];
+                System.arraycopy(args, 0, cmd, 3, args.length);
+            } else {
+                cmd = new String[args.length + 4];
+                cmd[3] = "-J-Djava.security.egd=file:/dev/./urandom";
+                System.arraycopy(args, 0, cmd, 4, args.length);
+            }
 
-            String[] cmd = new String[args.length + 4];
             cmd[0] = toolPath;
             cmd[1] = "-J-Duser.language=en";
             cmd[2] = "-J-Duser.country=US";
-            cmd[3] = "-J-Djava.security.egd=file:/dev/./urandom";
-            System.arraycopy(args, 0, cmd, 4, args.length);
             return ProcessTools.executeCommand(cmd);
 
         } finally {
@@ -1179,9 +1199,9 @@ public class Compatibility {
 
             // defaults
             if (RSA.equals(keyAlgorithm) || DSA.equals(keyAlgorithm)) {
-                return 2048;
+                return 3072;
             } else if (EC.equals(keyAlgorithm)) {
-                return 256;
+                return 384;
             } else {
                 throw new RuntimeException("problem determining key size");
             }
