@@ -29,70 +29,90 @@ import jdk.jpackage.test.PackageTest;
 import jdk.jpackage.test.PackageType;
 import jdk.jpackage.test.Annotations.Test;
 import jdk.jpackage.test.Annotations.Parameters;
+
+import java.util.Arrays;
 import java.util.List;
 import java.util.function.Predicate;
 import java.util.stream.Stream;
-import java.util.stream.Collectors;
 import jdk.jpackage.test.Executor;
 
+import static jdk.jpackage.test.WindowsHelper.getTempDirectory;
+
 /*
  * @test
  * @summary Custom l10n of msi installers in jpackage
- * @library /test/jdk/tools/jpackage/helpers
+ * @library ../helpers
  * @key jpackagePlatformPackage
  * @requires (jpackage.test.SQETest == null)
  * @build jdk.jpackage.test.*
  * @requires (os.family == "windows")
+ * @modules jdk.jpackage/jdk.jpackage.internal
  * @compile WinL10nTest.java
- * @run main/othervm/timeout=1440 -Xmx512m jdk.jpackage.test.Main
+ * @run main/othervm/timeout=360 -Xmx512m jdk.jpackage.test.Main
  *  --jpt-run=WinL10nTest
  */
 
 public class WinL10nTest {
 
     public WinL10nTest(WixFileInitializer wxlFileInitializers[],
-            String expectedCulture, String expectedErrorMessage) {
+            String expectedCulture, String expectedErrorMessage,
+            String userLanguage, String userCountry,
+            boolean enableWixUIExtension) {
         this.wxlFileInitializers = wxlFileInitializers;
         this.expectedCulture = expectedCulture;
         this.expectedErrorMessage = expectedErrorMessage;
+        this.userLanguage = userLanguage;
+        this.userCountry = userCountry;
+        this.enableWixUIExtension = enableWixUIExtension;
     }
 
     @Parameters
     public static List<Object[]> data() {
         return List.of(new Object[][]{
-            {null, "en-us", null},
+            {null, "en-us", null, null, null, false},
+            {null, "en-us", null, "en", "US", false},
+            {null, "en-us", null, "en", "US", true},
+            {null, "de-de", null, "de", "DE", false},
+            {null, "de-de", null, "de", "DE", true},
+            {null, "ja-jp", null, "ja", "JP", false},
+            {null, "ja-jp", null, "ja", "JP", true},
+            {null, "zh-cn", null, "zh", "CN", false},
+            {null, "zh-cn", null, "zh", "CN", true},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("a.wxl", "en-us")
-            }, "en-us", null},
+            }, "en-us", null, null, null, false},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("a.wxl", "fr")
-            }, "fr;en-us", null},
+            }, "fr;en-us", null, null, null, false},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("a.wxl", "fr"),
                 WixFileInitializer.create("b.wxl", "fr")
-            }, "fr;en-us", null},
+            }, "fr;en-us", null, null, null, false},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("a.wxl", "it"),
                 WixFileInitializer.create("b.wxl", "fr")
-            }, "it;fr;en-us", null},
+            }, "it;fr;en-us", null, null, null, false},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("c.wxl", "it"),
                 WixFileInitializer.create("b.wxl", "fr")
-            }, "fr;it;en-us", null},
+            }, "fr;it;en-us", null, null, null, false},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("a.wxl", "fr"),
                 WixFileInitializer.create("b.wxl", "it"),
                 WixFileInitializer.create("c.wxl", "fr"),
                 WixFileInitializer.create("d.wxl", "it")
-            }, "fr;it;en-us", null},
+            }, "fr;it;en-us", null, null, null, false},
             {new WixFileInitializer[] {
                 WixFileInitializer.create("c.wxl", "it"),
                 WixFileInitializer.createMalformed("b.wxl")
-            }, null, null}
+            }, null, null, null, null, false},
+            {new WixFileInitializer[] {
+                WixFileInitializer.create("MsiInstallerStrings_de.wxl", "de")
+            }, "en-us", null, null, null, false}
         });
     }
 
-    private final static Stream<String> getLightCommandLine(
+    private static Stream<String> getLightCommandLine(
             Executor.Result result) {
         return result.getOutput().stream().filter(s -> {
             s = s.trim();
@@ -101,8 +121,16 @@ public class WinL10nTest {
         });
     }
 
+    private static List<TKit.TextStreamVerifier> createDefaultL10nFilesLocVerifiers(Path tempDir) {
+        return Arrays.stream(DEFAULT_L10N_FILES).map(loc ->
+                TKit.assertTextStream("-loc " + tempDir.resolve(
+                        String.format("config/MsiInstallerStrings_%s.wxl", loc)).normalize()))
+                .toList();
+    }
+
     @Test
     public void test() throws IOException {
+        final Path tempRoot = TKit.createTempDirectory("tmp");
 
         final boolean allWxlFilesValid;
         if (wxlFileInitializers != null) {
@@ -119,6 +147,26 @@ public class WinL10nTest {
             // 1. Set fake run time to save time by skipping jlink step of jpackage.
             // 2. Instruct test to save jpackage output.
             cmd.setFakeRuntime().saveConsoleOutput(true);
+
+            // Set JVM default locale that is used to select primary l10n file.
+            if (userLanguage != null) {
+                cmd.addArguments("-J-Duser.language=" + userLanguage);
+            }
+            if (userCountry != null) {
+                cmd.addArguments("-J-Duser.country=" + userCountry);
+            }
+
+            // Cultures handling is affected by the WiX extensions used.
+            // By default only WixUtilExtension is used, this flag
+            // additionally enables WixUIExtension.
+            if (enableWixUIExtension) {
+                cmd.addArgument("--win-dir-chooser");
+            }
+
+            // Preserve config dir to check the set of copied l10n files.
+            Path tempDir = getTempDirectory(cmd, tempRoot);
+            Files.createDirectories(tempDir.getParent());
+            cmd.addArguments("--temp", tempDir.toString());
         })
         .addBundleVerifier((cmd, result) -> {
             if (expectedCulture != null) {
@@ -134,8 +182,14 @@ public class WinL10nTest {
             if (wxlFileInitializers != null) {
                 if (allWxlFilesValid) {
                     for (var v : wxlFileInitializers) {
-                        v.createCmdOutputVerifier(resourceDir).apply(
-                                getLightCommandLine(result));
+                        if (!v.name.startsWith("MsiInstallerStrings_")) {
+                            v.createCmdOutputVerifier(resourceDir).apply(
+                                    getLightCommandLine(result));
+                        }
+                    }
+                    Path tempDir = getTempDirectory(cmd, tempRoot).toAbsolutePath();
+                    for (var v : createDefaultL10nFilesLocVerifiers(tempDir)) {
+                        v.apply(getLightCommandLine(result));
                     }
                 } else {
                     Stream.of(wxlFileInitializers)
@@ -168,9 +222,12 @@ public class WinL10nTest {
         test.run();
     }
 
-    final private WixFileInitializer wxlFileInitializers[];
+    final private WixFileInitializer[] wxlFileInitializers;
     final private String expectedCulture;
     final private String expectedErrorMessage;
+    final private String userLanguage;
+    final private String userCountry;
+    final private boolean enableWixUIExtension;
     private Path resourceDir;
 
     private static class WixFileInitializer {
@@ -221,7 +278,7 @@ public class WinL10nTest {
 
         TKit.TextStreamVerifier createCmdOutputVerifier(Path root) {
             return TKit.assertTextStream(
-                    root.resolve(name).toAbsolutePath().toString());
+                    "-loc " + root.resolve(name).toAbsolutePath().normalize());
         }
 
         boolean isValid() {
@@ -236,4 +293,6 @@ public class WinL10nTest {
         private final String name;
         private final String culture;
     }
+
+    private static final String[] DEFAULT_L10N_FILES = { "de", "en", "ja", "zh_CN" };
 }
