@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@
  * @bug 8141492 8071982 8141636 8147890 8166175 8168965 8176794 8175218 8147881
  *      8181622 8182263 8074407 8187521 8198522 8182765 8199278 8196201 8196202
  *      8184205 8214468 8222548 8223378 8234746 8241219 8254627 8247994 8263528
- *      8266808
+ *      8266808 8248863
  * @summary Test the search feature of javadoc.
  * @library ../../lib
  * @modules jdk.javadoc/jdk.javadoc.internal.tool
@@ -64,7 +64,8 @@ public class TestSearch extends JavadocTester {
                 "module-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -87,7 +88,8 @@ public class TestSearch extends JavadocTester {
                 "module-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -108,7 +110,8 @@ public class TestSearch extends JavadocTester {
                 "member-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -129,7 +132,9 @@ public class TestSearch extends JavadocTester {
                 "type-search-index.js",
                 "index-all.html",
                 "allpackages-index.html",
-                "allclasses-index.html");
+                "allclasses-index.html",
+                "search-page.js",
+                "search.html");
     }
 
     @Test
@@ -150,7 +155,8 @@ public class TestSearch extends JavadocTester {
                 "member-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -170,7 +176,9 @@ public class TestSearch extends JavadocTester {
                 "package-search-index.js",
                 "tag-search-index.js",
                 "type-search-index.js",
-                "index-all.html");
+                "index-all.html",
+                "search-page.js",
+                "search.html");
     }
 
     @Test
@@ -190,7 +198,8 @@ public class TestSearch extends JavadocTester {
                 "member-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -211,7 +220,8 @@ public class TestSearch extends JavadocTester {
                 "member-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -233,7 +243,8 @@ public class TestSearch extends JavadocTester {
                 "member-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -255,7 +266,8 @@ public class TestSearch extends JavadocTester {
                 "module-search-index.js",
                 "package-search-index.js",
                 "tag-search-index.js",
-                "type-search-index.js");
+                "type-search-index.js",
+                "search.html");
     }
 
     @Test
@@ -411,7 +423,7 @@ public class TestSearch extends JavadocTester {
                     <link rel="stylesheet" type="text/css" href="script-dir/jquery-ui.min.css" title="Style">
                     """,
                 """
-                    <script type="text/javascript" src="script-dir/jquery-3.5.1.min.js"></script>
+                    <script type="text/javascript" src="script-dir/jquery-3.6.0.min.js"></script>
                     """,
                 """
                     <script type="text/javascript" src="script-dir/jquery-ui.min.js"></script>""",
@@ -420,7 +432,7 @@ public class TestSearch extends JavadocTester {
                     loadScripts(document, 'script');""",
                 "<div class=\"nav-list-search\">",
                 """
-                    <label for="search-input">SEARCH:</label>
+                    <div class="nav-list-search"><a href="search.html">SEARCH</a>
                     <input type="text" id="search-input" disabled placeholder="Search">
                     <input type="reset" id="reset-button" disabled value="reset">
                     """);
@@ -683,22 +695,9 @@ public class TestSearch extends JavadocTester {
     void checkJqueryAndImageFiles(boolean expectedOutput) {
         checkFiles(expectedOutput,
                 "search.js",
-                "jquery-ui.overrides.css",
-                "script-dir/jquery-3.5.1.min.js",
+                "script-dir/jquery-3.6.0.min.js",
                 "script-dir/jquery-ui.min.js",
                 "script-dir/jquery-ui.min.css",
-                "script-dir/jquery-ui.structure.min.css",
-                "script-dir/images/ui-bg_glass_65_dadada_1x400.png",
-                "script-dir/images/ui-icons_454545_256x240.png",
-                "script-dir/images/ui-bg_glass_95_fef1ec_1x400.png",
-                "script-dir/images/ui-bg_glass_75_dadada_1x400.png",
-                "script-dir/images/ui-bg_highlight-soft_75_cccccc_1x100.png",
-                "script-dir/images/ui-icons_888888_256x240.png",
-                "script-dir/images/ui-icons_2e83ff_256x240.png",
-                "script-dir/images/ui-icons_cd0a0a_256x240.png",
-                "script-dir/images/ui-bg_glass_55_fbf9ee_1x400.png",
-                "script-dir/images/ui-icons_222222_256x240.png",
-                "script-dir/images/ui-bg_glass_75_e6e6e6_1x400.png",
                 "resources/x.png",
                 "resources/glass.png");
     }
@@ -709,29 +708,13 @@ public class TestSearch extends JavadocTester {
                 "##REPLACE:");
 
         checkOutput("search.js", true,
-                "function searchIndexWithMatcher(indexArray, matcher, category, nameFunc) {",
-                """
-                    function getURLPrefix(ui) {
-                        var urlPrefix="";
-                        var slash = "/";
-                        if (ui.item.category === catModules) {
-                            return ui.item.l + slash;
-                        } else if (ui.item.category === catPackages && ui.item.m) {
-                            return ui.item.m + slash;
-                        } else if (ui.item.category === catTypes || ui.item.category === catMembers) {
-                            if (ui.item.m) {
-                                urlPrefix = ui.item.m + slash;
-                            } else {
-                                $.each(packageSearchIndex, function(index, item) {
-                                    if (item.m && ui.item.p === item.l) {
-                                        urlPrefix = item.m + slash;
-                                    }
-                                });
-                            }
-                        }
-                        return urlPrefix;
-                    }""",
-                "url += ui.item.l;");
+                "function searchIndex(indexArray, category) {",
+                "function getURLPrefix(item, category) {",
+                "url += item.l;");
+
+        checkOutput("search-page.js", true,
+                "function renderResults(result) {",
+                "function selectTab(category) {");
 
         checkCssClasses("search.js", "stylesheet.css");
     }
