@@ -201,21 +201,11 @@ class APITest {
             "help-doc.html",
             "index-all.html",
             "index.html",
-            "script-dir/jquery-3.5.1.min.js",
+            "script-dir/jquery-3.6.0.min.js",
             "script-dir/jquery-ui.min.js",
             "script-dir/jquery-ui.min.css",
-            "script-dir/jquery-ui.structure.min.css",
-            "script-dir/images/ui-bg_glass_65_dadada_1x400.png",
-            "script-dir/images/ui-icons_454545_256x240.png",
-            "script-dir/images/ui-bg_glass_95_fef1ec_1x400.png",
-            "script-dir/images/ui-bg_glass_75_dadada_1x400.png",
-            "script-dir/images/ui-bg_highlight-soft_75_cccccc_1x100.png",
-            "script-dir/images/ui-icons_888888_256x240.png",
-            "script-dir/images/ui-icons_2e83ff_256x240.png",
-            "script-dir/images/ui-icons_cd0a0a_256x240.png",
-            "script-dir/images/ui-bg_glass_55_fbf9ee_1x400.png",
-            "script-dir/images/ui-icons_222222_256x240.png",
-            "script-dir/images/ui-bg_glass_75_e6e6e6_1x400.png",
+            "search.html",
+            "search-page.js",
             "member-search-index.js",
             "module-search-index.js",
             "overview-tree.html",
@@ -228,7 +218,6 @@ class APITest {
             "resources/x.png",
             "script.js",
             "search.js",
-            "jquery-ui.overrides.css",
             "stylesheet.css",
             "tag-search-index.js",
             "type-search-index.js"
@@ -240,7 +229,8 @@ class APITest {
                          && !s.endsWith("-search-index.js")
                          && !s.equals("index-all.html")
                          && !s.equals("search.js")
-                         && !s.equals("jquery-ui.overrides.css")
+                         && !s.equals("search.html")
+                         && !s.equals("search-page.js")
                          && !s.equals("allclasses-index.html")
                          && !s.equals("allpackages-index.html")
                          && !s.equals("system-properties.html"))
