@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -1142,6 +1142,12 @@ public class DPrinter {
             return visitBlockTag(node, null);
         }
 
+        public Void visitSpec(SpecTree node, Void p) {
+            printDocTree("url", node.getURL());
+            printList("title", node.getTitle());
+            return visitBlockTag(node, null);
+        }
+
         public Void visitStartElement(StartElementTree node, Void p) {
             printName("name", node.getName());
             printList("attrs", node.getAttributes());
@@ -1186,6 +1192,7 @@ public class DPrinter {
         }
 
         public Void visitValue(ValueTree node, Void p) {
+            printDocTree("format", node.getFormat());
             printDocTree("value", node.getReference());
             return visitInlineTag(node, null);
         }
