@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -752,7 +752,7 @@ public class ToolBox {
         private final static Pattern packagePattern =
                 Pattern.compile("package\\s+(((?:\\w+\\.)*)\\w+)");
         private final static Pattern classPattern =
-                Pattern.compile("(?:public\\s+)?(?:class|enum|interface|record)\\s+(\\w+)");
+                Pattern.compile("(?:public\\s+)?(?:class|enum|interface|record)\\s+((\\w|\\$)+)");
 
         /**
          * Extracts the Java file name from the class declaration.
