@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -82,12 +82,10 @@ public class EventNames {
     public final static String OldObjectSample = PREFIX + "OldObjectSample";
     public final static String SymbolTableStatistics = PREFIX + "SymbolTableStatistics";
     public final static String StringTableStatistics = PREFIX + "StringTableStatistics";
-    public final static String PlaceholderTableStatistics = PREFIX + "PlaceholderTableStatistics";
-    public final static String LoaderConstraintsTableStatistics = PREFIX + "LoaderConstraintsTableStatistics";
-    public final static String ProtectionDomainCacheTableStatistics = PREFIX + "ProtectionDomainCacheTableStatistics";
     public static final String RedefineClasses = PREFIX + "RedefineClasses";
     public static final String RetransformClasses = PREFIX + "RetransformClasses";
     public static final String ClassRedefinition = PREFIX + "ClassRedefinition";
+    public static final String FinalizerStatistics = PREFIX + "FinalizerStatistics";
 
     // This event is hard to test
     public final static String ReservedStackActivation = PREFIX + "ReservedStackActivation";
@@ -148,6 +146,7 @@ public class EventNames {
     public final static String ZUnmap = PREFIX + "ZUnmap";
     public final static String GCLocker = PREFIX + "GCLocker";
     public static final String SystemGC = PREFIX + "SystemGC";
+    public static final String GCCPUTime = PREFIX + "GCCPUTime";
 
     // Compiler
     public final static String Compilation = PREFIX + "Compilation";
@@ -158,14 +157,12 @@ public class EventNames {
     public final static String CompilerConfiguration = PREFIX + "CompilerConfiguration";
     public final static String CodeCacheStatistics = PREFIX + "CodeCacheStatistics";
     public final static String CodeCacheConfiguration = PREFIX + "CodeCacheConfiguration";
-    public final static String CodeSweeperStatistics = PREFIX + "CodeSweeperStatistics";
-    public final static String CodeSweeperConfiguration = PREFIX + "CodeSweeperConfiguration";
-    public final static String SweepCodeCache = PREFIX + "SweepCodeCache";
     public final static String CodeCacheFull = PREFIX + "CodeCacheFull";
     public final static String ObjectAllocationInNewTLAB = PREFIX + "ObjectAllocationInNewTLAB";
     public final static String ObjectAllocationOutsideTLAB = PREFIX + "ObjectAllocationOutsideTLAB";
     public final static String ObjectAllocationSample = PREFIX + "ObjectAllocationSample";
     public final static String Deoptimization = PREFIX + "Deoptimization";
+    public final static String JitRestart = PREFIX + "JitRestart";
 
     // OS
     public final static String OSInformation = PREFIX + "OSInformation";
@@ -196,8 +193,13 @@ public class EventNames {
     public final static String X509Certificate = PREFIX + "X509Certificate";
     public final static String X509Validation = PREFIX + "X509Validation";
     public final static String SecurityProperty = PREFIX + "SecurityPropertyModification";
+    public final static String SecurityProviderService = PREFIX + "SecurityProviderService";
     public final static String DirectBufferStatistics = PREFIX + "DirectBufferStatistics";
     public final static String Deserialization = PREFIX + "Deserialization";
+    public static final String VirtualThreadStart = PREFIX + "VirtualThreadStart";
+    public static final String VirtualThreadEnd = PREFIX + "VirtualThreadEnd";
+    public static final String VirtualThreadPinned = PREFIX + "VirtualThreadPinned";
+    public static final String VirtualThreadSubmitFailed = PREFIX + "VirtualThreadSubmitFailed";
 
     // Containers
     public static final String ContainerConfiguration = PREFIX + "ContainerConfiguration";
