@@ -36,18 +36,6 @@
 #include <unistd.h>
 #endif
 
-#include "jvmti.h"
-
-/**
- * Additional Java basic types
- */
-
-#ifdef _WIN32
-    typedef unsigned __int64 julong;
-#else
-    typedef unsigned long long julong;
-#endif
-
 #define LOG(...) \
   { \
     printf(__VA_ARGS__); \
@@ -63,14 +51,8 @@ const char* TranslateError(jvmtiError err);
 static jvmtiExtensionFunction GetVirtualThread_func = NULL;
 static jvmtiExtensionFunction GetCarrierThread_func = NULL;
 
-/**
- * Convert the digits of the given value argument to a null-terminated
- * character string and store the result (up to 32 bytes) in string.
- * If value is negative, the first character of the stored string is
- * the minus sign (-). The function returns a pointer to the begining
- * of the result string.
- */
-char* jlong_to_string(jlong value, char *string) {
+char*
+jlong_to_string(jlong value, char *string) {
   char buffer[32];
   char *pbuf, *pstr;
 
@@ -96,33 +78,6 @@ char* jlong_to_string(jlong value, char *string) {
   return string;
 }
 
-/**
- * Convert the digits of the given value argument to a null-terminated
- * character string and store the result (up to 32 bytes) in string.
- * The function returns a pointer to the begining of the result string.
- */
-char* julong_to_string(julong value, char *string) {
-    char buffer[32];
-    char *pbuf, *pstr;
-
-    pstr = string;
-    if (value == 0) {
-        *pstr++ = '0';
-    } else {
-        pbuf = buffer;
-        while (value != 0) {
-            *pbuf++ = '0' + (char)(value % 10);
-            value = value / 10;
-        }
-        while (pbuf != buffer) {
-            *pstr++ = *--pbuf;
-        }
-    }
-    *pstr = '\0';
-
-    return string;
-}
-
 static void
 fatal(JNIEnv* jni, const char* msg) {
   jni->FatalError(msg);
@@ -245,8 +200,8 @@ print_thread_info(jvmtiEnv *jvmti, JNIEnv* jni, jthread thread_obj) {
   check_jvmti_status(jni, jvmti->GetThreadInfo(thread_obj, &thread_info), "Error in GetThreadInfo");
   check_jvmti_status(jni, jvmti->GetThreadState(thread_obj, &thread_state), "Error in GetThreadInfo");
   const char* state = TranslateState(thread_state);
-  LOG("Thread: %p, name: %s, state(%x): %s, attr: %s\n", thread_obj, thread_info.name, thread_state, TranslateState(thread_state),
-         (thread_info.is_daemon ? "daemon": ""));
+  LOG("Thread: %p, name: %s, state(%x): %s, attrs: %s %s\n", thread_obj, thread_info.name, thread_state, TranslateState(thread_state),
+         (jni->IsVirtualThread(thread_obj)? "virtual": "kernel"), (thread_info.is_daemon ? "daemon": ""));
 }
 
 static void
@@ -294,16 +249,7 @@ get_thread_name(jvmtiEnv *jvmti, JNIEnv* jni, jthread thread) {
   }
   check_jvmti_status(jni, err, "get_thread_name: error in JVMTI GetThreadInfo call");
 
-  static const char* UNNAMED_STR = "<Unnamed thread>";
-  static size_t UNNAMED_LEN = strlen(UNNAMED_STR);
-  char* tname = thr_info.name;
-  if (tname == NULL) {
-    err = jvmti->Allocate((jlong)(UNNAMED_LEN + 1), (unsigned char**)&tname);
-    check_jvmti_status(jni, err, "get_method_class_name: error in JVMTI Allocate");
-    strncpy(tname, UNNAMED_STR, UNNAMED_LEN);
-    tname[UNNAMED_LEN] = '\0';
-  }
-  return tname;
+  return thr_info.name == NULL ? (char*)"<Unnamed thread>" : thr_info.name;
 }
 
 static char*
